#ifndef IA_HDR_H
#define	IA_HDR_H

#define COLOR_CONTROL_VALID 0x0001
#define COLOR_CONTROL_UPDATED 0x0002
#define COLOR_CONTROL_OVERWRITE 0x0004
struct color_control_rec
   {
    unsigned short status;
    int lb,ub;  /* if range is set lb = ub = range / 2 */
    unsigned long first_color_index,num_colors;
    int mid_value,range;
   };

struct patient_rec
    {char *name;
     char *ssnum;
     char *patient_id;  /* medical record# */
     char *study_num;    /* study # */
     char *patient_num;  /* patient # */
     char sex;
     int age;
     int height; /* in mm */
     int weight; /* in grams */
    };

/*  char *image_id */
#define MRI "MRI"
#define SPECT "SPECT"
#define PET "PET"
#define FMRI "FMRI"
#define PIXEL_TYPE "PIXEL TYPE"
#define PIXEL_COUNT "PIXEL COUNT"
struct data_info_rec
   {
    char *image_id;
    char *default_datafn;
    unsigned long num_scans; /* old = num_dims */
                  /* used as num_scans when NUM_SCANS_USED in var_use_status
                     is set.
                     Ex. for activation study like fmri, this
                     field is used to indicate the # of scans over time
                     associated with each slice */
    unsigned long dimx,dimy;
    unsigned long max_scans;
    unsigned long pixval_max,pixval_min;
    long actual_pixval_min;  /* old = num_vols */
 
    unsigned long histo_dimx,histo_dimy;
    unsigned long pixel_depth_bits;
    float xpixel_size,ypixel_size;
    float pixel_thickness; /* thickness + spacing */
    float spacing;
    short plane_type;  
          /* -1 = unknown,0 = axial, 1 = sagittal, 2 = coronal, 3 = oblique */
    float trans_matrix[4][4]; /* unused yet */
    short unused2;   /* old = pulse_sequence */
    unsigned long var_use_status;     /* old = orientation &  position */
#define ACTUAL_PIXVAL_MIN_USED 0x00000001
#define NUM_SCANS_USED 0x00000002
#define UNUSED2_USED 0x00000004
#define UNUSED3_USED 0x00000008
#define UNUSED4_USED 0x00000010
#define UNUSED5_USED 0x00000020
#define UNUSED6_USED 0x00000040
#define UNUSED7_USED 0x00000080
#define TRANS_MATRIX_USED 0x00000100
#define _3D_IMAGE_ORDER_SET 0x00000200
#define _3D_IMAGE_BOTTOM_TO_TOP 0x00000400
#define INPUT_DATA_FLOAT 0x00001000
#define INPUT_DATA_DOUBLE 0x00002000
#define INPUT_DATA_SIGNED 0x00004000
#define POS_NEG_ACTIVATION 0x00010000

#define INPUT_DATA_TYPE 0x00007000
    short num_echoes;
    short unused3; /* old = flip_angle */
    short unused4; /* old = scan_dimx */
    short unused5; /* old = scan_dimy */

    float unused6;    /* old = field_of_view */
    float unused7;    /* old = num_excitations */
    float tr;    /* repetition/Recovery time */
    char *te;    /* echo delay */

    char *date_of_study;
    char *time_of_study;

    struct patient_rec patient;

    char *refered_by;
    char *description;
    char *hosp_name;

   };


struct coord_3d_rec
   {
    float x,y,z;
   };

struct slice_plane_rec
   {
    struct coord_3d_rec vector;
    float constant;
   };

#define SLICE_ORIGINAL 0x0001
#define SLICE_BUILT 0x0002
#define SLICE_FIRST 0x0004
#define SLICE_FIRST_ECHO 0x0008
#define SLICE_HISTOGRAM_1D 0x0010
#define SLICE_HISTOGRAM_2D 0x0020
#define SLICE_CHECKED 0x4000
#define SLICE_PROCESSED 0x8000
struct hdr_scan_entry_rec
   {
    unsigned short status;
    char *img_label;
    unsigned long num;   /* slice # - starting with 1 */
    unsigned long img_addr;

    struct slice_plane_rec plane;

    struct hdr_scan_entry_rec *prior_slice;
    struct hdr_scan_entry_rec *next_slice;
    struct hdr_scan_entry_rec *partner_slice;

/* the following variables are computed, not read from the header file.
   user defined for associating the slice with other user defined slice
        attributes  */
    unsigned long slice_num,scan_num;
    unsigned char echo_num;
    struct slice_attr_rec *slice;
   };

struct analysis_group_rec
   {
    unsigned long num_members;
    unsigned long *member_img_label_indx;
   };

struct hdr_analysis_rec
   {
    unsigned long num_img_labels;
    char **img_labels;
    unsigned long num_analysis_groups;
    struct analysis_group_rec *groups;
   };

struct input_field_offset_rec
   {
    unsigned long info_start,info_size;
    unsigned long extra_info_start,extra_info_size;
    unsigned long comment_start,comment_size;
    unsigned long user_defined_start,user_defined_size;
   };

struct extra_file_info_rec
   {char *color_bundle_label;
    struct color_control_rec color_control; 

    char *last_edit_begin;
    char *last_edit_end;
   };

/* id options */
#define HDR_ID_V1 "H1.0"
#define HDR_ID_V1 "H2.0"
#define DAT_ID_V1 "D1.0"

/* unsigned short status */
#define _1_ECHO_IMAGE 0x0001
#define _2_ECHO_IMAGE 0x0002
#define _2_ECHO_INDIVIDUAL_MIN_MAX 0x1000
#define GLOBAL_MIN_MAX_UNDEFINED 0x2000
       /* if (!(hdr.status & GLOBAL_MIN_MAX_UNDEFINED))
             if the data_info.pixval_min >= data_info.pixval_max then
                hdr.status |= GLOBAL_MIN_MAX_UNDEFINED will be set by the
                program
          if (hdr.status & GLOBAL_MIN_MAX_UNDEFINED)
             the slice pixels will be displayed based on the slice MIN & MAX
             by default
       */

struct hdr_rec
   {
    unsigned long size;
    char *id;
    struct input_field_offset_rec offset;
    unsigned short status;
    struct data_info_rec data_info;
    struct hdr_analysis_rec analysis;
    unsigned long num_scans;
    struct hdr_scan_entry_rec *scans;
    struct extra_file_info_rec extra_info;
    long comment_size;
    char *comment;
    long user_defined_size;
    unsigned char *user_defined;
   };

#endif IA_HDR_H
