/****************************************************************************
Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.
****************************************************************************/

#ifndef GEOMETRY_H
#define	GEOMETRY_H
#define RADIAN (double) 3.141592654
#define NO_INTERSECTION 0
#define ONE_INTERSECTION 1
#define PLANE_LINE 2

struct line_rec
   {
    float gradient_x;
    float gradient_y;
    float intercept;
   };

struct int_pt_rec
   {
    int x,y;
   };

struct line_segment_rec
   {
    struct line_rec eq;
    struct int_pt_rec start,end;
   };

struct point_rec
   {
    long x,y;
   };

#define VECTOR_DIRECTION_COSINE 1
#define VECTOR_ORIGIN 2
#define VECTOR_X_PLANE 4
#define VECTOR_Y_PLANE 8
#define VECTOR_Z_PLANE 16
#define VECTOR_SLOPE_PLANE 32

#define VECTOR_RIGHT_ANGLE_PLANE 0x0000001c
struct vector_3d_rec
   {
    unsigned long status;
    float x,y,z;
   };

#define MATRIX_ROTATED 1
#define MATRIX_TRANSLATED 2
#define MATRIX_SCALED 4
#define MATRIX_INITIALIZED 8

#define MATRIX_DIRTY 7
struct matrix_2d_rec
   {
    unsigned long status;
    float buf[3][3];
   };

struct matrix_3d_rec
   {
    unsigned long status;
    float buf[4][4];
   };

#define PLANE_FROM_3_PTS 1
#define PLANE_LINE 2
#define PLANE_BOUNDED 4
#define PLANE_CONST_COMPUTED 8

#define PLANE_BOUND_VERTICES_COMPUTED 16
#define PLANE_P1_VALID 32
#define PLANE_P2_VALID 64
#define PLANE_P3_VALID 128

#define PLANE_FROM_PT_NORMAL 0x00000100
#define PLANE_PROJ_DIM_VALID 0x00000200
#define PLANE_SAME_COORD_AFTER_RESLICE 0x00000400
#define PLANE_ADJUST_TO_CENTER 0x00000800

#define PLANE_ROTATION_RAD_VALID 0x00001000
#define PLANE_PROJ_BUF_DIM_VALID 0x00002000
#define PLANE_BASE_VALID 0x00004000
#define PLANE_COORD_RATIO_VALID 0x00008000

#define PLANE_TRANSLATE_VALID 0x00010000
#define PLANE_SCALE_VALID 0x00020000
#define PLANE_ROTATE_VALID 0x00040000
#define PLANE_ACTUAL_PROJ_DIM_VALID 0x00080000

#define PLANE_ACTUAL_VALID 0x00100000
#define PLANE_TRANS_USED 0x00200000
#define PLANE_PLANE_PROJ_VALID 0x00400000
#define PLANE_CENTER_DEFAULT 0x00800000
#define PLANE_PLANE_GRAVITY_VALID 0x01000000
#define PLANE_TRANS_VALID 0x02000000
#define PLANE_TRANS_PROJ_DIM_VALID 0x04000000
#define PLANE_APPROX_VALUE 0x08000000

#define PLANE_PLANE_GRAVITY_IDENTITY 0x10000000
#define PLANE_TRANS_IDENTITY 0x20000000
#define PLANE_INCRE_LOW_TO_HIGH 0x40000000
#define PLANE_NORMAL_REDEFINED 0x80000000

#define PLANE_PROJ_DIM_DATA_OK 0x00480204
#define PLANE_PROJ_READY 0x00400000


#define PLANE_M_PARAMS_COMPUTED 0x00000001
#define PLANE_REVERSED 0x00000002
#define PLANE_ADJUST_TO_CENTER_DURING_NEXT_CHANGE 0x00000004
struct plane_3d_rec
   {
    struct vector_3d_rec base,actual_base,p2,p3;
    unsigned long status;
    unsigned long more_status;
    struct vector_3d_rec normal,normal_cosine,coord_ratio;
    float constk;

    struct vector_3d_rec global_start,global_end;
    struct vector_3d_rec local_start,local_end;
    struct vector_3d_rec center;
    struct vector_3d_rec bound_vertices[4];

    struct matrix_3d_rec global;
    struct matrix_3d_rec xyztoplane;
    struct matrix_3d_rec plane_trans;
    struct matrix_3d_rec planetoxyz;
    float obl_deg,obj_proj_deg;
    struct matrix_3d_rec obl,obl_proj;

/*
   select_plane_gravity = 1 use the bottom left corner as the origin
                        = 2 use the bottom right corner as the origin
                        = 4 use the top right corner as the origin
                        = 8 use the top left corner as the origin
     1,2,4,8 are mutually exclusive and they can co-exist with 16
                        = 16 reflection about the line y = x
*/
#define GY_REFLECTION 1
#define GYX_REFLECTION 2
#define GX_REFLECTION 4
#define GY_X_REFLECTION 8
#define GNO_REFLECTION 16
#define GDEFAULT 32
    unsigned char plane_gravity_status;
    struct matrix_3d_rec plane_gravity;
    struct matrix_3d_rec trans;

    long firstx,lastx,firsty,lasty;
    long proj_dimx,proj_dimy;
    long actual_proj_dimx,actual_proj_dimy;
    long trans_proj_dimx,trans_proj_dimy;
    long proj_buf_dimx,proj_buf_dimy;

    struct vector_3d_rec translate,scale,rotate;

    struct vector_3d_rec plane_0_vector,plane_max_vector;
    float move_unit_ratio;
    long total_planes,plane_num;
   };

#define PI_RADIAN (double) 3.141592654

#define LINE_NO_PARALLEL 0
#define LINE_X_EQUAL_K_SAME 1
#define LINE_X_EQUAL_K_PARALLEL 2
#define LINE_Y_EQUAL_C_SAME 3
#define LINE_Y_EQUAL_C_PARALLEL 4
#define LINE_PARALLEL 5


#define LINE_SEGMENT_NOT_INTERSECTING 0
#define LINE_SEGMENT_EQUAL 1
#define LINE_SEGMENT_INTERSECTS 2
#define LINE_SEGMENT_PARALLEL 3

#endif GEOMETRY_H
