/****************************************************************************
Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.
****************************************************************************/


#ifndef SLICE_ROI_REC_H
#define	SLICE_ROI_REC_H
#include "geometry.h"
#include "rect.h"

/* 
for simplicity, once a circle is drawn, it is treated as a series of points...
*/
/*
struct coordinate_xy_rec
   {
    short x,y;
   };

struct bound_rec
   {
    short left,top,right,bottom;
   };

#define DRAW_FREE_CLOSED 0x0001
#define DRAW_FREE 0x0002
#define DRAW_LINE 0x0004
#define DRAW_POLYGON 0x0008
#define DRAW_CIRCLE 0x0010
#define DRAW_RECT 0x0020
#define DRAW_POINT 0x0040
#define DRAW_ELLIPSE 0x0080

#define CURVE_OR 0x0800

#define COORDINATE_BOUND_COMPUTED 0x1000
#define CURVE_NEVER_CURRENT 0x2000
#define CURVE_MATRIX_IN_USE 0x4000
#define CURVE_BOUND_TRANSFORMED 0x8000

#define DRAW_MODE_BACKSPACE_ALL 0x00B0
#define DRAW_FORCE_CLOSED 0x000f
#define DRAW_MODE 0x00ff
#define DRAW_CIRCLE_OR_ELLIPSE 0x0090
#define DRAW_CIRCLE_OR_ELLIPSE_OR_RECT 0x00B0
#define DRAW_MODE_COMPATIBLE 0x000F

struct coordinate_2d_rec
   {
    unsigned short status,more_status;
    unsigned long total_xys;
    unsigned long num_xys;
    struct bound_rec bound;
    struct coordinate_xy_rec *pts;
    struct matrix_2d_rec matrix,inv;
   };
*/

#define ALLOCATE_MAX_XY_PTS 50
struct float_xy_rec
   {
    float x,y;
   };


#define DRAW_FREE_CLOSED 0x0001
#define DRAW_FREE 0x0002
#define DRAW_LINE 0x0004
#define DRAW_POLYGON 0x0008
#define DRAW_CIRCLE 0x0010
#define DRAW_RECT 0x0020
#define DRAW_POINT 0x0040
#define DRAW_ELLIPSE 0x0080

#define CURVE_OR 0x0800

#define COORDINATE_BOUND_COMPUTED 0x1000
#define CURVE_NEVER_CURRENT 0x2000
#define CURVE_MATRIX_IN_USE 0x4000
#define CURVE_BOUND_TRANSFORMED 0x8000

#define DRAW_MODE_BACKSPACE_ALL 0x00B0
#define DRAW_FORCE_CLOSED 0x000f
#define DRAW_MODE 0x00ff
#define DRAW_CIRCLE_OR_ELLIPSE 0x0090
#define DRAW_CIRCLE_OR_ELLIPSE_OR_RECT 0x00B0
#define DRAW_MODE_COMPATIBLE 0x000F

struct curve_rec
   {
    unsigned short status;

#define CURVE_NEW 0x0001
    unsigned short more_status;
    unsigned long total_xys;
    unsigned long num_xys;
    struct float_bound_rec fl_bound;
    struct float_xy_rec *xys;
    struct matrix_2d_rec matrix,inv;
   };

/*
value for pen type
*/
struct curve_2d_rec
   {
    unsigned short pen_type;
    unsigned long total_curves;
    unsigned long cur_curve;
    unsigned long num_curves;
    struct curve_rec **curves;
   };

struct user_defined_rec
   {
    void *ptr;
    int (*exit)();
   };

/*
   roi_rec status
*/

/* rois have type HISTO_PIXEL_TYPE_ROI when there are created as a result of
     partitioning the rois which are neither of type HISTO_PIXEL_TYPE_ROI nor
     of type HISTO_EDITED_ROI */
/* rois of type HISTO_EDITED_ROI are created when they are being edited
      and a key
      is pressed to signify they are to be of type HISTO_ROI  -- they have
      higher priority to be used in constructing the pixel type map than rois 
      of type HISTO_PIXEL_TYPE_ROI;  that is their regions will be overwriting
      the HISTO_PIXEL_TYPE_ROI regions.  Among them, the oldest used rois
      rois created by partitioning
      have higher priorities than others
*/

#define OWN_ROI 0x0001
#define ROI_VISIBLE 0x0002
#define FOREIGN_ROI 0x0004
#define ROI_NEVER_CURRENT 0x0008
#define ROI_EXCLUDE 0x0010
#define HISTO_STAT_COMPUTED 0x0100
#define ROI_UNION_CURVE_REGIONS 0x0200
#define ROI_USER_MARKED 0x0400

#define HISTO_SEGMENTED_ROI 0x1000
#define HISTO_EDITED_ROI 0x2000
#define BREAKER_ROI 0x4000
#define ROI_MARKER 0x8000
#define ROI_TYPE 0xf000

/* prog_status */
#define ROI_MUST_UPDATE 0x0001
#define ROI_MARK_DELETE 0x0002
#define ROI_3D_ROI 0x0004
#define ROI_3D_ROI_WID_MARKER 0x0008

struct roi_rec
   {
    char *label;
    unsigned short status,prog_status; 

    unsigned long total_pens;
    unsigned long cur_pen;
    unsigned long num_pens;
    struct curve_2d_rec **pens;
    struct user_defined_rec user;
   };

/*
 for slice_roi_rec status
*/
#define SLICE_ROI_NON_EMPTY 0x001
#define SLICE_ROI_MUST_SAVE 0x0002
#define SLICE_ROI_SHOW_EDITED 0x0004
#define SLICE_ROI_MUST_UPDATE 0x0008
#define SLICE_ROI_DIM_VALID 0x0010
#define SLICE_ROI_MARK_DELETE 0x0020
#define SLICE_ROI_PIXEL_SIZE_VALID 0x0040
#define SLICE_ROI_PROCESSED 0x0080
#define SLICE_ROI_ALREADY_COUNTED 0x1000
#define SLICE_ROI_TO_BE_SAVED 0x000A
/*
prog_status is user defined and used in the program.  Its meaning is
independent from other slice_roi_rec depending solely user defined
interpretation */
struct slice_roi_rec
   {
    unsigned short status,prog_status;
    float xpixel_size,ypixel_size;
    unsigned long dimx,dimy;
    unsigned long total_rois;
    unsigned long cur_roi;
    unsigned long num_rois;
    struct roi_rec **rois;
   };

#endif SLICE_ROI_REC_H
