/****************************************************************************
2dbufvalrec.c - ???

Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.
****************************************************************************/

static const char rcsid[] = "$Id: 2dbufvalrec.c,v 1.7 1999/07/07 18:01:05 hughett Exp $ Release $Name: nria_1_0_6 $";


#include <math.h>
#include <stdio.h>

#include <sys/types.h>

#include "error_constants.h"
#include "buf_2d_val_rec.h"
#include "common.h"
#include "nria_lib.h"

/*
#define TRACE(x) x; fflush(stdout);
*/
#define TRACE(x)


/* Private functions */
extern void init_val_rec(struct val_rec *val);
extern void free_val_rec(struct val_rec *val);
extern void init_buf_2d_val_rec(struct buf_2d_val_rec *pixels);
extern void free_buf_2d_val_rec(struct buf_2d_val_rec *pixels);
extern int add_coord_to_val_rec(struct val_rec *val, unsigned long row, 
    unsigned long col);
extern int setup_all_val_recs(struct buf_2d_val_rec *pixels, 
    struct buf_2d_rec *_2d);
extern int sort_val_rec_proc(struct val_rec *val1, struct val_rec *val2);
extern int setup_val_recs(struct buf_2d_val_rec *pixels, 
    struct buf_2d_rec *_2d);
extern struct val_rec *find_val_rec(struct buf_2d_val_rec *pixels, 
    unsigned long value);
extern int find_val_recs(struct buf_2d_val_rec *pixels, 
    unsigned long start_value, unsigned long end_value, struct val_rec **vals,
    unsigned long *num_vals);
extern int setup_buf_2d_val_rec(struct buf_2d_val_rec *pixels, 
    struct buf_2d_rec *_2d, unsigned long min, unsigned long max);
extern void pr_val_rec(struct val_rec *val);
extern void pr_buf_2d_val_rec(struct buf_2d_val_rec *pixels);



/*------------------------------------------------------------------------
init_val_rec - ???
------------------------------------------------------------------------*/

void
init_val_rec(val)
 struct val_rec *val;
{
 val->total_coords = val->num_coords = 0;
 val->coords = NULL;
}


/*------------------------------------------------------------------------
free_val_rec - ???
------------------------------------------------------------------------*/

void
free_val_rec(val)
 struct val_rec *val;
{
 if (val->total_coords > 0)
    c_free(val->coords);
 init_val_rec(val);
}


/*------------------------------------------------------------------------
init_buf_2d_val_rec - ???
------------------------------------------------------------------------*/

void
init_buf_2d_val_rec(pixels)
 struct buf_2d_val_rec *pixels;
{
 pixels->actual_min = 0xffffffff;
 pixels->actual_max = 0; 
 pixels->pixel_depth = 0; 
 pixels->total_vals = pixels->num_vals = 0;
 pixels->vals = NULL;
}


/*------------------------------------------------------------------------
free_buf_2d_val_rec - ???
------------------------------------------------------------------------*/

void
free_buf_2d_val_rec(pixels)
 struct buf_2d_val_rec *pixels;
{
 unsigned long lcv;

 if (pixels->total_vals > 0)
    {for (lcv = 0; lcv < pixels->total_vals; lcv++)
        free_val_rec(&(pixels->vals[lcv]));
     c_free(pixels->vals);
    }
 init_buf_2d_val_rec(pixels);
}


/*------------------------------------------------------------------------
add_coord_to_val_rec - ???
------------------------------------------------------------------------*/

int
add_coord_to_val_rec(val,row,col)
 struct val_rec *val;
 unsigned long row,col;
{
 int err;

 if (val->total_coords == val->num_coords)
    {get_ptr_memory(&(val->coords),&(val->total_coords),100,
        (size_t) sizeof(*(val->coords)),&err);
     if (err != NO_ERROR) return err;
    }
 val->coords[val->num_coords].x = col;
 val->coords[val->num_coords++].y = row;
 return NO_ERROR;
}


/*------------------------------------------------------------------------
setup_all_val_recs - ???
------------------------------------------------------------------------*/

int
setup_all_val_recs(pixels,_2d)
 struct buf_2d_val_rec *pixels;
 struct buf_2d_rec *_2d;
{
 unsigned long row,col,indx;
 double val;
 int err;

 TRACE(printf("setup_all_val_recs...\n"))
 for (row = 0; row < _2d->dimy; row++)
    for (col = 0; col < _2d->dimx; col++)
       {val = _2d->GetVal(_2d,row,col);
        indx = (unsigned long) rint(val);
        if (indx < pixels->min) indx = pixels->min;
          else if (indx > pixels->max) indx = pixels->max;
        if ((err = add_coord_to_val_rec(&(pixels->vals[indx]),row,col)) !=
                 NO_ERROR) return err;
        if (pixels->actual_min > indx)
           pixels->actual_min = indx;
        if (pixels->actual_max < indx)
           pixels->actual_max = indx;
       }
 TRACE(printf("setup_all_val_recs...end\n"))
 return NO_ERROR;
}


/*------------------------------------------------------------------------
sort_val_rec_proc - ???
------------------------------------------------------------------------*/

int
sort_val_rec_proc(val1,val2)
 struct val_rec *val1,*val2;
{
 if (val1->value > val2->value) return 1;
 if (val1->value < val2->value) return -1;
 return 0;
}


/*------------------------------------------------------------------------
setup_val_recs - ???
------------------------------------------------------------------------*/

int
setup_val_recs(pixels,_2d)
 struct buf_2d_val_rec *pixels;
 struct buf_2d_rec *_2d;
{
 unsigned long row,col,posi,lcv,next;
 double val;
 struct val_rec the_val;
 int err;

 TRACE(printf("setup_val_recs...\n"))
 for (row = 0; row < _2d->dimy; row++)
    for (col = 0; col < _2d->dimx; col++)
       { 
        val = _2d->GetVal(_2d,row,col);
        the_val.value = (unsigned long) rint(val);
        if (the_val.value < pixels->min) the_val.value = pixels->min;
          else if (the_val.value > pixels->max) the_val.value = pixels->max;
        if (obj_binary_search(pixels->vals,pixels->num_vals,
              (size_t) sizeof(the_val),&the_val,sort_val_rec_proc,&posi) ==
                 NOT_FOUND)
           {if (pixels->total_vals == pixels->num_vals)
               {get_ptr_memory(&(pixels->vals),&(pixels->total_vals),100,
                    (size_t) sizeof(*(pixels->vals)),&err);
                if (err != NO_ERROR) return err;
               }
            for (lcv = pixels->num_vals; lcv > posi; )
               {next = lcv - 1;
                pixels->vals[lcv] = pixels->vals[next];
                lcv = next;}
            pixels->vals[posi].value = the_val.value;
            if (pixels->actual_min > the_val.value)
               pixels->actual_min = the_val.value;
            if (pixels->actual_max < the_val.value)
               pixels->actual_max = the_val.value;
           }
        if ((err = add_coord_to_val_rec(&(pixels->vals[posi]),row,col)) !=
                 NO_ERROR) return err;
       }
 TRACE(printf("setup_val_recs...end\n"))
 return NO_ERROR;
}


/*------------------------------------------------------------------------
find_val_rec - ???
------------------------------------------------------------------------*/

struct val_rec *
find_val_rec(pixels,value)
 struct buf_2d_val_rec *pixels;
 unsigned long value;
{
 unsigned long posi;
 struct val_rec the_val;

 if (value < pixels->min) value = pixels->min;
   else if (value > pixels->max) value = pixels->max;
 if (pixels->pixel_depth <= 8)
    {
     if (pixels->vals[value].value != value) return NULL;
     return &(pixels->vals[value]);
    }
 the_val.value = value;
 if (obj_binary_search(pixels->vals,pixels->num_vals,
     (size_t) sizeof(the_val),&the_val,sort_val_rec_proc,&posi) ==
                 NO_ERROR)
    return &(pixels->vals[posi]);
 return NULL;
}


/*------------------------------------------------------------------------
find_val_recs - ???
------------------------------------------------------------------------*/

int
find_val_recs(pixels,start_value,end_value,vals,num_vals)
 struct buf_2d_val_rec *pixels;
 unsigned long start_value,end_value,*num_vals;
 struct val_rec **vals;
{
 unsigned long posi,posi1;
 struct val_rec the_val;
/*
 if (start_value < pixels->min) start_value = pixels->min;
   else if (start_value > pixels->max) start_value = pixels->max;
 if (end_value < pixels->min) end_value = pixels->min;
   else if (end_value > pixels->max) end_value = pixels->max;

 if (pixels->pixel_depth <= 8)
    {*vals = &(pixels->vals[start_value]);
     *num_vals = end_value - start_value + 1;
     return NO_ERROR;
    }
*/
 the_val.value = start_value;
 obj_binary_search(pixels->vals,pixels->num_vals,
     (size_t) sizeof(the_val),&the_val,sort_val_rec_proc,&posi);
 if (posi >= pixels->num_vals)
    {*vals = NULL;
     *num_vals = 0;}
   else
    {*vals = &(pixels->vals[posi]);
     the_val.value = end_value;
     obj_binary_search(pixels->vals,pixels->num_vals,
         (size_t) sizeof(the_val),&the_val,sort_val_rec_proc,&posi1);
     if (posi1 >= pixels->num_vals) posi = pixels->num_vals - 1;
     *num_vals = posi1 - posi + 1;
     if (pixels->vals[*num_vals - 1].value > end_value) *num_vals -= 1;
    }
 return NO_ERROR;

}


/*------------------------------------------------------------------------
setup_buf_2d_val_rec - ???
------------------------------------------------------------------------*/

int
setup_buf_2d_val_rec(pixels,_2d,min,max)
 struct buf_2d_val_rec *pixels;
 struct buf_2d_rec *_2d;
 unsigned long min,max;
{
 unsigned long total_values,lcv;
 int err;

 free_buf_2d_val_rec(pixels);
 if (_2d->pixel_depth <= 8)
    {total_values = num_bit_values(_2d->pixel_depth);
     get_ptr_memory(&(pixels->vals),&(pixels->total_vals),total_values,
         (size_t) sizeof(*(pixels->vals)),&err);
     if (err != NO_ERROR) return err;
     for (lcv = 0; lcv < total_values; lcv++)
        {init_val_rec(&(pixels->vals[lcv]));
         pixels->vals[lcv].value = lcv;}
     pixels->num_vals = pixels->total_vals;
     pixels->min = min;
     pixels->max = max;
     pixels->pixel_depth = _2d->pixel_depth;
     setup_all_val_recs(pixels,_2d);
     return NO_ERROR;
    }
 pixels->min = min;
 pixels->max = max;
 return setup_val_recs(pixels,_2d);

}


/*------------------------------------------------------------------------
pr_val_rec - ???
------------------------------------------------------------------------*/

void
pr_val_rec(val)
 struct val_rec *val;
{
 printf("val = %p value = %ld  total = %ld  num = %ld  coords = %p\n",
            val,val->value,val->total_coords,val->num_coords,val->coords);
}


/*------------------------------------------------------------------------
pr_buf_2d_val_rec - ???
------------------------------------------------------------------------*/

void
pr_buf_2d_val_rec(pixels)
 struct buf_2d_val_rec *pixels;
{
 unsigned long lcv;

 printf("\npr_buf_2d_val_rec...\n");
 printf("min = %ld max = %ld   actual min = %ld  actual max = %ld\n",
     pixels->min,pixels->max,pixels->actual_min,pixels->actual_max);
 printf("total_vals = %ld num_vals = %ld\n",pixels->total_vals,pixels->num_vals);
 for (lcv = 0; lcv < pixels->num_vals; lcv++)
    pr_val_rec(&(pixels->vals[lcv]));
 printf("pr_buf_2d_val_rec...end\n\n");

}
