/****************************************************************************
coord_rec.c - ???

Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.
****************************************************************************/

static const char rcsid[] = "$Id: coord_rec.c,v 1.8 1999/07/07 18:01:06 hughett Exp $ Release $Name: nria_1_0_6 $";

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>

#include "common.h"
#include "error_constants.h"
#include "coord_rec.h"
#include "typedef.h"
#include "nria_lib.h"

/*
#define TRACE(x) x; fflush(stdout);
*/
#define TRACE(x)


/*
init_coord_rec(coords)
get_coord_rec_memory(coords,add_num_pts)
free_coord_rec(coords)
find_coord_rec_min_max(coords,minx,maxx,miny,maxy)
pr_coord_rec(coords)
coord_rec_sort_proc(pt1,pt2)
sort_coord_rec(coord)
set_buf_by_coord_rec(_2d,coords,val)
xor_set_buf_by_sorted_coord_rec(_2d,coords)

sort_coord_pt_rec_proc(pt1,pt2)
get_common_coords(dest,src)
exclude_common_coords(dest,src)
have_common_coords(dest,src)
*/


/*-------------------------------------------------------------------------
init_coord_rec - ???
-------------------------------------------------------------------------*/

void
init_coord_rec(coords)
 struct coord_rec *coords;
{
 coords->total_pts = coords->num_pts = 0;
 coords->pts = NULL;
}


/*-------------------------------------------------------------------------
get_coord_rec_memory - ???
-------------------------------------------------------------------------*/

int
get_coord_rec_memory(coords,add_num_pts)
 struct coord_rec *coords;
 unsigned int add_num_pts;
{
 unsigned int extra;
 int err;

 if (coords != NULL)
    {extra = coords->total_pts - coords->num_pts;
     if (extra < add_num_pts)
        {get_ptr_memory(&(coords->pts),&(coords->total_pts),
              (unsigned int) add_num_pts,
              (size_t) sizeof(struct coord_pt_rec),&err);
         if (err != NO_ERROR) return err;
        }
    }
 return NO_ERROR;
}


/*-------------------------------------------------------------------------
free_coord_rec - ???
-------------------------------------------------------------------------*/

void
free_coord_rec(coords)
 struct coord_rec *coords;
{
 if ((coords->total_pts > 0) && (coords->pts != NULL)) c_free(coords->pts);
 init_coord_rec(coords);
}


/*-------------------------------------------------------------------------
find_coord_rec_min_max - ???
-------------------------------------------------------------------------*/

int
find_coord_rec_min_max(coords,minx,maxx,miny,maxy)
 struct coord_rec *coords;
 unsigned int *minx,*maxx,*miny,*maxy;
{
 unsigned int lcv;

 *minx = *miny = 0xffffffff;
 *maxx = *maxy = 0;
 if (coords->num_pts == 0) return CANCEL;
 for (lcv = 0; lcv < coords->num_pts; lcv++)
    {if (coords->pts[lcv].x > *maxx) *maxx = coords->pts[lcv].x;
     if (coords->pts[lcv].x < *minx) *minx = coords->pts[lcv].x;
     if (coords->pts[lcv].y > *maxy) *maxy = coords->pts[lcv].y;
     if (coords->pts[lcv].y < *miny) *miny = coords->pts[lcv].y;}

 return NO_ERROR;

}


/*-------------------------------------------------------------------------
pr_coord_rec - ???
-------------------------------------------------------------------------*/

void
pr_coord_rec(coords)
 struct coord_rec *coords;
{
 unsigned long lcv;

 printf("pr_coord_rec coords = %p\n",coords);
 if (coords != NULL)
    {printf("  total_pts = %ld   num_pts = %ld\n",coords->total_pts,
          coords->num_pts);
     for (lcv = 0; lcv < coords->num_pts; lcv++)
        printf("%ld) x = %ld  y = %ld\n",lcv,(long) coords->pts[lcv].x,
          (long) coords->pts[lcv].y);
    }
}


/*-------------------------------------------------------------------------
coord_rec_sort_proc - ???
-------------------------------------------------------------------------*/

int
coord_rec_sort_proc(pt1,pt2)
 struct coord_pt_rec *pt1,*pt2;
{
 if (pt1->y > pt2->y) return 1;
 if (pt1->y < pt2->y) return -1;
 if (pt1->x > pt2->x) return 1;
 if (pt1->x < pt2->x) return -1;
 return 0;
}


/*-------------------------------------------------------------------------
sort_coord_rec - ???
-------------------------------------------------------------------------*/

void
sort_coord_rec(coords)
 struct coord_rec *coords;
{
 if (coords->num_pts > 1)
    qsort(coords->pts,coords->num_pts,
        (size_t) sizeof(*(coords->pts)),coord_rec_sort_proc);
}


/*-------------------------------------------------------------------------
set_buf_by_coord_rec - ???
-------------------------------------------------------------------------*/

void
set_buf_by_coord_rec(_2d,coords,val)
 struct buf_2d_rec *_2d;
 struct coord_rec *coords;
 long val;
{
 unsigned long lcv;

 for (lcv = 0; lcv < coords->num_pts; lcv++)
    _2d->SetVal(_2d,(unsigned long) coords->pts[lcv].y,
         (unsigned long) coords->pts[lcv].x,(double) val);
}


/*-------------------------------------------------------------------------
xor_set_buf_by_sorted_coord_rec - ???
-------------------------------------------------------------------------*/

int
xor_set_buf_by_sorted_coord_rec(_2d,coords)
 struct buf_2d_rec *_2d;
 struct coord_rec *coords;
{
 unsigned long lcv,indx;
 double val;

 if (coords->num_pts == 0) return NO_ERROR;
 sort_coord_rec(coords);
 val = _2d->GetVal(_2d,(unsigned long) coords->pts[0].y,
           (unsigned long) coords->pts[0].x);
 if (val != .0) val = 0;
   else val = 1.0;
 _2d->SetVal(_2d,(unsigned long) coords->pts[0].y,
           (unsigned long) coords->pts[0].x,val);
 for (lcv = 0,indx = 1; indx < coords->num_pts; indx++)
    {if ((coords->pts[lcv].y != coords->pts[indx].y) ||
         (coords->pts[lcv].x != coords->pts[indx].x))
        {if ((val = _2d->GetVal(_2d,(unsigned long) coords->pts[indx].y,
              (unsigned long) coords->pts[indx].x)) != .0)
            val = 0;
           else val = 1.0;
         _2d->SetVal(_2d,(unsigned long) coords->pts[indx].y,
              (unsigned long) coords->pts[indx].x,val);
         lcv = indx;}
    }
 return NO_ERROR;
}


/*-------------------------------------------------------------------------
sort_coord_pt_rec_proc - ???
-------------------------------------------------------------------------*/

int
sort_coord_pt_rec_proc(pt1,pt2)
 struct coord_pt_rec *pt1,*pt2;
{
 if (pt1->y > pt2->y) return 1;
 if (pt1->y < pt2->y) return -1;
 if (pt1->x > pt2->x) return 1;
 if (pt1->x < pt2->x) return -1;
 return 0;
}


/*-------------------------------------------------------------------------
get_common_coords - ???
-------------------------------------------------------------------------*/

int
get_common_coords(dest,src)
 struct coord_rec *dest,*src;
{
 unsigned long lcv,cnt,indx;

 if (src->num_pts == 0)
    {dest->num_pts = 0;
     return NO_ERROR;}
   else if (dest->num_pts == 0) return NO_ERROR;

 qsort(dest->pts,dest->num_pts,(size_t) sizeof(struct coord_pt_rec),
           sort_coord_pt_rec_proc);
 qsort(src->pts,src->num_pts,(size_t) sizeof(struct coord_pt_rec),
           sort_coord_pt_rec_proc);

 for (indx = cnt = lcv = 0; (cnt < src->num_pts) && (lcv < dest->num_pts); )
    if (src->pts[cnt].y > dest->pts[lcv].y) lcv++;
      else if (src->pts[cnt].y < dest->pts[lcv].y) cnt++;
      else if (src->pts[cnt].x > dest->pts[lcv].x) lcv++;
      else if (src->pts[cnt].x < dest->pts[lcv].x) cnt++;
      else
       {dest->pts[indx++] = dest->pts[lcv++];
       }
 dest->num_pts = indx;
 return NO_ERROR;

}


/*-------------------------------------------------------------------------
exclude_common_coords - ???
-------------------------------------------------------------------------*/

int
exclude_common_coords(dest,src)
 struct coord_rec *dest,*src;
{
 unsigned long lcv,cnt,indx;

 if (src->num_pts == 0)
    {
     return NO_ERROR;}
   else if (dest->num_pts == 0) return NO_ERROR;

 qsort(dest->pts,dest->num_pts,(size_t) sizeof(*(dest->pts)),
           sort_coord_pt_rec_proc);
 qsort(src->pts,src->num_pts,(size_t) sizeof(*(src->pts)),
           sort_coord_pt_rec_proc);

 for (indx = cnt = lcv = 0; (cnt < src->num_pts) && (lcv < dest->num_pts); )
    {if (src->pts[cnt].y > dest->pts[lcv].y)
        dest->pts[indx++] = dest->pts[lcv++];
       else if (src->pts[cnt].y < dest->pts[lcv].y) cnt++;
       else if (src->pts[cnt].x > dest->pts[lcv].x)
        dest->pts[indx++] = dest->pts[lcv++];
       else if (src->pts[cnt].x < dest->pts[lcv].x) cnt++;
       else lcv++;
    }

 for (;(lcv < dest->num_pts);)
    dest->pts[indx++] = dest->pts[lcv++];
 dest->num_pts = indx;
 return NO_ERROR;

}


/*-------------------------------------------------------------------------
have_common_coords - ???
-------------------------------------------------------------------------*/

int
have_common_coords(dest,src)
 struct coord_rec *dest,*src;
{
 unsigned long lcv,cnt;

 if (src->num_pts == 0)
    {dest->num_pts = 0;
     return 0;}
   else if (dest->num_pts == 0) return 0;

 qsort(dest->pts,dest->num_pts,(size_t) sizeof(struct coord_pt_rec),
           sort_coord_pt_rec_proc);
 qsort(src->pts,src->num_pts,(size_t) sizeof(struct coord_pt_rec),
           sort_coord_pt_rec_proc);

 for (cnt = lcv = 0; (cnt < src->num_pts) && (lcv < dest->num_pts); )
    if (src->pts[cnt].y > dest->pts[lcv].y) lcv++;
      else if (src->pts[cnt].y < dest->pts[lcv].y) cnt++;
      else if (src->pts[cnt].x > dest->pts[lcv].x) lcv++;
      else if (src->pts[cnt].x < dest->pts[lcv].x) cnt++;
      else
       {return 1;
       }
 return 0;

}
