/****************************************************************************
dir_fns.c - ???

Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.
****************************************************************************/

static const char rcsid[] = "$Id: dir_fns.c,v 1.8 1999/07/07 18:01:06 hughett Exp $ Release $Name: nria_1_0_6 $";

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <unistd.h>

#include "error_constants.h"
#include "common_constants.h"
#include "typedef.h"
#include "str_def.h"
#include "nlib.h"
#include "debug.h"
#include "nria_lib.h"

#define DIR_ELEMS	10
#define FN_ELEMS	60
#define OTHERFNS_ELEMS	20


int fb_match();

/*
valid_dir(dir)
char *default_valid_dir(dir_fn,default_dir_fn)
char *parent_dir(dir)
char *complete_fn(fn)
char *cur_obsolute_fn(dir_fn,fn,status)
father_dir(dir,fatherdir)
brother_dirs(dir,patt_dirfn,brotherdir)
son_dirs(dir,patt_dirfn,sondir)
sort_strs_proc(str1,str2)
int get_fns(dir,dirfns,num_dirfns,select_dirfns,patt_dirfn,
       fns,num_fns,select_fns,patt_fn,
       otherfns,num_otherfns,select_otherfns,patt_otherfn)
*/


/*----------------------------------------------------------------------
valid_dir - ???
----------------------------------------------------------------------*/

int
valid_dir(dir)
     char *dir;
{
 char *cur_dir;
 int err;

 if (dir == NULL) return -2;
 if ((cur_dir = (char *) getcwd(NULL,256)) == NULL)
    {return -2;}
 err = chdir(dir);

 chdir(cur_dir);
 free(cur_dir);
 return err;
}


/*----------------------------------------------------------------------
default_valid_dir - ???
----------------------------------------------------------------------*/

char *
default_valid_dir( dir_fn, default_dir_fn )
     char *dir_fn,*default_dir_fn;
{
  char *ptr;

  if (valid_dir(dir_fn) != 0)
    {if (cp_str(&ptr,default_dir_fn) != 0)
       return NULL;}
  else
    {if (cp_str(&ptr,dir_fn) != 0)
       return NULL;}
  return ptr;
}


/*----------------------------------------------------------------------
parent_dir - ???
----------------------------------------------------------------------*/

char *
parent_dir( dir )
     char *dir;
{
  static char str[256];
  unsigned long len,lcv;
  char ch;

  if (dir == NULL) return NULL;
  if (dir[0] == '\0') return NULL;
  if ((dir[0] == '/') && (dir[1] == '\0'))
    {strcpy(str,"/");
     return str;}
  len = strlen(dir);
  for (lcv = len - 1; lcv > 0; lcv--)
    if (dir[lcv] == '/') break;
  ch = dir[lcv];
  dir[lcv] = '\0';
  strcpy(str,dir);
  dir[lcv] = ch;
  return str;
}


/*----------------------------------------------------------------------
complete_fn - ???
----------------------------------------------------------------------*/

char *
complete_fn( fn )
     char *fn;
{
  char *fn1,*dir,*cur_dir,*par_dir,*temp,*str;
  int err=0;

  if (fn == NULL)
    return NULL;
  if (fn[0] == '\0')
    return NULL;
  if ((dir = (char *) locate_dirfn(fn)) == NULL)
    {
      if ((dir = (char *) getcwd(NULL,256)) == NULL)
	return NULL;
    }
  else
    {
      if (dir[0] == '.')
	{
	  if (dir[1] == '\0')
	    {
	      c_free(dir);
	      if ((dir = (char *) getcwd(NULL,256)) == NULL)
		return NULL;
	    }
	  else if (dir[1] == '/')
            {
	      temp = dir;
	      cur_dir = (char *) getcwd(cur_dir,256);
	      err = concat_strs(&dir,2,cur_dir,&(temp[1]));
	      c_free(cur_dir);
	      c_free(temp);
	      if (err != NO_ERROR)
		return NULL;
            }
	  else if (dir[1] == '.')
            {
	      cur_dir = (char *) getcwd(cur_dir,256);
	      par_dir = parent_dir(cur_dir);
	      temp = dir;
	      if (dir[2] == '\0')
		dir = par_dir;
	      else
                err = concat_strs(&dir,2,par_dir,&(temp[2]));
	      c_free(temp);
	      c_free(par_dir);
	      c_free(cur_dir);
	      if (err != NO_ERROR)
		return NULL;
            }
        }
    }
  if ((err = cp_str(&str,fn)) == NO_ERROR)
    {
      drop_dirfn(str);
      if ((dir[0] == '/') && (dir[1] == '\0'))
        err = concat_strs(&fn1,2,"/",str);
      else
	err = concat_strs(&fn1,3,dir,"/",str);
      c_free(str);
    }

  c_free(dir);
  if (err == NO_ERROR)
    return fn1;
  else
    return NULL;
}


/*----------------------------------------------------------------------
cur_obsolute_fn - ???

 status = either FATHER_DIR or BROTHER_DIR or SON_DIR or CUR_DIR
 default = CUR_DIR

 Ex. assume that current dir = /h/g/f
    1) cur_obsolute_fn( NULL, "a",   FATHER_DIR)             returns /h/a
    2) cur_obsolute_fn( NULL, NULL,  FATHER_DIR)             returns /h/g
    3) cur_obsolute_fn( NULL, "",    FATHER_DIR)             returns /h/g
    4) cur_obsolute_fn( NULL, NULL,  BROTHER_DIR || SON_DIR) returns NULL
    5) cur_obsolute_fn( NULL, "a",   BROTHER_DIR)            returns /h/g/a
    6) cur_obsolute_fn( NULL, "a",   SON_DIR)                returns /h/g/f/a
    7) cur_obsolute_fn( NULL, "a" || NULL || "", CUR_DIR)    returns /h/g/f
    8) cur_obsolute_fn returns /h/g/f


    if current_dir = /
       cur_obsolute_fn(NULL,NULL || "",FATHER_DIR) returns /.
    if the first parameter != NULL || empty then
       this parameter is the reference for the directory specification
           other than the current directory.
----------------------------------------------------------------------*/

char *
cur_obsolute_fn(dir_fn,fn,status)
     char *dir_fn;
     char *fn;
     int status;
{
  char *cur_dir,*fn1,len,len2;
  unsigned char free_cur_dir;

  if ((fn == NULL) && ((status != FATHER_DIR) && (status != CUR_DIR)))
    return NULL;
  free_cur_dir = 0;
  if ((dir_fn != NULL) && (strlen(dir_fn) != 0))
    cur_dir = dir_fn;
  else if ((cur_dir = (char *) getcwd(NULL,256)) == NULL)
    {
      return NULL;}
  else free_cur_dir = 1;

  switch (status)
    {case FATHER_DIR : {
      drop_last_element(cur_dir,'/');
      if (fn != NULL) len = strlen(fn);
      else len = 0;
      if (len != 0) drop_last_element(cur_dir,'/');
      len2 = strlen(cur_dir);
      if (len2 == 0)
	{if (len == 0) copy_str(&fn1,"/",(size_t) 1);
	else concat_strs(&fn1,2,"/",fn);}
      else if (len != 0) concat_strs(&fn1,3,cur_dir,"/",fn);
      else copy_str(&fn1,cur_dir,(size_t) len2);
      break;
    }

    case BROTHER_DIR : {
      drop_last_element(cur_dir,'/');
      concat_strs(&fn1,3,cur_dir,"/",fn);
      break;
    }
    case SON_DIR : {
      concat_strs(&fn1,3,cur_dir,"/",fn);
      break;
    }
    default : 
      if ((fn == NULL)  || (strlen(fn) == 0))
	copy_str(&fn1,cur_dir,(size_t) strlen(cur_dir));
      else concat_strs(&fn1,3,cur_dir,"/",fn);
    }
  if (free_cur_dir) free(cur_dir);
  return fn1;
}

#if 0        /******************* chop ************/
char *
obsolute_fn(dir_fn,fn,status)
     char *fn;
     int status;
{
  char *cur_dir,*ptr;
  int same_dir;

  if ((cur_dir = (char *) getcwd(NULL,256)) == NULL)
    {printf("getcwd error...\n");
     return NULL;}
  same_dir = strcmp(cur_dir,dir_fn);

  if ((same_dir != 0) && (chdir(dir_fn) != 0))
    {free(cur_dir);
     return NULL;}
  ptr = cur_obsolute_fn(fn,status);
  if ((same_dir != 0) && (chdir(cur_dir) != 0))
    {free(cur_dir);
     if (ptr != NULL) c_free(ptr);
     return NULL;}
  free(cur_dir);
  return ptr;
}
#endif      /************** end chop *******************/



/*----------------------------------------------------------------------
father_dir - ???
----------------------------------------------------------------------*/

int
father_dir(dir,fatherdir)
     char *dir,**fatherdir;
{
  char *cur_dir;
  int err;

  *fatherdir = NULL;
  if ((cur_dir = (char *) getcwd(NULL,256)) == NULL)
    return CANCEL;
  err = NO_ERROR;
  if (str_cmp(dir,cur_dir) != 0) err = chdir(dir);

  if ((err == NO_ERROR) && (err = chdir("..")) == NO_ERROR)
    *fatherdir = (char *) getcwd(NULL,256);

  chdir(cur_dir);
  free(cur_dir);
  if (err != NO_ERROR) return err;

  if (*fatherdir == NULL) 
    return CANCEL;
  return NO_ERROR;
}


/*----------------------------------------------------------------------
son_dirs - ???
----------------------------------------------------------------------*/

int
son_dirs(dir,patt_dirfn,sondir)
     char *dir;
     char *patt_dirfn;
     struct strs_rec **sondir;
{
  char **dirfns;
  unsigned int num_dirfns;
  int err;
  struct strs_rec *ptr;

  err = get_fns(dir,&dirfns,&num_dirfns,1,patt_dirfn,
		NULL,0,0,NULL,NULL,0,0,NULL);
  *sondir = NULL;
  if (num_dirfns > 0)
    {
      if (((err = setup_strs_rec(&ptr,num_dirfns,0)) != NO_ERROR) ||
	  ((err = add_str_ptr_ary_to_strs_rec(ptr,dirfns,num_dirfns))
	   != NO_ERROR))
        {deallocate_2d_ary(dirfns,num_dirfns);
         if (ptr != NULL)
	   {free_strs_rec_ptrs(ptr);
	    c_free(ptr);
	  }
       }
      else *sondir = ptr;
    }
  return err;
}


/*----------------------------------------------------------------------
brother_dirs - ???
----------------------------------------------------------------------*/

int
brother_dirs(dir,patt_dirfn,brotherdir)
     char *dir;
     char *patt_dirfn;
     struct strs_rec **brotherdir;
{
  char *ancestor;
  int err;

  if ((err = father_dir(dir,&ancestor)) != NO_ERROR) return 0;
  err = son_dirs(ancestor,patt_dirfn,brotherdir);
  c_free(ancestor);
  return err;
}


/*----------------------------------------------------------------------
sort_strs_proc - ???
----------------------------------------------------------------------*/

int
sort_strs_proc(str1,str2)
     char **str1,**str2;
{
  return no_case_str_cmp(*str1,*str2);
}


/*----------------------------------------------------------------------
get_fns - ???

1) sort file names ascendingly
2) can use a wild card specification  "*" to match file name
----------------------------------------------------------------------*/

int
get_fns( dir, dirfns, num_dirfns, select_dirfns, patt_dirfn,
	fns, num_fns, select_fns, patt_fn,
	otherfns, num_otherfns, select_otherfns, patt_otherfn)
     char *dir,***dirfns,***fns,***otherfns;
     char *patt_dirfn,*patt_fn,*patt_otherfn;
     unsigned char select_dirfns,select_fns,select_otherfns;
     unsigned int *num_dirfns,*num_fns,*num_otherfns;
{
  DIR *dirp;
  struct dirent *dp;
  unsigned int fn_elems,dir_elems,other_elems;
  char fn[300];
  struct stat buf;
  int len,err;

  if (select_dirfns != 0)
    {*num_dirfns = dir_elems = 0;
     *dirfns = NULL;}
  if (select_fns != 0)
    {*num_fns = fn_elems = 0;
     *fns = NULL;}
  if (select_otherfns != 0)
    {*num_otherfns = other_elems = 0;
     *otherfns = NULL;}

  if ((select_dirfns == 0)
      && (select_fns == 0)
      && (select_otherfns == 0))
    return 1;

  NOPRT(printf("DEBUG: opening directory %s\n", dir););

  if ((dirp = opendir(dir)) == NULL) {
    TRACE(printf("DEBUG: ERROR opening directory.\n", dir););
    return CANCEL;
  }

  strcpy(fn,dir);
  strcat(fn,"/");
  len = strlen(fn);

  err = NO_ERROR;

  NOPRT(printf("DEBUG: traversing...\n"););

  for (dp = readdir(dirp); dp != NULL; dp = readdir(dirp))
    {

      NOPRT(printf("DEBUG: readdir found %s\n", dp->d_name););

      strcpy(&(fn[len]),dp->d_name);
      if (stat(fn,&buf) == 0)
	{ 
	  if (buf.st_mode & S_IFDIR)
	    {
	      if ((select_dirfns != 0) && fb_match(patt_dirfn,dp->d_name))
		{
		  if (*num_dirfns == dir_elems)
		    {
		      get_ptr_memory( dirfns, &dir_elems,
				     (unsigned int) DIR_ELEMS,
				     (size_t) sizeof(char *), &err );
		      if (err != NO_ERROR) break;
		    }
		  copy_str(&((*dirfns)[*num_dirfns]),
			   dp->d_name,strlen(dp->d_name));
		  *num_dirfns += 1;
		}
	    }
/*
          else if  ((buf.st_mode & S_IFREG) || (buf.st_mode & S_IFCHR) ||
                    (buf.st_mode & S_IFBLK))
*/
          else if  (buf.st_mode & S_IFREG)
	    {
	      if ((select_fns != 0) && fb_match(patt_fn,dp->d_name))
		{
		  if (*num_fns == fn_elems)
		    {
		      get_ptr_memory( fns, &fn_elems,
				     (unsigned int) FN_ELEMS,
				     (size_t) sizeof(char *), &err );
		      if (err != NO_ERROR) break;
		    }
		  copy_str(&((*fns)[*num_fns]),
			   dp->d_name,strlen(dp->d_name));
		  *num_fns += 1;
		}
	    }
          else
	    {
	      if ((select_otherfns != 0) && fb_match(patt_otherfn,dp->d_name))
		{
		  if (*num_otherfns == other_elems)
		    {
		      get_ptr_memory(otherfns,&other_elems,
				     (unsigned int) OTHERFNS_ELEMS,
				     (size_t) sizeof(char *),&err);
		      if (err != NO_ERROR) break;
		    }
		  copy_str(&((*otherfns)[*num_otherfns]),
			   dp->d_name,strlen(dp->d_name));
		  *num_otherfns += 1;
		}
	    }
        }
    }
  closedir(dirp);
  if ((select_dirfns != 0) && (*num_dirfns > 1))
    qsort(*dirfns,*num_dirfns,(size_t) sizeof(char *),sort_strs_proc);
  if ((select_fns != 0) && (*num_fns > 1))
    qsort(*fns,*num_fns,(size_t) sizeof(char *),sort_strs_proc);
  if ((select_otherfns != 0) && (*num_otherfns > 1))
    qsort(*otherfns,*num_otherfns,(size_t) sizeof(char *),sort_strs_proc);
  return err;
}

