/****************************************************************************
more_str_opr.c - ???

Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.
****************************************************************************/

static const char rcsid[] = "$Id: more_str_opr.c,v 1.10 1999/07/07 18:01:06 hughett Exp $ Release $Name: nria_1_0_6 $";

#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <sys/types.h>

#include "error_constants.h"
#include "str_def.h"
#include "debug.h"
#include "nria_lib.h"


#define NUM_ADDED_ENTRIES 5


/*-----------------------------------------------------------------------
init_strs_rec - ???
-----------------------------------------------------------------------*/

void
init_strs_rec(ptr)
 struct strs_rec *ptr;
{
 ptr->status = 0;
 ptr->total_strs = ptr->cur_str = ptr->num_strs = 0;
 ptr->strs = NULL;
}


/*-----------------------------------------------------------------------
free_strs_rec - ???
-----------------------------------------------------------------------*/

void
free_strs_rec(ptr)
 struct strs_rec *ptr;
{
 unsigned long lcv;

 if (ptr != NULL)
    {if (ptr->total_strs > 0) 
       {for (lcv = 0; lcv < ptr->num_strs; lcv++)
           c_free(ptr->strs[lcv]);
        c_free(ptr->strs);}
     init_strs_rec(ptr);}
}


/*-----------------------------------------------------------------------
free_strs_rec_ptrs - ???
-----------------------------------------------------------------------*/

void
free_strs_rec_ptrs(ptr)
 struct strs_rec *ptr;
{
 if (ptr->total_strs > 0) c_free(ptr->strs);
 init_strs_rec(ptr);
}


/*-----------------------------------------------------------------------
add_strs_rec_entries - ???
-----------------------------------------------------------------------*/

int
add_strs_rec_entries(ptr,add_num_entries)
 struct strs_rec *ptr;
 unsigned int add_num_entries;
{
 int err;
 unsigned long temp,total;

 if (add_num_entries == 0) return NO_ERROR;
 total = ptr->num_strs + add_num_entries;
 if (total > ptr->total_strs)
    {temp = ptr->total_strs;
     get_ptr_memory(&(ptr->strs),&(ptr->total_strs),add_num_entries,
            (size_t) sizeof(char *),&err);
     if (err == NO_ERROR)
        init_ptr_ary(&(ptr->strs[temp]),add_num_entries,NULL);
     return err;
    }
 return NO_ERROR;
}


/*-----------------------------------------------------------------------
add_str_ptr_to_strs_rec - ???
-----------------------------------------------------------------------*/

int
add_str_ptr_to_strs_rec(ptr,str)
 struct strs_rec *ptr;
 char *str;
{
 int err;

 if (str == NULL) return NO_ERROR;
 if (ptr->num_strs >= ptr->total_strs)
    {get_ptr_memory(&(ptr->strs),&(ptr->total_strs),NUM_ADDED_ENTRIES,
          (size_t) sizeof(*(ptr->strs)),&err);
     if (err != NO_ERROR) return err;}
 ptr->strs[(ptr->num_strs)++] = str;
 return NO_ERROR;
}


/*-----------------------------------------------------------------------
AddStrPtrToSortedStrsRec - ???
-----------------------------------------------------------------------*/

int
AddStrPtrToSortedStrsRec(ptr,str,set_cur,found_is_noerror)
 struct strs_rec *ptr;
 char *str;
 int set_cur,found_is_noerror;
{
 int err;
 unsigned long insert_posi;
 long lcv;

 if (obj_binary_search(ptr->strs,ptr->num_strs,
        sizeof(*(ptr->strs)),&str,sort_no_case_str_proc,&insert_posi)
      == NOT_FOUND)
    {TRACE(printf("NOT_FOUND str = %s\n",str))
     TRACE(pr_strs_rec(ptr))
     if ((err = add_strs_rec_entries(ptr,1)) == NO_ERROR)
        {
         for (lcv = ptr->num_strs; lcv > insert_posi; lcv--)
            ptr->strs[lcv] = ptr->strs[lcv - 1];
         ptr->strs[lcv] = str;
         ptr->num_strs += 1;
         if (set_cur) ptr->cur_str = insert_posi;
           else if (ptr->cur_str >= insert_posi) ptr->cur_str += 1;
        }
     TRACE(printf(":after...\n");pr_strs_rec(ptr))
    }
   else
    {if (set_cur) ptr->cur_str = insert_posi;
     if (found_is_noerror) return NO_ERROR;
     return FOUND;}
 return err;
}


/*-----------------------------------------------------------------------
add_str_to_strs_rec - ???
-----------------------------------------------------------------------*/

int
add_str_to_strs_rec(ptr,str)
 struct strs_rec *ptr;
 char *str;
{
 int err;
 char *str1;

 if ((err = cp_str(&str1,str)) == NO_ERROR) 
    if ((err = add_str_ptr_to_strs_rec(ptr,str1)) != NO_ERROR) 
       c_free(str1);
 return err;
}


/*-----------------------------------------------------------------------
AddStrToSortedStrsRec - ???
-----------------------------------------------------------------------*/

int
AddStrToSortedStrsRec(ptr,str,set_cur,found_is_noerror)
 struct strs_rec *ptr;
 char *str;
 int set_cur,found_is_noerror;
{
 int err;
 char *str1;

 if ((err = cp_str(&str1,str)) == NO_ERROR) 
    if ((err = AddStrPtrToSortedStrsRec(ptr,str1,set_cur,
               found_is_noerror)) != NO_ERROR)
       c_free(str1);
        
 return err;
}


/*-----------------------------------------------------------------------
add_str_ary_to_strs_rec - ???
-----------------------------------------------------------------------*/

int
add_str_ary_to_strs_rec(ptr,strs,num_strs)
 struct strs_rec *ptr;
 char **strs;
 long num_strs;
{
 int err;
 long cnt;
 
 if ((err = add_strs_rec_entries(ptr,num_strs)) == NO_ERROR)
    for (cnt = 0; cnt < num_strs; cnt++)
       if ((err = add_str_to_strs_rec(ptr,strs[cnt])) != NO_ERROR)
          return err;
 return err;
}


/*-----------------------------------------------------------------------
add_str_ptr_ary_to_strs_rec - ???
-----------------------------------------------------------------------*/

int
add_str_ptr_ary_to_strs_rec(ptr,strs,num_strs)
 struct strs_rec *ptr;
 char **strs;
 long num_strs;
{
 int err;
 long cnt;
 
 if (num_strs == 0) return NO_ERROR;
 if ((err = add_strs_rec_entries(ptr,num_strs)) == NO_ERROR)
    for (cnt = 0; cnt < num_strs; cnt++)
       ptr->strs[(ptr->num_strs)++] = strs[cnt];
 return err;
}


/*-----------------------------------------------------------------------
add_str_tokens_to_strs_rec - ???
-----------------------------------------------------------------------*/

int
add_str_tokens_to_strs_rec(ptr,str)
 struct strs_rec *ptr;
 char *str;
{
 static char delimiter[] = {' ',9,'\n','\0'};
 char *token;
 int err;

 if ((token = (char *) strtok(str,delimiter)) == NULL) return NO_ERROR;
 for (;;)
    {
     if ((err = add_str_to_strs_rec(ptr,token)) != NO_ERROR) return err;
     if ((token = (char *) strtok(NULL,delimiter)) == NULL) break;
    }
 return NO_ERROR;
}


/*-----------------------------------------------------------------------
add_str_ptr_ary_to_sorted_strs_rec - ???
-----------------------------------------------------------------------*/

int
add_str_ptr_ary_to_sorted_strs_rec(sorted,strs,num_strs,set_cur)
 struct strs_rec *sorted;
 char **strs;
 long num_strs;
 int set_cur;
{
 int err;
 long cnt;

 for (cnt = 0; cnt < num_strs; cnt++)
    if ((err = AddStrPtrToSortedStrsRec(sorted,strs[cnt],set_cur,1))
          != NO_ERROR)
       if (err != FOUND) return err;
 return NO_ERROR;
}


/*-----------------------------------------------------------------------
add_sort_no_dup_strs_rec - ???

add_sort_no_dup_strs_rec(result,is_sort_and_no_dup,num_srcs[,src1,src2,...])

FIXME:  This function makes two passes over the argments, which may or
may not be legal and portable.  -PWH 19990405
-----------------------------------------------------------------------*/

int
add_sort_no_dup_strs_rec(
    struct strs_rec *result,
    int is_sort_and_no_dup,
    unsigned long num_srcs,
    ...)
{
 va_list args;
 struct strs_rec *src,all,new;
 unsigned long lcv,cnt,num;
 int err;
 char *last;

 cnt = 0;
 va_start(args, num_srcs);
 if (num_srcs > 0)
    {for (lcv = 0; lcv < num_srcs; lcv++)
        if ((src = va_arg(args, struct strs_rec *)) != NULL)
           cnt += src->num_strs;
    }
 va_end(args);

 err = NO_ERROR;
 if ((cnt > 0) || ((result->num_strs > 0) && !is_sort_and_no_dup))
    {
     cnt += result->num_strs;
     init_strs_rec(&all);
     if ((err = add_strs_rec_entries(&all,cnt)) != NO_ERROR)
        {free_strs_rec_ptrs(&all);
         return err;}
     va_start(args, num_srcs);
     for (cnt = 0; cnt < result->num_strs; cnt++)
        all.strs[cnt] = result->strs[cnt];
     for (lcv = 0; lcv < num_srcs; lcv++)
        if ((src = va_arg(args, struct strs_rec *)) != NULL)
           {
            for (num = 0; num < src->num_strs; num++)
               all.strs[cnt++] = src->strs[num];
           }
     va_end(args);
     all.num_strs = cnt;

     if (all.num_strs > 1) sort_strs_rec_ascendingly(&all);
     last = all.strs[0];
     for (lcv = 1,cnt = 1; cnt < all.num_strs; cnt++)
        if (str_cmp(last,all.strs[cnt]) == 0) all.strs[cnt] = NULL;
          else
           {lcv++;
            last = all.strs[cnt];}
     init_strs_rec(&new);
     if ((err = add_strs_rec_entries(&new,lcv)) == NO_ERROR)
        {
         for (cnt = 0; cnt < all.num_strs; cnt++)
            if (all.strs[cnt] != NULL)
               {if ((err = cp_str(&(new.strs[new.num_strs]),all.strs[cnt])) 
                       != NO_ERROR)
                   break;
                new.num_strs += 1;}
        }
     if (err != NO_ERROR)
        {free_strs_rec(&new);
         return err;}
       else
        {free_strs_rec(result);
         *result = new;}
     free_strs_rec_ptrs(&all);
    }
 return err;
}


#if 0        /* Old varargs version */
int
add_sort_no_dup_strs_rec(va_alist)
 va_dcl
{
 va_list args;
 struct strs_rec *result,*src,all,new;
 unsigned long num_srcs,lcv,cnt,num;
 int err,is_sort_and_no_dup;
 char *last;

 va_start(args);
 result = va_arg(args, struct strs_rec *);
 is_sort_and_no_dup = va_arg(args, int);
 num_srcs = va_arg(args, long);
 cnt = 0;
 if (num_srcs > 0)
    {for (lcv = 0; lcv < num_srcs; lcv++)
        if ((src = va_arg(args, struct strs_rec *)) != NULL)
           cnt += src->num_strs;
    }
 va_end(args);

 err = NO_ERROR;
 if ((cnt > 0) || ((result->num_strs > 0) && !is_sort_and_no_dup))
    {
     cnt += result->num_strs;
     init_strs_rec(&all);
     if ((err = add_strs_rec_entries(&all,cnt)) != NO_ERROR)
        {free_strs_rec_ptrs(&all);
         return err;}
     va_start(args);
     result = va_arg(args, struct strs_rec *);
     is_sort_and_no_dup = va_arg(args, int);
     num_srcs = va_arg(args, long);
     for (cnt = 0; cnt < result->num_strs; cnt++)
        all.strs[cnt] = result->strs[cnt];
     for (lcv = 0; lcv < num_srcs; lcv++)
        if ((src = va_arg(args, struct strs_rec *)) != NULL)
           {
            for (num = 0; num < src->num_strs; num++)
               all.strs[cnt++] = src->strs[num];
           }
     va_end(args);
     all.num_strs = cnt;

     if (all.num_strs > 1) sort_strs_rec_ascendingly(&all);
     last = all.strs[0];
     for (lcv = 1,cnt = 1; cnt < all.num_strs; cnt++)
        if (str_cmp(last,all.strs[cnt]) == 0) all.strs[cnt] = NULL;
          else
           {lcv++;
            last = all.strs[cnt];}
     init_strs_rec(&new);
     if ((err = add_strs_rec_entries(&new,lcv)) == NO_ERROR)
        {
         for (cnt = 0; cnt < all.num_strs; cnt++)
            if (all.strs[cnt] != NULL)
               {if ((err = cp_str(&(new.strs[new.num_strs]),all.strs[cnt])) 
                       != NO_ERROR)
                   break;
                new.num_strs += 1;}
        }
     if (err != NO_ERROR)
        {free_strs_rec(&new);
         return err;}
       else
        {free_strs_rec(result);
         *result = new;}
     free_strs_rec_ptrs(&all);
    }
 return err;
}
#endif


/*-----------------------------------------------------------------------
cp_strs_rec_1 - ???
-----------------------------------------------------------------------*/

int
cp_strs_rec_1(dest,src)
 struct strs_rec *dest,*src;
{
 int err;
 unsigned long lcv,indx;

 if (src->num_strs == 0) return NO_ERROR;
 if ((err = add_strs_rec_entries(dest,src->num_strs)) != NO_ERROR)
    return err;
 for (indx = dest->num_strs,lcv = 0; lcv < src->num_strs; lcv++)
    {if ((err = cp_str(&(dest->strs[indx]),src->strs[lcv])) != NO_ERROR) 
        {for (lcv = dest->num_strs; lcv < indx; lcv++)
            {c_free(dest->strs[lcv]);
             dest->strs[lcv] = NULL;}
         return err;}
     indx++;}
 dest->num_strs = indx;
 return NO_ERROR;
}


/*-----------------------------------------------------------------------
cp_strs_rec - ???
-----------------------------------------------------------------------*/

int
cp_strs_rec(dest,src)
 struct strs_rec *dest,*src;
{
 free_strs_rec(dest);
 return cp_strs_rec_1(dest,src);
}


/*-----------------------------------------------------------------------
append_strs_rec - ???
-----------------------------------------------------------------------*/

int
append_strs_rec(dest,src)
 struct strs_rec *dest,*src;
{
 return cp_strs_rec_1(dest,src);
}


/*-----------------------------------------------------------------------
cp_strs_rec_ptrs_1 - ???
-----------------------------------------------------------------------*/

int
cp_strs_rec_ptrs_1(dest,src)
 struct strs_rec *dest,*src;
{
 int err;
 unsigned long lcv,indx;

 if (src->num_strs == 0) return NO_ERROR;
 if ((err = add_strs_rec_entries(dest,src->num_strs)) != NO_ERROR)
    return err;
 for (indx = dest->num_strs,lcv = 0; lcv < src->num_strs; lcv++)
    dest->strs[indx++] = src->strs[lcv];

 dest->num_strs = indx;
 return NO_ERROR;
}


/*-----------------------------------------------------------------------
cp_strs_rec_ptrs - ???
-----------------------------------------------------------------------*/

int
cp_strs_rec_ptrs(dest,src)
 struct strs_rec *dest,*src;
{
 free_strs_rec(dest);
 return cp_strs_rec_ptrs_1(dest,src);
}


/*-----------------------------------------------------------------------
append_strs_rec_ptrs - ???
-----------------------------------------------------------------------*/

int
append_strs_rec_ptrs(dest,src)
 struct strs_rec *dest,*src;
{
 return cp_strs_rec_ptrs_1(dest,src);
}


/*-----------------------------------------------------------------------
common_ptrs_in_sorted_strs_rec - ???

 str ptrs in sorted will be compared with those in src and result is
set as follows :-
  1) for those in src and not found in sorted, the corresonding entries in result will be set to NULL.
  2) for those in src and found in sorted, the corresonding entries in 
     result will be set to those string ptrs in src.
  3) result has an increase of sorted->num_strs entries.
-----------------------------------------------------------------------*/

int
common_ptrs_in_sorted_strs_rec(result,sorted,src)
 struct strs_rec *result,*sorted,*src;
{
 unsigned long lcv,insert_posi;
 int err;

 if ((err = add_strs_rec_entries(result,sorted->num_strs)) != NO_ERROR)
    return err;
 for (lcv = 0; lcv < src->num_strs; lcv++)
    if (obj_binary_search(sorted->strs,sorted->num_strs,
        sizeof(*(sorted->strs)),&(src->strs[lcv]),
        sort_no_case_str_proc,&insert_posi) == NO_ERROR)
       result->strs[result->num_strs + insert_posi] = src->strs[lcv];
/*
 for (cnt = lcv = 0; lcv < src->num_strs; lcv++)
    if (result->strs[result->num_strs + lcv] != NULL) 
       {if (lcv != cnt)
           result->strs[result->num_strs + cnt] = 
                  result->strs[result->num_strs + lcv];
        cnt++;}
*/
 result->num_strs += sorted->num_strs;

 return NO_ERROR;
}


/*-----------------------------------------------------------------------
diff_ptrs_in_sorted_strs_rec - ???

 str ptrs in sorted will be compared with those in src and result is
set as follows :-
  1) for those in src and found in sorted, the corresonding entries in result
     will be set to NULL.
  2) for those in sorted and not found in src, the corresonding entries in 
     result will be set to those string ptrs in sorted.
  3) result has an increase of sorted->num_strs entries.
-----------------------------------------------------------------------*/

int
diff_ptrs_in_sorted_strs_rec(result,sorted,src)
 struct strs_rec *result,*sorted,*src;
{
 unsigned long lcv,insert_posi;
 int err;

 if ((err = add_strs_rec_entries(result,sorted->num_strs)) != NO_ERROR)
    return err;
 for (lcv = 0; lcv < sorted->num_strs; lcv++)
    result->strs[result->num_strs + lcv] = sorted->strs[lcv];

 for (lcv = 0; lcv < src->num_strs; lcv++)
    if (obj_binary_search(sorted->strs,sorted->num_strs,
        sizeof(*(sorted->strs)),&(src->strs[lcv]),
        sort_no_case_str_proc,&insert_posi) == NO_ERROR)
       result->strs[result->num_strs + insert_posi] = NULL;
/*
 for (cnt = lcv = 0; lcv < src->num_strs; lcv++)
    if (result->strs[result->num_strs + lcv] != NULL) 
       {if (lcv != cnt)
           result->strs[result->num_strs + cnt] = 
                  result->strs[result->num_strs + lcv];
        cnt++;}
*/
 result->num_strs += sorted->num_strs;
 return NO_ERROR;
}


/*-----------------------------------------------------------------------
sort_strs_ascendingly_proc - ???
-----------------------------------------------------------------------*/

int
sort_strs_ascendingly_proc(str1,str2)
 char **str1,**str2;
{
 return no_case_str_cmp(*str1,*str2);
}


/*-----------------------------------------------------------------------
sort_strs_rec_ascendingly - ???
-----------------------------------------------------------------------*/

void
sort_strs_rec_ascendingly(ptr)
 struct strs_rec *ptr;
{
 if (ptr->num_strs > 1)
    qsort(ptr->strs,ptr->num_strs,
        (size_t) sizeof(char *),sort_strs_ascendingly_proc);
}


/*-----------------------------------------------------------------------
sort_strs_rec - ???
-----------------------------------------------------------------------*/

void
sort_strs_rec(ptr,sort_proc)
 struct strs_rec *ptr;
 int (*sort_proc)();
{
 if (ptr->num_strs > 1)
    qsort(ptr->strs,ptr->num_strs,(size_t) sizeof(char *),sort_proc);
}


/*-----------------------------------------------------------------------
position_str_in_strs_rec - ???
-----------------------------------------------------------------------*/

int
position_str_in_strs_rec(ptr,str,posi)
 struct strs_rec *ptr;
 char *str;
 unsigned long posi;
{
 int err;
 long indx,lcv;
 char *str1;

 if (str == NULL) return NO_ERROR;
 indx = FindStrInHandleAryByContent(ptr->strs,ptr->num_strs,str,&err);
 if (err == NO_ERROR)
    {str1 = ptr->strs[indx];
    }
   else
    {if ((err = cp_str(&str1,str)) != NO_ERROR) return err;
     indx = ptr->num_strs;
     if ((err = add_strs_rec_entries(ptr,1)) != NO_ERROR)
        {c_free(str1);
         return err;}
     ptr->num_strs += 1;    
    }

 if (posi > indx)
    {for (lcv = indx + 1; lcv <= posi; lcv++)
        ptr->strs[lcv - 1] = ptr->strs[lcv];
    }
   else 
    for (lcv = indx; lcv > posi; lcv--)
       ptr->strs[lcv] = ptr->strs[lcv - 1];
 ptr->strs[posi] = str1;
 return NO_ERROR;
}


/*-----------------------------------------------------------------------
find_str_in_strs_rec - ???
-----------------------------------------------------------------------*/

int
find_str_in_strs_rec(ptr,str,indx)
 struct strs_rec *ptr;
 char *str;
 unsigned long *indx;
{
 int err;
 unsigned long indx1;

 if (ptr->num_strs == 0)
    {if (indx != NULL) *indx = 0;
     return NOT_FOUND;}
 indx1 = FindStrInHandleAryByContent(ptr->strs,ptr->num_strs,str,&err);
 if (indx != NULL) *indx = indx1;
 return err;
}




/*-----------------------------------------------------------------------
FindStrInSortedStrsRec - ???
-----------------------------------------------------------------------*/

int
FindStrInSortedStrsRec(sorted,str,indx)
     struct strs_rec *sorted;
     char *str;
     unsigned long *indx;
{
  unsigned long indx1;
  int err;

  if (sorted->num_strs == 0)
  {
    if (indx != NULL)
      *indx = 0;
    return NOT_FOUND;
  }
  err = obj_binary_search(sorted->strs,sorted->num_strs,
			  sizeof(*(sorted->strs)),&str,
			  sort_no_case_str_proc,&indx1);
  if (indx != NULL)
    *indx = indx1;
  return err;
}





/*-----------------------------------------------------------------------
drop_indx_str_ptr_from_strs_rec - ???
-----------------------------------------------------------------------*/

int
drop_indx_str_ptr_from_strs_rec(ptr,indx)
 struct strs_rec *ptr;
 unsigned long indx;
{
 unsigned long lcv;

 if (indx < ptr->num_strs)
    { 
     ptr->num_strs -= 1;
     for (lcv = indx + 1; lcv <= ptr->num_strs; )
        {ptr->strs[indx] = ptr->strs[lcv];
         indx = lcv;
         lcv += 1;}
     ptr->strs[ptr->num_strs] = NULL;}

 return NO_ERROR;
}


/*-----------------------------------------------------------------------
drop_str_ptr_from_strs_rec - ???
-----------------------------------------------------------------------*/

int
drop_str_ptr_from_strs_rec(ptr,str)
 struct strs_rec *ptr;
 char *str;
{
 unsigned long lcv;

 if (ptr->num_strs == 0) return NO_ERROR;
 for (lcv = 0; lcv < ptr->num_strs; lcv++)
    if (ptr->strs[lcv] == str)
       {drop_indx_str_ptr_from_strs_rec(ptr,lcv);
        break;};

 return NO_ERROR;
}


/*-----------------------------------------------------------------------
drop_str_from_strs_rec - ???
-----------------------------------------------------------------------*/

int
drop_str_from_strs_rec(ptr,str)
 struct strs_rec *ptr;
 char *str;
{
 unsigned long lcv;
 int err;

 lcv = FindStrInHandleAryByContent(ptr->strs,ptr->num_strs,str,&err);
 if (err == NO_ERROR)
    {c_free(ptr->strs[lcv]);
     drop_indx_str_ptr_from_strs_rec(ptr,lcv);
    }
 return NO_ERROR;
}


/*-----------------------------------------------------------------------
DropStrPtrInSortedStrsRec - ???
-----------------------------------------------------------------------*/

int
DropStrPtrInSortedStrsRec(sorted,str)
 struct strs_rec *sorted;
 char *str;
{
 unsigned long indx;

 if (obj_binary_search(sorted->strs,sorted->num_strs,
        sizeof(*(sorted->strs)),&str,
        sort_no_case_str_proc,&indx) == NO_ERROR)
    {
     drop_indx_str_ptr_from_strs_rec(sorted,indx);
    }
 return NO_ERROR;
}


/*-----------------------------------------------------------------------
DropStrInSortedStrsRec - ???
-----------------------------------------------------------------------*/

int
DropStrInSortedStrsRec(sorted,str)
 struct strs_rec *sorted;
 char *str;
{
 unsigned long indx;

 if (obj_binary_search(sorted->strs,sorted->num_strs,
        sizeof(*(sorted->strs)),&str,
        sort_no_case_str_proc,&indx) == NO_ERROR)
    {c_free(sorted->strs[indx]);
     drop_indx_str_ptr_from_strs_rec(sorted,indx);
    }
 return NO_ERROR;
}


/*-----------------------------------------------------------------------
drop_dup_str_from_strs_rec - ???
-----------------------------------------------------------------------*/

int
drop_dup_str_from_strs_rec(ptr)
 struct strs_rec *ptr;
{
 unsigned long indx,lcv;

 PRT(printf("drop_dup_str_from_strs_rec...num_strs = %d\n",ptr->num_strs))
 if ((ptr == NULL) || (ptr->num_strs <= 1)) return NO_ERROR;
 sort_strs_rec_ascendingly(ptr);
 PRT(printf("...num_strs = %d\n",ptr->num_strs))
 for (lcv = 0,indx = 1; indx < ptr->num_strs; )
    if (str_cmp(ptr->strs[lcv],ptr->strs[indx]) == 0)
       {PRT(printf("%d) dup = %s\n",indx,ptr->strs[indx]))
        c_free(ptr->strs[indx]);
        drop_indx_str_ptr_from_strs_rec(ptr,indx);
       }
      else
       {indx++;
        lcv++;}
 PRT(printf("end...num_strs = %d\n",ptr->num_strs))
 return NO_ERROR;
}


/*-----------------------------------------------------------------------
setup_strs_rec - ???
-----------------------------------------------------------------------*/

int
setup_strs_rec(ptr,num_strs,cur_str)
 struct strs_rec **ptr;
 unsigned int num_strs,cur_str;
{
 unsigned int cnt;
 int err;

 cnt = 0;
 get_ptr_memory(ptr,&cnt,(unsigned int) 1,
                     (size_t) sizeof(struct strs_rec),&err);
 if (err != NO_ERROR)
    {*ptr = NULL;
     return err;}
 init_strs_rec(*ptr);
 if (num_strs == 0) return NO_ERROR;

 if ((err = add_strs_rec_entries(*ptr,num_strs)) != NO_ERROR)
    {free_strs_rec_ptrs(ptr);
     c_free(ptr);
     return err;}

 if (cur_str >= (*ptr)->num_strs) (*ptr)->cur_str = 0;
   else (*ptr)->cur_str = cur_str;

 return NO_ERROR;
}


/*-----------------------------------------------------------------------
build_strs_rec - ???

build_strs_rec(ptr,cur_str,total_strs,str1,str2,...,strn)
-----------------------------------------------------------------------*/

int
build_strs_rec(
    struct strs_rec *ptr,
    unsigned long cur_str,
    unsigned long total_strs,
    ...)
{
 va_list args;
 unsigned long lcv,indx;
 int err;

 /* FIXME:  Is the order of the next two statements correct? -PWH 19990405 */
 ptr->cur_str = cur_str;
 if (total_strs <= 0) return 0;

 va_start(args, total_strs);

 if ((err = add_strs_rec_entries(ptr,total_strs)) != NO_ERROR) return err;
 for (indx = ptr->num_strs,lcv = 0; lcv < total_strs; lcv++)
    {if ((err = cp_str(&(ptr->strs[indx]),va_arg(args, char *) )) != NO_ERROR) 
        {for (lcv = ptr->num_strs; lcv < indx; lcv++)
            {c_free(ptr->strs[lcv]);
             ptr->strs[lcv] = NULL;}
         return err;}
     indx++;}
 ptr->num_strs = indx;

 va_end(args);

 return err;
}

#if 0        /* Old varargs version */
int
build_strs_rec(va_alist)
 va_dcl
{
 va_list args;
 struct strs_rec *ptr;
 unsigned long lcv,indx,total_strs;
 int err;

 va_start(args);
 ptr = va_arg(args,struct strs_rec *);
 ptr->cur_str = va_arg(args,unsigned long);
 total_strs = va_arg(args,unsigned long);
 if (total_strs <= 0) return 0;
 va_start(args);
 va_arg(args,struct strs_rec *);
 va_arg(args,unsigned long);
 va_arg(args,unsigned long);

 if ((err = add_strs_rec_entries(ptr,total_strs)) != NO_ERROR) return err;
 for (indx = ptr->num_strs,lcv = 0; lcv < total_strs; lcv++)
    {if ((err = cp_str(&(ptr->strs[indx]),va_arg(args, char *) )) != NO_ERROR) 
        {for (lcv = ptr->num_strs; lcv < indx; lcv++)
            {c_free(ptr->strs[lcv]);
             ptr->strs[lcv] = NULL;}
         return err;}
     indx++;}
 ptr->num_strs = indx;

 va_end(args);

 return err;
}
#endif


/*-----------------------------------------------------------------------
drop_strs_rec_suffix - ???
-----------------------------------------------------------------------*/

void
drop_strs_rec_suffix(labels)
 struct strs_rec *labels;
{
 unsigned int lcv;

 for (lcv = 0; lcv < labels->num_strs; lcv++)
    drop_suffix(labels->strs[lcv]);
}


/*-----------------------------------------------------------------------
get_strs_rec_max_str_len - ???
-----------------------------------------------------------------------*/

int
get_strs_rec_max_str_len(labels)
 struct strs_rec *labels;
{
 unsigned int lcv,len,temp;

 if (labels == NULL) return 0;
 for (len = lcv = 0; lcv < labels->num_strs; lcv++)
    {temp = get_strlen(labels->strs[lcv]);
     if (temp > len) len = temp;}
 return len;
}


/*-----------------------------------------------------------------------
get_strs_rec_max_some_str_len - ???
-----------------------------------------------------------------------*/

int
get_strs_rec_max_some_str_len(labels,selected)
 struct strs_rec *labels;
 unsigned int *selected;
{
 unsigned int lcv,len,temp;

 if (labels == NULL) return 0;
 for (len = lcv = 0; lcv < labels->num_strs; lcv++)
    if (selected[lcv] > 0)
       {temp = get_strlen(labels->strs[lcv]);
        if (temp > len) len = temp;}
 return len;
}


/*-----------------------------------------------------------------------
pr_strs_rec - ???
-----------------------------------------------------------------------*/

void
pr_strs_rec(ptr)
 struct strs_rec *ptr;
{
 unsigned int lcv,cnt;

 printf("print_str_rec...ptr = %p status = %lx  total_strs = %ld "
	"cur_str = %ld num_strs = %ld\n",
	ptr,ptr->status,ptr->total_strs,ptr->cur_str,ptr->num_strs);
 for (lcv = 0,cnt = 0; lcv < ptr->num_strs; lcv++)
    printf("lcv = %d strlen = %d str = %p %s\n",lcv,
              get_strlen(ptr->strs[lcv]),ptr->strs[lcv],ptr->strs[lcv]);
 printf("print_str_rec...end\n");
 
}


/*-----------------------------------------------------------------------
init_str_value_rec - ???
-----------------------------------------------------------------------*/

void
init_str_value_rec(ptr)
 struct str_value_rec *ptr;
{
 ptr->str = NULL;
}


/*-----------------------------------------------------------------------
free_str_value_rec - ???
-----------------------------------------------------------------------*/

void
free_str_value_rec(ptr)
 struct str_value_rec *ptr;
{
 c_free(ptr->str);
 init_str_value_rec(ptr);
}


/*-----------------------------------------------------------------------
sort_str_value_rec_proc - ???
-----------------------------------------------------------------------*/

int
sort_str_value_rec_proc(val1,val2)
 struct str_value_rec *val1,*val2;
{
 if (val1->value > val2->value) return 1;
 if (val1->value < val2->value) return -1;
 return 0;
}


/*-----------------------------------------------------------------------
setup_str_value_rec - ???
-----------------------------------------------------------------------*/

int
setup_str_value_rec(ptr,str,value)
 struct str_value_rec *ptr;
 char *str;
 long value;
{
 int err;

 if ((err = cp_str(&(ptr->str),str)) != NO_ERROR) return err;
 ptr->value = value;
 return NO_ERROR;
}


/*-----------------------------------------------------------------------
setup_str_value_rec_ary - ???
-----------------------------------------------------------------------*/

int
setup_str_value_rec_ary(ary,num_elems,str,value)
 struct str_value_rec **ary;
 long *num_elems;
 char *str;
 long value;
{
 struct str_value_rec ptr;
 int err;

 init_str_value_rec(&ptr);
 if ((err = setup_str_value_rec(&ptr,str,value)) != NO_ERROR)
    return err;

 get_handle_memory(ary,num_elems,1,
       (size_t) sizeof(struct str_value_rec),&err);
 if (err != NO_ERROR)
    {free_str_value_rec(&ptr);
     return err;}
 (*ary)[*num_elems - 1] = ptr;
 return NO_ERROR;
}


/*-----------------------------------------------------------------------
free_str_value_rec_ary - ???
-----------------------------------------------------------------------*/

void
free_str_value_rec_ary(ary,num_elems)
 struct str_value_rec *ary;
 long num_elems;
{
 long lcv;

 for (lcv = 0; lcv < num_elems; lcv++)
    free_str_value_rec(&(ary[lcv]));
 c_free(ary);
}


/*-----------------------------------------------------------------------
selected_strs - ???

return a string of 1s indicating the presence of a selected member in the set.
Note that selected[0] is always 0. If a member is selected,
 selected[order(member) + 1] is 1.
-----------------------------------------------------------------------*/

int
selected_strs(set,selected_set,selected)
 struct strs_rec *set,*selected_set;
 unsigned char **selected;
{
 unsigned long total,indx,lcv;
 int err;

 *selected = NULL;
 if ((set->num_strs == 0) || (selected_set->num_strs == 0)) return NO_ERROR;
 total = 0;
 get_ptr_memory(selected,&total,set->num_strs + 1,
                    (size_t) sizeof(**selected),&err);
 if (err != NO_ERROR) return err;
 for (lcv = 0; lcv <= set->num_strs; lcv++)
    (*selected)[lcv] = 0;
 if (set->num_strs == 0) return NO_ERROR;
 for (lcv = 0; lcv < set->num_strs; lcv++)
    {for (indx = 0; indx < selected_set->num_strs; indx++)
        if (str_cmp(set->strs[lcv],selected_set->strs[indx]) == 0)
           {(*selected)[lcv + 1] = 1;
            break;}
    }
 return NO_ERROR;
}


/****************
selected_strs(set,selected_set,selected)
 struct strs_rec *set,*selected_set;
 unsigned char **selected;
{
 unsigned long total,indx,lcv,cnt;
 unsigned char *picks;
 int err;

 total = 0;
 get_ptr_memory(selected,&total,set->num_strs + 1,
                    (size_t) sizeof(**selected),&err);
 if (err != NO_ERROR) return err;
 (*selected)[0] = 0;
 if (set->num_strs == 0) return NO_ERROR;
 if (selected_set->num_strs == 0)
    {for (lcv = 1; lcv <= set->num_strs; lcv++)
        (*selected)[lcv] = 0;
     return NO_ERROR;}

 total = 0;
 get_ptr_memory(&picks,&total,selected_set->num_strs,
                    (size_t) sizeof(*picks),&err);
 if (err != NO_ERROR)
    {c_free(*selected);
     *selected = NULL;
     return err;}

 for (indx = 0; indx < selected_set->num_strs; indx++)
    picks[indx] = 0;

 for (cnt = lcv = 0; lcv < set->num_strs; lcv++)
    {for (cnt = indx = 0; indx < selected_set->num_strs; indx++)
        if (picks[indx]) cnt += 1;
          else if (str_cmp(set->strs[lcv],selected_set->strs[indx]) == 0)
           {(*selected)[lcv + 1] = 1;
            picks[indx] = 1;
            break;}
     if (cnt == selected_set->num_strs)
        {for (; lcv < set->num_strs; lcv++) (*selected)[lcv + 1] = 0;
         break;
        }
       else if (indx == selected_set->num_strs)
            (*selected)[lcv + 1] = 0;
    }
 c_free(picks);
 return NO_ERROR;
}
*****************/


/*-----------------------------------------------------------------------
cmp_str_arys_no_order - ???
-----------------------------------------------------------------------*/

int
cmp_str_arys_no_order(ary1,num_elems1,ary2,num_elems2,result)
 char **ary1,**ary2;
 long num_elems1,num_elems2;
 int *result;
{
 struct strs_rec ptr1,ptr2;
 long lcv;
 int err;

 err = NO_ERROR;
 if (num_elems1 != num_elems2) *result = 1;
   else
    {
     init_strs_rec(&ptr1);
     init_strs_rec(&ptr2);
     if (((err = add_str_ptr_ary_to_strs_rec(&ptr1,ary1,num_elems1)) == NO_ERROR) &&
         ((err = add_str_ptr_ary_to_strs_rec(&ptr2,ary2,num_elems2)) == NO_ERROR))
         {sort_strs_rec_ascendingly(&ptr1);
          sort_strs_rec_ascendingly(&ptr2);
          for (*result = 0,lcv = 0; lcv < ptr1.num_strs; lcv++)
             if (str_cmp(ptr1.strs[lcv],ptr2.strs[lcv]) != 0)
                {*result = 1;
                 break;}
         }
     free_strs_rec_ptrs(&ptr1);
     free_strs_rec_ptrs(&ptr2);
    }
 return err;
}


/*-----------------------------------------------------------------------
cmp_str_arys - ???
-----------------------------------------------------------------------*/

int
cmp_str_arys(ary1,num_elems1,ary2,num_elems2,result)
 char **ary1,**ary2;
 long num_elems1,num_elems2;
 int *result;
{
 long lcv;

 if (num_elems1 != num_elems2) *result = 1;
   else
    {
     for (*result = 0,lcv = 0; lcv < num_elems1; lcv++)
        if ((*result = str_cmp(ary1[lcv],ary2[lcv])) != 0)
           {break;}
    }
 return NO_ERROR;
}


/*-----------------------------------------------------------------------
 rd_strs_rec- ???
-----------------------------------------------------------------------*/

int
rd_strs_rec(fid,ptr)
 FILE *fid;
 struct strs_rec *ptr;
{
 unsigned long lcv,cur_str,num_strs;
 int err;

 TRACE(printf("rd_strs_rec 1\n"))
 if ((fread(&cur_str,(size_t) sizeof(cur_str),1,fid) != 1) ||
     (fread(&num_strs,(size_t) sizeof(num_strs),1,fid) != 1))
    return READ_ERROR;
   else
    {free_strs_rec(ptr);

     if (num_strs == 0) return NO_ERROR;
     if ((err = add_strs_rec_entries(ptr,num_strs)) != NO_ERROR)
        return err;

     if ((err = rd_file_strs(fid,ptr->strs,num_strs)) != NO_ERROR)
        {for (lcv = 0; lcv < num_strs; lcv++)
            {c_free(ptr->strs[lcv]);
             ptr->strs[lcv] = NULL;}
         return err;}
     ptr->num_strs = num_strs;
     ptr->cur_str = cur_str;}
 TRACE(printf("rd_strs_rec 2\n"))
 return NO_ERROR;
}


/*-----------------------------------------------------------------------
rd_ascii_file_strs_to_strs_rec - ???
-----------------------------------------------------------------------*/

int
rd_ascii_file_strs_to_strs_rec(label_fn,labels,is_sort)
 char *label_fn;
 struct strs_rec *labels;
 int is_sort;
{
 int err;
 char **strs;
 unsigned long num_strs,lcv;

 err = rd_all_file_texts(label_fn,&strs,&num_strs);
 if (num_strs > 0)
    if ((err = add_str_ptr_ary_to_strs_rec(labels,strs,num_strs)) ==
                NO_ERROR)
       {if ((labels->num_strs > 1) && is_sort) 
           sort_strs_rec_ascendingly(labels);
       }
      else
       {for (lcv = 0; lcv < num_strs; lcv++)
           c_free(strs[lcv]);
        c_free(strs);}
 return err;
}


/*-----------------------------------------------------------------------
add_double_to_strs_rec - ???
-----------------------------------------------------------------------*/

int
add_double_to_strs_rec(ptr,fl_num,num_decimals)
 struct strs_rec *ptr;
 double fl_num;
 int num_decimals;
{
 char str[200],format[20];

 sprintf(format,"%%-.%df",num_decimals);
 sprintf(str,format,fl_num);
 return add_str_to_strs_rec(ptr,str);
}


/*-----------------------------------------------------------------------
add_long_to_strs_rec - ???
-----------------------------------------------------------------------*/

int
add_long_to_strs_rec(ptr,long_num)
 struct strs_rec *ptr;
 long long_num;
{
 char str[200];

 sprintf(str,"%-ld",long_num);
 return add_str_to_strs_rec(ptr,str);
}


/*-----------------------------------------------------------------------
longest_str_len - ???
-----------------------------------------------------------------------*/

int
longest_str_len(ptr,longest_indx)
 struct strs_rec *ptr;
 long *longest_indx;
{
 long lcv,longest,len;

 *longest_indx = -1;
 for (longest = 0,lcv = 0; lcv < ptr->num_strs; lcv++)
    {len = get_strlen(ptr->strs[lcv]);
     if (len > longest)
        {longest = len;
         *longest_indx = lcv;}
    }
 return longest;
}


/*-----------------------------------------------------------------------
wr_strs_rec - ???
-----------------------------------------------------------------------*/

int
wr_strs_rec(fid,ptr)
 FILE *fid;
 struct strs_rec *ptr;
{
 if ((fwrite(&(ptr->cur_str),(size_t) sizeof(ptr->cur_str),1,fid) != 1) ||
     (fwrite(&(ptr->num_strs),(size_t) sizeof(ptr->num_strs),1,fid) != 1))
    return WRITE_ERROR;
   else if (ptr->num_strs > 0)
    return wr_file_strs(fid,ptr->strs,ptr->num_strs);
 return NO_ERROR;
}

