/****************************************************************************
scale.c - ???

Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.
****************************************************************************/

static const char rcsid[] = "$Id: scale.c,v 1.7 1999/07/07 18:01:07 hughett Exp $ Release $Name: nria_1_0_6 $";

#include <stdio.h>
#include <math.h>
#include <string.h>

#include <sys/types.h>
#include <sys/timeb.h>

#include "common.h"
#include "error_constants.h"
#include "nria_lib.h"



/*-----------------------------------------------------------------------
scale_1d_byte_values - ???
-----------------------------------------------------------------------*/

int
scale_1d_byte_values(out_1d,in_1d,dimx,dimy,min,max,new_min,new_max)
 unsigned char *out_1d,*in_1d;
 unsigned long dimx,dimy;
 unsigned long min,max,new_min,new_max;
{
 unsigned long row,col,cnt,height,new_height;
 float ratio;
 static unsigned char buf[256];
 static unsigned long last_min,last_max,last_new_min,last_new_max,first = 1;

 if ((min == new_min) && (max == new_max) && (in_1d == out_1d))
    return NO_ERROR;
 if ((min > max) || (new_min > new_max)) return CANCEL;
 if ((new_min == new_max) || (min == max))
    {
     for (cnt = row = 0; row < dimy; row++)
        for (col = 0; col < dimx; col++,cnt++)
           out_1d[cnt] = new_min;
     return NO_ERROR;}
 height = max - min;
 new_height = new_max - new_min;
 ratio = (float) new_height / (float) height;
 if (first || (last_min != min) || (last_max != max) ||
     (last_new_min != new_min) || (last_new_max != new_max))
    {first = 0;
     last_min = min;
     last_max = max;
     last_new_min = new_min;
     last_new_max = new_max;

     for (row = 0; row < min; row++)
         buf[row] = new_min;
     for (row = min; row <= max; row++)
        buf[row] = new_min +
                      (long) ceil((double)((row - min) * ratio));
     for (row = max; row < 256; row++)
         buf[row] = new_max;
    }
 for (cnt = row = 0; row < dimy; row++)
    for (col = 0; col < dimx; col++,cnt++)
       out_1d[cnt] = buf[in_1d[cnt]];
 return NO_ERROR;
}


/*-----------------------------------------------------------------------
scale_1d_short_values - ???
-----------------------------------------------------------------------*/

int
scale_1d_short_values(out_1d,in_1d,dimx,dimy,min,max,new_min,new_max)
 unsigned short *out_1d,*in_1d;
 unsigned long dimx,dimy;
 unsigned long min,max,new_min,new_max;
{
 unsigned long row,col,cnt,height,new_height;
 float ratio;

 if ((min == new_min) && (max == new_max) && (in_1d == out_1d))
    return NO_ERROR;
 if ((min > max) || (new_min > new_max)) return CANCEL;
 if ((new_min == new_max) || (min == max))
    {for (cnt = row = 0; row < dimy; row++)
        for (col = 0; col < dimx; col++,cnt++)
           out_1d[cnt] = new_min;
     return NO_ERROR;}

 height = max - min;
 new_height = new_max - new_min;
 ratio = (float) new_height / (float) height;

 for (cnt = row = 0; row < dimy; row++)
    for (col = 0; col < dimx; col++,cnt++)
       {if (in_1d[cnt] >= max) out_1d[cnt] = new_max;
          else out_1d[cnt] = new_min +  
                (long) ceil((double)((in_1d[cnt] - min) * ratio));
       }
 return NO_ERROR;
}


/*-----------------------------------------------------------------------
scale_1d_int_values - ???
-----------------------------------------------------------------------*/

int
scale_1d_int_values(out_1d,in_1d,dimx,dimy,min,max,new_min,new_max)
 unsigned int *out_1d,*in_1d;
 unsigned long dimx,dimy;
 unsigned long min,max,new_min,new_max;
{
 unsigned long row,col,cnt,height,new_height;
 float ratio;

 if ((min == new_min) && (max == new_max) && (in_1d == out_1d))
    return NO_ERROR;
 if ((min > max) || (new_min > new_max)) return CANCEL;
 if ((new_min == new_max) || (min == max))
    {for (cnt = row = 0; row < dimy; row++)
        for (col = 0; col < dimx; col++,cnt++)
           out_1d[cnt] = new_min;
     return NO_ERROR;}
 height = max - min;
 new_height = new_max - new_min;
 ratio = (float) new_height / (float) height;

 for (cnt = row = 0; row < dimy; row++)
    for (col = 0; col < dimx; col++,cnt++)
       {if (in_1d[cnt] >= max) out_1d[cnt] = new_max;
          else out_1d[cnt] = new_min +  
                (long) ceil((double)((in_1d[cnt] - min) * ratio));
       }
 return NO_ERROR;
}


/*-----------------------------------------------------------------------
scale_values - ???
-----------------------------------------------------------------------*/

int
scale_values(out_1d,in_1d,dimx,dimy,min,max,new_min,new_max,elem_bytes)
 unsigned char *out_1d,*in_1d;
 unsigned long dimx,dimy;
 unsigned long min,max,new_min,new_max;
 unsigned long elem_bytes;
{
 if (elem_bytes == 1)
    return scale_1d_byte_values(out_1d,in_1d,dimx,dimy,min,max,new_min,new_max);
   else if (elem_bytes == 2)
    return scale_1d_short_values(out_1d,in_1d,dimx,dimy,min,max,new_min,
              new_max);
   else if (elem_bytes == 4)
    return scale_1d_int_values(out_1d,in_1d,dimx,dimy,min,max,new_min,new_max);
   else return CANCEL;
}


/*-----------------------------------------------------------------------
assign_values - ???
-----------------------------------------------------------------------*/

int
assign_values(dest,dest_elem_len,src,src_elem_len,num_elems)
 unsigned char *dest,*src;
 unsigned long dest_elem_len,src_elem_len,num_elems;
{
 unsigned long offset,lcv;

 if (dest_elem_len == src_elem_len)
    {if (dest != src) memcpy(dest,src,num_elems * src_elem_len);
    }
   else if (dest_elem_len < src_elem_len)
    {offset = src_elem_len - dest_elem_len;
     for (lcv = 0; lcv < num_elems; lcv++)
        {
         memcpy(dest,src + offset,dest_elem_len);
         dest += dest_elem_len;
         src += src_elem_len;
        }
    }
   else
    {offset = dest_elem_len - src_elem_len;
     for (lcv = 0; lcv < num_elems; lcv++)
        {memset(dest,0,offset);
         memcpy(dest + offset,src,src_elem_len);
         dest += dest_elem_len;
         src += src_elem_len;
        }
    }
 return NO_ERROR;
}


/*-----------------------------------------------------------------------
get_byte_ary_min_max - ???
-----------------------------------------------------------------------*/

int
get_byte_ary_min_max(in_1d,num_elems,min,max)
 unsigned char *in_1d;
 unsigned long num_elems,*min,*max;
{
 unsigned long cnt;

 *min = 0xffffffff;
 *max = 0;
 for (cnt = 0; cnt < num_elems; cnt++)
    {if (in_1d[cnt] > *max) *max = in_1d[cnt];
     if (in_1d[cnt] < *min) *min = in_1d[cnt];}
 return NO_ERROR;
}


/*-----------------------------------------------------------------------
get_short_ary_min_max - ???
-----------------------------------------------------------------------*/

int
get_short_ary_min_max(in_1d,num_elems,min,max)
 unsigned short *in_1d;
 unsigned long num_elems,*min,*max;
{
 unsigned long cnt;

 *min = 0xffffffff;
 *max = 0;
 for (cnt = 0; cnt < num_elems; cnt++)
    {if (in_1d[cnt] > *max) *max = in_1d[cnt];
     if (in_1d[cnt] < *min) *min = in_1d[cnt];}
 return NO_ERROR;
}


/*-----------------------------------------------------------------------
get_int_ary_min_max - ???
-----------------------------------------------------------------------*/

int
get_int_ary_min_max(in_1d,num_elems,min,max)
 unsigned short *in_1d;
 unsigned long num_elems,*min,*max;
{
 unsigned long cnt;

 *min = 0xffffffff;
 *max = 0;
 for (cnt = 0; cnt < num_elems; cnt++)
    {if (in_1d[cnt] > *max) *max = in_1d[cnt];
     if (in_1d[cnt] < *min) *min = in_1d[cnt];}
 return NO_ERROR;
}


/*-----------------------------------------------------------------------
get_1d_ary_min_max - ???
-----------------------------------------------------------------------*/

int
get_1d_ary_min_max(in_1d,num_elems,min,max,elem_bytes)
 unsigned short *in_1d;
 unsigned long num_elems,*min,*max;
{

 if (elem_bytes == 1)
    return get_byte_ary_min_max(in_1d,num_elems,min,max);
   else if (elem_bytes == 2)
    return get_short_ary_min_max(in_1d,num_elems,min,max);
   else if (elem_bytes == 4)
    return get_int_ary_min_max(in_1d,num_elems,min,max);
   else return CANCEL;
}
