/****************************************************************************
vector.c - ???

Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.
****************************************************************************/

static const char rcsid[] = "$Id: vector.c,v 1.5 1999/07/07 18:01:07 hughett Exp $ Release $Name: nria_1_0_6 $";


#include <stdio.h>
#include <sys/types.h>
#include <math.h>

#include "typedef.h"
#include "geometry.h"
#include "error_constants.h"
#include "nria_lib.h"


/*
cc -c vector.c
cc -o vector vector.c $lib/math_opr.o -lm
*/




/*--------------------------------------------------------------------------
add_xyz_to_vector_3d - ???
---------------------------------------------------------------------------*/

void
add_xyz_to_vector_3d(v1,x,y,z)
 struct vector_3d_rec *v1;
 float x,y,z;
{
 v1->x += x;
 v1->y += y;
 v1->z += z;
}


/*--------------------------------------------------------------------------
add_vector_3d - ???
---------------------------------------------------------------------------*/

void
add_vector_3d(v1,v2)
 struct vector_3d_rec *v1,*v2;
{
 v1->x += v2->x;
 v1->y += v2->y;
 v1->z += v2->z;
}


/*--------------------------------------------------------------------------
sub_vector_3d - ???
---------------------------------------------------------------------------*/

void
sub_vector_3d(v1,v2)
 struct vector_3d_rec *v1,*v2;
{
 v1->x -= v2->x;
 v1->y -= v2->y;
 v1->z -= v2->z;
} 


/*--------------------------------------------------------------------------
sub_vectors - ???
---------------------------------------------------------------------------*/

void
sub_vectors(result,v1,v2)
 struct vector_3d_rec *result,*v1,*v2;
{
 result->x = v1->x - v2->x;
 result->y = v1->y - v2->y;
 result->z = v1->z - v2->z;
}


/*--------------------------------------------------------------------------
add_vectors - ???
---------------------------------------------------------------------------*/

void
add_vectors(result,v1,v2)
 struct vector_3d_rec *result,*v1,*v2;
{
 result->x = v1->x + v2->x;
 result->y = v1->y + v2->y;
 result->z = v1->z + v2->z;
}


/*--------------------------------------------------------------------------
mult_vector_with_const - ???
---------------------------------------------------------------------------*/

void
mult_vector_with_const(v1,fl_const)
 struct vector_3d_rec *v1;
 float fl_const;
{
 v1->x *= fl_const;
 v1->y *= fl_const;
 v1->z *= fl_const;
}



/*--------------------------------------------------------------------------
cross_product - ???
---------------------------------------------------------------------------*/

void
cross_product(x,y,z,x1,y1,z1,x2,y2,z2)
 float *x,*y,*z,x1,y1,z1,x2,y2,z2;
{
 float tempx,tempy,tempz;

 tempx = y1 * z2 - z1 * y2;
 tempy = z1 * x2 - x1 * z2;
 tempz = x1 * y2 - y1 * x2;
 *x = tempx;
 *y = tempy;
 *z = tempz;
}


/*--------------------------------------------------------------------------
vector_cross_product - ???
---------------------------------------------------------------------------*/

void
vector_cross_product(result,v1,v2)
 struct vector_3d_rec *result,*v1,*v2;
{
 cross_product(&(result->x),&(result->y),&(result->z),
	       v1->x,v1->y,v1->z,v2->x,v2->y,v2->z);
 return;
}



/*--------------------------------------------------------------------------
dot_product - ???
---------------------------------------------------------------------------*/

float 
dot_product(x1,y1,z1,x2,y2,z2)
 float x1,y1,z1,x2,y2,z2;
{
 return x1 * x2 + y1 * y2 + z1 * z2;
}


/*--------------------------------------------------------------------------
vector_dot_product - ???
---------------------------------------------------------------------------*/

float 
vector_dot_product(v1,v2)
 struct vector_3d_rec *v1,*v2;
{
 return dot_product(v1->x,v1->y,v1->z,v2->x,v2->y,v2->z);
}


/*--------------------------------------------------------------------------
setup_vector_3d_rec - ???
---------------------------------------------------------------------------*/

void
setup_vector_3d_rec(vector,x,y,z)
 struct vector_3d_rec *vector;
 float x,y,z;
{
 vector->status = 0;
 vector->x = x;
 vector->y = y;
 vector->z = z;
}


/*--------------------------------------------------------------------------
length_vector - ???
---------------------------------------------------------------------------*/

float 
length_vector(vector)
 struct vector_3d_rec *vector;
{
 float length;

 length = sqrt((double) vector->x * (double) vector->x + 
               (double) vector->y * (double) vector->y + 
               (double) vector->z * (double) vector->z);
 return length;
}


/*--------------------------------------------------------------------------
degree_angle_vectors - ???
---------------------------------------------------------------------------*/

float 
degree_angle_vectors(v1,v2)
 struct vector_3d_rec *v1,*v2;
{
 float dot,len1,len2;

 dot = vector_dot_product(v1,v2);
 len1 = length_vector(v1);
 len2 = length_vector(v2);
 len1 = len1 * len2;
 if (len1 == .0) return .0;
 dot = dot / len1;
 return (float) acos_to_degree(dot);
}


/*--------------------------------------------------------------------------
vector_distance - ???
---------------------------------------------------------------------------*/

float 
vector_distance(v1,v2)
 struct vector_3d_rec *v1,*v2;
{
 double x,y,z;
 float length;

 x = (double) (v1->x - v2->x);
 y = (double) (v1->y - v2->y);
 z = (double) (v1->z - v2->z);
 length = (float) sqrt(x * x + y * y + z * z);
 return length;
}


/*--------------------------------------------------------------------------
vector_distance_from_abs - ???
---------------------------------------------------------------------------*/

float 
vector_distance_from_abs(v1)
 struct vector_3d_rec *v1;
{
 float len1,len2;
 struct vector_3d_rec v2;

 len1 = length_vector(v1);
 v2.x = (float) floor((double) (v1->x + .5));
 v2.y = (float) floor((double) (v1->y + .5));
 v2.z = (float) floor((double) (v1->z + .5));
 len2 = length_vector(&v2);
 len1 -= len2;
 if (len1 < .0) len1 -= len1;
 return len1;
}


/*--------------------------------------------------------------------------
mid_vectors - ???
---------------------------------------------------------------------------*/

void
mid_vectors(result,v1,v2)
 struct vector_3d_rec *result,*v1,*v2;
{
 result->x = (v1->x + v2->x) / 2.0;
 result->y = (v1->y + v2->y) / 2.0;
 result->z = (v1->z + v2->z) / 2.0;
}


/*--------------------------------------------------------------------------
unit_vector - ???
---------------------------------------------------------------------------*/

void
unit_vector(ux,uy,uz,x,y,z,is_infinity)
 float *ux,*uy,*uz,x,y,z;
 int *is_infinity;
{
 float length;

 if ((x == .0) && (y == .0) && (z == .0))
    *is_infinity = 1;
   else
    {*is_infinity = 0;
     length = sqrt((double) (x * x + y * y + z * z));
     *ux = x / length;
     *uy = y / length;
     *uz = z / length;}
}


/*--------------------------------------------------------------------------
direction_cosine - ???
---------------------------------------------------------------------------*/

void
direction_cosine(vector,cosx,cosy,cosz,is_infinity)
 struct vector_3d_rec *vector;
 float *cosx,*cosy,*cosz;
 int *is_infinity;
{
 float length;

 if ((vector->x == 0) && (vector->y == 0) && (vector->z == 0))
    {
     *is_infinity = 1;
    }
   else
    {*is_infinity = 0;
     length = length_vector(vector);
     *cosx = (float) vector->x / length;
     *cosy = (float) vector->y / length;
     *cosz = (float) vector->z / length;
    }
}


/*--------------------------------------------------------------------------
direction_cosine_vector - ???
---------------------------------------------------------------------------*/

void
direction_cosine_vector(vector)
 struct vector_3d_rec *vector;
{
 int is_infinity;

 direction_cosine(vector,&(vector->x),&(vector->y),&(vector->z),&is_infinity);
 if (is_infinity) vector->status |= VECTOR_ORIGIN;
   else vector->status |= VECTOR_DIRECTION_COSINE;
}


/*--------------------------------------------------------------------------
floor_vector_3d_rec - ???
---------------------------------------------------------------------------*/

void
floor_vector_3d_rec(vector)
 struct vector_3d_rec *vector;
{
 vector->x = (float) floor((double) vector->x);
 vector->y = (float) floor((double) vector->y);
 vector->z = (float) floor((double) vector->z);
}


/*--------------------------------------------------------------------------
truncate_vector_3d_rec - ???
---------------------------------------------------------------------------*/

void
truncate_vector_3d_rec(vector)
 struct vector_3d_rec *vector;
{
 double one;

 one = 1.0;
 vector->x -= (float) fmod((double) vector->x,one);
 vector->y -= (float) fmod((double) vector->y,one);
 vector->z -= (float) fmod((double) vector->z,one);
}


/*--------------------------------------------------------------------------
round_vector_3d_rec - ???
---------------------------------------------------------------------------*/

void
round_vector_3d_rec(vector)
 struct vector_3d_rec *vector;
{
 vector->x = (float) floor((double) (vector->x + .5));
 vector->y = (float) floor((double) (vector->y + .5));
 vector->z = (float) floor((double) (vector->z + .5));
}


/*--------------------------------------------------------------------------
pos_vector_3d_rec - ???
---------------------------------------------------------------------------*/

void
pos_vector_3d_rec(vector)
 struct vector_3d_rec *vector;
{
 if (vector->x < .0) vector->x = -vector->x;
 if (vector->y < .0) vector->y = -vector->y;
 if (vector->z < .0) vector->z = -vector->z;
}


/*--------------------------------------------------------------------------
reverse_vector_3d_rec - ???
---------------------------------------------------------------------------*/

void
reverse_vector_3d_rec(vector)
 struct vector_3d_rec *vector;
{
 vector->x = -vector->x;
 vector->y = -vector->y;
 vector->z = -vector->z;
}


/*--------------------------------------------------------------------------
same_direction_vectors - ???
---------------------------------------------------------------------------*/

int
same_direction_vectors(v1,v2)
 struct vector_3d_rec *v1,*v2;
{
 struct vector_3d_rec temp1,temp2;

 temp1 = *v1;
 temp2 = *v2;
 direction_cosine_vector(&temp1);
 direction_cosine_vector(&temp2);
 if ((temp1.x == temp2.x) && (temp1.y == temp2.y) && (temp1.z == temp2.z))
    return 1;
   else return 0;
}


/*--------------------------------------------------------------------------
same_dimension_vectors - ???
---------------------------------------------------------------------------*/

int
same_dimension_vectors(v1,v2)
 struct vector_3d_rec *v1,*v2;
{
 if ((v1->x < .0) && (v2->x >= .0)) return 0;
 if ((v1->x > .0) && (v2->x <= .0)) return 0;
 if ((v1->x == .0) && (v2->x != .0)) return 0;

 if ((v1->y < .0) && (v2->y >= .0)) return 0;
 if ((v1->y > .0) && (v2->y <= .0)) return 0;
 if ((v1->y == .0) && (v2->y != .0)) return 0;

 if ((v1->z < .0) && (v2->z >= .0)) return 0;
 if ((v1->z > .0) && (v2->z <= .0)) return 0;
 if ((v1->z == .0) && (v2->z != .0)) return 0;
 printf("same_dimension_vectors...\n");
 return 1;
}


/*--------------------------------------------------------------------------
is_reverse_vector - ???
---------------------------------------------------------------------------*/

int
is_reverse_vector(v1,v2)
 struct vector_3d_rec *v1,*v2;
{
 if ((v1->x == -(v2->x)) && (v1->y == -(v2->y)) && (v1->z == -(v2->z)))
    return 1;
   else return 0;
}


/*--------------------------------------------------------------------------
same_3d_pt - ???
---------------------------------------------------------------------------*/

int
same_3d_pt(pt1,pt2)
 struct vector_3d_rec *pt1,*pt2;
{
 if ((pt1->x == pt2->x) && (pt1->y == pt2->y) && (pt1->z == pt2->z))
    return 1;
   else return 0;
}


/*--------------------------------------------------------------------------
fl_in_range - ???
---------------------------------------------------------------------------*/

int
fl_in_range(fl,range1,range2)
 float fl,range1,range2;
{
 if (range1 > range2)
    {if ((fl < (range2 - TOLERANCE)) || (fl > (range1 + TOLERANCE))) return 0;}
   else if ((fl < (range1 - TOLERANCE)) || (fl > (range2 + TOLERANCE))) return 0;

 return 1;
}


/*--------------------------------------------------------------------------
pt_in_range - ???
---------------------------------------------------------------------------*/

int
pt_in_range(pt,v1,v2)
 struct vector_3d_rec *pt,*v1,*v2;
{
 if (fl_in_range(pt->x,v1->x,v2->x) &&
     fl_in_range(pt->y,v1->y,v2->y) &&
     fl_in_range(pt->z,v1->z,v2->z)) return 1;
 return 0;
}


/*--------------------------------------------------------------------------
num_pt_coords_in_range - ???
---------------------------------------------------------------------------*/

void
num_pt_coords_in_range(pt,v1,v2,num_coords_in_range)
 struct vector_3d_rec *pt,*v1,*v2;
 int *num_coords_in_range;
{
 *num_coords_in_range = 0;
 if (fl_in_range(pt->x,v1->x,v2->x)) *num_coords_in_range += 1;
 if (fl_in_range(pt->y,v1->y,v2->y)) *num_coords_in_range += 1;
 if (fl_in_range(pt->z,v1->z,v2->z)) *num_coords_in_range += 1;
}


/*--------------------------------------------------------------------------
rounded_pt_in_range - ???
---------------------------------------------------------------------------*/

int
rounded_pt_in_range(pt,v1,v2)
 struct vector_3d_rec *pt,*v1,*v2;
{
 struct vector_3d_rec t1,t2,t3;

 t3 = *pt;
 round_vector_3d_rec(&t3);
 t1 = *v1;
 round_vector_3d_rec(&t1);
 t2 = *v2;
 round_vector_3d_rec(&t2);
 return pt_in_range(&t3,&t1,&t2);
}


/*--------------------------------------------------------------------------
vector_colinear - ???
---------------------------------------------------------------------------*/

int
vector_colinear(vector1,vector2)
 struct vector_3d_rec *vector1,*vector2;
{
 float fl,dot;

 fl = length_vector(vector1) * length_vector(vector2);
 dot = dot_product(vector1->x,vector1->y,vector1->z,
                   vector2->x,vector2->y,vector2->z);
 if (dot == fl) return 1;
 return 0;

}


/*--------------------------------------------------------------------------
colinear_3_pts - ???
---------------------------------------------------------------------------*/

int
colinear_3_pts(x1,y1,z1,x2,y2,z2,x3,y3,z3)
 float x1,y1,z1,x2,y2,z2,x3,y3,z3;
{
 struct vector_3d_rec p2,p3;

 setup_vector_3d_rec(&p2,x2 - x1,y2 - y1,z2 - z1);
 setup_vector_3d_rec(&p3,x3 - x1,y3 - y1,z3 - z1);
 return vector_colinear(&p2,&p3);
}


/*--------------------------------------------------------------------------
pt_line_dist - ???
---------------------------------------------------------------------------*/

float 
pt_line_dist(base_vector,direction_vector,pt)
 struct vector_3d_rec *base_vector,*direction_vector,*pt;
{
 struct vector_3d_rec temp,temp1,temp2;

 temp.x = pt->x - base_vector->x;
 temp.y = pt->y - base_vector->y;
 temp.z = pt->z - base_vector->z;
 temp2 = *direction_vector;
 direction_cosine_vector(&temp2);
 vector_cross_product(&temp1,&temp,&temp2);
 return length_vector(&temp1);
}


/*--------------------------------------------------------------------------
get_vector_plane_type - ???
---------------------------------------------------------------------------*/

int
get_vector_plane_type(vector)
 struct vector_3d_rec *vector;
{
 if (vector->x == .0)
    {if (vector->y == .0)
        {if (vector->z == .0) return VECTOR_ORIGIN;
         return VECTOR_Z_PLANE;}
     if (vector->z == .0) return VECTOR_Y_PLANE;}
   else if (vector->y == .0)
    {if (vector->z == .0) return VECTOR_X_PLANE;}
 return VECTOR_SLOPE_PLANE;
}


/*--------------------------------------------------------------------------
is_vectors_ordered - ???
---------------------------------------------------------------------------*/

int
is_vectors_ordered(vectors,num_vectors)
 struct vector_3d_rec **vectors;
 unsigned long num_vectors;
{
 struct vector_3d_rec vec1,vec2;
 float dot,len1,len2,rad_degree,total_degree;
 unsigned long lcv,last,next;

 total_degree = 0;
 last = num_vectors - 1;
 setup_vector_3d_rec(&vec1,vectors[last]->x - vectors[0]->x,
         vectors[last]->y - vectors[0]->y,vectors[last]->z - vectors[0]->z);
 next = 1;
 for (lcv = 0; lcv < num_vectors; lcv++)
    {
     setup_vector_3d_rec(&vec2,vectors[next]->x - vectors[lcv]->x,
         vectors[next]->y - vectors[lcv]->y,vectors[next]->z - vectors[lcv]->z);
     dot = vector_dot_product(&vec1,&vec2);
     len1 = length_vector(&vec1);
     len2 = length_vector(&vec2);
     rad_degree = dot / (len1 * len2);
     rad_degree = acos((double) rad_degree);
     total_degree += degree(rad_degree);
     vec1 = vec2;
     next++;
     if (next == num_vectors) next = 0;
    }
 if (total_degree == 360.0) return 1;
   else return 0;
}


/*--------------------------------------------------------------------------
arrange_vectors - ???
---------------------------------------------------------------------------*/

int
arrange_vectors(vectors,num_vectors)
 struct vector_3d_rec *vectors;
 unsigned long num_vectors;
{
 struct vector_3d_rec **vecs,*vecs1;
 unsigned long cnt,total,lcv,num,indx;
 int err;

 if (num_vectors < 4) return NO_ERROR;
 total = 0;
 get_ptr_memory(&vecs,&total,num_vectors,
     (size_t) sizeof(*vecs),&err);
 if (err != NO_ERROR) return err;
 total = 0;
 get_ptr_memory(&vecs1,&total,num_vectors,
     (size_t) sizeof(*vecs1),&err);
 if (err != NO_ERROR)
    {c_free(vecs);
     return err;}
 for (lcv = 0; lcv < 3; lcv++) vecs[lcv] = &(vectors[lcv]);
 num = 3;
 for (lcv = 3; lcv < num_vectors; lcv++)
    {
     for (cnt = lcv; cnt > 1; cnt--)
        {
         for (indx = lcv; indx > cnt; indx--)
            vecs[indx] = vecs[indx - 1];
         vecs[cnt] = &(vectors[lcv]);
         if (is_vectors_ordered(vecs,lcv)) break;
        }
     if (cnt == 1) break;
    }
 if (lcv == num_vectors)
    {for (lcv = 0; lcv < num_vectors; lcv++)
        vecs1[lcv] = *(vecs[lcv]);
     for (lcv = 0; lcv < num_vectors; lcv++)
        vectors[lcv] = vecs1[lcv];
     err = NO_ERROR;}
   else
    {
     err = CANCEL;}
 c_free(vecs);
 c_free(vecs1);
 return err;
}


/*--------------------------------------------------------------------------
pr_vector_3d_rec - ???
---------------------------------------------------------------------------*/

void
pr_vector_3d_rec(vector)
 struct vector_3d_rec *vector;
{
 printf("   vector status = %lx  [x,y,z] = %.6f  %.6f  %.6f\n",
          vector->status,vector->x,vector->y,vector->z);
}


/*--------------------------------------------------------------------------
cuboid_vertices - ???
---------------------------------------------------------------------------*/

void
cuboid_vertices(vertices,startx,endx,starty,endy,startz,endz)
 struct vector_3d_rec vertices[8];
 float startx,endx,starty,endy,startz,endz;
{
 setup_vector_3d_rec(&(vertices[0]),startx,starty,startz);
 setup_vector_3d_rec(&(vertices[1]),endx,starty,startz);
 setup_vector_3d_rec(&(vertices[2]),startx,endy,startz);
 setup_vector_3d_rec(&(vertices[3]),endx,endy,startz);
 setup_vector_3d_rec(&(vertices[4]),startx,starty,endz);
 setup_vector_3d_rec(&(vertices[5]),endx,starty,endz);
 setup_vector_3d_rec(&(vertices[6]),startx,endy,endz);
 setup_vector_3d_rec(&(vertices[7]),endx,endy,endz);
}


/*--------------------------------------------------------------------------
cuboid_edges - ???
---------------------------------------------------------------------------*/

void
cuboid_edges(vertices,startx,endx,starty,endy,startz,endz)
 struct vector_3d_rec vertices[24];
 float startx,endx,starty,endy,startz,endz;
{
 setup_vector_3d_rec(&(vertices[0]),startx,starty,startz);
 setup_vector_3d_rec(&(vertices[1]),endx,starty,startz);
 setup_vector_3d_rec(&(vertices[2]),startx,starty,startz);
 setup_vector_3d_rec(&(vertices[3]),startx,endy,startz);
 setup_vector_3d_rec(&(vertices[4]),startx,starty,startz);
 setup_vector_3d_rec(&(vertices[5]),startx,starty,endz);

 setup_vector_3d_rec(&(vertices[6]),endx,starty,startz);
 setup_vector_3d_rec(&(vertices[7]),endx,endy,startz);
 setup_vector_3d_rec(&(vertices[8]),endx,starty,endz);
 setup_vector_3d_rec(&(vertices[9]),startx,endy,startz);

 setup_vector_3d_rec(&(vertices[10]),startx,endy,startz);
 setup_vector_3d_rec(&(vertices[11]),endx,endy,startz);
 setup_vector_3d_rec(&(vertices[12]),startx,endy,startz);
 setup_vector_3d_rec(&(vertices[13]),startx,endy,endz);

 setup_vector_3d_rec(&(vertices[14]),endx,endy,startz);
 setup_vector_3d_rec(&(vertices[15]),endx,endy,endz);

 setup_vector_3d_rec(&(vertices[16]),startx,starty,endz);
 setup_vector_3d_rec(&(vertices[17]),endx,starty,endz);
 setup_vector_3d_rec(&(vertices[18]),startx,starty,endz);
 setup_vector_3d_rec(&(vertices[19]),startx,endy,endz);

 setup_vector_3d_rec(&(vertices[20]),endx,starty,endz);
 setup_vector_3d_rec(&(vertices[21]),endx,endy,endz);

 setup_vector_3d_rec(&(vertices[22]),startx,endy,endz);
 setup_vector_3d_rec(&(vertices[23]),endx,endy,endz);
}


/*--------------------------------------------------------------------------
cuboid_plane_vertices - ???
---------------------------------------------------------------------------*/

void
cuboid_plane_vertices(bounds,vertices,indx_v1,indx_v2,indx_v3,indx_v4)
 struct vector_3d_rec bounds[4],vertices[8];
 int indx_v1,indx_v2,indx_v3,indx_v4;
{
 bounds[0] = vertices[indx_v1];
 bounds[1] = vertices[indx_v2];
 bounds[2] = vertices[indx_v3];
 bounds[3] = vertices[indx_v4];
}


/*--------------------------------------------------------------------------
cuboid_planes - ???
---------------------------------------------------------------------------*/

void
cuboid_planes(bases,normals,vertices,startx,endx,starty,endy,startz,endz)
 struct vector_3d_rec bases[6],normals[6],vertices[6][4];
 float startx,endx,starty,endy,startz,endz;
{
 setup_vector_3d_rec(&(bases[0]),startx,starty,startz);
 setup_vector_3d_rec(&(normals[0]),.0,.0,1.0);
 setup_vector_3d_rec(&(vertices[0][0]),startx,starty,startz);
 setup_vector_3d_rec(&(vertices[0][1]),endx,starty,startz);
 setup_vector_3d_rec(&(vertices[0][2]),endx,endy,startz);
 setup_vector_3d_rec(&(vertices[0][3]),startx,endy,startz);

 setup_vector_3d_rec(&(bases[1]),endx,starty,startz);
 setup_vector_3d_rec(&(normals[1]),1.0,.0,.0);
 setup_vector_3d_rec(&(vertices[1][0]),endx,starty,startz);
 setup_vector_3d_rec(&(vertices[1][1]),endx,starty,endz);
 setup_vector_3d_rec(&(vertices[1][2]),endx,endy,endz);
 setup_vector_3d_rec(&(vertices[1][3]),endx,endy,startz);

 setup_vector_3d_rec(&(bases[2]),startx,starty,endz);
 setup_vector_3d_rec(&(normals[2]),.0,.0,1.0);
 setup_vector_3d_rec(&(vertices[2][0]),startx,starty,endz);
 setup_vector_3d_rec(&(vertices[2][1]),endx,starty,endz);
 setup_vector_3d_rec(&(vertices[2][2]),endx,endy,endz);
 setup_vector_3d_rec(&(vertices[2][3]),startx,endy,endz);

 setup_vector_3d_rec(&(bases[3]),startx,starty,startz);
 setup_vector_3d_rec(&(normals[3]),1.0,.0,.0);
 setup_vector_3d_rec(&(vertices[3][0]),startx,starty,startz);
 setup_vector_3d_rec(&(vertices[3][1]),startx,starty,endz);
 setup_vector_3d_rec(&(vertices[3][2]),startx,endy,endz);
 setup_vector_3d_rec(&(vertices[3][3]),startx,endy,startz);

 setup_vector_3d_rec(&(bases[4]),startx,endy,startz);
 setup_vector_3d_rec(&(normals[4]),0.0,1.0,.0);
 setup_vector_3d_rec(&(vertices[4][0]),startx,endy,startz);
 setup_vector_3d_rec(&(vertices[4][1]),endx,endy,startz);
 setup_vector_3d_rec(&(vertices[4][2]),endx,endy,endz);
 setup_vector_3d_rec(&(vertices[4][3]),startx,endy,endz);

 setup_vector_3d_rec(&(bases[5]),startx,starty,startz);
 setup_vector_3d_rec(&(normals[5]),0.0,1.0,.0);
 setup_vector_3d_rec(&(vertices[5][0]),startx,starty,startz);
 setup_vector_3d_rec(&(vertices[5][1]),endx,starty,startz);
 setup_vector_3d_rec(&(vertices[5][2]),endx,starty,endz);
 setup_vector_3d_rec(&(vertices[5][3]),startx,starty,endz);
}


/*--------------------------------------------------------------------------
closest_2_vertices - ???
---------------------------------------------------------------------------*/

void
closest_2_vertices(base_vector,direction_vector,startx,endx,starty,endy,
                 startz,endz,closest1,closest2)
 struct vector_3d_rec *base_vector,*direction_vector,*closest1,*closest2;
 float startx,endx,starty,endy,startz,endz;
{
 struct vector_3d_rec vertices[8];
 float dists[8],min;
 int cnt,indx,indx1;

 cuboid_vertices(vertices,startx,endx,starty,endy,startz,endz);
 for (cnt = 0; cnt < 8; cnt++)
    dists[cnt] = pt_line_dist(base_vector,direction_vector,&(vertices[cnt]));
 for (min = dists[0],indx = 0,cnt = 1; cnt < 8; cnt++)
    if (min > dists[cnt])
       {indx = cnt;
        min = dists[cnt];}
 for (cnt = 0; cnt < 8; cnt++)
    if (cnt != indx)
       {indx1 = cnt;
        min = dists[cnt];}
 for (cnt = 0; cnt < 8; cnt++)
    if (cnt != indx)
       if (min > dists[cnt])
          {indx1 = cnt;
           min = dists[cnt];}
 memcpy(closest1,&(vertices[indx]),(size_t) sizeof(struct vector_3d_rec));
 memcpy(closest2,&(vertices[indx1]),(size_t) sizeof(struct vector_3d_rec));
}


/*--------------------------------------------------------------------------
get_line_move_ratio - ???
---------------------------------------------------------------------------*/

int
get_line_move_ratio(base,direction,last_pt,ratio)
 struct vector_3d_rec *base,*direction,*last_pt;
 float *ratio;
{
 if (direction->x != 0)
    *ratio = (last_pt->x - base->x) / direction->x;
   else if (direction->y != 0)
    *ratio = (last_pt->y - base->y) / direction->y;
   else if (direction->z != 0)
    *ratio = (last_pt->z - base->z) / direction->z;
   else return 1;
 return NO_ERROR;
}


/*--------------------------------------------------------------------------
get_vector_angle_str - ???
---------------------------------------------------------------------------*/

char *
get_vector_angle_str(vector)
 struct vector_3d_rec *vector;
{
 static char str[70];
 struct vector_3d_rec temp;

 temp = *vector;
 direction_cosine_vector(&temp);
 temp.x = (float) acos_to_degree(temp.x);
 temp.y = (float) acos_to_degree(temp.y);
 temp.z = (float) acos_to_degree(temp.z);

 sprintf(str,"[%-3.0f/%-3.0f/%-3.0f]",temp.x,temp.y,temp.z);
 no_blank_in_str(str);
 return str;
}


/*--------------------------------------------------------------------------
get_vector_angle_str1 - ???
---------------------------------------------------------------------------*/

char *
get_vector_angle_str1(vector)
 struct vector_3d_rec *vector;
{
 static char str[70];
 struct vector_3d_rec temp;

 temp = *vector;
 direction_cosine_vector(&temp);
 temp.x = (float) acos_to_degree(temp.x);
 temp.y = (float) acos_to_degree(temp.y);
 temp.z = (float) acos_to_degree(temp.z);

 sprintf(str,"[t=%-3.2f,%-3.2f/%-3.2f/%-3.2f]",temp.x+temp.y+temp.z,
       temp.x,temp.y,temp.z);
 no_blank_in_str(str);
 return str;
}


/*--------------------------------------------------------------------------
cos_degree_vector - ???
---------------------------------------------------------------------------*/

void
cos_degree_vector(vector)
 struct vector_3d_rec *vector;
{
 vector->x = (float) cos_degree(vector->x);
 vector->y = (float) cos_degree(vector->y);
 vector->z = (float) cos_degree(vector->z);
}


/*--------------------------------------------------------------------------
vector_angle_more_than_90 - ???
---------------------------------------------------------------------------*/

int
vector_angle_more_than_90(vector)
 struct vector_3d_rec *vector;
{
 struct vector_3d_rec temp;
 int cnt;

 temp = *vector;
 direction_cosine_vector(&temp);
 temp.x = (float) acos_to_degree(temp.x);
 temp.y = (float) acos_to_degree(temp.y);
 temp.z = (float) acos_to_degree(temp.z);
 cnt = 0;
 if (temp.x > 90.0) cnt++;
 if (temp.y > 90.0) cnt++;
 if (temp.z > 90.0) cnt++;
 return cnt;
}


/*--------------------------------------------------------------------------
direction_3d_index - ???
---------------------------------------------------------------------------*/

int
direction_3d_index(num)
 float num;
{if (num == .0) return 0;
   else if (num > .0) return 1;
 return 2;
}


/*--------------------------------------------------------------------------
vectors_direction - ???
---------------------------------------------------------------------------*/

int
vectors_direction(v1,v2)
 struct vector_3d_rec *v1,*v2;
{
 struct vector_3d_rec normal;
 static unsigned char dire_table[3][3] = {{0,1,2},{1,2,2},{2,1,1}};
 int x,y,final;
 static unsigned char same_vector_dires[27] = 
     {0,1,2,1,1,2,2,2,2,
      1,1,2,1,1,2,2,2,2,
      2,2,1,1.1,1,1,1,1};

 vector_cross_product(&normal,v1,v2);
 if (get_vector_plane_type(&normal) == VECTOR_ORIGIN)
    {final = direction_3d_index(v1->x) * 9 + direction_3d_index(v1->y) * 3 +
           direction_3d_index(v1->z);
     final = same_vector_dires[final];
     return 0;
    }
   else if (normal.z == .0)
    {if (normal.x == .0) x = 0;
       else if (normal.x > .0) x = 1;
       else x = 2;
     if (normal.y == .0) y = 0;
       else if (normal.y > .0) y = 1;
       else y = 2;

     final = dire_table[x][y];
     return final;
    }
   else if (normal.z < .0)
    {return 1;}
   else
    {return 2;}
}


/*--------------------------------------------------------------------------
is_rect_vertices_clockwise - ???

 precondition :-
  1) pts must be in order in that when they are connected in order, no lines
     are intersected.
---------------------------------------------------------------------------*/

int
is_rect_vertices_clockwise(pts)
 struct vector_3d_rec pts[4];
{
 struct vector_3d_rec p1,p2;
 int ans1,ans2;

 sub_vectors(&p1,&(pts[1]),&(pts[0]));
 sub_vectors(&p2,&(pts[2]),&(pts[1]));
 ans1 = vectors_direction(&p1,&p2);

 sub_vectors(&p1,&(pts[3]),&(pts[2]));
 sub_vectors(&p2,&(pts[0]),&(pts[3]));
 ans2 = vectors_direction(&p1,&p2);


 if (ans1 == 0)
    {if (ans2 == 1)
        {return 1;}
       else 
        {return 0;}
    }
   else if (ans2 == 0)
    {if (ans1 == 1)
        {return 1;}
       else 
        {return 0;}
    }
   else if (ans1 == 1)
    {if (ans2 == 1)
        {return 1;}
       else
        {return 0;}
    }
   else if (ans2 == 1)
        {return 1;}
   else
    {return 0;}

}


/*--------------------------------------------------------------------------
find_pts_cuboid - ???
---------------------------------------------------------------------------*/

int
find_pts_cuboid(pts,num_pts,min,max)
 struct vector_3d_rec *pts,*min,*max;
 unsigned long num_pts;
{
 unsigned long lcv;

 if (num_pts == 0) return CANCEL;
 *min = pts[0];
 *max = pts[0];
 for (lcv = 1; lcv < num_pts; lcv++)
    {if (min->x > pts[lcv].x) min->x = pts[lcv].x;
     if (min->y > pts[lcv].y) min->y = pts[lcv].y;
     if (min->z > pts[lcv].z) min->z = pts[lcv].z;
     if (max->x < pts[lcv].x) max->x = pts[lcv].x;
     if (max->y < pts[lcv].y) max->y = pts[lcv].y;
     if (max->z < pts[lcv].z) max->x = pts[lcv].z;
   }
 return NO_ERROR;
}


/*--------------------------------------------------------------------------
closest_pt_from - ???
---------------------------------------------------------------------------*/

int
closest_pt_from(ref_pt,pts,num_pts,indx)
 struct vector_3d_rec *ref_pt,*pts;
 unsigned long num_pts,*indx;
{
 float dist,dist1;
 unsigned long lcv;

 if (num_pts == 0) return CANCEL;
 
 dist = vector_distance(&(pts[0]),ref_pt);
 for (*indx = 0,lcv = 1; lcv < num_pts; lcv++)
    {dist1 = vector_distance(&(pts[lcv]),ref_pt);
     if (dist1 < dist)
        {dist = dist1;
         *indx = lcv;}
    }

 return NO_ERROR;
}


/*--------------------------------------------------------------------------
lines_intersection_2 - ???
---------------------------------------------------------------------------*/

int
lines_intersection_2(px1,py1,dx1,dy1,px2,py2,dx2,dy2,ratio1,ratio2)
 double px1,py1,dx1,dy1,px2,py2,dx2,dy2,*ratio1,*ratio2;
{
 double det,b1,b2;

 det = dy1 * dx2 - dx1 * dy2;
 if (det == .0)
    {
     return 0;}
 b1 = px2 - px1;
 b2 = py2 - py1;
 *ratio1 = (-dy2 * b1 + dx2 * b2) / det;
 *ratio2 = (dx1 * b2 - dy1 * b1) / det;

 return 1;
}


/*--------------------------------------------------------------------------
lines_intersection - ???
---------------------------------------------------------------------------*/

int
lines_intersection(base1,direction1,base2,direction2,and)
 struct vector_3d_rec *base1,*direction1,*base2,*direction2,*and;
{
 double ratio1,ratio2,temp1,temp2;
 double x1,y1,z1,x2,y2,z2,dx1,dy1,dz1,dx2,dy2,dz2;

 x1 = base1->x;
 y1 = base1->y;
 dx1 = direction1->x;
 dy1 = direction1->y;
 x2 = base2->x;
 y2 = base2->y;
 dx2 = direction2->x;
 dy2 = direction2->y;
 z1 = base1->z;
 z2 = base2->z;
 dz1 = direction1->z;
 dz2 = direction2->z;

 if (!lines_intersection_2(x1,y1,dx1,dy1,x2,y2,dx2,dy2,&ratio1,&ratio2))
    {
     if (!lines_intersection_2(x1,z1,dx1,dz1,x2,z2,dx2,dz2,&ratio1,&ratio2))
        {
         if (!lines_intersection_2(y1,z1,dy1,dz1,y2,z2,dy2,dz2,&ratio1,&ratio2))
            {
             return 0;}
         temp1 = x1 + ratio1 * dx1;
         temp2 = x2 + ratio2 * dx2;
        }
       else
        {temp1 = y1 + ratio1 * dy1;
         temp2 = y2 + ratio2 * dy2;
        }
    }
   else
    {
     temp1 = z1 + ratio1 * dz1;
     temp2 = z2 + ratio2 * dz2;
    }
 temp2 -= temp1;
 if (temp2 < .0) temp2 = -temp2;
 printf("temp2 = %f\n",temp2);
 if (temp2 > .5)
    {
     return 0;}

 and->x = (float) (x1 + ratio1 * dx1);
 and->y = (float) (y1 + ratio1 * dy1);
 and->z = (float) (z1 + ratio1 * dz1);
 return 1;
}

