/****************************************************************************
3d_roi_types.c - ???

Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.
****************************************************************************/

static const char rcsid[] = "$Id: 3d_roi_types.c,v 1.9 1999/07/07 17:41:13 hughett Exp $ Release $Name: nria_1_0_6 $";


#include <xview/xview.h>
#include <xview/cms.h>
#include <xview/canvas.h>
#include <xview/panel.h>
#include <xview/notice.h>
#include <xview/wmgr.h>

#include "buf_3d_rec.h"
#include "geometry.h"
#include "mri_format.h"
#include "nria_procedures.h"

/*
#define TRACE(x) x; fflush(stdout);
#define PRT(x) x; fflush(stdout);
#define PRT1(x) x; fflush(stdout);
*/
#define PRT(x)
#define TRACE(x)
#define PRT1(x)


static char *_3DROI_ERROR_MSG = "Error occurred when updating the 3D ROIs !!";

#define CLEAN_UP_3D_MENU 0
#define SETUP_3D_MENU 1
#define GET_ALL_3D_MENU_PTR 2

struct roi_type_3d_rec
   {
    unsigned short status;
    struct image_display_window *win;
    struct display_win_rec *dwin;
    Frame frame;
    Panel panel;
    Panel_item display,apply_to,show_roi,roi_type,show_pixel_type,
               is_verify,hide,show;
   };

struct all_roi_type_3d_recs
   {
    int num_3ds;
    struct roi_type_3d_rec **_3ds;
   };



void init_roi_type_3d_rec(struct roi_type_3d_rec *_3d);
void free_roi_type_3d_rec(struct roi_type_3d_rec *_3d);
void init_all_roi_type_3d_recs(struct all_roi_type_3d_recs *all_3ds);
int add_window_list(Panel_item item, struct view_rec *view);
void applied_proc_to_selected_plane(struct roi_type_3d_rec *_3d,
    int (*proc)());
void _3d_roi_type_done_proc(Frame frame);
int AllocRoi2ds(struct view_rec *view, struct strs_rec *roi);
int AllocPixelType2ds(struct view_rec *view, unsigned long num_bufs);
int check_roi_pixel_types(struct roi_type_3d_rec *_3d, struct view_rec *view,
    int *change);
int SetupRoiPixelTypes(struct roi_type_3d_rec *_3d, struct list_rec *list,
    long num_items);
void now_on_roi_and_types_proc(Panel_item item, Event *event);
void setup_3d_pixel_type_proc(struct roi_type_3d_rec *_3d);
int setup_file_rois_list_item(struct img_buf_rec *img, Panel_item roi_item);
int WidsToListAry(struct input_file_rec *file, struct strs_rec *wids,
    struct list_rec **list, unsigned long *num_elems);
int get_applied_planes(struct input_file_rec *file, Panel_item apply_to_item,
    struct list_rec **list, int *num_elems);
int WinIs3dRoisInUse(struct image_display_window *win);
int WinRecompute3dRois(struct image_display_window *win);
int ProcessUpdate3dRois(int *err, struct image_display_window *win,
    struct input_file_rec *file, struct strs_rec **update_rois,
    struct strs_rec **load_rois);
int WinUpdate3dRois(struct image_display_window *win);
void update_cur_3d_rois_proc(Panel_item item, Event *event);
void win_select_cur_wid_item(struct image_display_window *win,
    Panel_item list_item);
int win_select_cur_group_items(struct image_display_window *win,
    Panel_item list_item);
void cur_wid_proc(Panel_item item, Event *event);
void cur_group_proc(Panel_item item, Event *event);
int select_group_proc(Panel_item item, Event *event);
void now_off_roi_and_types_proc(Panel_item item, Event *event);
int show_hide_superimpose(struct view_rec *view, struct roi_type_3d_rec *_3d);
void show_hide_superimpose_proc(Panel_item item, Event *event);
int setup_roi_type_3d_rec(struct roi_type_3d_rec *_3d,
    struct image_display_window *win);
struct roi_type_3d_rec *find_roi_type_3d_rec(
   struct all_roi_type_3d_recs *all_3ds, struct image_display_window *win);
void add_roi_type_window_list(struct image_display_window *win,
    struct view_rec *view);
unsigned long enter_3d_rois_and_pixel_types(struct image_display_window *win,
    int status);



/*-------------------------------------------------------------------------
init_roi_type_3d_rec - ???
-------------------------------------------------------------------------*/

void
init_roi_type_3d_rec(_3d)
 struct roi_type_3d_rec *_3d;
{
 _3d->status = 0;
 _3d->win = NULL;
 _3d->frame  = 0;
}


/*-------------------------------------------------------------------------
free_roi_type_3d_rec - ???
-------------------------------------------------------------------------*/

void
free_roi_type_3d_rec(_3d)
 struct roi_type_3d_rec *_3d;
{
 struct all_roi_type_3d_recs *all_3ds;

 TRACE(printf("free_roi_type_3d_rec...\n"))
 if (_3d->frame != 0) xv_destroy_safe(_3d->frame);
 all_3ds = (struct all_roi_type_3d_recs *) 
          enter_3d_rois_and_pixel_types(NULL,GET_ALL_3D_MENU_PTR);
 c_free(_3d);
 DropHandleAryPtr(&(all_3ds->_3ds),&(all_3ds->num_3ds),_3d);
}


/*-------------------------------------------------------------------------
init_all_roi_type_3d_recs - ???
-------------------------------------------------------------------------*/

void
init_all_roi_type_3d_recs(all_3ds)
 struct all_roi_type_3d_recs *all_3ds;
{
 all_3ds->num_3ds = 0;
 all_3ds->_3ds = NULL;
}


/*-------------------------------------------------------------------------
add_window_list - ???
-------------------------------------------------------------------------*/

int
add_window_list(item,view)
 Panel_item item;
 struct view_rec *view;
{
 struct strs_rec labels,selected;
 int err;

 init_strs_rec(&labels);
 init_strs_rec(&selected);
 if (((err = list_items(&labels,item)) == NO_ERROR) &&
     ((err = selected_list_items(&selected,item)) == NO_ERROR) &&
     ((err = GroupViewWids(view,&labels,1,1)) == NO_ERROR) && 
       (labels.num_strs > 0))
    {
     delete_all_list_items(item);
     setup_list_items(&labels,item,0);
     select_list_items(&selected,item);
    }
 TRACE(printf("err = %d\n",err);pr_strs_rec(&labels))
 free_strs_rec(&labels);
 free_strs_rec(&selected);
 return err;
}


/*-------------------------------------------------------------------------
applied_proc_to_selected_plane - ???
-------------------------------------------------------------------------*/

void
applied_proc_to_selected_plane(_3d,proc)
 struct roi_type_3d_rec *_3d;
 int (*proc)();
{
 unsigned long lcv,num_rows;
 char *str;
 struct img_buf_rec *img;
 struct view_rec *result_view;

 if ((img = win_rec_cur_img_buf(_3d->win->owner)) == NULL) return;

 num_rows = (int) xv_get(_3d->apply_to,PANEL_LIST_NROWS);
 for (lcv = 0; lcv < num_rows; lcv++)
    if (xv_get(_3d->apply_to,PANEL_LIST_SELECTED,lcv)) 
       {if ((str = (char *) xv_get(_3d->apply_to,PANEL_LIST_STRING,lcv)) != 
               NULL)
           if ((result_view = find_file_view_rec(img->file,str)) != NULL)
              proc(result_view,_3d);
       }
}


/*-------------------------------------------------------------------------
_3d_roi_type_done_proc - ???
-------------------------------------------------------------------------*/

void
_3d_roi_type_done_proc(frame)
 Frame frame;
{
 struct roi_type_3d_rec *_3d;

 _3d = (struct roi_type_3d_rec *)
                     xv_get(frame,WIN_CLIENT_DATA);
 free_roi_type_3d_rec(_3d);
}


/*-------------------------------------------------------------------------
AllocRoi2ds - ???
-------------------------------------------------------------------------*/

int
AllocRoi2ds(view,roi)
 struct view_rec *view;
 struct strs_rec *roi;
{
 unsigned long lcv,num;
 struct buf_2d_rec **_2ds;
 int err;

 PRT(printf(" AllocRoi2ds....num_2ds = %d total = %d roi->num_strs = %d\n",
           view->num_roi_2ds,view->total_roi_2ds,roi->num_strs))
 err = NO_ERROR;
 if (roi->num_strs > view->num_roi_2ds)
    {if (roi->num_strs > view->total_roi_2ds)
        {num = roi->num_strs - view->total_roi_2ds;
         lcv = view->total_roi_2ds;
         if ((err = get_ptr(&(view->roi_2ds),&(view->total_roi_2ds),num,
                 (size_t) sizeof(*(view->roi_2ds)))) != NO_ERROR)
            return err;
         for (; lcv < view->total_roi_2ds; lcv++) view->roi_2ds[lcv] = NULL;
        }
     num = roi->num_strs - view->num_roi_2ds;
     if ((err = get_handle(&_2ds,NULL,num,(size_t) sizeof(**_2ds)))
              != NO_ERROR)
        return err;
     for (lcv = 0,num = view->num_roi_2ds; num < roi->num_strs; num++,lcv++)
         {init_buf_2d_rec(_2ds[lcv]);
          view->roi_2ds[num] = _2ds[lcv];
         }
     c_free(_2ds);
     view->num_roi_2ds = roi->num_strs;
    }
   else if (roi->num_strs < view->num_roi_2ds)
   {for (num = roi->num_strs; num < view->num_roi_2ds; num++)
       {free_buf_2d_rec(view->roi_2ds[num]);
        c_free(view->roi_2ds[num]);
        view->roi_2ds[num] = NULL;}
    view->num_roi_2ds = roi->num_strs;}
 return err;
}


/*-------------------------------------------------------------------------
AllocPixelType2ds - ???
-------------------------------------------------------------------------*/

int
AllocPixelType2ds(view,num_bufs)
 struct view_rec *view;
 unsigned long num_bufs;
{
 unsigned long lcv,num;
 struct buf_2d_rec **_2ds;
 int err;

 PRT(printf(" AllocPixelType2ds....num_2ds = %d total = %d\n",
           view->num_type_2ds,view->total_type_2ds))
 err = NO_ERROR;
 if (num_bufs > view->num_type_2ds)
    {if (num_bufs > view->total_type_2ds)
        {num = num_bufs - view->total_type_2ds;
         lcv = view->total_type_2ds;
         if ((err = get_ptr(&(view->type_2ds),&(view->total_type_2ds),num,
                 (size_t) sizeof(*(view->type_2ds)))) != NO_ERROR)
            return err;
         for (; lcv < view->total_type_2ds; lcv++) view->type_2ds[lcv] = NULL;
        }
     num = num_bufs - view->num_type_2ds;
     if ((err = get_handle(&_2ds,NULL,num,(size_t) sizeof(**_2ds)))
              != NO_ERROR)
        return err;
     for (lcv = 0,num = view->num_type_2ds; num < num_bufs; num++,lcv++)
         {init_buf_2d_rec(_2ds[lcv]);
          view->type_2ds[num] = _2ds[lcv];
         }
     c_free(_2ds);
     view->num_type_2ds = num_bufs;
    }
   else if (num_bufs < view->num_type_2ds)
   {for (num = num_bufs; num < view->num_type_2ds; num++)
       {free_buf_2d_rec(view->type_2ds[num]);
        c_free(view->type_2ds[num]);
        view->type_2ds[num] = NULL;}
    view->num_type_2ds = num_bufs;}
 return err;
}


/*-------------------------------------------------------------------------
check_roi_pixel_types - ???
-------------------------------------------------------------------------*/

int
check_roi_pixel_types(_3d,view,change)
 struct roi_type_3d_rec *_3d;
 struct view_rec *view;
 int *change;
{
 int err;
 struct strs_rec roi,roi_type,pixel_type,all_rois;
 unsigned long lcv,num,last_roi_num_strs,last_pixel_type_num_strs;
 struct buf_3d_rec *the_3d;

 PRT(printf(" check_roi_pixel_types....0\n"))
 init_strs_rec(&all_rois);
 init_strs_rec(&roi);
 init_strs_rec(&roi_type);
 init_strs_rec(&pixel_type);
 err = NO_ERROR;
 *change = 0;
 if (((err = selected_list_items(&roi,_3d->show_roi)) == NO_ERROR) &&
     ((err = selected_list_items(&roi_type,_3d->roi_type)) == NO_ERROR) &&
     ((err = selected_list_items(&pixel_type,_3d->show_pixel_type)) == NO_ERROR))
    {sort_strs_rec_ascendingly(&roi);
     sort_strs_rec_ascendingly(&roi_type);
     sort_strs_rec_ascendingly(&pixel_type);
     if (pixel_type.num_strs == 0)
        free_strs_rec(&roi_type);
     for (lcv = 0; lcv < roi.num_strs; lcv++)
        {if ((err = AddStrToSortedStrsRec(&all_rois,roi.strs[lcv],0,1))
                 != NO_ERROR)
            break;}
     if ((err == NO_ERROR) && (roi_type.num_strs > 0))
        for (lcv = 0; lcv < roi_type.num_strs; lcv++)
           {if ((err = AddStrToSortedStrsRec(&all_rois,roi_type.strs[lcv],0,1))
                 != NO_ERROR)
               break;
           }
    }
 if (err != NO_ERROR)
    {PRT(printf("*** != NO_ERROR...\n"))
     free_strs_rec(&all_rois);
     free_strs_rec(&roi);
     free_strs_rec(&pixel_type);
     free_strs_rec(&roi_type);
     return err;}

 PRT(printf(" check_roi_pixel_types....1\n"))
 if (roi.num_strs == view->rois.num_strs)
    {for (lcv = 0; lcv < roi.num_strs; lcv++)
        if (str_cmp(roi.strs[lcv],view->rois.strs[lcv]) != 0)
           break;
     if (lcv != roi.num_strs) *change = 1;
    }
   else *change = 1;

 PRT(printf(" check_roi_pixel_types....roi.num_strs = %d\n",roi.num_strs))
 PRT(printf(" ....roi_type.num_strs = %d\n",roi_type.num_strs))
 PRT(printf(" ....pixel_type.num_strs = %d\n",pixel_type.num_strs))

 if (pixel_type.num_strs == view->pixel_types.num_strs)
    {for (lcv = 0; lcv < pixel_type.num_strs; lcv++)
        if (str_cmp(pixel_type.strs[lcv],view->pixel_types.strs[lcv]) != 0)
           break;
     if (lcv != pixel_type.num_strs) *change |= 2;
       else if (roi_type.num_strs == view->roi_pixel_types.num_strs)
        {for (lcv = 0; lcv < roi_type.num_strs; lcv++)
            if (str_cmp(roi_type.strs[lcv],
                    view->roi_pixel_types.strs[lcv]) != 0)
               break;
         if (lcv != roi_type.num_strs) *change |= 2;}
       else *change |= 2;
    }
   else *change |= 2;
 PRT(printf("&&&&&&&*change = %d\n",*change))
 if (*change == 0)
    {
     if ((view->status & PLANE_3D_ROIS_NEED_UPDATED) || 
         (view->status & PLANE_3D_PIXEL_TYPES_NEED_UPDATED) ||
         !(view->img->status4 & IMAGE_3D_SUPERIMPOSED_OPRS) ||
         (view->img->file->pixel_type.status & PIXEL_TYPE_3D_REDEFINED))
         *change = 1;
       else
        {for (lcv = 0; lcv < all_rois.num_strs; lcv++)
            if (find_file_3d(view->img->file,all_rois.strs[lcv],&the_3d) 
                     != NO_ERROR)
               {*change = 1;
                break;}
         if ((pixel_type.num_strs > 0) && ((*change) == 0))
            if (find_file_3d(view->img->file,
                       PIXEL_TYPE_3D_ID,&the_3d) != NO_ERROR)
               *change = 1;
        }
     free_strs_rec(&all_rois);
     free_strs_rec(&roi);
     free_strs_rec(&roi_type);
     free_strs_rec(&pixel_type);
     return NO_ERROR;}

 if (all_rois.num_strs != view->num_roi_2ds)
    err = AllocRoi2ds(view,&all_rois);
 if (err == NO_ERROR)
    {if (pixel_type.num_strs > 0) num = 1;
       else num = 0;
     if (view->num_type_2ds != num)
        err = AllocPixelType2ds(view,1);
    }

 PRT(printf(" check_roi_pixel_types....3\n"))
 view->status &= ~PLANE_3D_PIXEL_TYPES_READY;
 view->status &= ~PLANE_3D_ROIS_READY;
 if (err == NO_ERROR)
    {last_roi_num_strs = view->rois.num_strs;
     last_pixel_type_num_strs = view->pixel_types.num_strs;
     free_strs_rec(&(view->rois));
     free_strs_rec(&(view->roi_pixel_types));
     free_strs_rec(&(view->pixel_types));
     free_strs_rec(&(view->all_rois));
     if ((roi.num_strs == 0) && (pixel_type.num_strs == 0))
        {*change = 0;
         view->status &= ~IMAGE_3D_SUPERIMPOSED_OPRS;}
     if (roi.num_strs == 0)
        {if (view->img->win != NULL)
            WinDropColorDataRec1(view->img->win->image_display,
                   SUPERIMPOSE_3D_ID_ROIS,1,0,NULL,1);
         view->status &= ~PLANE_PROJECT_ROIS;
         view->status &= ~PLANE_3D_ROIS_NEED_UPDATED;}
       else 
        {view->status |= PLANE_PROJECT_ROIS;
         roi.cur_str = 0;
         view->rois = roi;}

     if (pixel_type.num_strs == 0)
        {if (view->img->win != NULL)
            WinDropColorDataRec1(view->img->win->image_display,
                   SUPERIMPOSE_3D_ID,1,0,NULL,1);
         view->status &= ~PLANE_PROJECT_PIXEL_TYPES;
        }
       else 
        {view->status |= PLANE_PROJECT_PIXEL_TYPES;
         roi_type.cur_str = 0;
         pixel_type.cur_str = 0;
         view->roi_pixel_types = roi_type;
         view->pixel_types = pixel_type;
        }
     all_rois.cur_str = 0;
     view->all_rois = all_rois;
     PRT(printf("roi...\n");pr_strs_rec(&roi))
     PRT(printf("all_rois...\n");pr_strs_rec(&all_rois))
     PRT(printf("roi_type...\n");pr_strs_rec(&roi_type))
     PRT(printf("pixel_type...\n");pr_strs_rec(&pixel_type))
    }
   else
    {free_strs_rec(&pixel_type);
     free_strs_rec(&roi_type);
     free_strs_rec(&all_rois);
     free_strs_rec(&roi);}
 PRT(printf(" .....err = %d  num_roi_2ds = %d total = %d change = %d\n",
           err,view->num_roi_2ds,view->total_roi_2ds,*change))
 PRT(printf(" check_roi_pixel_types..end.....num_roi_2ds = %d total = %d\n\n",
           view->num_type_2ds,view->total_type_2ds))
 return err;
}


/*-------------------------------------------------------------------------
SetupRoiPixelTypes - ???
-------------------------------------------------------------------------*/

int
SetupRoiPixelTypes(_3d,list,num_items)
 struct roi_type_3d_rec *_3d;
 struct list_rec *list;
 long num_items;
{
 int err;
 struct view_rec *view;
 int change,delete;
 struct list_rec *next;
 long lcv;

 for (lcv = 0; lcv < num_items; lcv++)
    {view = list->item;
     delete = 0;
     if (((err = check_roi_pixel_types(_3d,view,&change)) == NO_ERROR) && change)
        {PRT(printf("change = %d\n",change))
         view->rois.cur_str = 0;
         view->pixel_types.cur_str = 0;
        }
       else if (err != NO_ERROR)
        return err;
       else delete = 1;
     next = list->next;
     PRT(printf("delete = %d next = %u list = %u head = %u\n",delete,next,list,next->head))
     if (delete) delete_item_from_list(list);
     PRT(printf("num_lefts = %d\n",cnt_num_list_items(list)))
     list = next;
    }
 return NO_ERROR;
}


/*-------------------------------------------------------------------------
now_on_roi_and_types_proc - ???
-------------------------------------------------------------------------*/

void
now_on_roi_and_types_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct roi_type_3d_rec *_3d;
 static struct list_rec *list = NULL;
 int lcv,num_elems;
 struct view_rec *view;
 struct img_buf_rec *img;
 int err;

 _3d = (struct roi_type_3d_rec *) xv_get(item,PANEL_CLIENT_DATA);
 if ((img = win_rec_cur_img_buf(_3d->win->owner)) == NULL) return;
/*
 if ((view = img->cur_slice->slice->view) == NULL) return CANCEL;
 DeleteRoisOfType(&(view->sl.slice_roi_win.slice_roi),ROI_3D_ROI,2);
*/
 if (list != NULL) 
    {c_free(list);
     list = NULL;}
 PRT(printf("now_on_roi_and_types_proc...\n"))
 if ((get_applied_planes(img->file,_3d->apply_to,
         &list,&num_elems) == NO_ERROR) && (list != NULL)) 
    {PRT(pr_list_rec(list,NULL);)

     PRT(printf("now_on_roi_and_types_proc...num_elems = %d\n",num_elems))
     if ((err = SetupRoiPixelTypes(_3d,&(list[0]),num_elems)) != NO_ERROR)
        {notice_prompt(_3d->dwin->frame,(Event *) NULL,
            NOTICE_MESSAGE_STRINGS,
               "AN ERROR OCCURRED WHEN SETTING UP THE ROIS OR PIXEL TYPES !!!!",
               "TRY AGAIN !!!",
               0,
            NOTICE_BUTTON_YES," OK !",0);
         c_free(list);
         list = NULL;
         return;
        }
     PRT(pr_list_rec(list,NULL);)
     for (lcv = 0; lcv < num_elems; lcv++)
        if (list[lcv].next != NULL) break;
     PRT(printf("now_on_roi_and_types_proc...num_elems = %d lcv = %d\n",
              num_elems,lcv))
     if (lcv == num_elems)
        {c_free(list);
         list = NULL;}
       else
        {
         view = list[lcv].item;
         PRT(printf("now_on_roi_and_types_proc...rois.num_strs = %d pixel_types.num_strs = %d\n",
                   view->rois.num_strs,view->pixel_types.num_strs))
         if (view->status & PLANE_PROJECT_ROIS) 
            {PRT(printf("now_on_roi_and_types_proc..setup_and_display_3d_rois\n"))
             if (view->rois.num_strs > 0)
                setup_and_display_3d_rois(view,&(list[lcv]));
               else view->status |= PLANE_3D_ROIS_NEED_UPDATED;
            }
           else if (view->status & PLANE_PROJECT_PIXEL_TYPES)
            {PRT(printf("now_on_roi_and_types_proc..setup_pixel_types\n"))
             if (view->pixel_types.num_strs > 0)
                setup_and_display_3d_pixel_types(view,&(list[lcv]));
               else view->status |= PLANE_3D_PIXEL_TYPES_NEED_UPDATED;
            }
         if (!(view->status & PLANE_PROJECT_ROIS) &&
             !(view->status & PLANE_PROJECT_PIXEL_TYPES))
            view->img->status4 &= ~IMAGE_3D_SUPERIMPOSED_OPRS;
        }
    }
}


/*-------------------------------------------------------------------------
setup_3d_pixel_type_proc - ???
-------------------------------------------------------------------------*/

void
setup_3d_pixel_type_proc(_3d)
 struct roi_type_3d_rec *_3d;
{
 int cnt,lcv,indx;
 struct img_buf_rec *img;
 struct strs_rec *order,*type;

 if ((img = win_rec_cur_img_buf(_3d->win->owner)) == NULL)
    return;

 order = &(img->file->pixel_type.order);
 for (cnt = 0; cnt < order->num_strs; cnt++)
    xv_set(_3d->show_pixel_type,
          PANEL_LIST_STRING,cnt,order->strs[cnt],
          PANEL_LIST_SELECT,cnt,0,0);
 if (img->cur_slice->slice->view != NULL)
    {type = &(img->cur_slice->slice->view->pixel_types);
     for (lcv = 0; lcv < type->num_strs; lcv++)
        {for (indx = 0; indx < order->num_strs; indx++)
            if (str_cmp(order->strs[indx],type->strs[lcv]) == 0)
               break;
         if (indx < order->num_strs)
            xv_set(_3d->show_pixel_type,PANEL_LIST_SELECT,indx,1,0);
           else
            {xv_set(_3d->show_pixel_type,
                PANEL_LIST_STRING,cnt,type->strs[lcv],
                PANEL_LIST_SELECT,cnt,1,0);
             cnt++;}
        }
   }
}


/*-------------------------------------------------------------------------
setup_file_rois_list_item - ???
-------------------------------------------------------------------------*/

int
setup_file_rois_list_item(img,roi_item)
 struct img_buf_rec *img;
 Panel_item roi_item;
{
 struct strs_rec roi_labels,*roi;

 init_strs_rec(&roi_labels);
 FileGetAllRoiLabelPtrs(img->file,&roi_labels,0,1);
 PRT(pr_strs_rec(&roi_labels);)
 setup_list_items(&roi_labels,roi_item,0);

 if (img->cur_slice->slice->view != NULL)
    {roi = &(img->cur_slice->slice->view->mask_rois);
     select_list_items(roi,roi_item);
    }
 free_strs_rec_ptrs(&roi_labels);
 return NO_ERROR;
}


/*-------------------------------------------------------------------------
WidsToListAry - ???
-------------------------------------------------------------------------*/

int
WidsToListAry(file,wids,list,num_elems)
 struct input_file_rec *file;
 struct strs_rec *wids;
 struct list_rec **list;
 unsigned long *num_elems;
{
 struct view_rec *result_view;
 struct list_rec *next;
 int lcv,err;

 *list = NULL;
 *num_elems = 0;
 if (wids->num_strs == 0) return CANCEL;
 if ((err = get_ptr(list,NULL,wids->num_strs,(size_t) sizeof(**list)))
       != NO_ERROR) 
    return err;
 for (next = NULL,lcv = 0; lcv < wids->num_strs; lcv++)
    if ((result_view = find_file_view_rec(file,wids->strs[lcv])) != NULL)
       {PRT(printf("WidsToListAry str = %s\n",wids->strs[lcv]))
        if (next != NULL)
           {setup_list_rec(&((*list)[lcv]),result_view,
                     list,NULL,NULL,NULL);
            add_item_after_cur_list(next,&((*list)[lcv]));}
          else setup_list_rec(&((*list)[lcv]),result_view,
                     list,NULL,NULL,NULL);
        next = &((*list)[lcv]);
       }
 *num_elems = wids->num_strs;
 return err;
}


/*-------------------------------------------------------------------------
get_applied_planes - ???
-------------------------------------------------------------------------*/

int
get_applied_planes(file,apply_to_item,list,num_elems)
 struct input_file_rec *file;
 Panel_item apply_to_item;
 struct list_rec **list;
 int *num_elems;
{
 struct strs_rec wids;
 int err;

 *num_elems = 0;
 *list = NULL;
 PRT(printf("get_applied_planes..1\n"))
 init_strs_rec(&wids);
 if ((err = selected_list_item_ptrs(&wids,apply_to_item)) == NO_ERROR)
    {PRT(printf("get_applied_planes..wids.num_strs = %d\n",wids.num_strs))
     if (wids.num_strs > 0) 
        err = WidsToListAry(file,&wids,list,num_elems);}
 free_strs_rec_ptrs(&wids);
 PRT(printf("get_applied_planes..err = %d\n",err))
 return err;
}


/*-------------------------------------------------------------------------
WinIs3dRoisInUse - ???
-------------------------------------------------------------------------*/

int
WinIs3dRoisInUse(win)
 struct image_display_window *win;
{
 struct img_buf_rec *img;
 struct slice_info_rec *slice_info;
 struct view_rec *view;
 long lcv;

 if ((img = win_rec_cur_img_buf(win->owner)) == NULL) return 0;
 if ((view = img->cur_slice->slice->view) == NULL) return 0;

 slice_info = &(view->file->slice_info);
 for (lcv = 0; lcv < slice_info->num_views; lcv++)
    if ((slice_info->views[lcv]->all_rois.num_strs > 0) &&
        ((slice_info->views[lcv]->rois.num_strs > 0) ||
         (slice_info->views[lcv]->roi_pixel_types.num_strs > 0)))
       return 1;
 return 0;
}


/*-------------------------------------------------------------------------
WinRecompute3dRois - ???
-------------------------------------------------------------------------*/

int
WinRecompute3dRois(win)
 struct image_display_window *win;
{
 struct img_buf_rec *img;
 struct view_rec *view;
 struct slice_info_rec *slice_info;
 long lcv;

 PRT1(printf("WinRecompute3dRois...\n"))
 if ((img = win_rec_cur_img_buf(win->owner)) == NULL) return CANCEL;
 if ((view = img->cur_slice->slice->view) == NULL) return CANCEL;

 slice_info = &(view->file->slice_info);
 for (lcv = 0; lcv < slice_info->num_views; lcv++)
    if ((slice_info->views[lcv]->all_rois.num_strs > 0) &&
        ((slice_info->views[lcv]->rois.num_strs > 0) ||
         (slice_info->views[lcv]->roi_pixel_types.num_strs > 0)))
       {PRT1(printf("WinRecompute3dRois...wid = %s\n",slice_info->views[lcv]->wid))
        BuildAndDisplayPlaneData(slice_info->views[lcv]);}
 return NO_ERROR;
}


/*-------------------------------------------------------------------------
ProcessUpdate3dRois - ???
-------------------------------------------------------------------------*/

int
ProcessUpdate3dRois(err,win,file,update_rois,load_rois)
 int *err;
 struct image_display_window *win;
 struct input_file_rec *file;
 struct strs_rec **update_rois,**load_rois;
{
 unsigned long cur;
 struct buf_3d_rec *_3d;
 char *str;
 int inuse;

 if ((*err == NO_ERROR) &&
     ((cur = (*update_rois)->cur_str) < (*update_rois)->num_strs) &&
     (find_file_3d(file,(*update_rois)->strs[cur],&_3d) == NO_ERROR) )
    {(*update_rois)->cur_str += 1;
     concat_strs(&str,2,"Updating The 3D ROI ",(*update_rois)->strs[cur]);
     ShowMsgStrs(win->dwin->frame,1,NO_ERROR,0,2,str,
               PLEASE_WAIT,Setup3dRoi,9,file,(*update_rois)->strs[cur],_3d,
               ProcessUpdate3dRois,err,win,file,update_rois,load_rois);
     c_free(str);
    }
 else
    {
     free_strs_rec(*update_rois);
     c_free(*update_rois);
     *update_rois = NULL;
     inuse = WinIs3dRoisInUse(win);
     if (*err != NO_ERROR)
        if (inuse)
           ShowMsgStrs(win->dwin->frame,0,*err,0,2,_3DROI_ERROR_MSG,
                    "Will proceed to recompute new 3D ROI outlines",NULL,0);
	else ShowMsgStrs(win->dwin->frame,0,*err,0,1,_3DROI_ERROR_MSG,NULL,0);
     if (inuse)
        ShowMsgStrs(win->dwin->frame,1,NO_ERROR,0,2,
               "Recomputing new 3D ROI outlines",
               PLEASE_WAIT,WinRecompute3dRois,1,win);
     return *err;}
 return *err;      /* FIXME:  Added but not checked.  -PWH 19990319 */
}


/*-------------------------------------------------------------------------
WinUpdate3dRois - ???
-------------------------------------------------------------------------*/

int
WinUpdate3dRois(win)
 struct image_display_window *win;
{
 struct img_buf_rec *img;
 struct strs_rec roi_labels;
 static struct strs_rec *update_rois = NULL,*load_rois = NULL;
 struct buf_3d_rec *_3d;
 int err;
 unsigned long cnt;
 static int err1;

 if ((img = win_rec_cur_img_buf(win->owner)) == NULL) return CANCEL;
 if ((update_rois != NULL) || (load_rois != NULL)) return CANCEL;
 init_strs_rec(&roi_labels);
 FileGetAllRoiLabelPtrs(img->file,&roi_labels,0,1);
 PRT1(printf("WinUpdate3dRois...roi_labels\n"))
 PRT1(pr_strs_rec(&roi_labels))
 err = NO_ERROR;
 if (roi_labels.num_strs > 0)
    {
     if (((err = setup_strs_rec(&update_rois,0,0)) == NO_ERROR) &&
         ((err = setup_strs_rec(&load_rois,0,0)) == NO_ERROR))
        {for (cnt = 0; cnt < roi_labels.num_strs; cnt++)
            {if (find_file_3d(img->file,roi_labels.strs[cnt],&_3d) == NO_ERROR) 
                err = add_str_to_strs_rec(update_rois,roi_labels.strs[cnt]);
               else err = add_str_to_strs_rec(load_rois,roi_labels.strs[cnt]);
             if (err != NO_ERROR) break;
            }
        }

     PRT1(printf("WinUpdate3dRois...update_rois\n"))
     PRT1(pr_strs_rec(update_rois))
     if ((err == NO_ERROR) && (update_rois->num_strs > 0))
        {update_rois->cur_str = 0;
         err1 = NO_ERROR;
         ProcessUpdate3dRois(&err1,win,img->file,&update_rois,&load_rois);
        }
       else
        {free_strs_rec(update_rois);
         c_free(update_rois);
         update_rois = NULL;}

     PRT1(printf("WinUpdate3dRois...load_rois\n"))
     PRT1(pr_strs_rec(load_rois))
     if (load_rois != NULL) 
        {free_strs_rec(load_rois);
         c_free(load_rois);
         load_rois = NULL;}
     if (err != NO_ERROR)
        ShowMsgStrs(win->dwin->frame,0,err,0,1,_3DROI_ERROR_MSG,NULL,0);

    }
 free_strs_rec_ptrs(&roi_labels);
 return err;

}


/*-------------------------------------------------------------------------
update_cur_3d_rois_proc - ???
-------------------------------------------------------------------------*/

void
update_cur_3d_rois_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct roi_type_3d_rec *_3d;

 _3d = (struct roi_type_3d_rec *) xv_get(item,PANEL_CLIENT_DATA);
 WinUpdate3dRois(_3d->win);
}


/******************
all_none_proc(item,value,event)
 Panel_item item;
 int value;
 Event *event;
{
 struct roi_type_3d_rec *_3d;

 _3d = (struct roi_type_3d_rec *) xv_get(item,PANEL_CLIENT_DATA);
 if (value)
    {if (item == _3d->is_show_pixel_type)
        select_all_list_items(_3d->show_pixel_type);
       else select_all_list_items(_3d->show_roi);
    }
   else
    {if (item == _3d->is_show_pixel_type)
        deselect_all_list_items(_3d->show_pixel_type);
       else deselect_all_list_items(_3d->show_roi);
    }
}
****************/



/*-------------------------------------------------------------------------
win_select_cur_wid_item - ???
-------------------------------------------------------------------------*/

void
win_select_cur_wid_item(win,list_item)
 struct image_display_window *win;
 Panel_item list_item;
{
/*
 struct view_rec *view;
 struct img_buf_rec *img;

 if ((img = win_rec_cur_img_buf(_3d->win->owner)) == NULL) return CANCEL;
 if ((view = img->cur_slice->slice->view) == NULL) return CANCEL;
*/
 deselect_all_list_items(list_item);
 select_list_str_item(win->owner->wid,list_item,1);
}


/*-------------------------------------------------------------------------
win_select_cur_group_items - ???
-------------------------------------------------------------------------*/

int
win_select_cur_group_items(win,list_item)
 struct image_display_window *win;
 Panel_item list_item;
{
 struct view_rec *view,*view1;
 struct img_buf_rec *img;
 int num_rows,lcv;
 char *str;

 if ((img = win_rec_cur_img_buf(win->owner)) == NULL) return CANCEL;
 if ((view = img->cur_slice->slice->view) == NULL) return CANCEL;

 num_rows = (int) xv_get(list_item,PANEL_LIST_NROWS);
 for (lcv = 0; lcv < num_rows; lcv++)
    if ((str = (char *) xv_get(list_item,PANEL_LIST_STRING,lcv))  != NULL)
       if (((view1 = find_file_view_rec(img->file,str)) != NULL) &&
           (view->group_num == view1->group_num))
          xv_set(list_item,PANEL_LIST_SELECT,lcv,1,0);
         else xv_set(list_item,PANEL_LIST_SELECT,lcv,0,0);
 return NO_ERROR;
}


/**********************
verify_rois_proc(item,value,event)
 Panel_item item;
 int value;
 Event *event;
{
 struct roi_type_3d_rec *_3d;
 int selected_indx;
 struct view_rec *view;
 struct img_buf_rec *img;

 _3d = (struct roi_type_3d_rec *) xv_get(item,PANEL_CLIENT_DATA);
 if ((img = win_rec_cur_img_buf(_3d->win->owner)) == NULL) return CANCEL;
 if (value)
    if (next_selected_item(_3d->show_pixel_type,0,&selected_indx) == NO_ERROR)
       {xv_set(item,PANEL_VALUE,0,0);
        value = 0;}
 if (value)
    img->status4 |= IMAGE_3D_SUPERIMPOSED_FILLED;
   else img->status4 &= ~IMAGE_3D_SUPERIMPOSED_FILLED;
}
*************************/


/*-------------------------------------------------------------------------
cur_wid_proc - ???
-------------------------------------------------------------------------*/

void
cur_wid_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct roi_type_3d_rec *_3d;

 _3d = (struct roi_type_3d_rec *) xv_get(item,PANEL_CLIENT_DATA);
 win_select_cur_wid_item(_3d->win,_3d->apply_to);
}


/*-------------------------------------------------------------------------
cur_group_proc - ???
-------------------------------------------------------------------------*/

void
cur_group_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct roi_type_3d_rec *_3d;

 _3d = (struct roi_type_3d_rec *) xv_get(item,PANEL_CLIENT_DATA);
 win_select_cur_group_items(_3d->win,_3d->apply_to);
}


/*-------------------------------------------------------------------------
select_group_proc - ???
-------------------------------------------------------------------------*/

int
select_group_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct roi_type_3d_rec *_3d;
 struct view_rec *view;
 struct img_buf_rec *img;
 struct strs_rec labels;
 long num_gnums;
 int err,*gnums;
 int num_rows,lcv,cnt;
 char *str;

 _3d = (struct roi_type_3d_rec *) xv_get(item,PANEL_CLIENT_DATA);
 if ((img = win_rec_cur_img_buf(_3d->win->owner)) == NULL) return CANCEL;
 init_strs_rec(&labels);
 if (((err = selected_list_item_ptrs(&labels,_3d->apply_to)) == NO_ERROR) &&
     (labels.num_strs > 0))
    {if ((err = view_labels_to_group_nums(img->file,&labels,&gnums,&num_gnums))
                    == NO_ERROR)
        {num_rows = (int) xv_get(_3d->apply_to,PANEL_LIST_NROWS);
         for (lcv = 0;lcv < num_rows; lcv++)
            if ((str = (char *) xv_get(_3d->apply_to,PANEL_LIST_STRING,lcv)) 
                     != NULL)
               if ((view = find_file_view_rec(img->file,str)) != NULL)
                  for (cnt = 0; cnt < num_gnums; cnt++)
                     if (view->group_num == gnums[cnt])
                        {xv_set(_3d->apply_to,PANEL_LIST_SELECT,lcv,1,0);
                         break;}
        }
     c_free(gnums);
    }
 free_strs_rec_ptrs(&labels);
 return err;
}


/*-------------------------------------------------------------------------
now_off_roi_and_types_proc - ???
-------------------------------------------------------------------------*/

void
now_off_roi_and_types_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct roi_type_3d_rec *_3d;
 struct view_rec *view;
 struct img_buf_rec *img;
 struct strs_rec list,list1;

 _3d = (struct roi_type_3d_rec *) xv_get(item,PANEL_CLIENT_DATA);
 init_strs_rec(&list);
 init_strs_rec(&list1);
 selected_list_item_ptrs(&list,_3d->show_pixel_type);
 selected_list_item_ptrs(&list1,_3d->show_roi);
 deselect_all_list_items(_3d->show_pixel_type);
 deselect_all_list_items(_3d->show_roi);
 now_on_roi_and_types_proc(_3d->display,event);
 select_list_items(&list,_3d->show_pixel_type);
 select_list_items(&list1,_3d->show_roi);
 free_strs_rec_ptrs(&list);
 free_strs_rec_ptrs(&list1);
 PRT1(printf("now_off_roi_and_types_proc\n"))
 if ((img = win_rec_cur_img_buf(_3d->win->owner)) == NULL) return;
 if ((view = img->cur_slice->slice->view) == NULL) return;
 PRT1(printf("all_rois.num_strs = %d\n",view->all_rois.num_strs))
 PRT1(printf("rois.num_strs = %d\n",view->rois.num_strs))
 PRT1(printf("roi_pixel_type.num_strs = %d\n",view->roi_pixel_types.num_strs))
 PRT1(printf("pixel_types.num_strs = %d\n",view->pixel_types.num_strs))
 WinMultiScreenCanvas(_3d->win);
/*
 if ((img = win_rec_cur_img_buf(_3d->win->owner)) == NULL) return CANCEL;
 if ((view = img->cur_slice->slice->view) == NULL) return CANCEL;
 DeleteRoisOfType(&(view->sl.slice_roi_win.slice_roi),ROI_3D_ROI,2);
 win_redisplay_all_curves(_3d->win);
*/
}


/********************
show_superimpose(view,_3d)
 struct view_rec *view;
 struct roi_type_3d_rec *_3d;
{
 PRT(printf("show_superimpose..wid = %s vstatus = %x imgstatus = %x\n",
     view->wid,(unsigned long) view->status,(unsigned long) view->img->status4))
 PRT(if (view->status & PLANE_PROJECT_PIXEL_TYPES)
        printf("PLANE_PROJECT_PIXEL_TYPES...\n"))
 if ((view->img != NULL) && (view->img->win != NULL) &&
     (view->img->win->image_display != NULL))
    if ((view->status & PLANE_3D_PIXEL_TYPES_READY) &&
        (view->status & PLANE_PROJECT_PIXEL_TYPES))
       {view->img->status4 |= IMAGE_3D_SUPERIMPOSED_READY;
        if (view->status & PLANE_3D_PIXEL_TYPES_NEED_UPDATED)
           {
           }
          else
           {PRT(printf("show_superimpose...WinMultiScreenCanvas\n"))
            WinEnableDisableColorData(view->img->win->image_display,
                            SUPERIMPOSE_3D_ID,0);
            WinMultiScreenCanvas(view->img->win->image_display);
           }
       }
 return NO_ERROR;
}

show_superimpose_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct roi_type_3d_rec *_3d;

 _3d = (struct roi_type_3d_rec *) xv_get(item,PANEL_CLIENT_DATA);
 PRT(printf("show_superimpose_proc...\n"))
 applied_proc_to_selected_plane(_3d,show_superimpose);
}

win_hide_superimposed_3d_data(win)
 struct image_display_window *win;
{
 struct img_buf_rec *img;
 struct view_rec *view;

 if ((img = win_rec_cur_img_buf(win->owner)) == NULL) return CANCEL;
 PRT(printf("win_hide_superimposed_3d_data...wwid = %s\n",win->owner->wid))
 PRT(printf("img->status4 = %x\n",img->status4))
 view = img->cur_slice->slice->view;
 if ((view->status & PLANE_3D_PIXEL_TYPES_READY) &&
     (view->status & PLANE_PROJECT_PIXEL_TYPES) &&
     (img->status4 & IMAGE_3D_SUPERIMPOSED_READY))
    {img->status4 &= ~IMAGE_3D_SUPERIMPOSED_READY;
     WinEnableDisableColorData(win,SUPERIMPOSE_3D_ID,1);
     PRT(printf("disabled...\n"))
     WinMultiScreenCanvas(win);
     return NO_ERROR;
    }
 return CANCEL;
}

hide_superimpose(view,_3d)
 struct view_rec *view;
 struct roi_type_3d_rec *_3d;
{
 PRT(printf("hide_superimpose...vwid = %s\n",view->wid))
 if ((view->img != NULL) && (view->img->win != NULL) &&
     (view->img->win->image_display != NULL))
    win_hide_superimposed_3d_data(view->img->win->image_display);
}

hide_superimpose_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct roi_type_3d_rec *_3d;

 _3d = (struct roi_type_3d_rec *) xv_get(item,PANEL_CLIENT_DATA);
 applied_proc_to_selected_plane(_3d,hide_superimpose);
}
******************/


/*-------------------------------------------------------------------------
show_hide_superimpose - ???
-------------------------------------------------------------------------*/

int
show_hide_superimpose(view,_3d)
 struct view_rec *view;
 struct roi_type_3d_rec *_3d;
{
 struct color_data_rec *color_data;
 struct image_display_window *win;

 PRT1(printf("show_hide_superimpose...vwid = %s\n",view->wid))
 if ((view->img != NULL) && (view->img->win != NULL) &&
     ((win = view->img->win->image_display) != NULL) &&
     (WinFindColorDataRec1(win,SUPERIMPOSE_3D_ID,0,&color_data,NULL) == NO_ERROR))
    {if (color_data->status4 & COLOR_DATA_DISABLED)
        {PRT1(printf("show_hide_superimpose...enable...\n"))
         WinEnableDisableColorData(win,SUPERIMPOSE_3D_ID,0);}
       else
        {PRT1(printf("show_hide_superimpose...disable...\n"))
         WinEnableDisableColorData(win,SUPERIMPOSE_3D_ID,1);}
     WinMultiScreenCanvas(win);
    }
 return NO_ERROR;
}


/*-------------------------------------------------------------------------
show_hide_superimpose_proc - ???
-------------------------------------------------------------------------*/

void
show_hide_superimpose_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct roi_type_3d_rec *_3d;

 _3d = (struct roi_type_3d_rec *) xv_get(item,PANEL_CLIENT_DATA);
 PRT1(printf("show_hide_superimpose_proc...\n"))
 applied_proc_to_selected_plane(_3d,show_hide_superimpose);
}


/*-------------------------------------------------------------------------
setup_roi_type_3d_rec - ???
-------------------------------------------------------------------------*/

int
setup_roi_type_3d_rec(_3d,win)
 struct roi_type_3d_rec *_3d;
 struct image_display_window *win;
{
 Panel_item item,item1,item2;
 int row,offset,temp,temp1;
 struct img_buf_rec *img;
 char *str;
 Rect *rr,*rr1;

 TRACE(printf("setup_roi_type_3d_rec...1\n"))
 if ((img = win_rec_cur_img_buf(win->owner)) == NULL) return CANCEL;
 _3d->frame = xv_create(win->dwin->frame, FRAME,
              FRAME_INHERIT_COLORS,TRUE,
              WIN_CLIENT_DATA,_3d,
              0);
 if (_3d->frame == 0)
    {free_roi_type_3d_rec(_3d);
     return ~NO_ERROR;}
 _3d->win = win;
 _3d->dwin = win->dwin;
 str = win_rec_win_id_title(win->owner,_3D_ROI_TYPE_MENU);
 xv_set(_3d->frame,
              FRAME_SHOW_LABEL, TRUE,
              XV_LABEL,str,
              FRAME_NO_CONFIRM,TRUE,
              FRAME_DONE_PROC,_3d_roi_type_done_proc,
              0);
 c_free(str);
 xv_set(_3d->frame,FRAME_CLOSED,FALSE,0);

 _3d->panel = xv_create(_3d->frame,PANEL,
               FRAME_INHERIT_COLORS,TRUE,
               0);
 if (_3d->panel == 0)
    {free_roi_type_3d_rec(_3d);
     return ~NO_ERROR;}

 row = 5;
 offset = xv_rows(_3d->panel,1) - xv_rows(_3d->panel,0) - 10;
 item = xv_create(_3d->panel,PANEL_MESSAGE,
	PANEL_LABEL_STRING,"SHOW ROIS & PIXEL TYPES",
        XV_X,5,XV_Y,row,
	0);
 rr = (Rect *) xv_get(item,XV_RECT);

 TRACE(printf("setup_roi_type_3d_rec...2\n"))
 _3d->display = xv_create(_3d->panel,PANEL_BUTTON,
	PANEL_LABEL_STRING,"NOW ON",
	PANEL_NOTIFY_PROC,now_on_roi_and_types_proc,
        XV_X,rr->r_left + rr->r_width + 5,XV_Y,row,
        PANEL_CLIENT_DATA,_3d,
	0);
 rr = (Rect *) xv_get(_3d->display,XV_RECT);
 xv_create(_3d->panel,PANEL_BUTTON,
	PANEL_LABEL_STRING,"NOW OFF",
	PANEL_NOTIFY_PROC,now_off_roi_and_types_proc,
        XV_X,rr->r_left + rr->r_width + 5,XV_Y,row,
        PANEL_CLIENT_DATA,_3d,
	0);

 row += offset;

 item = xv_create(_3d->panel,PANEL_BUTTON,
	PANEL_LABEL_STRING,"ALL",
	PANEL_NOTIFY_PROC,select_list_proc,
        XV_X,5,XV_Y,row,
	0);
 rr = (Rect *) xv_get(item,XV_RECT);
 item1 = xv_create(_3d->panel,PANEL_BUTTON,
	PANEL_LABEL_STRING,"NONE",
	PANEL_NOTIFY_PROC,deselect_list_proc,
        XV_X,rr->r_left + rr->r_width,XV_Y,row,
	0);
 rr = (Rect *) xv_get(item1,XV_RECT);
 item2 = xv_create(_3d->panel,PANEL_BUTTON,
	PANEL_LABEL_STRING,"CUR WID",
	PANEL_NOTIFY_PROC,cur_wid_proc,
        XV_X,rr->r_left + rr->r_width,XV_Y,row,
        PANEL_CLIENT_DATA,_3d,
	0);
 rr = (Rect *) xv_get(item2,XV_RECT);
 item2 = xv_create(_3d->panel,PANEL_BUTTON,
	PANEL_LABEL_STRING,"CUR GROUP",
	PANEL_NOTIFY_PROC,cur_group_proc,
        XV_X,rr->r_left + rr->r_width,XV_Y,row,
        PANEL_CLIENT_DATA,_3d,
	0);
 TRACE(printf("setup_roi_type_3d_rec...3..1\n"))
 rr = (Rect *) xv_get(item2,XV_RECT);
 xv_create(_3d->panel,PANEL_BUTTON,
	PANEL_LABEL_STRING,"GROUP",
	PANEL_NOTIFY_PROC,select_group_proc,
        XV_X,rr->r_left + rr->r_width + 5,XV_Y,row,
        PANEL_CLIENT_DATA,_3d,
	0);

 TRACE(printf("setup_roi_type_3d_rec...4\n"))

 row += offset;
 _3d->apply_to = xv_create(_3d->panel,PANEL_LIST,
                 PANEL_CHOOSE_ONE,FALSE,
                 PANEL_LIST_DISPLAY_ROWS,4,
                 PANEL_LIST_WIDTH,70,
                 PANEL_LAYOUT,PANEL_VERTICAL,
                 XV_X,5,XV_Y,row,
                 0);
 xv_set(item,PANEL_CLIENT_DATA,_3d->apply_to,0);
 xv_set(item1,PANEL_CLIENT_DATA,_3d->apply_to,0);
 setup_view_window_list(img,_3d->apply_to,0,1,1,1,1);
 win_select_cur_wid_item(_3d->win,_3d->apply_to);

 rr = (Rect *) xv_get(_3d->apply_to,XV_RECT);
 item = xv_create(_3d->panel,PANEL_MESSAGE,
                  XV_X,rr->r_left + rr->r_width + 5,XV_Y,row + 3,
 	           PANEL_LABEL_STRING, "ROIS",PANEL_VALUE,0,
                   0);
 rr1 = (Rect *) xv_get(item,XV_RECT);
 item = xv_create(_3d->panel,PANEL_BUTTON,
	PANEL_LABEL_STRING,"ALL",
	PANEL_NOTIFY_PROC,select_list_proc,
        XV_X,rr1->r_left + rr1->r_width + 5,XV_Y,row,
	0);
 rr1 = (Rect *) xv_get(item,XV_RECT);
 item1 = xv_create(_3d->panel,PANEL_BUTTON,
	PANEL_LABEL_STRING,"NONE",
	PANEL_NOTIFY_PROC,deselect_list_proc,
        XV_X,rr1->r_left + rr1->r_width + 5,XV_Y,row,
	0);
 rr1 = (Rect *) xv_get(item1,XV_RECT);
 xv_create(_3d->panel,PANEL_BUTTON,
	PANEL_LABEL_STRING,"UPDATE ROIS - <> /",
	PANEL_NOTIFY_PROC,update_cur_3d_rois_proc,
        XV_X,rr1->r_left + rr1->r_width + 5,XV_Y,row,
        PANEL_CLIENT_DATA,_3d,0);

 TRACE(printf("setup_roi_type_3d_rec...5\n"))
/*
 rr1 = (Rect *) xv_get(item1,XV_RECT);
 _3d->is_verify = xv_create(_3d->panel,PANEL_CHECK_BOX,
	PANEL_LABEL_STRING,"FILLED",
	PANEL_NOTIFY_PROC,verify_rois_proc,
        PANEL_CLIENT_DATA,_3d,
        XV_X,rr1->r_left + rr1->r_width + 5,XV_Y,row,
	0);
*/
 row += offset;
 _3d->show_roi = xv_create(_3d->panel,PANEL_LIST,
        PANEL_LAYOUT,PANEL_VERTICAL,
        PANEL_CHOOSE_ONE,FALSE,
        PANEL_LIST_DISPLAY_ROWS,3,
        PANEL_LIST_WIDTH,230,
        XV_X,rr->r_left + rr->r_width + 5,XV_Y,row,
        PANEL_CLIENT_DATA,_3d,0);
 setup_file_rois_list_item(img,_3d->show_roi);
 xv_set(item,PANEL_CLIENT_DATA,_3d->show_roi,0);
 xv_set(item1,PANEL_CLIENT_DATA,_3d->show_roi,0);

 TRACE(printf("setup_roi_type_3d_rec...6\n"))

 rr = (Rect *) xv_get(_3d->show_roi,XV_RECT);
 row = rr->r_top + rr->r_height + 5;
 item = xv_create(_3d->panel,PANEL_MESSAGE,
                   XV_X,5,XV_Y,row,
 	           PANEL_LABEL_STRING, "ROI TYPES",PANEL_VALUE,0,
                   0);
 rr1 = (Rect *) xv_get(item,XV_RECT);
 item = xv_create(_3d->panel,PANEL_BUTTON,
	PANEL_LABEL_STRING,"ALL",
	PANEL_NOTIFY_PROC,select_list_proc,
        XV_X,rr1->r_left + rr1->r_width,XV_Y,row,
	0);
 rr1 = (Rect *) xv_get(item,XV_RECT);
 item1 = xv_create(_3d->panel,PANEL_BUTTON,
	PANEL_LABEL_STRING,"NONE",
	PANEL_NOTIFY_PROC,deselect_list_proc,
        XV_X,rr1->r_left + rr1->r_width + 5,XV_Y,row,
	0);

 temp = row;
 row += offset;
 _3d->roi_type = xv_create(_3d->panel,PANEL_LIST,
        PANEL_LAYOUT,PANEL_VERTICAL,
        PANEL_CHOOSE_ONE,FALSE,
        PANEL_LIST_DISPLAY_ROWS,3,
        PANEL_LIST_WIDTH,165,
        XV_X,5,XV_Y,row,
        PANEL_CLIENT_DATA,_3d,0);
 setup_file_rois_list_item(img,_3d->roi_type);
 xv_set(item,PANEL_CLIENT_DATA,_3d->roi_type,0);
 xv_set(item1,PANEL_CLIENT_DATA,_3d->roi_type,0);

 rr = (Rect *) xv_get(_3d->roi_type,XV_RECT);
 _3d->show_pixel_type = xv_create(_3d->panel,PANEL_LIST,
        PANEL_LAYOUT,PANEL_VERTICAL,
        PANEL_CHOOSE_ONE,FALSE,
        PANEL_LIST_DISPLAY_ROWS,3,
        PANEL_LIST_WIDTH,160,
        XV_X,rr->r_left + rr->r_width + 5,XV_Y,row,
        PANEL_CLIENT_DATA,_3d,0);
 setup_3d_pixel_type_proc(_3d);
 rr = (Rect *) xv_get(_3d->show_pixel_type,XV_RECT);
 TRACE(printf("setup_roi_type_3d_rec...7\n"))

 item = xv_create(_3d->panel,PANEL_MESSAGE,PANEL_LABEL_STRING, "TYPES",
                   XV_Y,temp + 3,
 	           0);
 rr1 = (Rect *) xv_get(item,XV_RECT);
 temp1 = rr->r_left + rr->r_width - rr1->r_width;
 xv_set(item,XV_X,temp1,0);
 item1 = xv_create(_3d->panel,PANEL_BUTTON,
	PANEL_LABEL_STRING,"NONE",
	PANEL_NOTIFY_PROC,deselect_list_proc,
        XV_Y,temp,
        PANEL_CLIENT_DATA,_3d->show_pixel_type,
	0);
 rr = (Rect *) xv_get(item,XV_RECT);
 temp1 = temp1 - rr->r_width - 15;
 xv_set(item1,XV_X,temp1,0);

 item = xv_create(_3d->panel,PANEL_BUTTON,
	PANEL_LABEL_STRING,"ALL",
	PANEL_NOTIFY_PROC,select_list_proc,
        XV_Y,temp,
        PANEL_CLIENT_DATA,_3d->show_pixel_type,
	0);
 TRACE(printf("setup_roi_type_3d_rec...8\n"))
 rr1 = (Rect *) xv_get(item,XV_RECT);
 rr = (Rect *) xv_get(item1,XV_RECT);
 temp1 = rr->r_left - rr1->r_width - 3;
 xv_set(item,XV_X,temp1,0);

 rr = (Rect *) xv_get(_3d->show_pixel_type,XV_RECT);
 row = rr->r_top + rr->r_height + 2;
 _3d->show = xv_create(_3d->panel,PANEL_BUTTON,
	PANEL_LABEL_STRING,"SHOW/HIDE TYPES",
	PANEL_NOTIFY_PROC,show_hide_superimpose_proc,
        XV_X,5,XV_Y,row,
        PANEL_CLIENT_DATA,_3d,
	0);
/*
 rr = (Rect *) xv_get(_3d->show,XV_RECT);
 _3d->hide = xv_create(_3d->panel,PANEL_BUTTON,
	PANEL_LABEL_STRING,"HIDE TYPES",
	PANEL_NOTIFY_PROC,hide_superimpose_proc,
        XV_X,rr->r_left + rr->r_width + 5,XV_Y,row,
        PANEL_CLIENT_DATA,_3d,
	0);
*/
 TRACE(printf("setup_roi_type_3d_rec...3\n"))

 rr = (Rect *) xv_get(_3d->show_pixel_type,XV_RECT);
 window_fit(_3d->panel);
 xv_set(_3d->panel,XV_WIDTH,rr->r_left + rr->r_width + 5,0);
 rr = (Rect *) xv_get(_3d->panel,XV_RECT);
 xv_set(_3d->panel,XV_HEIGHT,rr->r_height - 5,0);
 TRACE(printf("setup_roi_type_3d_rec...9\n"))
 window_fit(_3d->frame);
 make_2_wins_adjacent(_3d->frame,win->dwin->frame,0);
 win_set_mouse_at_center(_3d->frame);

 return NO_ERROR;
}


/*-------------------------------------------------------------------------
find_roi_type_3d_rec - ???
-------------------------------------------------------------------------*/

struct roi_type_3d_rec *
find_roi_type_3d_rec(all_3ds,win)
 struct all_roi_type_3d_recs *all_3ds;
 struct image_display_window *win;
{
 int lcv;
 struct input_file_rec *f1;
 struct img_buf_rec *img;

 if ((img = win_rec_cur_img_buf(win->owner)) == NULL) return NULL;
 f1 = img->file;
 for (lcv = 0; lcv < all_3ds->num_3ds; lcv++)
    if ((img = win_rec_cur_img_buf(all_3ds->_3ds[lcv]->win->owner)) != NULL)
       if (img->file == f1)
          return all_3ds->_3ds[lcv];
 return NULL;
}


/*-------------------------------------------------------------------------
add_roi_type_window_list - ???
-------------------------------------------------------------------------*/

void
add_roi_type_window_list(win,view)
 struct image_display_window *win;
 struct view_rec *view;
{
 struct all_roi_type_3d_recs *all_3ds;
 struct roi_type_3d_rec *_3d;

 if ((all_3ds = (struct all_roi_type_3d_recs *) 
             enter_3d_rois_and_pixel_types(win,GET_ALL_3D_MENU_PTR)) != NULL)
    {
     if ((_3d = find_roi_type_3d_rec(all_3ds,win)) != NULL)
        {add_window_list(_3d->apply_to,view);
        }
    }
}


/*-------------------------------------------------------------------------
enter_3d_rois_and_pixel_types - ???
-------------------------------------------------------------------------*/

unsigned long 
enter_3d_rois_and_pixel_types(win,status)
 struct image_display_window *win;
 int status;
{
 static struct all_roi_type_3d_recs all_3ds;
 static char first_time = 1;
 int err;
 struct roi_type_3d_rec *_3d;

 err = NO_ERROR;
 if (status == GET_ALL_3D_MENU_PTR)
    {if (first_time) return 0;
     return (unsigned long) &all_3ds;}     /* FIXME:  ptr as long */
  else if (status == CLEAN_UP_3D_MENU)
   {if (!first_time)
       if ((_3d = find_roi_type_3d_rec(&all_3ds,win)) != NULL)
          free_roi_type_3d_rec(_3d);
    return NO_ERROR;}

 if (first_time)
    {init_all_roi_type_3d_recs(&all_3ds);
     first_time = 0;}
   else
    {if ((_3d = find_roi_type_3d_rec(&all_3ds,win)) != NULL)
        if (_3d->win == win)
           {xv_set(_3d->frame,XV_SHOW,TRUE,FRAME_CLOSED,FALSE,WIN_ALARM,0);
            win_set_mouse_at_center(_3d->frame);
            wmgr_top(_3d->frame);
            return NO_ERROR;}
          else free_roi_type_3d_rec(_3d);
    }
/*
    {
     if ((_3d = find_roi_type_3d_rec(&all_3ds,win)) != NULL)
        {_3d->win = win;
         _3d->dwin = win->dwin;
         xv_set(_3d->frame,XV_SHOW,TRUE,0);
         xv_set(_3d->frame,XV_OWNER,win->dwin->frame,
              XV_SHOW,TRUE,FRAME_CLOSED,FALSE,WIN_ALARM,0);
         str = win_rec_win_id_title(win->owner,_3D_ROI_TYPE_MENU);
         xv_set(_3d->frame,XV_LABEL,str,0);
         c_free(str);
         win_set_mouse_at_center(_3d->frame);

         wmgr_top(_3d->frame);
         return NO_ERROR;}
    }
*/
 get_handle_memory(&(all_3ds._3ds),&(all_3ds.num_3ds),1,
     (size_t) sizeof(struct roi_type_3d_rec),&err);
 if (err != NO_ERROR) return err;
 _3d = all_3ds._3ds[all_3ds.num_3ds - 1];
 init_roi_type_3d_rec(_3d);
 if ((err = setup_roi_type_3d_rec(_3d,win)) == NO_ERROR)
    xv_set(_3d->frame,XV_SHOW,TRUE,FRAME_CLOSED,FALSE,0);
 return err;
}
