/****************************************************************************
analysis.c - ???

Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.
****************************************************************************/

static const char rcsid[] = "$Id: analysis.c,v 1.8 1999/07/07 17:41:14 hughett Exp $ Release $Name: nria_1_0_6 $";


#include "debug.h"
#include "mri_format.h"
#include "nria_procedures.h"


/*------------------------------------------------------------------------
USE_SPLIT_SHARED_PIXELS_ALGORITHM

  DRW  960909:

   If this is defined, then if two (or more) regions overlap, the
pixels which are shared between them will be distributed evenly
among the sharing regions.

   If this is not defined, then if two (or more) regions overlap,
the pixels which are shared between them will simply be counted
multiple times.

   This is only for compiling and testing purposes.  If a runtime
switch is desired, this ought to be changed to a global (package)
variable.

   To define this macro, simply uncomment the #define.

NB: as of September 1996, the MHCRC is NOT using the split shared
pixels algorithm.  Therefore, this should NOT be defined.  
---------------------------------------------------------------------------*/

/*  #define USE_SPLIT_SHARED_PIXELS_ALGORITHM  */


/* Functions defined in this file */
void init_analysis_rec(struct analysis_rec *anal);
void free_analysis_rec(struct analysis_rec *analysis);
int setup_analysis_params(struct analysis_rec *anal,
    struct output_text_format_rec *format, struct pixel_type_rec *pixel_type);
void init_common_pixel_rec(struct common_pixel_rec *common,
    int is_init_total_indxs);
void init_common_pixel_rec_matrix(struct buf_2d_rec *common,
    int is_init_total_indxs);
void free_common_pixel_rec(struct common_pixel_rec *share);
void free_common_pixel_rec_matrix(struct buf_2d_rec *common);
int add_global_pixel_rec(struct common_pixel_rec *share,
    unsigned int roi_indx);
int add_global_pixel_by_mask_proc(struct buf_2d_rec *common,
    unsigned long row, unsigned long col, unsigned long *pixel_size,
    struct buf_2d_rec *pixel_types, unsigned long *roi_indx);
int add_global_pixel_rec_matrix_by_mask(struct buf_2d_rec *common,
    struct buf_2d_rec *mask, struct int_bound_rec *affected,
    struct buf_2d_rec *pixel_types, unsigned long roi_indx);
int count_shared_pixels(struct common_pixel_rec *common,
    float *cnt_shared_pixels, unsigned long pixel_type_indx,
    unsigned char *group_roi_yes, struct output_text_format_rec *format);
int is_cnt_share(struct common_pixel_rec *share_pixel);
int correct_slice_counts(struct analysis_rec *anal,
    struct output_text_format_rec *format, struct slice_roi_rec *slice_roi);
void check_analysis_status(struct analysis_rec *analysis);
int is_roi_in_analysis_groups(struct text_output_analysis_group_rec *groups,
    unsigned int num_groups, char *roi_label);
int cnt_pixel_types_by_mask_proc(struct buf_2d_rec *_2d, unsigned long row,
    unsigned long col, unsigned int *pixel_type_cnts,
    unsigned int num_pixel_types);
int cnt_pixel_types_by_mask(struct buf_2d_rec *_2d, struct buf_2d_rec *mask,
    struct int_bound_rec *affected, unsigned int *pixel_type_cnts,
    unsigned int num_pixel_types);
int setup_pixel_type_cnt_rec(struct pixel_type_cnt_rec **ptr,
    struct pixel_type_rec *pixel_type);
int unset_slice_counts(struct slice_roi_rec *slice_roi,
    struct output_text_format_rec *format, unsigned long last_roi_indx);



/*-----------------------------------------------------------------------
init_analysis_rec - ???
-----------------------------------------------------------------------*/

void
init_analysis_rec(anal)
 struct analysis_rec *anal;
{
 anal->next_slice_num = 0;
 anal->err = NO_ERROR;
 anal->num_type_indxs = 0;
 anal->type_indxs = NULL;
 anal->total_group_rois = 0;
 anal->is_shared_pixel = 0;
 anal->group_roi_yes = NULL;
 anal->cnt_shared_pixels = NULL;
 init_buf_2d_rec(&(anal->_2d));
 init_draw_mask_rec(&(anal->mask));
 init_buf_2d_rec(&(anal->common));
}



/*-----------------------------------------------------------------------
free_analysis_rec - ???
-----------------------------------------------------------------------*/

void
free_analysis_rec(analysis)
     struct analysis_rec * analysis;
{
  TRACE(printf("free_analysis_rec()\n"););
  TRACE(printf("	analysis: %p\n",
	       (void *) analysis);
	printf("		type_indxs: %p\n",
	       (void *) analysis->type_indxs);
	printf("		group_roi_yes: %p\n",
	       (void *) analysis->group_roi_yes);
	printf("		cnt_shared_pixels: %p\n",
	       (void *) analysis->cnt_shared_pixels););

  c_free(analysis->type_indxs);
  c_free(analysis->group_roi_yes);
  c_free(analysis->cnt_shared_pixels);
  free_buf_2d_rec(&(analysis->_2d));
  free_draw_mask_rec(&(analysis->mask));
  free_common_pixel_rec_matrix(&(analysis->common));
  free_buf_2d_rec(&(analysis->common));
  TRACE(printf("free_analysis_rec() end.\n"););
}


/*-----------------------------------------------------------------------
setup_analysis_params - ???
-----------------------------------------------------------------------*/

int
setup_analysis_params(anal,format,pixel_type)
 struct analysis_rec *anal;
 struct output_text_format_rec *format;
 struct pixel_type_rec *pixel_type;
{
 unsigned long indx,total,start_indx;
 int err;
 long offset,cnt,whch;

 anal->total_group_rois = format->roi_labels.num_strs * format->num_groups;

 total = 0;
 anal->num_type_indxs = pixel_type->order.num_strs + 1;
 if ((err = get_ptr(&(anal->type_indxs),&total,anal->num_type_indxs,
            sizeof(*(anal->type_indxs)))) != NO_ERROR)
     return err;
 for (cnt = 0; cnt < pixel_type->order.num_strs; cnt++)
    anal->type_indxs[cnt] = 0;

 for (cnt = 0; cnt < pixel_type->order.num_strs; cnt++)
    if (find_str_in_strs_rec(&(format->pixel_type_labels),
             pixel_type->order.strs[cnt],&indx) == NO_ERROR)
       anal->type_indxs[cnt + 1] = format->pixel_type_nums[indx];
      else if (find_sub_str(pixel_type->order.strs[cnt],".TYPE",&start_indx) ==
                      NO_ERROR)
       {pixel_type->order.strs[cnt][start_indx] = '\0';
        if (find_str_in_strs_rec(&(format->pixel_type_labels),
             pixel_type->order.strs[cnt],&indx) == NO_ERROR)
           anal->type_indxs[cnt + 1] = format->pixel_type_nums[indx];
        pixel_type->order.strs[cnt][start_indx] = '.';}

/*
 pr_strs_rec(&(pixel_type->order));
 printf("setup_dp_analysis_rec...1\n");
 pr_strs_rec(&(format->pixel_type_labels));
 for (cnt = 0; cnt <= pixel_type->order.num_strs; cnt++)
    printf("%d) %d\n",cnt,anal->type_indxs[cnt]);

 printf("anal->total_group_rois = %d num_strs = %d format->num_groups = %d\n",anal->total_group_rois,format->roi_labels.num_strs,format->num_groups);
 for (cnt = 0; cnt < format->pixel_type_labels.num_strs; cnt++)
    printf("%d) %d %s\n",cnt,format->pixel_type_nums[cnt],
        format->pixel_type_labels.strs[cnt]);
 for (cnt = 0; cnt < format->num_groups; cnt++)
    printf("G#%d) type#%d\n",cnt+1,format->groups[cnt].pixel_type_indx);
*/

 for (cnt = 0; cnt < format->num_groups; cnt++)
    if (format->groups[cnt].num_roi_indxs > 1)
       {anal->is_shared_pixel = 1;
        break;}
 if (anal->is_shared_pixel)
    {total = 0;
     if ((err = get_ptr(&(anal->group_roi_yes),&total,anal->total_group_rois,
            sizeof(*(anal->group_roi_yes)))) != NO_ERROR)
        return err;
     
     for (cnt = 0; cnt < anal->total_group_rois; cnt++)
        anal->group_roi_yes[cnt] = 0;
     for (offset = -1,cnt = 0; cnt < format->num_groups; cnt++,
            offset += format->roi_labels.num_strs)
        if (format->groups[cnt].num_roi_indxs > 1)
           for (whch = 0; whch < format->groups[cnt].num_roi_indxs; whch++)
               {anal->group_roi_yes[offset + 
                     format->groups[cnt].roi_indxs[whch]] = 1;
/*
                printf("groups[%d].roi_indxs[%d] group_roi_yes[%d] == 1\n",cnt,whch,
                       offset + format->groups[cnt].roi_indxs[whch]);
*/
               }
     total = 0;
     if ((err = get_ptr(&(anal->cnt_shared_pixels),&total,
         anal->total_group_rois,sizeof(*(anal->cnt_shared_pixels)))) 
             != NO_ERROR)
        return err;}
 return NO_ERROR;
}



/*-----------------------------------------------------------------------
init_common_pixel_rec - ???
-----------------------------------------------------------------------*/

void
init_common_pixel_rec(common,is_init_total_indxs)
 struct common_pixel_rec *common;
 int is_init_total_indxs;
{
 if (is_init_total_indxs) common->total_indxs = 0;
 common->num_indxs = 0;
}


/*-----------------------------------------------------------------------
init_common_pixel_rec_matrix - ???
-----------------------------------------------------------------------*/

void
init_common_pixel_rec_matrix(common,is_init_total_indxs)
 struct buf_2d_rec *common;
 int is_init_total_indxs;
{
 unsigned row,col,size;

 size = sizeof(struct common_pixel_rec);
 for (row = 0; row < common->dimy; row++)
    for (col = 0; col < common->dimx; col++)
       init_common_pixel_rec(&(common->buf[row][col * size]),
          is_init_total_indxs);                 
}



/*-----------------------------------------------------------------------
free_common_pixel_rec - ???

FIXME:  Do NOT trace this routine -- when crashing, outputs voluminous
text.  DRW 961121
-----------------------------------------------------------------------*/

void
free_common_pixel_rec(share)
     struct common_pixel_rec *share;
{
  if (share->total_indxs > 0)
  {
    share->total_indxs = share->num_indxs = 0;
    c_free(share->indxs);
  }
}



/*-----------------------------------------------------------------------
free_common_pixel_rec_matrix - ???
-----------------------------------------------------------------------*/

void
free_common_pixel_rec_matrix(common)
     struct buf_2d_rec *common;
{
  unsigned row,col,size;

  TRACE(printf("free_common_pixel_rec_matrix()\n"););
  size = sizeof(struct common_pixel_rec);
  NOPRT(printf("	size = %u\n", size);
	printf("	dimy = %u\n", common->dimy);
	printf("	dimx = %u\n", common->dimx););
  for (row = 0; row < common->dimy; row++)
  {
    for (col = 0; col < common->dimx; col++)
    {
      free_common_pixel_rec(&(common->buf[row][col * size]));
      NOPRT(printf("\r	(%u,%u) freed",row,col););
    }
  }
  NOPRT(printf("\nfree_common_pixel_rec_matrix() end.\n"););
}


/*-----------------------------------------------------------------------
add_global_pixel_rec - ???
-----------------------------------------------------------------------*/

int
add_global_pixel_rec(share,roi_indx)
 struct common_pixel_rec *share;
 unsigned int roi_indx;
{
 int err;

 if (share->num_indxs >= share->total_indxs)
    {get_ptr_memory(&(share->indxs),&(share->total_indxs),(unsigned int) 1,
          (size_t) sizeof(*(share->indxs)),&err);
     if (err != NO_ERROR) return err;}
 share->indxs[share->num_indxs] = roi_indx;
 share->num_indxs++;

 return NO_ERROR;
}


/*-----------------------------------------------------------------------
add_global_pixel_by_mask_proc - ???
-----------------------------------------------------------------------*/

int
add_global_pixel_by_mask_proc(common,row,col,pixel_size,pixel_types,roi_indx)
 struct buf_2d_rec *common;
 unsigned long row,col,*pixel_size,*roi_indx;
 struct buf_2d_rec *pixel_types;
{
 if (get_1d_buf_value(pixel_types->buf[row],col,pixel_types->pixel_depth))
    return add_global_pixel_rec(&(common->buf[row][col * (*pixel_size)]),
               *roi_indx);
 return NO_ERROR;
}


/*-----------------------------------------------------------------------
add_global_pixel_rec_matrix_by_mask - ???
-----------------------------------------------------------------------*/

int
add_global_pixel_rec_matrix_by_mask(common,mask,affected,pixel_types,roi_indx)
 struct buf_2d_rec *common,*mask;
 struct int_bound_rec *affected;
 struct buf_2d_rec *pixel_types;
 unsigned long roi_indx;
{
 unsigned long size;
 int err;

 TRACE(printf("add_global_pixel_rec_matrix_by_mask...\n"))
 /*printf("add_global_pixel_rec_matrix_by_mask...pixel_types = %u  roi_indx = %d\n",pixel_types,roi_indx);fflush(stdout);*/

 size = sizeof(struct common_pixel_rec);
 err = process_buf_2d_rec_by_mask(common,0,0,mask,affected,
           add_global_pixel_by_mask_proc,&size,pixel_types,&roi_indx,
           0,0,0,0,0,0);
 /* printf("add_global_pixel_rec_matrix_by_mask...err...end = %d\n",err);
    fflush(stdout);*/
 return err;
}


/*-----------------------------------------------------------------------
count_shared_pixels - ???
-----------------------------------------------------------------------*/

int
count_shared_pixels(common,cnt_shared_pixels,pixel_type_indx,group_roi_yes,format)
 struct common_pixel_rec *common;
 float *cnt_shared_pixels;
 unsigned long pixel_type_indx;
 unsigned char *group_roi_yes;
 struct output_text_format_rec *format;
{
 float ratio;
 unsigned long lcv,cnt;
 long offset,whch,num,last_whch,last;
 int yes;

 if (common->num_indxs <= 1) return NO_ERROR;
 for (yes = 0,lcv = 0; lcv < common->num_indxs; lcv++)
    for (offset = -1,cnt = 0; cnt < format->num_groups; cnt++,
          offset += format->roi_labels.num_strs)
       if ((format->groups[cnt].pixel_type_indx == pixel_type_indx) &&
           (format->groups[cnt].num_roi_indxs > 1))
          {whch = offset + common->indxs[lcv];
           if (group_roi_yes[whch])
              {yes = 1;
               group_roi_yes[whch] = 2;}
          }
 if (yes == 0) return NO_ERROR;
 for (offset = 0,cnt = 0; cnt < format->num_groups; cnt++,
          offset += format->roi_labels.num_strs)
     if ((format->groups[cnt].pixel_type_indx == pixel_type_indx) &&
         (format->groups[cnt].num_roi_indxs > 1))
        {last = offset + format->roi_labels.num_strs;
         for (num = 0,whch = offset; whch < last; whch++)
             if (group_roi_yes[whch] == 2)
                {last_whch = whch;
                 num += 1;}
         if (num == 1)
            group_roi_yes[last_whch] = 1;
           else if (num > 1)
            {ratio = (float) (num - 1) / (float) num;
             for (whch = offset; whch < last; whch++)
                if (group_roi_yes[whch] == 2)
                   {group_roi_yes[whch] = 1;
                    cnt_shared_pixels[whch] += ratio;}
            }
        }
 return NO_ERROR;
}


/*-----------------------------------------------------------------------
is_cnt_share - ???
-----------------------------------------------------------------------*/

int
is_cnt_share(share_pixel)
 struct common_pixel_rec *share_pixel;
{
 if (share_pixel->num_indxs > 1) return 1;
 return 0;
}






/*-----------------------------------------------------------------------
correct_slice_counts - ???
-----------------------------------------------------------------------*/

int
correct_slice_counts( anal, format, slice_roi )
     struct analysis_rec *anal;
     struct output_text_format_rec *format;
     struct slice_roi_rec *slice_roi;
{
  struct buf_2d_rec *common, *pixel_types;
  unsigned long row, col, size, cnt, pixel_type_indx;

  float *cnt_shared_pixels;
  unsigned long fnum, lcv;
  unsigned long roi_indx;
  long shared_cnt;
  struct roi_rec *roi;
  struct pixel_type_cnt_rec *ptr;
  long offset,whch;
  struct output_count_group_rec *count;



  if (anal->is_shared_pixel)
  {
    cnt_shared_pixels = anal->cnt_shared_pixels;
    for (col = 0; col < anal->total_group_rois; col++)
      cnt_shared_pixels[col] = 0.0;
    common = &(anal->common);
    pixel_types = anal->_2d_ptr;

    TRACE(printf("correct_slice_counts()\n");
	  printf("	is_shared_pixel is true, total_group_rois = %d\n",
		 anal->total_group_rois););

    size = sizeof(struct common_pixel_rec) * common->dimx;
    for (row = 0;
	 row < common->dimy;
	 row++)
    {
      for (col = 0, cnt = 0;
	   col < size;
	   col += sizeof(struct common_pixel_rec), cnt++)
      {
	if (is_cnt_share(&(common->buf[row][col])))
	{
	  pixel_type_indx = get_1d_buf_value(pixel_types->buf[row],cnt,
					     pixel_types->pixel_depth);
	  if (anal->type_indxs[pixel_type_indx])
	  {
	    count_shared_pixels(&(common->buf[row][col]),
				cnt_shared_pixels,
				anal->type_indxs[pixel_type_indx],
				anal->group_roi_yes,format);
	  }
	}
      }
    }

#if 0
    size = sizeof(struct common_pixel_rec);
    for (row = 0; row < common->dimy; row++)
      for (col = 0; col < common->dimx; col++)
      {
	if (is_cnt_share(&(common->buf[row][col * size])))
	{
	  TRACE(printf("%d/%d\n",row,col));
	  pixel_type_indx = get_1d_buf_value(pixel_types->buf[row],col,
					     pixel_types->pixel_depth);
	  if (anal->type_indxs[pixel_type_indx])
	    count_shared_pixels(&(common->buf[row][col * size]),
				cnt_shared_pixels,
				anal->type_indxs[pixel_type_indx],
				anal->group_roi_yes,format);
	}
      }
#endif


#if 0
    /* DEBUGGING  DRW  960909 */
    for (offset = 0,cnt = 0; cnt < format->num_groups; cnt++,
	   offset += format->roi_labels.num_strs)
    {
      for (col = 0; col < format->roi_labels.num_strs; col++)
	printf("grp%d,reg%d %.3f  ",cnt,col,cnt_shared_pixels[offset + col]);
      printf("\n");
    }
#endif

  } /* if (anal->is_shared_pixel) */
  else
  {
    TRACE(printf("correct_slice_counts()\n");
	  printf("	is_shared_pixel is false\n"););
  }

  TRACE(printf("	next_slice_num = %d  \n",anal->next_slice_num + 1););
  count = &(format->counts[anal->next_slice_num]);

  /*
   *  DRW 19971121
   *
   *  FIXME: when something crashes NRIA, bad data can be saved in .ndat file,
   *    and when num_groups is not zero and count->groups is NULL, the
   *    code in the else{ below creates a segmentation fault.  Added this
   *    check to forestall this problem, but I don't know what the real
   *    problem behind it is.
   */
  if (!(count->groups)) {
    fprintf( stderr, "NRIA: Error: slice %ld, num_groups=%ld, but no groups!\n",
	     anal->next_slice_num + 1, count->num_groups );
    goto skip_correction;
  }


  for (lcv = 0; lcv < count->num_groups; lcv++)
  {

    for (fnum = 0; fnum < count->groups[lcv].num_fields; fnum++)
    {

      for (row = 1; row < anal->num_type_indxs; row++)
      {
	if (format->groups[lcv].pixel_type_indx == anal->type_indxs[row])
	{
	  count->groups[lcv].fields[fnum].valid = 0;
	  break;
	}
      }

      /*
       *  If we went through all indexes, we did not find a match between
       *    format->groups[lcv].pixel_type_indx and anal->type_indxs[row]
       */
      if (row == anal->num_type_indxs)
      {
	count->groups[lcv].fields[fnum].valid = COUNT_PIXEL_TYPE_NOT_DEFINED;
      }  /*  if (row == anal->num_type_indxs)  */

    }  /* for (fnum) */

  }  /*  for (lcv)  */

  NOPRT(printf("correct_slice_counts correct_counts..4.\n"));
  for (cnt = 0; cnt < format->roi_labels.num_strs; cnt++)
  {

    roi_indx = format->roi_nums[cnt];
    if ((roi_indx == 0) ||
	((roi = get_roi(slice_roi,format->roi_labels.strs[cnt])) == NULL))
      continue;

    if ((ptr = (struct pixel_type_cnt_rec *) roi->user.ptr) == NULL)
      continue;

    for (row = 1; row < ptr->num_pixel_type_cnts; row++)
    {

      if (anal->type_indxs[row] <= 0)
	continue;

      offset = -1;
      for (lcv = 0; lcv < format->num_groups; lcv++)
      {

	if (format->groups[lcv].pixel_type_indx != anal->type_indxs[row])
	  goto next_format_group;

	if ((format->groups[lcv].num_roi_indxs > 1) &&
	    anal->is_shared_pixel)
	{
	  whch = offset + roi_indx;
	  shared_cnt = (long) rint((double) cnt_shared_pixels[whch]);
	}
	else
	  shared_cnt = 0;

	for (fnum = 0; fnum < count->groups[lcv].num_fields; fnum++)
	{

	  if (roi_indx != format->groups[lcv].roi_indxs[fnum])
	    continue;

	  /*  FIXME:
	   *  ptr->pixel_type_cnts[] holds the original pixel counts.
	   *   shared_cnt contains the number of pixels shared with any
	   *   other region(s).
	   *
	   *  Oddly enough, even before this point,
	   *   count->groups[lcv].fields[fnum].value already holds a
	   *   "shared-pixel-corrected" count, so this subtraction is almost
	   *   redundant.  However, using the ptr-> value at this point seems
	   *   to effectively turn off the shared-pixel algorithm.
	   *
	   *  DRW 19960909
	   */

#ifdef USE_SPLIT_SHARED_PIXELS_ALGORITHM
	  count->groups[lcv].fields[fnum].value = 
	    ptr->pixel_type_cnts[row] - shared_cnt;
#else
	  count->groups[lcv].fields[fnum].value = 
	    ptr->pixel_type_cnts[row];
#endif
	  count->groups[lcv].fields[fnum].valid = COUNT_ROI_DEFINED;
	  break;

	}  /*  for (fnum = 0; fnum < count->groups[lcv].num_fields;...)  */

	if (fnum == count->groups[lcv].num_fields)
	  count->groups[lcv].fields[fnum].valid = COUNT_ROI_NOT_COMPUTED;

      next_format_group:
	offset += format->roi_labels.num_strs;

      }  /*  for (lcv = 0; lcv < format->num_groups; ...)  */

    }  /*  for (row = 1; row < ptr->num_pixel_type_cnts; row++)  */

  }  /*  for (cnt = 0; cnt < format->roi_labels.num_strs; cnt++)  */


skip_correction:


#ifdef USE_SPLIT_SHARED_PIXELS_ALGORITHM
  if (anal->next_slice_num == 1)
    fprintf( stderr,
	     "nria: Compiled to use split shared pixels algorithm.\n" );
#else
  if (anal->next_slice_num == 1)
    fprintf( stderr,
	     "nria: Compiled to use absolute pixel count algorithm.\n" );
#endif

  format->status |= OUT_FORMAT_COUNT_DEFINED;
  NOPRT(printf("correct_slice_counts correct_counts..end.\n"));
  return NO_ERROR;
}






/*-----------------------------------------------------------------------
check_analysis_status - ???
-----------------------------------------------------------------------*/

void
check_analysis_status(analysis)
 struct analysis_rec *analysis;
{

 if ((analysis->err == NO_ERROR) &&
     (analysis->cnt_slice == analysis->num_slices))
    {
     if (analysis->total_slices == analysis->num_slices)
        analysis->file->pixel_type.status |= PIXEL_TYPE_ALL_ROIS_COMPUTED;
     analysis->file->pixel_type.status |= PIXEL_TYPE_COMPUTED |
                    PIXEL_TYPE_AND_ANALYSIS_RELATED;
     analysis->file->output.status |= FILE_TO_BE_SAVED;
    }
}


/*-----------------------------------------------------------------------
is_roi_in_analysis_groups - ???
-----------------------------------------------------------------------*/

int
is_roi_in_analysis_groups(groups,num_groups,roi_label)
 struct text_output_analysis_group_rec *groups;
 unsigned int num_groups;
 char *roi_label;
{
 unsigned int lcv;
 int err;

 for (lcv = 0; lcv < num_groups; lcv++)
    if (FindStrInHandleAryByContent(groups[lcv].roi_labels,
         groups[lcv].num_rois,roi_label,&err) == NO_ERROR)
       return 1;
 return 0;
}


/*-----------------------------------------------------------------------
cnt_pixel_types_by_mask_proc - ???
-----------------------------------------------------------------------*/

int
cnt_pixel_types_by_mask_proc(_2d,row,col,pixel_type_cnts,num_pixel_types)
 struct buf_2d_rec *_2d;
 unsigned long row,col;
 unsigned int *pixel_type_cnts,num_pixel_types;
{
 double val;
 unsigned long value;

 val = _2d->GetVal(_2d,row,col);
 value = (unsigned long) rint(val);
 if (value >= num_pixel_types) 
    return NO_ERROR;
 pixel_type_cnts[value] += 1;

 return NO_ERROR;
}


/*-----------------------------------------------------------------------
cnt_pixel_types_by_mask - ???
-----------------------------------------------------------------------*/

int
cnt_pixel_types_by_mask(_2d,mask,affected,pixel_type_cnts,num_pixel_types)
 struct buf_2d_rec *_2d,*mask;
 struct int_bound_rec *affected;
 unsigned int *pixel_type_cnts,num_pixel_types;
{
 unsigned int row;
 int err;

 TRACE(printf("cnt_pixel_types_by_mask...\n"))
 /* printf("cnt_pixel_types_by_mask...num_pixel_types = %d\n",num_pixel_types);fflush(stdout);*/

 for (row = 0; row < num_pixel_types; row++)
    pixel_type_cnts[row] = 0;
 err = process_buf_2d_rec_by_mask(_2d,0,0,mask,affected,
           cnt_pixel_types_by_mask_proc,pixel_type_cnts,num_pixel_types,
           0,0,0,0,0,0,0);
 /* printf("cnt_pixel_types_by_mask...err = %d...end\n",err);fflush(stdout);*/
 return err;
}


/*-----------------------------------------------------------------------
setup_pixel_type_cnt_rec - ???
-----------------------------------------------------------------------*/

int
setup_pixel_type_cnt_rec(ptr,pixel_type)
 struct pixel_type_cnt_rec **ptr;
 struct pixel_type_rec *pixel_type;
{
 unsigned int total;
 int err;

 total = 0;
 get_ptr_memory(ptr,&total,(unsigned int) 1,
     (size_t) sizeof(struct pixel_type_cnt_rec),&err);
 if (err) return err;
 init_pixel_type_cnt_rec(*ptr);

 (*ptr)->status = 0;
 get_ptr_memory(&((*ptr)->pixel_type_cnts),&((*ptr)->num_pixel_type_cnts),
     (unsigned int) pixel_type->order.num_strs + 1,
     (size_t) sizeof(unsigned int),&err);
 if (err)
    {free_ptr(ptr);
     return err;}
 for (total = 0; total < (*ptr)->num_pixel_type_cnts; total++)
    (*ptr)->pixel_type_cnts[total] = 0;
 return NO_ERROR;
}


/*-----------------------------------------------------------------------
unset_slice_counts - ???
-----------------------------------------------------------------------*/

int
unset_slice_counts(slice_roi,format,last_roi_indx)
 struct slice_roi_rec *slice_roi;
 struct output_text_format_rec *format;
 unsigned long last_roi_indx;
{
 unsigned long cnt,roi_indx;
 int err1;
 struct pixel_type_cnt_rec *ptr;

 if (last_roi_indx >= slice_roi->num_rois) return NO_ERROR;
 for (cnt = 0; cnt <= last_roi_indx; cnt++)
    {roi_indx = FindStrInHandleAryByContent(
              format->roi_labels.strs,format->roi_labels.num_strs,
              slice_roi->rois[cnt]->label,&err1);
     if (err1 != NO_ERROR) continue;
     TRACE(printf("roi_nums[roi_indx] = %d roi_indx = %d\n",
              format->roi_nums[roi_indx],roi_indx))
     roi_indx = format->roi_nums[roi_indx];
     if (roi_indx == 0) continue;
     ptr = (struct pixel_type_cnt_rec *)
                 slice_roi->rois[cnt]->user.ptr;
     ptr->status &= ~PIXEL_TYPE_CNTS_COMPUTED;
    }
 return NO_ERROR;
}

