/****************************************************************************
display_init_free.c - Create/destroy NRIA menus ???

Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.


This file depends on XView and the NRIA GUI.
****************************************************************************/

static const char rcsid[] = "$Id: display_init_free.c,v 1.10 1999/07/07 17:41:15 hughett Exp $ Release $Name: nria_1_0_6 $";


#include <xview/xview.h>
#include <xview/cms.h>
#include <xview/canvas.h>
#include <xview/panel.h>
#include <xview/notice.h>
#include <xview/scrollbar.h>
#include <xview/notify.h>
#include <xview/font.h>

#include "mri_format.h"
#include "debug.h"

#include "nria_procedures.h"

/*
 *  DRW 19970811
 *
 *  Took all WIN_CMS attributes out of xv_create functions.  Bug (?) in
 *    XView code: cms not assigned correctly if WIN_CMS is in xv_create.
 *    If WIN_CMS is placed in subsequent xv_set, then cms is assigned
 *    correctly.  Will research when I have time...
 *  NOTE: This seems only to apply when creating frames... ???
 */



/* FIXME!  The next prototype cannot be put in a header file because
   it breaks other code; stick it here as a temporary kluge until
   I have time to fix things properly.  -PWH 19990607 */
extern void slave_win_proc();


/* Functions defined in this file */
void init_display_win_rec(struct display_win_rec *win);
void free_display_win_rec(struct display_win_rec *win);
void init_image_display_window(struct image_display_window *win);
void free_image_display_window(struct image_display_window *win);
int win_error_exit_cur_win(struct image_display_window *win, int err);
int win_done_it_proc(Xv_window win);
static int win_done_proc(Xv_window win);
static Menu setup_co_reg_file_menu(void);
static Menu setup_overlay_menu(void);
static Menu setup_resize_menu(void);
static Menu setup_img_menu(void);
static Menu setup_draw_list_menu(void);
Menu setup_roi_menu(void);
static Menu setup_GA_menu(void);
static Menu setup_TA_menu(void);
static Menu setup_histo_GA_menu(void);
static Menu setup_histo_TA_menu(void);
static Menu setup_pixel_count_anal_menu(void);
void do_TA_proc(Menu menu, Menu_item item, void *arg, Xv_Window window,
    Event *last_event);
void do_GA_proc(Menu menu, Menu_item item, void *arg, Xv_Window window,
    Event *last_event, int sel);
void do_CA_proc(Menu menu, Menu_item item, void *arg, Xv_Window window,
    Event *last_event);
static Menu setup_more_win_menu(void);
static Menu setup_close_win_menu(void);
static Menu setup_open_win_menu(void);
static Menu setup_exit_win_menu(void);
static Menu setup_win_menu(void);
Menu setup_color_pallette_menu(int select, struct pallette_rec *pallette,
    Xv_Window xvwin);
static Menu main_help_menu(void);
Menu setup_3d_cursor_menu(void);
static Menu setup_3d_menu(void);
static Menu setup_file_menu(struct pallette_rec *pallette, Xv_Window xvwin);
static void button_proc(Panel_item item, Event *event);
int SetupDisplayWinRec(struct display_win_rec *win,
    struct image_display_window *owner, Frame ancestor,
    struct display_win_rec *master);



/*------------------------------------------------------------------------
init_display_win_rec - ???
------------------------------------------------------------------------*/

void
init_display_win_rec(win)
     struct display_win_rec *win;
{
  win->owner = NULL;
  win->status2 = 0;
  win->analysis_status = 0;
  win->display_status = 0;
  win->resize_status = 0;
  win->ancestor = 0;
  win->frame = 0;
  win->font = 0;
  win->default_cursor = 0;
  win->panel = win->verti_scroll = win->horiz_scroll = 0;

  setup_int_rect_rec(&(win->label_area),0,0,0,0);
  setup_int_rect_rec(&(win->row_col_area),0,0,0,0);
  setup_int_rect_rec(&(win->right_angle_label_area),0,0,0,0);

  win->master = NULL;
  win->slave = NULL;

  init_screen_data_rec(&(win->screen));
}


/*------------------------------------------------------------------------
free_display_win_rec - ???
------------------------------------------------------------------------*/

void
free_display_win_rec(win)
     struct display_win_rec *win;
{
  if (win != NULL)
  {
    win->owner->dwin = win;
    PRT3(printf("free_display_win_rec...1\n"));

    destroy_all_subframes(win->frame);

    if ((win->owner->type2 & MANY_SLICES_WINDOWS) &&
	(win->select_slice.auto_on == (unsigned long*) win))
      win_auto_arg(win->owner,0);

    if (win->frame != 0)
    {
      if (win->frame_canvas.canvas != (Canvas)NULL)
      {
	xv_destroy(canvas_paint_window(win->frame_canvas.canvas));
	xv_destroy(win->frame_canvas.canvas);
      }
      xv_destroy_safe(win->frame);}
    drop_dwin_from_master_slave_relationship(win);
    c_free(win->frame_label);
    free_screen_data_rec(&(win->screen));
    init_display_win_rec(win);
    PRT3(printf("free_display_win_rec...7\n"));
  }
}



/*------------------------------------------------------------------------
init_image_display_window - ???
------------------------------------------------------------------------*/

void
init_image_display_window(win)
     struct image_display_window *win;
{
 win->status4 = 0;
 win->process_status4 = 0;
 win->type2 = 0;
 win->owner = NULL;
 win->ancestor = 0;
 win->num_display_wins = 0;
 win->dwins = NULL;
 win->dwin = NULL;
 win->intensity = NULL;

 setup_list_rec(&(win->partner),win,NULL,NULL,NULL,NULL);
 setup_list_rec(&(win->link),win,NULL,NULL,NULL,NULL);
 setup_list_rec(&(win->list),win,NULL,NULL,NULL,NULL);
 setup_list_rec(&(win->proc_after_dp),NULL,NULL,NULL,NULL,NULL);
 setup_list_rec(&(win->overlay),NULL,NULL,NULL,NULL,NULL);
 setup_list_rec(&(win->draw_list),NULL,NULL,NULL,NULL,NULL);
 init_draw_setting_rec1(&(win->control),0,0);
 init_roi_rec(&(win->image_view_roi),NULL);
 init_buf_2d_rec(&(win->image_view_2d));
 init_strs_rec(&(win->data_filter_types));
 init_color_screen_rec(&(win->data));

 return /* NO_ERROR */;
}



/*------------------------------------------------------------------------
free_image_display_window - ???
------------------------------------------------------------------------*/

void
free_image_display_window(win)
 struct image_display_window *win;
{
 unsigned int lcv;
 int num_display_wins;

 if (win != NULL)
    {
     delete_item_from_list(&(win->partner));
     delete_item_from_list(&(win->link));
     delete_item_from_list(&(win->list));
     delete_item_from_list(&(win->proc_after_dp));

     win_delete_overlay_list(win,0);
     win_delete_draw_lists(win);

     if (win->num_display_wins > 0)
        {num_display_wins = win->num_display_wins;
         win->num_display_wins = 1;
         for (lcv = 0; lcv < num_display_wins; lcv++)
            {
             if (lcv != 0) win->dwins[0] = win->dwins[lcv];
             PRT3(printf("%d) dwins[0] = %u\n",lcv,win->dwins[0]))
             free_display_win_rec(win->dwins[0]);
             c_free(win->dwins[0]);}
         c_free(win->dwins);}

     win_release_slice_roi_win(win);
     /* FIXME!  Added the two final arguments "0,0" below to match
	the definition of the function, but I suspect there is a
	serious bug in the definition.  -PWH 19990607 */
     free_draw_setting_rec(&(win->control),0,0);
     free_roi_rec(&(win->image_view_roi));
     free_buf_2d_rec(&(win->image_view_2d));
     free_strs_rec(&(win->data_filter_types));
     free_color_screen_rec(&(win->data));
     init_image_display_window(win);
     PRT3(printf("free_image_display_window...7\n"))
    }
}


/*------------------------------------------------------------------------
win_error_exit_cur_win - ???
------------------------------------------------------------------------*/

int
win_error_exit_cur_win(win,err)
 struct image_display_window *win;
 int err;
{
 int result;
 char *str,str1[10],*str2;
 static char *str3 = "ABORTING THE DISPLAY WINDOW !!!";
 Frame *frame;

 if (win->dwin == NULL)
    {
     str2 = win->dwins[0]->frame_label;}
   else
    {
     str2 = win->dwin->frame_label;}
 concat_strs(&str,3,"WHEN CREATING THE ",
                  str2," DISPLAY WINDOW");
 sprintf(str1,"%d",err);
 concat_strs(&str2,2,"AN ERROR OCCURRED...ERROR ID = ",str1);

 frame = (Frame *) main_display(2,NULL,0,0);
 result = notice_prompt(*frame,(Event *) NULL,
            NOTICE_MESSAGE_STRINGS,str2,str,str3,0,
            NOTICE_BUTTON_YES," OK !",
            0);
 if (win->dwin != NULL)
    ShowMsgStrs(*frame,1,NO_ERROR,1,1,
                        str3,delete_dwin,1,win->dwin);

 c_free(str);
 c_free(str2);
 return err;
}


/*------------------------------------------------------------------------
win_done_it_proc - ???
------------------------------------------------------------------------*/

int
win_done_it_proc(win)
 Xv_window win;
{
 struct display_win_rec *win1;
 int err;

 win1 = (struct display_win_rec *) xv_get(win,WIN_CLIENT_DATA);
 PRT3(printf("win_done_it_proc...in..win1 = %s\n",win1->owner->owner->wid))
 err = delete_dwin(win1);
 PRT3(printf("win_done_it_proc...err = %d\n",err))
 return err;
}


/*------------------------------------------------------------------------
win_done_proc - ???
------------------------------------------------------------------------*/

static int
win_done_proc(win)
 Xv_window win;
{
 (void) notify_set_itimer_func((Notify_client) win,
             (Notify_func) win_done_it_proc,ITIMER_REAL,
             &NOTIFY_POLLING_ITIMER,(struct itimerval *) 0);
 return XV_OK;
}


/*------------------------------------------------------------------------
setup_co_reg_file_menu - ???
------------------------------------------------------------------------*/

static Menu 
setup_co_reg_file_menu()
{
 static Menu menu = 0;

 if ((menu == 0) || ((Xv_pkg *) xv_get(menu,XV_TYPE) != MENU))
    {
     menu = xv_create(0,MENU,
        MENU_TITLE_ITEM,CO_REGISTER_FILE,
        MENU_RELEASE,
       MENU_ITEM,
         MENU_STRING,CO_WID_FILE,
         MENU_VALUE,CO_WID_FILE_VALUE,
         MENU_NOTIFY_PROC,co_register_file_proc,0,
       MENU_ITEM,
         MENU_STRING,UNCO_WID_FILE,
         MENU_VALUE,UNCO_WID_FILE_VALUE,
         MENU_NOTIFY_PROC,co_register_file_proc,0,
        MENU_ITEM,
         MENU_STRING,SAVE_CO_FILE,
         MENU_VALUE,SAVE_CO_FILE_VALUE,
         MENU_NOTIFY_PROC,co_register_file_proc,0,
        MENU_ITEM,
         MENU_STRING,LOAD_CO_FILE,
         MENU_VALUE,LOAD_CO_FILE_VALUE,
         MENU_NOTIFY_PROC,co_register_file_proc,0,
        MENU_ITEM,
         MENU_STRING,REVERSE_CO_FILE_PARAMS,
         MENU_VALUE,REVERSE_CO_FILE_PARAMS_VALUE,
         MENU_NOTIFY_PROC,co_register_file_proc,0,
       0);
    }
 return menu;
}



/*------------------------------------------------------------------------ 
setup_overlay_menu - ???
------------------------------------------------------------------------*/

static Menu 
setup_overlay_menu()
{
 static Menu menu = 0;

 if ((menu == 0) || ((Xv_pkg *) xv_get(menu,XV_TYPE) != MENU))
    {
     menu = xv_create(0,MENU,
        MENU_TITLE_ITEM,OVERLAY_IMAGE,
        MENU_RELEASE,
       MENU_ITEM,
         MENU_STRING,OVERLAYED_BY_WID,
         MENU_VALUE,OVERLAYED_BY_WID_VALUE,
         MENU_NOTIFY_PROC,overlay_proc,0,
       MENU_ITEM,
         MENU_STRING,OVERLAYED_BY_WID_STACK,
         MENU_VALUE,OVERLAYED_BY_WID_STACK_VALUE,
         MENU_NOTIFY_PROC,overlay_proc,0,
       MENU_ITEM,
         MENU_STRING,OVERLAYED_BY_ALL_ECHO_WID,
         MENU_VALUE,OVERLAYED_BY_ALL_ECHO_WID_VALUE,
         MENU_NOTIFY_PROC,overlay_proc,0,
       MENU_ITEM,
         MENU_STRING,OVERLAYED_BY_ALL_ECHO_WID_STACK,
         MENU_VALUE,OVERLAYED_BY_ALL_ECHO_WID_STACK_VALUE,
         MENU_NOTIFY_PROC,overlay_proc,0,
        MENU_ITEM,
         MENU_STRING,DELETE_OVERLAY,
         MENU_VALUE,DELETE_OVERLAY_VALUE,
         MENU_NOTIFY_PROC,overlay_proc,0,
        MENU_ITEM,
         MENU_STRING,DELETE_OVERLAY_STACK,
         MENU_VALUE,DELETE_OVERLAY_STACK_VALUE,
         MENU_NOTIFY_PROC,overlay_proc,0,
        MENU_ITEM,
         MENU_STRING,SHOW_HIDE_OVERLAY,
         MENU_VALUE,SHOW_HIDE_OVERLAY_VALUE,
         MENU_NOTIFY_PROC,overlay_proc,0,
        MENU_ITEM,
         MENU_STRING,SHOW_HIDE_OVERLAY_STACK,
         MENU_VALUE,SHOW_HIDE_OVERLAY_STACK_VALUE,
         MENU_NOTIFY_PROC,overlay_proc,0,
       0);
    }
 return menu;
}


/*------------------------------------------------------------------------
setup_resize_menu - ???
------------------------------------------------------------------------*/

static Menu 
setup_resize_menu()
{
 static Menu menu = 0;

 if ((menu == 0) || ((Xv_pkg *) xv_get(menu,XV_TYPE) != MENU))
    {
     menu = xv_create(0,MENU,
        MENU_TITLE_ITEM,RESIZE_IMAGE,
        MENU_RELEASE,
       MENU_ITEM,
         MENU_STRING,RESIZE_TO_WID_SIZE,
         MENU_VALUE,RESIZE_TO_WID_SIZE_VALUE,
         MENU_NOTIFY_PROC,resize_proc,0,
       MENU_ITEM,
         MENU_STRING,RESIZE_DEFAULT,
         MENU_VALUE,RESIZE_DEFAULT_VALUE,
         MENU_NOTIFY_PROC,resize_proc,0,
       MENU_ITEM,
         MENU_STRING,RESIZE_DEFAULT_SIZE,
         MENU_VALUE,RESIZE_DEFAULT_SIZE_VALUE,
         MENU_NOTIFY_PROC,resize_proc,0,
       MENU_ITEM,
         MENU_STRING,RESIZE_LAST,
         MENU_VALUE,RESIZE_LAST_VALUE,
         MENU_NOTIFY_PROC,resize_proc,0,
       MENU_ITEM,
         MENU_STRING,RESIZE_ZOOM_IN,
         MENU_VALUE,RESIZE_ZOOM_IN_VALUE,
         MENU_NOTIFY_PROC,resize_proc,0,
       MENU_ITEM,
         MENU_STRING,RESIZE_ZOOM_OUT,
         MENU_VALUE,RESIZE_ZOOM_OUT_VALUE,
         MENU_NOTIFY_PROC,resize_proc,0,
       0);
    }
 return menu;
}


/*------------------------------------------------------------------------
setup_img_menu - ???
------------------------------------------------------------------------*/

static Menu 
setup_img_menu()
{
 static Menu menu = 0;

 if ((menu == 0) || ((Xv_pkg *) xv_get(menu,XV_TYPE) != MENU))
    menu = (Menu) xv_create(0,MENU,
         MENU_RELEASE,
         MENU_TITLE_ITEM,IMAGE_MENU,
         MENU_ITEM,
            MENU_STRING,SLICE_SELECTION_MENU,
            MENU_NOTIFY_PROC,select_slice_proc,0,
        MENU_ITEM,
           MENU_STRING,IMAGE_CONTRAST,
           MENU_NOTIFY_PROC,image_contrast_proc,0,
        MENU_ITEM,MENU_STRING,PIXEL_SELECTION,
           MENU_NOTIFY_PROC,pixel_selection_proc,0,
        MENU_ITEM,MENU_STRING,GROW_REGION,
           MENU_NOTIFY_PROC,pt_to_region_proc,0,
        MENU_ITEM,
         MENU_STRING,IMAGE_VIEW_FILTER,
         MENU_NOTIFY_PROC,image_view_filter_proc,0,
        MENU_ITEM,
         MENU_STRING,_3D,
         MENU_VALUE,0,
         MENU_NOTIFY_PROC,win_3d_proc,0,
        MENU_ITEM,
         MENU_STRING,_3D_CLOSEST_VALUE,
         MENU_VALUE,1,
         MENU_NOTIFY_PROC,win_3d_proc,0,
       MENU_ITEM,
         MENU_STRING,MODIFY_IMAGE,
         MENU_NOTIFY_PROC,modify_image_proc,0,
       MENU_ITEM,
         MENU_STRING,REVERSE_IMAGES,
         MENU_NOTIFY_PROC,reverse_images_proc,0,
       MENU_ITEM,
         MENU_STRING,RESIZE_IMAGE,
         MENU_NOTIFY_PROC,resize_image_proc,
          MENU_PULLRIGHT,setup_resize_menu(),0,
        MENU_ITEM,
           MENU_STRING,IMAGE_FUSION_MENU,
           MENU_NOTIFY_PROC,fusion_proc,0,
        MENU_ITEM,
           MENU_STRING,CO_REGISTER_FILE,
           MENU_NOTIFY_PROC,enter_co_register_file_proc,
           MENU_PULLRIGHT,setup_co_reg_file_menu(),0,
        MENU_ITEM,
           MENU_STRING,OVERLAY_IMAGE,
          MENU_PULLRIGHT,setup_overlay_menu(),0,
        MENU_ITEM,
         MENU_STRING,IMAGE_ENHANCEMENT,
         MENU_NOTIFY_PROC,image_enhancement_proc,0,
        MENU_ITEM,
         MENU_STRING,EDGE_DETECTION,
         MENU_NOTIFY_PROC,edge_detection_proc,0,
/*
        MENU_ITEM,
         MENU_STRING,POLARITY,
         MENU_NOTIFY_PROC,polarity_proc,0,
*/
        MENU_ITEM,
         MENU_STRING,PRINT_OR_SAVE_IMAGE,
         MENU_VALUE,VALUE_SAVE_CURRENT_WIN_IMAGES,
         MENU_NOTIFY_PROC,print_or_save_image_proc,
         MENU_PULLRIGHT,setup_print_or_save_image_menu(),0,
        MENU_ITEM,
             MENU_STRING,MENU_SEPARATOR,0,
         MENU_ITEM,
             MENU_STRING,PROCESS_SLICE_SCANS_STR,
             MENU_NOTIFY_PROC,process_slice_scans_proc,0,
         0);
 return menu;
}


/*------------------------------------------------------------------------
setup_draw_list_menu - ???
------------------------------------------------------------------------*/

static Menu 
setup_draw_list_menu()
{
 static Menu menu = 0;

 if ((menu == 0) || ((Xv_pkg *) xv_get(menu,XV_TYPE) != MENU))
    {
     menu = xv_create(0,MENU,
        MENU_TITLE_ITEM,DRAW_LINK,
        MENU_RELEASE,
       MENU_ITEM,
         MENU_STRING,DRAW_LINK_WITH_WID,
         MENU_VALUE,0,
         MENU_NOTIFY_PROC,draw_link_with_wid_proc,0,
        MENU_ITEM,
         MENU_STRING,DELETE_DRAW_LINK_WITH_WID,
         MENU_VALUE,1,
         MENU_NOTIFY_PROC,draw_link_with_wid_proc,0,
       0);
    }
 return menu;
}


/*------------------------------------------------------------------------
setup_roi_menu - ???
------------------------------------------------------------------------*/

Menu 
setup_roi_menu()
{
 static Menu menu = 0;

 if ((menu == 0) || ((Xv_pkg *) xv_get(menu,XV_TYPE) != MENU))
    menu = (Menu) xv_create(0,MENU,
         MENU_TITLE_ITEM,ROI_MENU,
         MENU_ITEM,
             MENU_STRING,PEN_DISABLED,
             MENU_NOTIFY_PROC,activate_pen_proc,
             MENU_GEN_PROC,gen_pen_type_proc,
             0,
         MENU_ITEM,
             MENU_STRING,LABEL_ROI_AND_CURVE,
             MENU_NOTIFY_PROC,label_roi_proc,0,
         MENU_ITEM,
             MENU_STRING,SHOW_ROI,
             MENU_GEN_PROC,gen_show_roi_proc1,0,
         MENU_ITEM,
             MENU_STRING,HIGHLIGHT_NEXT_CURVE,
             MENU_NOTIFY_PROC,highlight_next_curve_proc,0,
         MENU_ITEM,
             MENU_STRING,HIGHLIGHT_NEXT_ROI,
             MENU_NOTIFY_PROC,highlight_next_roi_proc,0,
         MENU_ITEM,
             MENU_STRING,EDIT_NEXT_NEW_CURVE,
             MENU_NOTIFY_PROC,edit_next_empty_curve_proc,0,
         MENU_ITEM,
             MENU_STRING,EDIT_SCRATCH,
             MENU_NOTIFY_PROC,edit_scratch_proc,0,
         MENU_ITEM,
             MENU_STRING,ALL_WINS_CUR_ROI,
             MENU_NOTIFY_PROC,all_wins_cur_roi_proc,0,
         MENU_ITEM,
             MENU_STRING,SLICE_RECT,
             MENU_NOTIFY_PROC,slice_rect_proc,0,
         MENU_ITEM,
             MENU_STRING,DELETE_ROI,
             MENU_NOTIFY_PROC,dp_delete_roi_proc,
             MENU_PULLRIGHT,setup_delete_roi_menu(),0,

         MENU_ITEM,
             MENU_STRING,TRANS_ROI,
             MENU_NOTIFY_PROC,dp_trans_roi_proc,0,

         MENU_ITEM,
             MENU_STRING,DRAW_LINK,
             MENU_NOTIFY_PROC,dp_draw_list_proc,
             MENU_PULLRIGHT,setup_draw_list_menu(),0,
         MENU_ITEM,
             MENU_STRING,EDIT_ROI_BOUND,
             MENU_NOTIFY_PROC,dp_more_rois_proc,0,
         MENU_ITEM,
             MENU_STRING,ROI_OPTIONS,
             MENU_GEN_PROC,gen_more_roi_proc,0,
         MENU_ITEM,
             MENU_STRING,REVERSE_SLICE_ROIS,
             MENU_NOTIFY_PROC,reverse_slice_rois_proc,0,
         MENU_ITEM,
             MENU_STRING,IMPORT_SLICE_ROIS,
             MENU_NOTIFY_PROC,import_slice_rois_proc,0,
        MENU_ITEM,
             MENU_STRING,BREAKER,
             MENU_NOTIFY_PROC,breaker_options_proc,
             MENU_PULLRIGHT,setup_breaker_menu(),0,
         MENU_ITEM,
             MENU_STRING,CLIPBOARD,
             MENU_NOTIFY_PROC,clipboard_proc,
             MENU_GEN_PROC,gen_clipboard_proc,0,
         0);
 return menu;
}


/*
Menu setup_2_echo_anal_menu()

         MENU_TITLE_ITEM,PIXEL_TYPE_2_ECHO_ANALYSIS_STR,
*/


/*------------------------------------------------------------------------
setup_GA_menu - ???
------------------------------------------------------------------------*/

static Menu 
setup_GA_menu()
{
 static Menu menu = 0;

 if ((menu == 0) || ((Xv_pkg *) xv_get(menu,XV_TYPE) != MENU))
    menu = (Menu) xv_create(0,MENU,
         MENU_TITLE_ITEM,PIXEL_TYPE_GA_STR,
         MENU_ITEM,
             MENU_STRING,BKG_PIXELS,
             MENU_NOTIFY_PROC,bkg_pixels_proc,0,
         MENU_ITEM,
             MENU_STRING,HISTOGRAM_ROI,
             MENU_NOTIFY_PROC,show_histogram_menu,
             MENU_PULLRIGHT,setup_histogram_menu(),0,
         MENU_ITEM,
             MENU_STRING,HISTOGRAM_OPTIONS,
             MENU_NOTIFY_PROC,histogram_options_proc,0,
         MENU_ITEM,
             MENU_STRING,SEGMENT_SLICE,
             MENU_VALUE,0,
             MENU_NOTIFY_PROC,segment_cur_slice_proc,0,
         MENU_ITEM,
             MENU_STRING,"SEGMENT CUR SLICE NEW",
             MENU_VALUE,1,
             MENU_NOTIFY_PROC,segment_cur_slice_proc,0,
         MENU_ITEM,
             MENU_STRING,SEGMENT_PIXEL_TYPES,
             MENU_NOTIFY_PROC,segment_pixel_type_proc,0,
         MENU_ITEM,
             MENU_STRING,GROW_REGION_OPTIONS,
             MENU_NOTIFY_PROC,dp_grow_region_params_proc,0,
         MENU_ITEM,
             MENU_STRING,SHOW_PIXEL_TYPE_ROI,
             MENU_GEN_PROC,gen_show_pixel_type_roi_proc,0,
        MENU_ITEM,
             MENU_STRING,ANALYSIS_STR,
             MENU_GEN_PROC,gen_analysis_proc,
             MENU_NOTIFY_PROC,analyze_all_slices_proc,0,
         MENU_ITEM,
             MENU_STRING,OUTPUT_ANALYSIS,
             MENU_GEN_PROC,gen_output_text_proc,0,
         MENU_ITEM,
             MENU_STRING,MENU_SEPARATOR,0,
         MENU_ITEM,
             MENU_STRING,PIXEL_TYPES_TO_HISTO_ROIS,
             MENU_NOTIFY_PROC,pixel_types_to_histo_rois_proc,
             MENU_PULLRIGHT,pixel_types_to_histo_rois_menu(),0,
         MENU_ITEM,
             MENU_STRING,CUR_ROI_TO_HISTO_ROI,
             MENU_GEN_PROC,gen_cur_roi_to_histo_roi_proc,0,
         MENU_ITEM,
             MENU_STRING,HISTO_ROI_TO_SLICE_ROI,
             MENU_NOTIFY_PROC,histo_roi_to_slice_roi_proc,0,
         MENU_ITEM,
             MENU_STRING,IMPORT_PIXEL_TYPE_MAP,
             MENU_NOTIFY_PROC,import_pixel_type_map_proc,
             0,
         MENU_ITEM,
             MENU_STRING,PIXEL_TYPE_FORMAT,
             MENU_NOTIFY_PROC,output_text_format_proc,0,
         MENU_ITEM,
             MENU_STRING,REDEFINE_PIXEL_TYPE_ORDER,
             MENU_NOTIFY_PROC,redefine_pixel_type_order_proc,0,
         MENU_ITEM,
             MENU_STRING,OUTPUT_BIN,
             MENU_NOTIFY_PROC,wr_bin_proc,0,
         MENU_ITEM,
             MENU_STRING,ANALYSIS_MENU_HELP,
             MENU_NOTIFY_PROC,analysis_menu_help_proc,0,
         MENU_ITEM,
             MENU_STRING,PIXEL_TYPE_TA_STR,
             MENU_NOTIFY_PROC,do_TA_proc,0,
         MENU_ITEM,
             MENU_STRING,PIXEL_TYPE_CA_STR,
             MENU_NOTIFY_PROC,do_CA_proc,0,
         0);
 return menu;
}


/*------------------------------------------------------------------------
setup_TA_menu - ???
------------------------------------------------------------------------*/

static Menu 
setup_TA_menu()
{
 static Menu menu = 0;

 if ((menu == 0) || ((Xv_pkg *) xv_get(menu,XV_TYPE) != MENU))
    menu = (Menu) xv_create(0,MENU,
         MENU_TITLE_ITEM,PIXEL_TYPE_TA_STR,
         MENU_ITEM,
             MENU_STRING,BKG_PIXELS,
             MENU_NOTIFY_PROC,bkg_pixels_proc,0,
         MENU_ITEM,
             MENU_STRING,HISTOGRAM_ROI,
             MENU_NOTIFY_PROC,show_histogram_menu,
             MENU_PULLRIGHT,setup_histogram_menu(),0,
         MENU_ITEM,
             MENU_STRING,HISTOGRAM_OPTIONS,
             MENU_NOTIFY_PROC,histogram_options_proc,0,
         MENU_ITEM,
             MENU_STRING,PARTITION_PIXEL_TYPES,
             MENU_NOTIFY_PROC,segment_pixel_type_proc,0,
         MENU_ITEM,
             MENU_STRING,SHOW_PIXEL_TYPE_ROI,
             MENU_GEN_PROC,gen_show_pixel_type_roi_proc,0,
        MENU_ITEM,
             MENU_STRING,ANALYSIS_STR,
             MENU_GEN_PROC,gen_analysis_proc,
             MENU_NOTIFY_PROC,analyze_all_slices_proc,0,
         MENU_ITEM,
             MENU_STRING,OUTPUT_ANALYSIS,
             MENU_GEN_PROC,gen_output_text_proc,0,
         MENU_ITEM,
             MENU_STRING,MENU_SEPARATOR,0,
         MENU_ITEM,
             MENU_STRING,CUR_ROI_TO_HISTO_ROI,
             MENU_GEN_PROC,gen_cur_roi_to_histo_roi_proc,0,
         MENU_ITEM,
             MENU_STRING,HISTO_ROI_TO_SLICE_ROI,
             MENU_NOTIFY_PROC,histo_roi_to_slice_roi_proc,0,
         MENU_ITEM,
             MENU_STRING,IMPORT_PIXEL_TYPE_MAP,
             MENU_NOTIFY_PROC,import_pixel_type_map_proc,
             0,
         MENU_ITEM,
             MENU_STRING,PIXEL_TYPE_FORMAT,
             MENU_NOTIFY_PROC,output_text_format_proc,0,
         MENU_ITEM,
             MENU_STRING,REDEFINE_PIXEL_TYPE_ORDER,
             MENU_NOTIFY_PROC,redefine_pixel_type_order_proc,0,
         MENU_ITEM,
             MENU_STRING,OUTPUT_BIN,
             MENU_NOTIFY_PROC,wr_bin_proc,0,
         MENU_ITEM,
             MENU_STRING,ANALYSIS_MENU_HELP,
             MENU_NOTIFY_PROC,analysis_menu_help_proc,0,
         MENU_ITEM,
             MENU_STRING,PIXEL_TYPE_GA_STR,
             MENU_NOTIFY_PROC,do_GA_proc,0,
         MENU_ITEM,
             MENU_STRING,PIXEL_TYPE_CA_STR,
             MENU_NOTIFY_PROC,do_CA_proc,0,
         0);
 return menu;
}


/*------------------------------------------------------------------------
setup_histo_GA_menu - ???
------------------------------------------------------------------------*/

static Menu 
setup_histo_GA_menu()
{
 static Menu menu = 0;

 if ((menu == 0) || ((Xv_pkg *) xv_get(menu,XV_TYPE) != MENU))
    menu = (Menu) xv_create(0,MENU,
         MENU_TITLE_ITEM,PIXEL_TYPE_GA_STR,
         MENU_ITEM,
             MENU_STRING,BKG_PIXELS,
             MENU_NOTIFY_PROC,bkg_pixels_proc,0,
         MENU_ITEM,
             MENU_STRING,SEGMENT_SLICE,
             MENU_VALUE,0,
             MENU_NOTIFY_PROC,segment_cur_slice_proc,0,
         MENU_ITEM,
             MENU_STRING,"SEGMENT CUR SLICE NEW",
             MENU_VALUE,1,
             MENU_NOTIFY_PROC,segment_cur_slice_proc,0,
         MENU_ITEM,
             MENU_STRING,SEGMENT_PIXEL_TYPES,
             MENU_NOTIFY_PROC,segment_pixel_type_proc,
             MENU_PULLRIGHT,setup_pixel_type_menu(),0,
         MENU_ITEM,
             MENU_STRING,GROW_REGION_OPTIONS,
             MENU_NOTIFY_PROC,dp_grow_region_params_proc,0,
        MENU_ITEM,
             MENU_STRING,ANALYSIS_STR,
             MENU_GEN_PROC,gen_analysis_proc,
             MENU_NOTIFY_PROC,analyze_all_slices_proc,0,
         MENU_ITEM,
             MENU_STRING,OUTPUT_ANALYSIS,
             MENU_GEN_PROC,gen_output_text_proc,0,
         MENU_ITEM,
             MENU_STRING,MENU_SEPARATOR,0,
          MENU_ITEM,
             MENU_STRING,PIXEL_TYPES_TO_HISTO_ROIS,
             MENU_NOTIFY_PROC,pixel_types_to_histo_rois_proc,
             MENU_PULLRIGHT,pixel_types_to_histo_rois_menu(),0,
        MENU_ITEM,
             MENU_STRING,CUR_ROI_TO_HISTO_ROI,
             MENU_GEN_PROC,gen_cur_roi_to_histo_roi_proc,0,
         MENU_ITEM,
             MENU_STRING,HISTO_ROI_TO_SLICE_ROI,
             MENU_NOTIFY_PROC,histo_roi_to_slice_roi_proc,0,
         MENU_ITEM,
             MENU_STRING,IMPORT_PIXEL_TYPE_MAP,
             MENU_NOTIFY_PROC,import_pixel_type_map_proc,
             0,
         MENU_ITEM,
             MENU_STRING,PIXEL_TYPE_FORMAT,
             MENU_NOTIFY_PROC,output_text_format_proc,0,
         MENU_ITEM,
             MENU_STRING,REDEFINE_PIXEL_TYPE_ORDER,
             MENU_NOTIFY_PROC,redefine_pixel_type_order_proc,0,
         MENU_ITEM,
             MENU_STRING,OUTPUT_BIN,
             MENU_NOTIFY_PROC,wr_bin_proc,0,
         MENU_ITEM,
             MENU_STRING,ANALYSIS_MENU_HELP,
             MENU_NOTIFY_PROC,analysis_menu_help_proc,0,
         MENU_ITEM,
             MENU_STRING,PIXEL_TYPE_TA_STR,
             MENU_NOTIFY_PROC,do_TA_proc,0,
         MENU_ITEM,
             MENU_STRING,PIXEL_TYPE_CA_STR,
             MENU_NOTIFY_PROC,do_CA_proc,0,
         0);
 return menu;
}


/*------------------------------------------------------------------------
setup_histo_TA_menu - ???
------------------------------------------------------------------------*/

static Menu 
setup_histo_TA_menu()
{
 static Menu menu = 0;

 if ((menu == 0) || ((Xv_pkg *) xv_get(menu,XV_TYPE) != MENU))
    menu = (Menu) xv_create(0,MENU,
         MENU_TITLE_ITEM,PIXEL_TYPE_TA_STR,
         MENU_ITEM,
             MENU_STRING,BKG_PIXELS,
             MENU_NOTIFY_PROC,bkg_pixels_proc,0,
         MENU_ITEM,
             MENU_STRING,PARTITION_PIXEL_TYPES,
             MENU_NOTIFY_PROC,segment_pixel_type_proc,
             MENU_PULLRIGHT,setup_pixel_type_menu(),0,
        MENU_ITEM,
             MENU_STRING,ANALYSIS_STR,
             MENU_GEN_PROC,gen_analysis_proc,
             MENU_NOTIFY_PROC,analyze_all_slices_proc,0,
         MENU_ITEM,
             MENU_STRING,OUTPUT_ANALYSIS,
             MENU_GEN_PROC,gen_output_text_proc,0,
         MENU_ITEM,
             MENU_STRING,MENU_SEPARATOR,0,
         MENU_ITEM,
             MENU_STRING,CUR_ROI_TO_HISTO_ROI,
             MENU_GEN_PROC,gen_cur_roi_to_histo_roi_proc,0,
         MENU_ITEM,
             MENU_STRING,HISTO_ROI_TO_SLICE_ROI,
             MENU_NOTIFY_PROC,histo_roi_to_slice_roi_proc,0,
         MENU_ITEM,
             MENU_STRING,IMPORT_PIXEL_TYPE_MAP,
             MENU_NOTIFY_PROC,import_pixel_type_map_proc,
             0,
         MENU_ITEM,
             MENU_STRING,PIXEL_TYPE_FORMAT,
             MENU_NOTIFY_PROC,output_text_format_proc,0,
         MENU_ITEM,
             MENU_STRING,REDEFINE_PIXEL_TYPE_ORDER,
             MENU_NOTIFY_PROC,redefine_pixel_type_order_proc,0,
         MENU_ITEM,
             MENU_STRING,OUTPUT_BIN,
             MENU_NOTIFY_PROC,wr_bin_proc,0,
         MENU_ITEM,
             MENU_STRING,ANALYSIS_MENU_HELP,
             MENU_NOTIFY_PROC,analysis_menu_help_proc,0,
         MENU_ITEM,
             MENU_STRING,PIXEL_TYPE_GA_STR,
             MENU_NOTIFY_PROC,do_GA_proc,0,
         MENU_ITEM,
             MENU_STRING,PIXEL_TYPE_CA_STR,
             MENU_NOTIFY_PROC,do_CA_proc,0,
         0);
 return menu;
}


/*------------------------------------------------------------------------
setup_pixel_count_anal_menu - ???
------------------------------------------------------------------------*/

static Menu 
setup_pixel_count_anal_menu()
{
 static Menu menu = 0;

 if ((menu == 0) || ((Xv_pkg *) xv_get(menu,XV_TYPE) != MENU))
    menu = (Menu) xv_create(0,MENU,
         MENU_TITLE_ITEM,PIXEL_TYPE_CA_STR,
         MENU_ITEM,
             MENU_STRING,CA_OPTIONS_STR,
             MENU_NOTIFY_PROC,ca_options_proc,0,
         MENU_ITEM,
             MENU_STRING,ANALYSIS_STR,
             MENU_GEN_PROC,gen_analysis_proc,
             MENU_NOTIFY_PROC,analyze_all_slices_proc,0,
         MENU_ITEM,
             MENU_STRING,OUTPUT_ANALYSIS,
             MENU_GEN_PROC,gen_output_text_proc,0,
         MENU_ITEM,
             MENU_STRING,MENU_SEPARATOR,0,
         MENU_ITEM,
             MENU_STRING,HISTOGRAM_ROI,
             MENU_NOTIFY_PROC,show_histogram_menu,
             MENU_PULLRIGHT,setup_histogram_menu(),0,
         MENU_ITEM,
             MENU_STRING,HISTOGRAM_OPTIONS,
             MENU_NOTIFY_PROC,histogram_options_proc,0,
         MENU_ITEM,
             MENU_STRING,CUR_ROI_TO_HISTO_ROI,
             MENU_NOTIFY_PROC,cur_roi_to_histo_roi_proc,0,
         MENU_ITEM,
             MENU_STRING,HISTO_ROI_TO_SLICE_ROI,
             MENU_NOTIFY_PROC,histo_roi_to_slice_roi_proc,0,
         MENU_ITEM,
             MENU_STRING,COUNT_FORMAT,
             MENU_NOTIFY_PROC,output_text_format_proc,0,
         MENU_ITEM,
             MENU_STRING,PIXEL_TYPE_TA_STR,
             MENU_NOTIFY_PROC,do_TA_proc,0,
         MENU_ITEM,
             MENU_STRING,PIXEL_TYPE_GA_STR,
             MENU_NOTIFY_PROC,do_GA_proc,0,
         0);
 return menu;
}


/*------------------------------------------------------------------------
do_TA_proc - ???
------------------------------------------------------------------------*/

void
do_TA_proc(menu,item,arg,window,last_event)
 Menu menu;
 Menu_item item;
 void *arg;
 Xv_Window window;
 Event *last_event;
{
 struct display_win_rec *win1;
 Menu menu1;
 static Event event;
 Rect *rr;
 struct img_buf_rec *img;
 Menu root;

 PRT2(printf("do_TA_proc...\n"))
 if (menu == 0)
    {win1 = (struct display_win_rec *) arg;}
   else
    {root = (Menu) root_menu(menu);
     if ((last_event = (Event *) xv_get(root,MENU_FIRST_EVENT)) != NULL)
        {window = (Xv_Window) event_window(last_event);
         win1 = (struct display_win_rec *) xv_get(window,WIN_CLIENT_DATA);
        }
       else
        {win1 = (struct display_win_rec *) xv_get(root,MENU_CLIENT_DATA);
         window = win1->panel;}
    }
 if (win1->owner->type2 & HISTOGRAM_WINDOW)
    menu1 = setup_histo_TA_menu();
   else menu1 = setup_TA_menu();
 if ((img = win_rec_cur_img_buf(win1->owner->owner)) == NULL) 
    return /* CANCEL   -PWH removed this on 19980529 */;
/* img->file->pixel_type.status &= ~PIXEL_TYPE_GROW_REGION;*/
 img->file->status &= ~DEF_ANAL_MENU;
 img->file->status |= DEF_TA_MENU;
 xv_set(win1->anal_button,PANEL_ITEM_MENU,menu1,0);
 win1->analysis_status &= ~ALL_PIXEL_ANALYSIS;
 win1->analysis_status |= PIXEL_TYPE_PARTITION_ANALYSIS;
 xv_set(win1->anal_button,PANEL_LABEL_STRING,TA_STR,0);
 rr = (Rect *) xv_get(win1->anal_button,XV_RECT);
 xv_set(win1->panel,WIN_MOUSE_XY,rr->r_left + rr->r_width / 2,
            rr->r_top + rr->r_height / 2,0);
 event = *last_event;
 notify_post_event((Notify_client) win1->panel,
        (Notify_event) &event,NOTIFY_SAFE);
}



/*------------------------------------------------------------------------
do_GA_proc - ???
------------------------------------------------------------------------*/

void
do_GA_proc(menu,item,arg,window,last_event,sel)
 Menu menu;
 Menu_item item;
 void *arg;
 Xv_Window window;
 Event *last_event;
 int sel;
{
 struct display_win_rec *win1;
 Menu menu1;
 static Event event;
 Rect *rr;
 struct img_buf_rec *img;
 Menu root;

 PRT2(printf("do_GA_proc...\n"))
 if (menu == 0)
    {win1 = (struct display_win_rec *) arg;}
   else
    {root = (Menu) root_menu(menu);
     if ((last_event = (Event *) xv_get(root,MENU_FIRST_EVENT)) != NULL)
        {window = (Xv_Window) event_window(last_event);
         win1 = (struct display_win_rec *) xv_get(window,WIN_CLIENT_DATA);
        }
       else
        {win1 = (struct display_win_rec *) xv_get(root,MENU_CLIENT_DATA);
         window = win1->panel;}
    }
 win_def_grow_region_params(win1->owner);

 if (win1->owner->type2 & HISTOGRAM_WINDOW)
    menu1 = setup_histo_GA_menu();
   else menu1 = setup_GA_menu();
 if ((img = win_rec_cur_img_buf(win1->owner->owner)) == NULL)
     return /* CANCEL -PWH removed this 19980529 */;
 img->file->status &= ~DEF_ANAL_MENU;
 img->file->status |= DEF_GA_MENU;

 xv_set(win1->anal_button,PANEL_ITEM_MENU,menu1,0);
 win1->analysis_status &= ~ALL_PIXEL_ANALYSIS;
 win1->analysis_status |= PIXEL_TYPE_GROW_REGION_ANALYSIS;
 xv_set(win1->anal_button,PANEL_LABEL_STRING,GA_STR,0);
 rr = (Rect *) xv_get(win1->anal_button,XV_RECT);
 xv_set(win1->panel,WIN_MOUSE_XY,rr->r_left + rr->r_width / 2,
            rr->r_top + rr->r_height / 2,0);
 event = *last_event;
 if (sel)
    notify_post_event((Notify_client) win1->panel,
               (Notify_event) &event,NOTIFY_SAFE);
}


/*------------------------------------------------------------------------
do_CA_proc - ???
------------------------------------------------------------------------*/

void
do_CA_proc(menu,item,arg,window,last_event)
 Menu menu;
 Menu_item item;
 void *arg;
 Xv_Window window;
 Event *last_event;
{
 struct display_win_rec *win1;
 Menu menu1;
 static Event event;
 Rect *rr;
 struct img_buf_rec *img;
 Menu root;

 PRT2(printf("do_CA_proc...\n"))
 if (menu == 0)
    {win1 = (struct display_win_rec *) arg;}
   else
    {root = (Menu) root_menu(menu);
     if ((last_event = (Event *) xv_get(root,MENU_FIRST_EVENT)) != NULL)
        {window = (Xv_Window) event_window(last_event);
         win1 = (struct display_win_rec *) xv_get(window,WIN_CLIENT_DATA);
        }
       else
        {win1 = (struct display_win_rec *) xv_get(root,MENU_CLIENT_DATA);
         window = win1->panel;}
    }
/*
 win1 = (struct display_win_rec *) xv_get(menu,MENU_CLIENT_DATA);
 if ((last_event = (Event *) xv_get(menu,MENU_FIRST_EVENT)) != NULL)
    window = (Xv_Window) event_window(last_event);
   else window = win1->panel;
*/
 menu1 = setup_pixel_count_anal_menu();
 if ((img = win_rec_cur_img_buf(win1->owner->owner)) == NULL)
    return /* CANCEL -PWH removed this 19980529 */ ;
 img->file->status &= ~DEF_ANAL_MENU;
 img->file->status |= DEF_CA_MENU;

 xv_set(win1->anal_button,PANEL_ITEM_MENU,menu1,0);
 win1->analysis_status &= ~ALL_PIXEL_ANALYSIS;
 win1->analysis_status |= PIXEL_COUNT_VALUE_ANALYSIS;
 xv_set(win1->anal_button,PANEL_LABEL_STRING,CA_STR,0);
 rr = (Rect *) xv_get(win1->anal_button,XV_RECT);
 xv_set(win1->panel,WIN_MOUSE_XY,rr->r_left + rr->r_width / 2,
            rr->r_top + rr->r_height / 2,0);
 event = *last_event;
 notify_post_event((Notify_client) win1->panel,
         (Notify_event) &event,NOTIFY_SAFE);
}


/*------------------------------------------------------------------------
setup_more_win_rec - ???
------------------------------------------------------------------------*/

static Menu
setup_more_win_menu()
{
 static Menu menu = 0;

 if ((menu == 0) || ((Xv_pkg *) xv_get(menu,XV_TYPE) != MENU))
    menu = (Menu) xv_create(0,MENU,
       MENU_ITEM,
         MENU_STRING,ALL_ECHO_WINS,
         MENU_NOTIFY_PROC,all_echo_wins_proc,0,
       MENU_ITEM,
         MENU_STRING,DUPLICATE_WINDOW,
         MENU_NOTIFY_PROC,duplicate_win_proc,0,
       MENU_ITEM,
         MENU_STRING,SLAVE_WINDOW,
         MENU_NOTIFY_PROC,slave_win_proc,0,
        MENU_ITEM,
         MENU_STRING,GET_4_WINS,
         MENU_CLIENT_DATA,4,
         MENU_NOTIFY_PROC,project_1_echoe_wins_proc,0,
        MENU_ITEM,
         MENU_STRING,GET_4_WINS_ALL_ECHOES,
         MENU_CLIENT_DATA,4,
         MENU_NOTIFY_PROC,project_all_echoe_wins_proc,0,
        MENU_ITEM,
         MENU_STRING,GET_9_WINS,
         MENU_CLIENT_DATA,9,
         MENU_NOTIFY_PROC,project_1_echoe_wins_proc,0,
        MENU_ITEM,
         MENU_STRING,GET_9_WINS_ALL_ECHOES,
         MENU_CLIENT_DATA,9,
         MENU_NOTIFY_PROC,project_all_echoe_wins_proc,0,
        MENU_ITEM,
         MENU_STRING,GET_12_WINS,
         MENU_CLIENT_DATA,12,
         MENU_NOTIFY_PROC,project_1_echoe_wins_proc,0,
        MENU_ITEM,
         MENU_STRING,GET_12_WINS_ALL_ECHOES,
         MENU_CLIENT_DATA,12,
         MENU_NOTIFY_PROC,project_all_echoe_wins_proc,0,
        MENU_ITEM,
         MENU_STRING,STACK_SLICE_OR_SCAN,
         MENU_CLIENT_DATA,12,
         MENU_NOTIFY_PROC,win_stack_slice_or_scan_proc,0,
         0);
 return menu;
}


/*------------------------------------------------------------------------
setup_close_win_menu - ???
------------------------------------------------------------------------*/

static Menu
setup_close_win_menu()
{
 static Menu menu = 0;

 if ((menu == 0) || ((Xv_pkg *) xv_get(menu,XV_TYPE) != MENU))
    menu = (Menu) xv_create(0,MENU,
       MENU_ITEM,
           MENU_STRING,CLOSE_STACK_WINS,
           MENU_NOTIFY_PROC,close_stack_wins_proc,0,
        MENU_ITEM,
           MENU_STRING,CLOSE_FILE_WINS,
           MENU_NOTIFY_PROC,close_file_wins_proc,0,
         0);
 return menu;
}


/*------------------------------------------------------------------------
setup_open_win_menu - ???
------------------------------------------------------------------------*/

static Menu
setup_open_win_menu()
{
 static Menu menu = 0;

 if ((menu == 0) || ((Xv_pkg *) xv_get(menu,XV_TYPE) != MENU))
    menu = (Menu) xv_create(0,MENU,
        MENU_ITEM,
           MENU_STRING,OPEN_STACK_WINS,
           MENU_NOTIFY_PROC,open_stack_wins_proc,0,
        MENU_ITEM,
           MENU_STRING,OPEN_FILE_WINS,
           MENU_NOTIFY_PROC,open_file_wins_proc,0,
         0);
 return menu;
}


/*------------------------------------------------------------------------
setup_exit_win_menu - ???
------------------------------------------------------------------------*/

static Menu
setup_exit_win_menu()
{
 static Menu menu = 0;

 if ((menu == 0) || ((Xv_pkg *) xv_get(menu,XV_TYPE) != MENU))
    menu = (Menu) xv_create(0,MENU,
        MENU_ITEM,
         MENU_STRING,EXIT_MULTIPLE_DISPLAY_WINDOW,
         MENU_NOTIFY_PROC,win_exit_proc,0,
        MENU_ITEM,
         MENU_STRING,EXIT_STACK_WINDOWS,
         MENU_NOTIFY_PROC,win_stack_exit_proc,0,
         0);
 return menu;
}


/*------------------------------------------------------------------------
setup_win_menu - ???
------------------------------------------------------------------------*/

static Menu 
setup_win_menu()
{
 static Menu menu = 0;

 if ((menu == 0) || ((Xv_pkg *) xv_get(menu,XV_TYPE) != MENU))
    {/*printf("setup_win_menu...xv_create\n");fflush(stdout);*/
     menu = xv_create(0,MENU,
        MENU_TITLE_ITEM,WINDOW_MENU,
        MENU_RELEASE,
       MENU_ITEM,
         MENU_STRING,MAGNIFY_PIXEL,
         MENU_NOTIFY_PROC,magnify_pixel_proc,
         MENU_PULLRIGHT,setup_magnify_pixel_menu(),0,
        MENU_ITEM,
         MENU_STRING,GET_NEW_WINS,
         MENU_NOTIFY_PROC,get_new_win_proc,
         MENU_PULLRIGHT,setup_more_win_menu(),0,
        MENU_ITEM,
             MENU_STRING,IMAGE_HISTOGRAM_WINDOW,
             MENU_NOTIFY_PROC,image_histogram_proc,0,
        MENU_ITEM,
             MENU_STRING,CHANNEL_MENU,
             MENU_NOTIFY_PROC,get_channel_menu_proc,0,
        MENU_ITEM,
             MENU_STRING,MAKE_WINDOW_CURRENT,
             MENU_NOTIFY_PROC,make_window_cur_proc,0,
        MENU_ITEM,
           MENU_STRING,CLOSE_WIN,
           MENU_NOTIFY_PROC,close_win_proc,
           MENU_PULLRIGHT,setup_close_win_menu(),0,
        MENU_ITEM,
           MENU_STRING,OPEN_WIN,
           MENU_NOTIFY_PROC,open_win_proc,
           MENU_PULLRIGHT,setup_open_win_menu(),0,
        MENU_ITEM,
         MENU_STRING,EXIT_DISPLAY_WINDOW,
         MENU_NOTIFY_PROC,win_exit_cur_win_proc,
         MENU_PULLRIGHT,setup_exit_win_menu(),0,
       0);
    }
 return menu;
}


/**************************************
Menu setup_image_enhancement_menu()
{
 static Menu menu = 0;

 if ((menu == 0) || ((Xv_pkg *) xv_get(menu,XV_TYPE) != MENU))
    menu = xv_create(0,MENU,
        MENU_RELEASE,
        MENU_ITEM,
         MENU_STRING,IMAGE_ENHANCEMENT,
         MENU_NOTIFY_PROC,image_enhancement_proc,0,
        MENU_ITEM,
         MENU_STRING,EDGE_DETECTION,
         MENU_NOTIFY_PROC,edge_detection_proc,0,
        MENU_ITEM,
         MENU_STRING,POLARITY,
         MENU_NOTIFY_PROC,polarity_proc,0,
        0);
 return menu;
}
***************************************/



/*------------------------------------------------------------------------
setup_color_pallette_menu - ???
------------------------------------------------------------------------*/

#define SETUP 1
#define GET_PTR 2
#define DESTROY 3

Menu 
setup_color_pallette_menu(select,pallette,xvwin)
 int select;
 struct pallette_rec *pallette;
 Xv_Window xvwin;
{
 static Menu menu = 0;
 static Menu parent;
 struct intensity_rec *intensity;
 Menu menu1;
 long lcv,cnt;
 struct strs_rec ptr;
 int err;
 char *str;

 PRT1(printf("setup_color_pallette_menu...select = %d\n",select))
 if (select == SETUP)
    {if ((menu == 0) || ((Xv_pkg *) xv_get(menu,XV_TYPE) != MENU))
        if ((menu = xv_create(0,MENU,MENU_TITLE_ITEM,
                COLOR_PALLETTES,MENU_RELEASE,0)) != 0)
           {
            PRT1(printf("...menu = %u\n",menu))
            for (lcv = 0; lcv < pallette->num_pallettes; lcv++)
               {init_strs_rec(&ptr);
                intensity = pallette->pallettes[lcv];
                for (cnt = 0; cnt < intensity->num_scales; cnt++)
                   if ((err = add_str_ptr_to_strs_rec(&ptr,
                               intensity->scales[cnt].label)) != NO_ERROR)
                      break;
                if (err == NO_ERROR)
                   err = SetupTheMenu(&menu1,intensity->label,1,xvwin,
                            &ptr,next_color_menu_proc,NULL);
                free_strs_rec_ptrs(&ptr);
                str = NULL;
                if ((err != NO_ERROR) ||
                    ((err = cp_str(&str,intensity->label)) != NO_ERROR))
                   {if (menu1 != 0) xv_destroy_safe(menu1);
                    c_free(str);
                    break;}
                  else
                   xv_set(menu1,
                         MENU_DEFAULT,intensity->cur_scale + 1,0);
                xv_set(menu,
                   MENU_ITEM,MENU_STRING,str,MENU_RELEASE_IMAGE,
                     MENU_PULLRIGHT,menu1,0,0);
                parent = (Menu) xv_get(menu1,MENU_PARENT);

                PRT1(printf("...str = %s menu1 = %u parent = %u\n",str,menu1,parent);)
               }
            if (err == NO_ERROR)
               xv_set(menu,MENU_DEFAULT,pallette->cur_pallette + 1,0);
           }
     }
   else if (select == GET_PTR)
    {if ((menu == 0) || ((Xv_pkg *) xv_get(menu,XV_TYPE) != MENU))
        menu = 0;
    }
   else
    {if ((menu != 0) && ((Xv_pkg *) xv_get(menu,XV_TYPE) == MENU))
        {destroy_menu_and_submenus(menu,0);
         PRT1(printf("xv_destroy menu = %u\n",menu))
        }
     menu = 0;
    }
 PRT1(printf("setup_color_pallette_menu...select = %d menu = %u\n",select,menu);)
 return menu;
}



/*------------------------------------------------------------------------
main_helmenu - ???
------------------------------------------------------------------------*/

static Menu 
main_help_menu()
{
 static Menu menu = 0;

 if ((menu == 0) || ((Xv_pkg *) xv_get(menu,XV_TYPE) != MENU))
    menu = xv_create(0,MENU,
        MENU_RELEASE,
        MENU_ITEM,
         MENU_STRING,DISPLAY_MENU_HELP,
         MENU_NOTIFY_PROC,display_menu_help_proc,0,
        MENU_ITEM,
         MENU_STRING,ABOUT_RIA,
         MENU_NOTIFY_PROC,about_ria_proc,0,
        MENU_ITEM,
         MENU_STRING,BEGINNER_HELP,
         MENU_NOTIFY_PROC,beginner_help_proc,0,
        0);
 return menu;
}


/*------------------------------------------------------------------------
setup_3d_cursor_menu - ???
------------------------------------------------------------------------*/

Menu 
setup_3d_cursor_menu()
{
 static Menu menu = 0;

 if ((menu == 0) || ((Xv_pkg *) xv_get(menu,XV_TYPE) != MENU))
    menu = xv_create(0,MENU,
        MENU_RELEASE,
        MENU_ITEM,
           MENU_STRING,_3D_CUT_MODE,
           MENU_CLIENT_DATA,CURSOR_CUT,
           MENU_NOTIFY_PROC,_3d_select_cursor_proc,0,
        MENU_ITEM,
           MENU_STRING,_3D_ANGLE_CUT_MODE,
           MENU_CLIENT_DATA,CURSOR_ANGLE_CUT,
           MENU_NOTIFY_PROC,_3d_select_cursor_proc,0,
        MENU_ITEM,
           MENU_STRING,_3D_ROTATE_VERTICAL_MODE,
           MENU_CLIENT_DATA,CURSOR_ROTATE_VERTICAL,
           MENU_NOTIFY_PROC,_3d_select_cursor_proc,0,
        MENU_ITEM,
           MENU_STRING,_3D_ROTATE_HORIZONTAL_MODE,
           MENU_CLIENT_DATA,CURSOR_ROTATE_HORIZONTAL,
           MENU_NOTIFY_PROC,_3d_select_cursor_proc,0,
        MENU_ITEM,
           MENU_STRING,_3D_ROTATE_MODE,
           MENU_CLIENT_DATA,CURSOR_ROTATE,
           MENU_NOTIFY_PROC,_3d_select_cursor_proc,0,
        0);
 return menu;
}


/***************************************************
Menu setup_3d_orientatition_menu()
{
 static Menu menu = 0;

 if ((menu == 0) || ((Xv_pkg *) xv_get(menu,XV_TYPE) != MENU))
    menu = xv_create(0,MENU,
        MENU_RELEASE,
        MENU_ITEM,
           MENU_STRING,_3D_0_ROTATION,MENU_CLIENT_DATA,5,
           MENU_NOTIFY_PROC,_3d_plane_orientation_proc,0,
        MENU_ITEM,
           MENU_STRING,_3D_90_ROTATION,MENU_CLIENT_DATA,7,
           MENU_NOTIFY_PROC,_3d_plane_orientation_proc,0,
        MENU_ITEM,
           MENU_STRING,_3D_180_ROTATION,MENU_CLIENT_DATA,3,
           MENU_NOTIFY_PROC,_3d_plane_orientation_proc,0,
        MENU_ITEM,
           MENU_STRING,_3D_270_ROTATION,MENU_CLIENT_DATA,1,
           MENU_NOTIFY_PROC,_3d_plane_orientation_proc,0,
        MENU_ITEM,
           MENU_STRING,_3D_Y_X_REFLECTION,MENU_CLIENT_DATA,6,
           MENU_NOTIFY_PROC,_3d_plane_orientation_proc,0,
        MENU_ITEM,
           MENU_STRING,_3D_YX_REFLECTION,MENU_CLIENT_DATA,8,
           MENU_NOTIFY_PROC,_3d_plane_orientation_proc,0,
        MENU_ITEM,
           MENU_STRING,_3D_NO_REFLECTION,MENU_CLIENT_DATA,4,
           MENU_NOTIFY_PROC,_3d_plane_orientation_proc,0,
        MENU_ITEM,
           MENU_STRING,_3D_Y_REFLECTION,MENU_CLIENT_DATA,0,
           MENU_NOTIFY_PROC,_3d_plane_orientation_proc,0,
        MENU_ITEM,
           MENU_STRING,_3D_X_REFLECTION,MENU_CLIENT_DATA,2,
           MENU_NOTIFY_PROC,_3d_plane_orientation_proc,0,
        MENU_ITEM,
           MENU_STRING,FLIP_PLANE_CLOCKWISE,MENU_CLIENT_DATA,9,
           MENU_NOTIFY_PROC,_3d_plane_orientation_proc,0,
        MENU_ITEM,
           MENU_STRING,FLIP_PLANE_ANTI_CLOCKWISE,MENU_CLIENT_DATA,15,
           MENU_NOTIFY_PROC,_3d_plane_orientation_proc,0,
        0);
 return menu;
}
***************************************************/



/*------------------------------------------------------------------------
setup_3d_menu - ???
------------------------------------------------------------------------*/

static Menu 
setup_3d_menu()
{
 static Menu menu = 0;

 if ((menu == 0) || ((Xv_pkg *) xv_get(menu,XV_TYPE) != MENU))
    {
     menu = xv_create(0,MENU,
        MENU_RELEASE,
        MENU_ITEM,
           MENU_STRING,_3D_FILE_MENU,
           MENU_NOTIFY_PROC,enter_3d_file_menu_proc,0,
        MENU_ITEM,
           MENU_STRING,_3D_PLANE_OPTIONS,
           MENU_NOTIFY_PROC,enter_3d_plane_proc,0,
        MENU_ITEM,
           MENU_STRING,_3D_MORE_PLANE_OPTIONS,
           MENU_NOTIFY_PROC,enter_3d_more_plane_proc,0,
        MENU_ITEM,
           MENU_STRING,_3D_RESIZE_IMAGE,
           MENU_NOTIFY_PROC,enter_3d_resize_proc,0,
        MENU_ITEM,
           MENU_STRING,_3D_CO_REGISTRATION,
           MENU_NOTIFY_PROC,enter_3d_co_menu_proc,0,
        MENU_ITEM,
           MENU_STRING,_3D_PARAM_MENU,
           MENU_NOTIFY_PROC,enter_3d_param_menu_proc,0,
        MENU_ITEM,
           MENU_STRING,_3D_SAVE_MENU,
           MENU_NOTIFY_PROC,enter_3d_save_menu_proc,0,
        MENU_ITEM,
           MENU_STRING,_3D_CURSOR_MENU,
           MENU_PULLRIGHT,setup_3d_cursor_menu(),0,
        MENU_ITEM,
           MENU_STRING,_3D_ROIS_PIXEL_TYPES,
           MENU_NOTIFY_PROC,enter_3d_rois_and_pixel_types_proc,0,
/*
        MENU_ITEM,
              MENU_STRING,_3D_SPLIT_ROIS,
              MENU_NOTIFY_PROC,enter_3d_split_rois_proc,0,
        MENU_ITEM,
           MENU_STRING,_3D_DISPLAY_OPTIONS,
           MENU_NOTIFY_PROC,enter_3d_display_menu_proc,0,
*/
      0);
    }
 return menu;
}


/*------------------------------------------------------------------------
setup_file_menu - ???
------------------------------------------------------------------------*/

static Menu
setup_file_menu(pallette,xvwin)
     struct pallette_rec *pallette;
     Xv_Window xvwin;
{ 
  static Menu menu = 0;
  Menu menu1;

  TRACE(printf("setup_file_menu(): blank menu -> %p\n",(void *)menu));
  if ((menu == 0) || ((Xv_pkg *) xv_get(menu,XV_TYPE) != MENU))
  {
    menu1 = setup_color_pallette_menu(1,pallette,xvwin);
    menu = xv_create(0,MENU,
		     MENU_TITLE_ITEM,FILE_MENU,
		     MENU_ITEM,
		     MENU_STRING,SELECT_FNS_MENU,
		     MENU_NOTIFY_PROC,win_select_fns_menu_proc,0,
		     MENU_ITEM,
		     MENU_STRING,IMPORT_IMG,
		     MENU_NOTIFY_PROC,import_image_file_proc,0,
		     MENU_ITEM,
		     MENU_STRING,SAVE_DATA,
		     MENU_PULLRIGHT,setup_file_info_menu(),
		     MENU_NOTIFY_PROC,enter_save_dat_proc,0,
		     MENU_ITEM,
		     MENU_STRING,CURSOR_MODE,
		     MENU_GEN_PROC,gen_cursor_menu_proc,0,
		     MENU_ITEM,
		     MENU_STRING,COLOR_PALLETTES,
		     MENU_PULLRIGHT,menu1,0,
		     MENU_ITEM,
		     MENU_STRING,DEF_IMAGE_COLOR_SCALE,
		     MENU_VALUE,0,
		     MENU_NOTIFY_PROC,image_color_scale_proc,0,
		     MENU_ITEM,
		     MENU_STRING,NTH_IMAGE_COLOR_SCALE,
		     MENU_VALUE,1,
		     MENU_NOTIFY_PROC,image_color_scale_proc,0,
		     MENU_ITEM,
		     MENU_STRING,COLOR_EDITOR,
		     MENU_NOTIFY_PROC,color_editor_proc,0,
		     MENU_ITEM,
		     MENU_STRING,QUIT_AND_SAVE_FILE,
		     MENU_NOTIFY_PROC,win_quit_and_save_file_proc,0,
		     MENU_ITEM,
		     MENU_STRING,EXIT_PROGRAM,
		     MENU_NOTIFY_PROC,program_exit_proc,0,
		     MENU_ITEM,
		     MENU_STRING,MENU_SEPARATOR,0,
		     MENU_ITEM,
		     MENU_STRING,HELP_STR,
		     MENU_PULLRIGHT,main_help_menu(),0,
		     0);
    PRT1(printf("	parent = %p\n",
		(void *) xv_get(menu1,MENU_PARENT)););
    }
  PRT(printf("setup_file_menu() end.\n"););
  return menu;
}


/*------------------------------------------------------------------------
button_proc - ???
------------------------------------------------------------------------*/

static void
button_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct display_win_rec *dwin;
 Menu menu;

 PRT(printf("button_proc...\n"))
 dwin = (struct display_win_rec *) xv_get(item,PANEL_CLIENT_DATA);
 PRT(printf("button_proc...dwin = %u\n",dwin))
 menu = (Menu) xv_get(item,PANEL_ITEM_MENU);
 PRT(printf("button_proc...menu = %u\n",menu))
 xv_set(menu,MENU_CLIENT_DATA,dwin,0);
 PRT(printf("button_proc..1.menu = %u\n",menu))
}


/*
 * 19970808  bookmark
 */


/*------------------------------------------------------------------------
SetupDisplayWinRec - ???
------------------------------------------------------------------------*/

int 
SetupDisplayWinRec( win, owner, ancestor, master )
     struct display_win_rec *win;
     struct image_display_window *owner;
     Frame ancestor;
     struct display_win_rec *master;
{
  Cms cms;
  struct color_rec *color;
  Rect *rr;
  struct int_rect_rec limitrr;
  struct img_buf_rec *img;
  int startup_pixel_mag_factor;
  Menu _3d_menu,window_menu,menu1;
  char *label,*str;
  Panel_item item;
  int err;
  unsigned short anal_status;
  unsigned long dimx,dimy;
  struct buf_2d_rec *cur_2d;
  

  if ((img = win_rec_cur_img_buf(owner->owner)) == NULL)
    return CANCEL;

  PRT(printf("SetupDisplayWinRec() entered, ancestor = %p\n",
	     (void *)ancestor));
  cms = (Cms) xv_get(ancestor,WIN_CMS);
  PRT(printf("	CMS of ancestor = %p\n", (void *) cms););

  color = &(owner->intensity->color);
  win->frame = xv_create( ancestor, FRAME,
			  FRAME_INHERIT_COLORS, TRUE,
			  WIN_BACKGROUND_COLOR, 1 + color->num_system_colors,
			  WIN_FOREGROUND_COLOR, 2 + color->num_system_colors,
			  WIN_EVENT_PROC, main_event_proc,
			  WIN_CLIENT_DATA, win,
			  FRAME_DONE_PROC, win_done_proc,
			  FRAME_SHOW_LABEL, TRUE,
			  FRAME_NO_CONFIRM, TRUE,
			  XV_SHOW, FALSE,
			  XV_FONT, win->font,
			  0 );
  if (win->frame == 0)
    return WINDOW_ERROR;

  xv_set( win->frame,
	  WIN_CMS, cms,
	  NULL );
  PRT(printf("	Created frame: parent = %p, id = %p, win = %p, cms = %p\n",
	     (void *) ancestor,
	     (void *) win->frame,
	     (void *) win,
	     (void *) cms );
      printf("	(xv_get cms -> %p)\n", (void *)xv_get(win->frame,WIN_CMS)););

  win->panel = xv_create(win->frame,PANEL,
			 WIN_CLIENT_DATA,win,
			 WIN_CMS, cms,
			 XV_X,0,
			 XV_Y,0,
			 0);
  if (win->panel == 0)
    return WINDOW_ERROR;

  PRT(printf("	Created subpanel: id = %p, cms = %p\n",
	     (void *) win->panel,
	     (void *) xv_get(win->panel, WIN_CMS)););

  xv_set(win->frame,
	 WIN_CONSUME_EVENTS,WIN_LEFT_KEYS,ACTION_AGAIN,
	 ACTION_STOP,WIN_KEY_FIND,WIN_RESIZE,
	 WIN_MOUSE_BUTTONS,
	 NULL,
	 0);

  win->display_status |= WIN_NOT_YET_PAINTED;
  win->owner = owner;
  win->ancestor = ancestor;
  win->master = master;
  if (master == NULL)
    win->status2 |= MASTER_WINDOW;
  else
    add_master_slave_relationship(master,win);

  if (owner->type2 & DISPLAY_WINDOW_3D)
    startup_pixel_mag_factor = 1;
  else
    sscanf(win->owner->owner->root->prog->defaults.whch.
	   startup_pixel_mag_factor,"%d",&startup_pixel_mag_factor);

  rr = (Rect *) xv_get(win->frame,WIN_SCREEN_RECT);
  Rect_to_int_rect_rec(rr,&limitrr);
  limitrr.width -= 30;
  limitrr.height -= 50;
  if ((cur_2d = win_rec_cur_data_buf(owner->owner,owner->type2)) == NULL)
  {
    dimx = img->_2d.dimx;
    dimy = img->_2d.dimy;
  }
  else
  {
    dimx = cur_2d->dimx;
    dimy = cur_2d->dimy;
  }
  if ((err = setup_def_screen_data_rec(&(win->screen),&limitrr,0,0,
				       dimx,dimy,startup_pixel_mag_factor,
				       startup_pixel_mag_factor,1))
      != NO_ERROR)
    return err;
  
  win->font = (Xv_Font)
    xv_find( win->frame, FONT,
	     FONT_NAME, "courier",
	     NULL );

  win->frame_label = NULL;

  win->dp.server = XV_SERVER_FROM_WINDOW(win->frame);
  win->dp.dp = (Display *) xv_get(win->frame,XV_DISPLAY);
  win->dp.screen_num = (int) DefaultScreen(win->dp.dp);

  win->menu = setup_file_menu(&(win->owner->owner->root->def_color),
			      win->frame);
  if (win->menu == 0)
  {
    xv_destroy_safe(win->frame);
    win->frame = 0;
    return WINDOW_ERROR;
  }
  xv_set(win->menu,MENU_CLIENT_DATA,win,0);

  item = xv_create(win->panel,PANEL_BUTTON,
		   PANEL_LABEL_STRING,"FILE",
		   PANEL_NOTIFY_PROC,button_proc,PANEL_CLIENT_DATA,win,
		   XV_X,0,XV_Y,0,PANEL_ITEM_MENU,win->menu,0);
  rr = (Rect *) xv_get(item,XV_RECT);
  item = xv_create(win->panel,PANEL_BUTTON,
		   PANEL_LABEL_STRING,"IMG",
		   XV_X,rr->r_left + rr->r_width + 1,XV_Y,0,
		   PANEL_NOTIFY_PROC,button_proc,PANEL_CLIENT_DATA,win,
		   PANEL_ITEM_MENU,setup_img_menu(),
		   0);
  rr = (Rect *) xv_get(item,XV_RECT);

  item = xv_create(win->panel,PANEL_BUTTON,
		   PANEL_LABEL_STRING,"ROI",
		   XV_X,rr->r_left + rr->r_width + 1,XV_Y,0,
		   PANEL_NOTIFY_PROC,button_proc,PANEL_CLIENT_DATA,win,
		   PANEL_ITEM_MENU,setup_roi_menu(),
		   0);
  rr = (Rect *) xv_get(item,XV_RECT);
#if 0
  PRT3(printf("SetupDisplayWinRec(): file->status = %x\n",
	      (unsigned long) img->file->status));
#endif

  if (win->owner->type2 & DISPLAY_WINDOW_3D)
  {
    _3d_menu = setup_3d_menu();
    win->anal_button = xv_create(win->panel,PANEL_BUTTON,
				 PANEL_LABEL_STRING,_3D_BUTTON_STR,
				 XV_X,rr->r_left + rr->r_width+1,XV_Y,0,
				 PANEL_NOTIFY_PROC,button_proc,
				 PANEL_CLIENT_DATA,win,
				 PANEL_ITEM_MENU,_3d_menu,
				 0);
  }
  else if ((img->file->status & DEF_CA_MENU) ||
	   (!(img->file->status & DEF_ANAL_MENU) &&
	    (str_cmp(img->file->hdr.data_info.image_id,MRI) != 0) &&
	    (str_cmp(img->file->hdr.data_info.image_id,PIXEL_TYPE) != 0)))
  {
    win->anal_button = xv_create(win->panel,PANEL_BUTTON,
				 PANEL_LABEL_STRING,CA_STR,
				 PANEL_NOTIFY_PROC,button_proc,
				 PANEL_CLIENT_DATA,win,
				 XV_X,rr->r_left + rr->r_width+1,XV_Y,0,
				 PANEL_ITEM_MENU,setup_pixel_count_anal_menu(),
				 0);
    win->analysis_status |= PIXEL_COUNT_VALUE_ANALYSIS;
    img->file->status &= ~DEF_ANAL_MENU;
    img->file->status |= DEF_CA_MENU;
  }
  else
  {
    str = TA_STR;
    anal_status = PIXEL_TYPE_PARTITION_ANALYSIS;
    if (win->owner->type2 & HISTOGRAM_WINDOW)
    {
      PRT(printf("	setup_histo_GA_menu() to be called\n"););
      if ((img->file->status & DEF_TA_MENU) ||
	  (img->file->pixel_type.status & PIXEL_TYPE_2D_PARTITION))
      {
	menu1 = setup_histo_TA_menu();
	img->file->status &= ~DEF_ANAL_MENU;
	img->file->status |= DEF_TA_MENU;
      }
      else if ((img->file->status & DEF_GA_MENU) ||
	       (img->file->pixel_type.status & PIXEL_TYPE_GROW_REGION) ||
	       (img->file->pixel_type.status & PIXEL_TYPE_PIXMAP_HDR_VALID))
      {
	win_def_grow_region_params(win->owner);
	menu1 = setup_histo_GA_menu();
	anal_status = PIXEL_TYPE_GROW_REGION_ANALYSIS;
	str = GA_STR;
	img->file->status &= ~DEF_ANAL_MENU;
	img->file->status |= DEF_GA_MENU;
      }
      else
      {
	menu1 = setup_histo_TA_menu();
	img->file->status &= ~DEF_ANAL_MENU;
	img->file->status |= DEF_TA_MENU;
      }
    }
    else 
    {
      if (img->file->pixel_type.status & PIXEL_TYPE_2D_PARTITION)
      {
	menu1 = setup_TA_menu();
	img->file->status &= ~DEF_ANAL_MENU;
	img->file->status |= DEF_TA_MENU;
      }
      else if ((img->file->status & DEF_GA_MENU) ||
	       (img->file->pixel_type.status & PIXEL_TYPE_GROW_REGION) ||
	       (img->file->pixel_type.status & PIXEL_TYPE_PIXMAP_HDR_VALID))
      {
	win_def_grow_region_params(win->owner);
	menu1 = setup_GA_menu();
	anal_status = PIXEL_TYPE_GROW_REGION_ANALYSIS;
	str = GA_STR;
	img->file->status &= ~DEF_ANAL_MENU;
	img->file->status |= DEF_GA_MENU;
      }
      else 
      {
	menu1 = setup_TA_menu();
	img->file->status &= ~DEF_ANAL_MENU;
	img->file->status |= DEF_TA_MENU;
      }
    }

    win->anal_button =
      xv_create(win->panel,PANEL_BUTTON,PANEL_LABEL_STRING,str,
		PANEL_NOTIFY_PROC,button_proc,PANEL_CLIENT_DATA,win,
		XV_X,rr->r_left + rr->r_width+1,XV_Y,0,
		PANEL_ITEM_MENU,menu1,0);
    win->analysis_status |= anal_status;

#if 0
    if (img->file->hdr.data_info.num_echoes == 2)
      win->analysis_status |= PIXEL_TYPE_2_ECHO_ANALYSIS;
    else
      win->analysis_status |= PIXEL_TYPE_1_ECHO_ANALYSIS;
#endif

  }
  PRT(printf("	win->anal_button = %p\n", (void *)win->anal_button););
  rr = (Rect *) xv_get(win->anal_button,XV_RECT);
  window_menu = setup_win_menu();
  item = xv_create( win->panel, PANEL_BUTTON,
		    PANEL_LABEL_STRING, WIN,
		    XV_X, rr->r_left + rr->r_width + 1,
		    XV_Y, 0,
		    PANEL_NOTIFY_PROC, button_proc,
		    PANEL_CLIENT_DATA, win,
		    PANEL_ITEM_MENU, window_menu,
		    0 );

  rr = (Rect *) xv_get(item,XV_RECT);
  xv_set(win->panel,XV_HEIGHT,rr->r_top + rr->r_height - 4,0);

  init_frame_canvas_rec(&(win->frame_canvas));
  if (win->owner->type2 & MANY_SLICES_WINDOWS)
    setup_select_slice_win(&(win->select_slice),win);

  notify_interpose_event_func((Notify_client) win->frame,
			      (Notify_func) frame_interposer,NOTIFY_SAFE);

  get_ria_icon(win->frame);
  label = WinRecGetWid(win->owner->owner,win->owner->type2);
  set_icon_label(win->frame,label);
  TRACE(printf("SetupDisplayWinRec() end.\n"));
  return NO_ERROR;
}

