/****************************************************************************
dp_breaker_options.c - ???

Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.
****************************************************************************/

static const char rcsid[] = "$Id: dp_breaker_options.c,v 1.9 1999/07/07 17:41:15 hughett Exp $ Release $Name: nria_1_0_6 $";


#include <xview/xview.h>
#include <xview/cms.h>
#include <xview/canvas.h>
#include <xview/panel.h>
#include <xview/notice.h>
#include <xview/wmgr.h>

#include "mri_format.h"
#include "nria_procedures.h"

/*
#define PRINT(x) printf(x); fflush(stdout);
#define PRT(x) printf(x); fflush(stdout);
#define TRACE(x) x; fflush(stdout);
*/
#define PRINT(x)
#define PRT(x)
#define TRACE(x)



#define CLEAN_UP_BREAKER 0
#define SETUP_BREAKER 1
#define GET_BREAKER_PTR 2
#define BRING_UP_BREAKER 3

struct breaker_rec
   {
    unsigned short status;
    struct image_display_window *win;
    struct display_win_rec *dwin;
    Frame frame;
    Panel panel;
   };



/* Functions defined in this file */
void init_breaker_rec(struct breaker_rec *breaker);
void free_breaker_rec(struct breaker_rec *breaker);
void breaker_done_proc(Frame frame);
void breaket_set_option_proc(Panel_item item, int value, Event *event);
int setup_breaker_proc(struct breaker_rec *breaker,
    struct image_display_window *win);
int enter_dp_breaker_options(struct image_display_window *win, int status);



/*------------------------------------------------------------------------
init_breaker_rec - ???
------------------------------------------------------------------------*/

void
init_breaker_rec(breaker)
 struct breaker_rec *breaker;
{
 breaker->frame = 0;
 breaker->win = NULL;
 breaker->status = 0;
}


/*------------------------------------------------------------------------
free_breaker_rec - ???
------------------------------------------------------------------------*/

void
free_breaker_rec(breaker)
 struct breaker_rec *breaker;
{
 if (breaker->frame != 0) xv_destroy_safe(breaker->frame);
 init_breaker_rec(breaker);
}


/*------------------------------------------------------------------------
breaker_done_proc - ???
------------------------------------------------------------------------*/

void
breaker_done_proc(frame)
 Frame frame;
{
 struct breaker_rec *breaker;

 breaker = (struct breaker_rec *)
                     xv_get(frame,WIN_CLIENT_DATA);
 free_breaker_rec(breaker);
}


/*------------------------------------------------------------------------
breaket_set_option_proc - ???
------------------------------------------------------------------------*/

void
breaket_set_option_proc(item,value,event)
 Panel_item item;
 int value;
 Event *event;
{
 struct breaker_rec *breaker;
 struct slice_roi_rec *slice_roi;
 unsigned long which_option;

 breaker = (struct breaker_rec *) xv_get(item,PANEL_CLIENT_DATA);
 if ((slice_roi = win_rec_get_breaker(breaker->win->owner)) != NULL)
    {which_option = xv_get(item,XV_KEY_DATA,1);
     printf("which_option = %lx status = %lx\n",which_option,
                (unsigned long) slice_roi->prog_status);
     if (value)
        slice_roi->prog_status |= which_option;
       else slice_roi->prog_status &= ~which_option;
    }
}


/*------------------------------------------------------------------------
setup_breaker_proc - ???
------------------------------------------------------------------------*/

int
setup_breaker_proc(breaker,win)
 struct breaker_rec *breaker;
 struct image_display_window *win;
{
 int row,offset;
 struct img_buf_rec *img;
 char *str;
 Rect *rr;
 struct slice_roi_rec *slice_roi;
 Panel_item item;

 if ((img = win_rec_cur_img_buf(win->owner)) == NULL) return CANCEL;
 breaker->frame = xv_create(win->dwin->frame, FRAME,
              FRAME_INHERIT_COLORS,TRUE,
              WIN_CLIENT_DATA,breaker,
              FRAME_DONE_PROC,breaker_done_proc,
              0);
 /* FIXME:  Guessed at return value below.  -PWH 19990407 */
 if (breaker->frame == 0) return ~NO_ERROR;
 breaker->win = win;
 breaker->dwin = win->dwin;
 str = win_rec_win_id_title(win->owner,BREAKER);
 xv_set(breaker->frame,
              FRAME_SHOW_LABEL, TRUE,
              XV_LABEL,str,
              FRAME_NO_CONFIRM,TRUE,
              0);
 c_free(str);
 xv_set(breaker->frame,FRAME_CLOSED,FALSE,0);

 breaker->panel = xv_create(breaker->frame,PANEL,
               FRAME_INHERIT_COLORS,TRUE,
               0);

 if (breaker->panel == 0)
    {free_breaker_rec(breaker);
     return 0;}

 row = 2;
 offset = xv_rows(breaker->panel,1) - xv_rows(breaker->panel,0) - 10;
 item = xv_create(breaker->panel,PANEL_CHECK_BOX,
        PANEL_CHOICE_STRINGS,"DELETE CUR REGION AFTER APPLYING BREAKER",NULL,
        XV_X,5,XV_Y,row,
        PANEL_CLIENT_DATA,breaker,
        XV_KEY_DATA,1,(unsigned long) BREAKER_DELETE_CUR_REGION,
	PANEL_NOTIFY_PROC,breaket_set_option_proc,
	0);
 if ((slice_roi = win_rec_get_breaker(breaker->win->owner)) != NULL)
    if (slice_roi->prog_status & BREAKER_DELETE_CUR_REGION)
       xv_set(item,PANEL_VALUE,1,0);
 row += offset;
/*
 item = xv_create(breaker->panel,PANEL_CHECK_BOX,
        PANEL_CHOICE_STRINGS,"OUTPUT REGIONS WILL SHARE COMMON BOUNDARY",NULL,
        XV_X,5,XV_Y,row,
        PANEL_CLIENT_DATA,breaker,
        XV_KEY_DATA,1,(unsigned long) BREAKER_COMMON_BOUNDARY,
	PANEL_NOTIFY_PROC,breaket_set_option_proc,
	0);
 if (slice_roi != NULL)
    if (slice_roi->prog_status & BREAKER_COMMON_BOUNDARY)
       xv_set(item,PANEL_VALUE,1,0);
 row += offset;
*/
 item = xv_create(breaker->panel,PANEL_CHECK_BOX,
        PANEL_CHOICE_STRINGS,"LABEL OUTPUT REGION FROM LEFT TO RIGHT",NULL,
        XV_X,5,XV_Y,row,
        XV_KEY_DATA,1,(unsigned long) BREAKER_LABEL_BREAKER_REGION,
        PANEL_CLIENT_DATA,breaker,
	PANEL_NOTIFY_PROC,breaket_set_option_proc,
	0);
 if (slice_roi != NULL)
    if (slice_roi->prog_status & BREAKER_LABEL_BREAKER_REGION)
       xv_set(item,PANEL_VALUE,1,0);
 row += offset;
 item = xv_create(breaker->panel,PANEL_CHECK_BOX,
        PANEL_CHOICE_STRINGS,"DISABLE BREAKER AFTER APPLYING",NULL,
        XV_X,5,XV_Y,row,
        XV_KEY_DATA,1,(unsigned long) BREAKER_DISABLE_BREAKER_AFTER_APPLY,
        PANEL_CLIENT_DATA,breaker,
	PANEL_NOTIFY_PROC,breaket_set_option_proc,
	0);
 if (slice_roi != NULL)
    if (slice_roi->prog_status & BREAKER_DISABLE_BREAKER_AFTER_APPLY)
       xv_set(item,PANEL_VALUE,1,0);

 window_fit(breaker->panel);
 rr = (Rect *) xv_get(item,XV_RECT);
 xv_set(breaker->panel,XV_HEIGHT,rr->r_top + rr->r_height + 5,0);
 window_fit(breaker->frame);
 make_2_wins_adjacent(breaker->frame,win->dwin->frame,0);
 win_set_mouse_at_center(breaker->frame);
 return NO_ERROR;
}


/*------------------------------------------------------------------------
enter_dp_breaker_options - ???
------------------------------------------------------------------------*/

int
enter_dp_breaker_options(win,status)
 struct image_display_window *win;
 int status;
{
 static struct breaker_rec breaker;
 static char first_time = 1;
 int err;

 err = NO_ERROR;
 if (first_time)
    {init_breaker_rec(&breaker);
     first_time = 0;}
 if (status == GET_BREAKER_PTR)
    {return (unsigned long) &breaker;}     /* FIXME:  ptr as long */
   else if (status == CLEAN_UP_BREAKER)
    {if (!first_time)
        if (breaker.win == win)
           free_breaker_rec(&breaker);
     return NO_ERROR;}

 if (breaker.win == win)
    {
     xv_set(breaker.frame,
                XV_SHOW,TRUE,WIN_ALARM,FRAME_CLOSED,FALSE,0);
     win_set_mouse_at_center(breaker.frame);
     wmgr_top(breaker.frame);
     return NO_ERROR;}
   else free_breaker_rec(&breaker);

 if ((err = setup_breaker_proc(&breaker,win)) != NO_ERROR)
    ShowMsgStrs(win->dwin->frame,0,err,0,2,
               "An error occured when setting up",
               "   the breaker window !!!",NULL,0);
   else xv_set(breaker.frame,XV_SHOW,TRUE,FRAME_CLOSED,FALSE,0);
 TRACE(printf("breaker...2\n"))
 return err;
}
