/****************************************************************************
fmri.c - ???

Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.
****************************************************************************/

static const char rcsid[] = "$Id: fmri.c,v 1.6 1999/07/07 17:41:16 hughett Exp $ Release $Name: nria_1_0_6 $";

#include "mri_format.h"
#include "nria_procedures.h"


/*
#define PRT(x) x; fflush(stdout);
*/
#define PRT(x)



/* Functions defined in this file */
int rd_scans(FILE *fid, struct hdr_rec *hdr, struct buf_2d_rec **scans,
    unsigned long num_scans, unsigned long slice_num, unsigned long echo_num);
int motion_correction(struct buf_2d_rec **scans, unsigned long num_scans,
    struct buf_2d_rec **temps, unsigned long num_temps);
int SliceMotionCorrection(FILE *fid, struct hdr_rec *hdr,
    struct buf_2d_rec **scans, unsigned long num_scans,
    unsigned long slice_num, unsigned long echo_num,
    struct buf_2d_rec **temps, unsigned long num_temps,
    double *min, double *max);
int correlation_coeffient(struct buf_2d_rec **scans, unsigned long num_scans,
    struct buf_2d_rec **temps, unsigned long num_temps, int is_ref_wave_form,
    float *phases, double phases_sd, int is_normalized,
    struct buf_2d_rec *slice);
int setup_ref_waveform(float **phases, float *phases_sd,
    unsigned long num_phases, unsigned long right_shift,
    unsigned long num_scans);
int SliceCorrelationCoeff(FILE *fid, struct hdr_rec *hdr,
    struct buf_2d_rec **scans, unsigned long num_scans,
    unsigned long slice_num, unsigned long echo_num,
    struct buf_2d_rec **temps, unsigned long num_temps, int is_ref_wave_form,
    float *phases, double phases_sd, int is_normalized,
    int is_motion_correction, struct buf_2d_rec *slice);
int SingleSamplePairTest(struct buf_2d_rec **scans, unsigned long num_scans,
    struct buf_2d_rec **temps, unsigned long num_temps, int num_pairs,
    int seq, struct buf_2d_rec *slice);
int SingleSamplePairPtsTest(struct buf_2d_rec **scans,
    unsigned long num_scans, struct buf_2d_rec **temps,
    unsigned long num_temps, int num_pairs, int seq, struct buf_2d_rec *slice);
int SingleSampleTTest(FILE *fid, struct hdr_rec *hdr,
    struct buf_2d_rec **scans, unsigned long num_scans,
    unsigned long slice_num, unsigned long echo_num,
    struct buf_2d_rec **temps, unsigned long num_temps, int num_pairs,
    int seq, int is_pair_pts, int is_motion_correction,
    struct buf_2d_rec *slice);
int setup_cc_buf(struct hdr_rec *hdr, struct buf_2d_rec ***scans,
    unsigned long *num_scans);
int StdSlice(FILE *fid, struct hdr_rec *hdr, struct buf_2d_rec **scans,
    unsigned long num_scans, unsigned long slice_num, unsigned long echo_num,
    struct buf_2d_rec **temps, unsigned long num_temps,
    struct buf_2d_rec *slice);



/*----------------------------------------------------------------------------
rd_scans - ???
----------------------------------------------------------------------------*/

int
rd_scans(fid,hdr,scans,num_scans,slice_num,echo_num)
 FILE *fid;
 struct hdr_rec *hdr;
 struct buf_2d_rec **scans;
 unsigned long num_scans,slice_num,echo_num;
{
 int err;
 struct buf_2d_rec _2d;
 unsigned long scan_num;
 struct hdr_scan_entry_rec *scan_entry;
/*
 double min,max;
*/
 init_buf_2d_rec(&_2d);
 if ((err = SetupBuf2dRec(&_2d,hdr->data_info.dimx,hdr->data_info.dimy,
       hdr->data_info.pixel_depth_bits,
       InputDataTypeToBufType(&(hdr->data_info)))) != NO_ERROR)
    return err;
 for (scan_num = 0; scan_num < num_scans; scan_num++)
    {if ((err = LocateScanEntry(hdr,slice_num,
                scan_num,echo_num,&scan_entry)) != NO_ERROR)
        break;
/*
     PRT(printf("%d) s# = %d e# = %d scan# = %d addr = %u\n",
        scan_num,slice_num,echo_num,scan_num,scan_entry->img_addr))
*/
     if ((err = rd_cur_slice_data(fid,scan_entry,&_2d)) != NO_ERROR)
        break;
     Assign2d(scans[scan_num],&_2d);
/*
     MinMax2dValues(scans[scan_num],0,(double) .0,(double) .0,&min,&max);
     PRT(printf("%d) %u min = %f max = %f\n",scan_num,scan_entry->img_addr,min,max))
*/
    }
 free_buf_2d_rec(&_2d);
 return err;
}


/*----------------------------------------------------------------------------
motion_correction - ???
----------------------------------------------------------------------------*/

int
motion_correction(scans,num_scans,temps,num_temps)
 struct buf_2d_rec **scans,**temps;
 unsigned long num_scans,num_temps;
{
 unsigned long row,col,lcv,cnt,scan_num;
 double tot1,tot2,totx,toty,totx2,toty2,mx,my,mz;

 if (num_scans <= 1) return NO_ERROR;
 for (row = 0; row < scans[0]->dimy; row++)
    for (lcv = scans[0]->dimx - 1,col = 0,cnt = 1; col < scans[0]->dimx; col++)
       {temps[0]->SetVal(temps[0],row,col,
          (double) (scans[0]->GetVal(scans[0],row,col) -
                    scans[0]->GetVal(scans[0],row,lcv)) / 2.0);
        lcv = (lcv + 1) % scans[0]->dimx;
        cnt = (cnt + 1) % scans[0]->dimx;
       }
 for (col = 0; col < scans[0]->dimx; col++)
    for (lcv = scans[0]->dimy - 1,row = 0,cnt = 1; row < scans[0]->dimy; row++)
       {temps[1]->SetVal(temps[1],row,col,
          (double) (scans[0]->GetVal(scans[0],cnt,col) -
                    scans[0]->GetVal(scans[0],lcv,col)) / 2.0);
        lcv = (lcv + 1) % scans[0]->dimy;
        cnt = (cnt + 1) % scans[0]->dimy;
       }
 totx2 = Total2dValues(&(temps[0]),NULL,1,1);
 toty2 = Total2dValues(&(temps[1]),NULL,1,1);
 for (scan_num = 1; scan_num < num_scans; scan_num++)
    {Sub2ds(temps[2],scans[scan_num],scans[0]);
     totx = Total2dValues(&(temps[0]),&(temps[2]),1,0);
     toty = Total2dValues(&(temps[1]),&(temps[2]),1,0);
     mx = totx / totx2;
     my = toty / toty2;
     Mult2dByConstant(temps[2],temps[0],mx);
     Mult2dByConstant(temps[3],temps[1],my);
     Add2ds(temps[2],temps[2],temps[3]);
     Sub2ds(scans[scan_num],scans[scan_num],temps[2]);
    }

 if (num_scans > 10) cnt = 10;
   else cnt = num_scans / 2;
 Add2dArys(temps[0],scans,cnt,2);
 Add2dArys(temps[1],&(scans[num_scans - cnt]),cnt,2);
 Sub2ds(temps[0],temps[1],temps[0]);
 tot2 = Total2dValues(&(temps[0]),NULL,1,1);
 for (scan_num = 1; scan_num < num_scans; scan_num++)
    {Sub2ds(temps[2],scans[scan_num],scans[0]);
     tot1 = Total2dValues(&(temps[0]),&(temps[2]),1,0);
     mz = tot1 / tot2;
     Mult2dByConstant(temps[2],temps[0],mz);
     Sub2ds(scans[scan_num],scans[scan_num],temps[2]);
    }
 return NO_ERROR;
}


/*----------------------------------------------------------------------------
SliceMotionCorrection - ???
----------------------------------------------------------------------------*/

int
SliceMotionCorrection(fid,hdr,scans,num_scans,slice_num,echo_num,temps,num_temps,
                      min,max)
 FILE *fid;
 struct hdr_rec *hdr;
 struct buf_2d_rec **scans,**temps;
 unsigned long num_scans,slice_num,echo_num,num_temps;
 double *min,*max;
{
 int err;
 unsigned long lcv;
 double min1,max1;

 PRT(printf("SliceMotionCorrection...\n"))
 if ((err = rd_scans(fid,hdr,scans,num_scans,slice_num,echo_num)) != NO_ERROR)
    return err;
 motion_correction(scans,num_scans,temps,num_temps);
 for (lcv = 0; lcv < num_scans; lcv++)
    {MinMax2dValues(scans[lcv],0,(double) .0,(double) .0,&min1,&max1);
     if (lcv == 0)
        {*min = min1;
         *max = max1;}
       else
        {if (min1 < *min) *min = min1;
         if (max1 < *max) *max = max1;}
    }
 PRT(printf("SliceMotionCorrection...*min = %f  *max = %f\n",*min,*max))
 return NO_ERROR;
}


/*----------------------------------------------------------------------------
correlation_coeffient - ???
----------------------------------------------------------------------------*/

int
correlation_coeffient(scans,num_scans,temps,num_temps,
                      is_ref_wave_form,phases,phases_sd,is_normalized,slice)
 struct buf_2d_rec **scans,**temps,*slice;
 float *phases,phases_sd;
 unsigned long num_scans,num_temps;
 int is_ref_wave_form,is_normalized;
{
 unsigned long scan_num;
 double min,max;

 PRT(printf("correlation_coeffient...is_ref = %d phases_sd = %f is_normalized = %d\n",
       is_ref_wave_form,phases_sd,is_normalized))
 Add2dArys(temps[0],scans,num_scans,2);
 MinMax2dValues(temps[0],0,(double) .0,(double) .0,&min,&max);
 PRT(printf("mean  min = %f max = %f\n",min,max))
 for (scan_num = 0; scan_num < num_scans; scan_num++)
    Sub2ds(scans[scan_num],scans[scan_num],temps[0]);

 if (!(is_ref_wave_form && !is_normalized))
    {Add2dArys(temps[1],scans,num_scans,1);
     Sqrt2d(temps[1],temps[1]);}

 if (is_ref_wave_form)
    for (scan_num = 0; scan_num < num_scans; scan_num++)
       Mult2dByConstant(scans[scan_num],scans[scan_num],(double) phases[scan_num]);
 Add2dArys(temps[2],scans,num_scans,0);

 if (is_ref_wave_form)
    {if (is_normalized)
        {Div2ds(slice,temps[2],temps[1],1);
         Div2dByConstant(slice,slice,(double) phases_sd,1);}
       else Div2dByConstant(slice,temps[2],(double) (phases_sd * phases_sd),1);
    }
   else Div2ds(slice,temps[2],temps[1],1);
 MinMax2dValues(slice,0,(double) .0,(double) .0,&min,&max);
 PRT(printf("slice min = %f max = %f\n",min,max))

 return NO_ERROR;
}


/*----------------------------------------------------------------------------
setup_ref_waveform - ???
----------------------------------------------------------------------------*/

int
setup_ref_waveform(phases,phases_sd,num_phases,right_shift,num_scans)
 float **phases,*phases_sd;
 unsigned long num_phases,right_shift,num_scans;
{
 int err;
 unsigned long lcv,cnt,num,last,total;
 float flip;
 double val,ave;

 total = 0;
 if ((err = get_ptr(phases,&total,num_scans,sizeof(**phases))) != NO_ERROR)
    return err;
 PRT(printf("setup_ref_waveform phases = %u num_phases = %d  num_scans= %d\n",
     phases,num_phases,num_scans))
 last = num_scans / num_phases;
 flip = 1.0;
 for (num = 0,lcv = 0; lcv < num_phases; lcv++)
    {for (cnt = 0; cnt < last; cnt++,num++)
        {if (num >= num_scans) break;
         (*phases)[num] = flip;}
     flip *= -1.0;}

 if (right_shift)
    for (lcv = num_scans - 1; lcv >= right_shift; lcv--)
       (*phases)[lcv] = (*phases)[lcv - right_shift];
 for (val = ave = .0,lcv = 0; lcv < num_scans; lcv++)
    ave += (*phases)[lcv];
 ave /= (double) num_scans;
 PRT(printf("ave = %f\n",ave))
 for (lcv = 0; lcv < num_scans; lcv++)
    (*phases)[lcv] = (*phases)[lcv] - ave;

 for (val = .0,lcv = 0; lcv < num_scans; lcv++)
     val += (*phases)[lcv] * (*phases)[lcv];
 *phases_sd = (float) sqrt(val);
 PRT(printf("num_phases = %d num_scans = %d *phases_sd = %f\n",num_phases,num_scans,*phases_sd))
 PRT(for (lcv = 0; lcv < num_scans; lcv++)
        {printf("%-d) %.4f ",lcv,(*phases)[lcv]);
         if (((lcv + 1) % 5) == 0) printf("\n");
        }
     printf("\n");)
 return NO_ERROR;
}


/*----------------------------------------------------------------------------
SliceCorrelationCoeff - ???
----------------------------------------------------------------------------*/

int
SliceCorrelationCoeff(fid,hdr,scans,num_scans,slice_num,echo_num,temps,num_temps,
                      is_ref_wave_form,phases,phases_sd,is_normalized,
                      is_motion_correction,slice)
 FILE *fid;
 struct hdr_rec *hdr;
 struct buf_2d_rec **scans,**temps,*slice;
 unsigned long num_scans,slice_num,echo_num,num_temps;
 float *phases,phases_sd;
 int is_ref_wave_form,is_motion_correction;
{
 int err;
 
 PRT(printf("SliceCorrelationCoeff...is_motion_correction = %d is_ref_wave_form = %d\n",is_motion_correction,is_ref_wave_form))
 if ((err = rd_scans(fid,hdr,scans,num_scans,slice_num,echo_num)) != NO_ERROR)
    return err;
 if (is_motion_correction)
    motion_correction(scans,num_scans,temps,num_temps);
 PRT(printf("SliceCorrelationCoeff...is_normalized = %d\n",is_normalized))
 err = correlation_coeffient(scans,num_scans,temps,num_temps,
                    is_ref_wave_form,phases,phases_sd,is_normalized,slice);
 PRT(printf("SliceCorrelationCoeff...correlation_coeffient..err = %d\n",err))
 return err;
}


/*----------------------------------------------------------------------------
SingleSamplePairTest - ???

seq == 0 then activation/no-activation pair
  else no-activation/activation pair
----------------------------------------------------------------------------*/

int
SingleSamplePairTest(scans,num_scans,temps,num_temps,num_pairs,seq,slice)
 struct buf_2d_rec **scans,**temps,*slice;
 unsigned long num_scans,num_temps;
 int num_pairs,seq;
{
 unsigned long num_seq_scans,lcv,cnt,indx,num_seqs;

 num_seqs = num_pairs * 2;
 num_seq_scans = num_scans / num_seqs;
 PRT(printf("SingleSamplePairTest...num_seqs = %d num_seq_scans = %d\n",
         num_seqs,num_seq_scans))
 for (indx = cnt = lcv = 0; lcv < num_seqs; lcv++,cnt += num_seq_scans,indx++)
    Add2dArys(scans[indx],&(scans[cnt]),num_seq_scans,0);

 PRT(printf("after Add2dArys indx = %d\n",indx))
 if (num_pairs == 1) 
    {if (seq == 1) Sub2ds(slice,scans[0],scans[1]);
       else Sub2ds(slice,scans[1],scans[0]);
     return NO_ERROR;
    }
 for (indx = lcv = 0; lcv < num_seqs; lcv += 2,indx++)
    {if (seq == 1) Sub2ds(scans[indx],scans[lcv],scans[lcv+1]);
       else Sub2ds(scans[indx],scans[lcv+1],scans[lcv]);
    }
 PRT(printf("after Sub2ds num_pairs = %d indx = %d\n",num_pairs,indx))
 Add2dArys(temps[0],scans,num_pairs,2);
 for (lcv = 0; lcv < num_pairs; lcv++)
    Sub2ds(scans[lcv],scans[lcv],temps[0]);
 Add2dArys(temps[1],scans,num_pairs,1);
 Div2dByConstant(temps[1],temps[1],(double) ((num_pairs - 1) * num_pairs),1);
 Sqrt2d(temps[1],temps[1]);
 Div2ds(slice,temps[0],temps[1],1);
 return NO_ERROR;
}


/*----------------------------------------------------------------------------
SingleSamplePairPtsTest - ???
----------------------------------------------------------------------------*/

int
SingleSamplePairPtsTest(scans,num_scans,temps,num_temps,num_pairs,seq,slice)
 struct buf_2d_rec **scans,**temps,*slice;
 unsigned long num_scans,num_temps;
 int num_pairs,seq;
{
 unsigned long num_seq_scans,next,lcv,cnt,last,indx,num_seqs;

 num_seqs = num_pairs * 2;
 num_seq_scans = num_scans / num_seqs;
 PRT(printf("SingleSamplePairPtsTest...num_pairs = %d num_seqs = %d num_seq_scans = %d\n",
         num_pairs,num_seqs,num_seq_scans))
 for (indx = lcv = 0; lcv < num_pairs; lcv++)
    {cnt = lcv * num_seq_scans * 2;
     last = next = cnt + num_seq_scans;
     if (seq == 1) 
        {PRT(printf("seq = 1 indx = %d cnt = %d next = %d\n",indx,cnt,next))
         for (; cnt < last; cnt++,next++,indx++)
            Sub2ds(scans[indx],scans[cnt],scans[next]);}
       else
        {PRT(printf("seq = 0 indx = %d next = %d cnt = %d\n",indx,next,cnt))
         for (; cnt < last; cnt++,next++,indx++)
            Sub2ds(scans[indx],scans[next],scans[cnt]);}
    }
 num_pairs *= num_seq_scans;
 PRT(printf("after Add2dArys indx = %d num_pairs = %d\n",indx,num_pairs))
 if (num_pairs == 1) 
    {if (seq == 1) Sub2ds(slice,scans[0],scans[1]);
       else Sub2ds(slice,scans[1],scans[0]);
     return NO_ERROR;
    }
 Add2dArys(temps[0],scans,num_pairs,2);
 for (lcv = 0; lcv < num_pairs; lcv++)
    Sub2ds(scans[lcv],scans[lcv],temps[0]);
 Add2dArys(temps[1],scans,num_pairs,1);
 Div2dByConstant(temps[1],temps[1],(double) ((num_pairs - 1) * num_pairs),1);
 Sqrt2d(temps[1],temps[1]);
 Div2ds(slice,temps[0],temps[1],1);
 return NO_ERROR;
}


/*----------------------------------------------------------------------------
SingleSampleTTest - ???
----------------------------------------------------------------------------*/

int
SingleSampleTTest(fid,hdr,scans,num_scans,slice_num,echo_num,temps,num_temps,
                  num_pairs,seq,is_pair_pts,is_motion_correction,slice)
 FILE *fid;
 struct hdr_rec *hdr;
 struct buf_2d_rec **scans,**temps,*slice;
 unsigned long num_scans,slice_num,echo_num,num_temps;
 int num_pairs,seq,is_pair_pts,is_motion_correction;
{
 int err;
 
 PRT(printf("SingleSampleTTest...motion = %d num_pairs = %d seq = %d is_pair_pts = %d\n",
         is_motion_correction,num_pairs,seq,is_pair_pts))
 if ((err = rd_scans(fid,hdr,scans,num_scans,slice_num,echo_num)) != NO_ERROR)
    return err;
 if (is_motion_correction)
    motion_correction(scans,num_scans,temps,num_temps);
 PRT(printf("SingleSampleTTest...\n"))
 if (is_pair_pts)
    err = SingleSamplePairPtsTest(scans,num_scans,temps,num_temps,
                num_pairs,seq,slice);
   else err = SingleSamplePairTest(scans,num_scans,temps,num_temps,
                  num_pairs,seq,slice);
 PRT(printf("SingleSampleTTest.....err = %d\n",err))
 return err;
}


/*----------------------------------------------------------------------------
setup_cc_buf - ???
----------------------------------------------------------------------------*/

int
setup_cc_buf(hdr,scans,num_scans)
 struct hdr_rec *hdr;
 struct buf_2d_rec ***scans;
 unsigned long *num_scans;
{
 unsigned long lcv,num,type4,depth;
 struct data_info_rec *info;
 int err;

 PRT(printf("setup_cc_buf...\n"))
 info = &(hdr->data_info);
 num = NumberOfSliceScans(info) + 4;
 *num_scans = 0;
 if ((err = get_handle(scans,num_scans,num,sizeof(***scans))) != NO_ERROR)
    return err;
 PRT(printf("setup_cc_buf...1\n"))

 for (lcv = 0; lcv < *num_scans; lcv++)
    init_buf_2d_rec((*scans)[lcv]);

 type4 = InputDataTypeToBufType(info);
 if (type4 == BUF_VALUE_DOUBLE)
    depth = 8 * sizeof(double);
   else 
    {type4 = BUF_VALUE_FLOAT;
     depth = 8 * sizeof(float);}
 for (lcv = 0; lcv < *num_scans; lcv++)
    {if ((err = SetupBuf2dRec((*scans)[lcv],
              info->dimx,info->dimy,depth,type4)) != NO_ERROR)
        break;
    }
 PRT(printf("setup_cc_buf...err = %d\n",err))
 return err;
}


/*----------------------------------------------------------------------------
StdSlice - ???
----------------------------------------------------------------------------*/

int
StdSlice(fid,hdr,scans,num_scans,slice_num,echo_num,temps,num_temps,slice)
 FILE *fid;
 struct hdr_rec *hdr;
 struct buf_2d_rec **scans,**temps,*slice;
 unsigned long num_scans,slice_num,echo_num,num_temps;
{
 int err;
 unsigned long scan_num;

 PRT(printf("StdSlice... \n"))
 if ((err = rd_scans(fid,hdr,scans,num_scans,slice_num,echo_num)) != NO_ERROR)
    return err;

 Add2dArys(temps[0],scans,num_scans,2);
 for (scan_num = 0; scan_num < num_scans; scan_num++)
    Sub2ds(scans[scan_num],scans[scan_num],temps[0]);
 Add2dArys(temps[0],scans,num_scans,1);
 Div2dByConstant(temps[0],temps[0],(double) (num_scans - 1),1);
 Sqrt2d(slice,temps[0]);
 return NO_ERROR;
}
