/****************************************************************************
histo_segment.c - ???

Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.
****************************************************************************/

static const char rcsid[] = "$Id: histo_segment.c,v 1.7 1999/07/07 17:41:17 hughett Exp $ Release $Name: nria_1_0_6 $";

#include "mri_format.h"
#include "nria_procedures.h"

/*
#define PRT(x) x; fflush(stdout);
#define PRINT(x) x; fflush(stdout);
#define TRACE(x) x; fflush(stdout);
*/
#define PRINT(x)
#define TRACE(x)
#define PRT(x)



/* Functions defined in this file */
int enter_3_center_colinear_pts(struct slice_roi_rec *slice_roi, char *str,
    char *str1, char *str2, long midx01, long midy01, long midx02,
    long midy02);
int is_colinear(long x, long y, long x1, long y1, long x2, long y2);
int enter_colinear_pts(struct slice_roi_rec *slice_roi, long avex, long avey,
    char *str, long avex1, long avey1, char *str1, long avex2, long avey2,
    char *str2, long midx01, long midy01, long midx02, long midy02,
    long midx12, long midy12);
void init_neighbor_elem_rec(struct neighbor_elem_rec *member);
void free_neighbor_elem_rec(struct neighbor_elem_rec *member);
void free_all_neighbor_elem_recs(struct neighbor_elem_rec *members,
    unsigned long num_members);
int enter_member(struct neighbor_elem_rec *member,
    struct pixel_center_rec *center1, struct pixel_center_rec *center2);
int might_be_neighbor(struct histo_table_elem_rec *alias_of_elem,
    struct histo_table_elem_rec *elem);
int neighbor(struct histo_table_elem_rec **elems, unsigned long row,
    unsigned long num_rows, struct histo_table_elem_rec *histo_elem1,
    struct histo_table_elem_rec *histo_elem2, struct draw_mask_rec *mask,
    struct buf_2d_rec *bitmap, struct buf_2d_rec *bitmap1, int *is_neighbor);
int get_2_closest_neighbors(struct histo_table_elem_rec **elems,
    unsigned long row, unsigned long num_rows, struct draw_mask_rec *mask,
    struct buf_2d_rec *bitmap, struct buf_2d_rec *bitmap1);
int is_overlap_but_not_joint(struct buf_2d_rec *dest,
    struct buf_2d_rec *source, struct int_bound_rec *source_dim);
int get_the_mid_pt_centers(struct neighbor_elem_rec *member,
    struct pixel_center_rec *center, unsigned long x, unsigned long y,
    struct pixel_center_rec **center1, struct pixel_center_rec **center2);
int get_2_intersections(int x, int y, int angular_gradient, int left,
    int right, int top, int bottom, int *x1, int *y1, int *x2, int *y2);
int valid_intersection(struct histo_table_elem_rec *elem,
    struct draw_mask_rec *mask, struct buf_2d_rec *bitmap, int x, int y,
    int *inter_x, int *inter_y);
void get_corner_pts_for_left_side(int left, int top, int right, int bottom,
    int x1, int y1, int x2, int y2, int ptsx[], int ptsy[], int *num_pts);
void get_corner_pts_for_right_side(int left, int top, int right, int bottom,
    int x1, int y1, int x2, int y2, int ptsx[], int ptsy[], int *num_pts);
void get_corner_pts(int which_side, int centerx, int centery, int left,
    int top, int right, int bottom, int ori_x1, int ori_y1, int ori_x2,
    int ori_y2, int ptsx[], int ptsy[], int *num_pts);
int get_center_side(struct curve_rec *curve, struct draw_mask_rec *mask,
    int centerx, int centery, int left, int top, int right, int bottom,
    int *which_size);
int close_partition(struct curve_rec *curve, struct draw_mask_rec *mask,
    int avex, int avey, unsigned long dimx, unsigned long dimy,
    int x1, int y1, int x2, int y2);
int closed_unbounded_centers(struct histo_table_elem_rec *elem,
    struct curve_rec *curve, struct draw_mask_rec *mask,
    struct buf_2d_rec *bitmap);
int get_pixel_type_regions(struct slice_roi_rec *slice_roi,
    struct neighbor_elem_rec *members, struct pixel_center_rec *centers,
    int num_centers);
int get_complete_pixel_type_regions(struct slice_roi_rec *slice_roi,
    struct histo_table_elem_rec **elems, int num_rows,
    struct draw_mask_rec *mask, struct buf_2d_rec *bitmap);
int PartitionPixelTypes(struct histo_table_elem_rec **elems,
    unsigned long num_rows, struct histo_params_rec *histo,
    struct neighbor_elem_rec *members, struct slice_roi_rec *slice_roi);



/*-------------------------------------------------------------------------
enter_3_center_colinear_pts - ???
-------------------------------------------------------------------------*/

int
enter_3_center_colinear_pts(slice_roi,
                            str,str1,str2,midx01, midy01,midx02,midy02)
 struct slice_roi_rec *slice_roi;
 char *str,*str1,*str2;
 long midx01,midx02,midy01,midy02;
{
 int err;

 if ((err = EnterPtsIntoSliceRoiRec(slice_roi,NULL,str,
                  (unsigned long) DRAW_LINE,0,2, 
               (double) midx01, (double) midy01,(double) midx02,(double) midy02))
                    != NO_ERROR) return err;
 return NO_ERROR;
}


/*-------------------------------------------------------------------------
is_colinear - ???
-------------------------------------------------------------------------*/

int
is_colinear(x,y,x1,y1,x2,y2)
 long x,y,x1,y1,x2,y2;
{
 float m,m1;

 if ((x == x1) || (x == x2))
    {if (x1 == x2) return 1;
     return 0;}

 if ((y == y1) || (y == y2))
    {if (y1 == y2) return 1;
     return 0;}

 m = (y - y1) / (float) (x - x1);
 m1 = (y - y2) / (float) (x - x2);
 if (m == m1) return 1;
   else return 0;
}


/*-------------------------------------------------------------------------
enter_colinear_pts - ???
-------------------------------------------------------------------------*/

int
enter_colinear_pts(slice_roi,avex,avey,str,avex1,avey1,str1,avex2,avey2,str2,
                   midx01, midy01,midx02,midy02,midx12,midy12)
 struct slice_roi_rec *slice_roi;
 long avex,avey,avex1,avey1,avex2,avey2,
              midx01,midx12,midx02,midy01,midy12,midy02;
 char *str,*str1,*str2;
{
/*
 if (is_between(avex,avey,avex1,avey1,avex2,avey2))
*/
 if (is_colinear(avex,avey,avex1,avey1,avex2,avey2))
    return enter_3_center_colinear_pts(slice_roi,
                            str,str1,str2,midx01,midy01,midx02,midy02);
 else
     return ~NO_ERROR;    /* FIXME:  Guessed return value.  -PWH 19990412 */
}


/*-------------------------------------------------------------------------
init_neighbor_elem_rec - ???
-------------------------------------------------------------------------*/

void
init_neighbor_elem_rec(member)
 struct neighbor_elem_rec *member;
{
 member->num_neighbors = 0;
 member->neighbors = NULL;
}


/*-------------------------------------------------------------------------
free_neighbor_elem_rec - ???
-------------------------------------------------------------------------*/

void
free_neighbor_elem_rec(member)
 struct neighbor_elem_rec *member;
{
 if (member->num_neighbors > 0) c_free(member->neighbors);
 init_neighbor_elem_rec(member);
}


/*-------------------------------------------------------------------------
free_all_neighbor_elem_recs - ???
-------------------------------------------------------------------------*/

void
free_all_neighbor_elem_recs(members,num_members)
 struct neighbor_elem_rec *members;
 unsigned long num_members;
{
 unsigned long lcv;

 for (lcv = 0; lcv < num_members; lcv++)
    free_neighbor_elem_rec(&(members[lcv]));
}


/*-------------------------------------------------------------------------
enter_member - ???
-------------------------------------------------------------------------*/

int
enter_member(member,center1,center2)
 struct neighbor_elem_rec *member;
 struct pixel_center_rec *center1,*center2;
{
 int err;
 unsigned long indx;

 for (indx = 0; indx < member->num_neighbors; indx++)
    {if (member->neighbors[indx].neighbor1 == center1)
        {if (member->neighbors[indx].neighbor2 == center2)
            {
             return NO_ERROR;}
        }
       else if (member->neighbors[indx].neighbor1 == center2)
        {if (member->neighbors[indx].neighbor2 == center1)
            {
             return NO_ERROR;}
        }
    }
 indx = member->num_neighbors;
 get_ptr_memory(&(member->neighbors),&(member->num_neighbors),
           (unsigned long) 1,(size_t) sizeof(struct neighbor_rec),&err);
 if (err) return err;
 TRACE(printf("enter_member = %d roi labels 1 = %s   2 = %s\n",
          member,center1->roi_label,center2->roi_label))
 member->neighbors[indx].status = 0;
 member->neighbors[indx].neighbor1 = center1;
 member->neighbors[indx].neighbor2 = center2;
 return NO_ERROR;
}


/*-------------------------------------------------------------------------
might_be_neighbor - ???
-------------------------------------------------------------------------*/

int
might_be_neighbor(alias_of_elem,elem)
 struct histo_table_elem_rec *alias_of_elem,*elem;
{
 unsigned long cnt,indx;
 int ok;

 if (elem->common->status & NO_MORE_NEIGHBORS) return 0;
 if (alias_of_elem->common->status & NO_MORE_NEIGHBORS) return 0;
 if (alias_of_elem->common->status & NOT_NEIGHBOR) return 0;
 ok = 1;
 for (cnt = 0; (cnt < elem->member->num_neighbors) && ok; cnt++)
    for (indx = 0; (indx < alias_of_elem->member->num_neighbors) && ok;indx++)
       if ((elem->member->neighbors[cnt].neighbor1 ==
            alias_of_elem->member->neighbors[indx].neighbor1) &&
           (elem->member->neighbors[cnt].neighbor2 ==
            alias_of_elem->member->neighbors[indx].neighbor2))
          {alias_of_elem->common->status |= NOT_NEIGHBOR;
           ok = 0;}
 return ok;
}


/*-------------------------------------------------------------------------
neighbor - ???
-------------------------------------------------------------------------*/

int
neighbor(elems,row,num_rows,histo_elem1,histo_elem2,mask,bitmap,bitmap1,
         is_neighbor)
 struct histo_table_elem_rec **elems,*histo_elem1,*histo_elem2;
 unsigned long row,num_rows;
 struct buf_2d_rec *bitmap,*bitmap1;
 struct draw_mask_rec *mask;
 int *is_neighbor;
{
 struct curve_rec curve,*curve_ptr;
 unsigned short status;
 struct int_bound_rec dimension;
 int err;

 *is_neighbor = NOT_NEIGHBOR;
 init_curve_rec(&curve);
 if ((err = EnterCurvePts(&curve,&status,NULL,4,
        (double) elems[row][0].center->avex,(double) elems[row][0].center->avey,
        (double) histo_elem1->center->avex,(double) histo_elem1->center->avey,
        (double) histo_elem2->center->avex,(double) histo_elem2->center->avey,
        (double) elems[row][0].center->avex,(double) elems[row][0].center->avey))
               != NO_ERROR)
    return err;

 clear_draw_masks(mask,1);
 curve_ptr = &curve;
 if (((err = FillEexclusiveObjRegions(&curve_ptr,1,NULL,
         &(mask->mask._2d),1,0)) != NO_ERROR) ||
     ((err = XorBitmapByCurves(&(mask->mask._2d),&curve_ptr,1,NULL,1))
                 != NO_ERROR))
    {free_curve_rec(&curve);
     return NO_ERROR;}

 TRACE(printf("after xor_paint_filled_mask_to_bitmap\n"))
 FindCurveLongMinMax(&curve,NULL,&(dimension.left),&(dimension.right),
                           &(dimension.top),&(dimension.bottom),0,0,0,0);
                 
 if (!is_overlap_but_not_joint(bitmap,&(mask->mask._2d),&dimension))
    {superimpose_bitmap_region(bitmap,&(mask->mask._2d),&dimension);
     *is_neighbor = NEIGHBOR;}
 free_curve_rec(&curve);
 return NO_ERROR;
}


/*-------------------------------------------------------------------------
get_2_closest_neighbors - ???
-------------------------------------------------------------------------*/

int
get_2_closest_neighbors(elems,row,num_rows,mask,bitmap,bitmap1)
 struct histo_table_elem_rec **elems;
 struct buf_2d_rec *bitmap,*bitmap1;
 struct draw_mask_rec *mask;
 unsigned long row,num_rows;
{
 unsigned long lcv,indx;
 int is_neighbor;
 int err;

 for (lcv = 1; lcv < num_rows; lcv++)
    {
     TRACE(
       printf("get_2_closest_neighbors [0] label = %s [lcv] = %s status = %d\n",
            elems[row][0].center->roi_label,
            elems[row][lcv].center->roi_label,
            (int) elems[row][lcv].common->status))

     if (!(elems[row][lcv].common->status & NEIGHBOR) &&
         might_be_neighbor(&(elems[row][lcv]),&(elems[row][0])))
        {for (indx = 1; indx < num_rows; indx++)
            if ((indx != lcv) &&
               ((elems[row][indx].common->status & NEIGHBOR) ||
                 might_be_neighbor(&(elems[row][indx]),&(elems[row][0]))))
               {

                if ((elems[elems[row][lcv].row_indx][indx].common->status
                    & NEIGHBOR) ||
                    might_be_neighbor(&(elems[elems[row][lcv].row_indx][indx]),
                      &(elems[elems[row][lcv].row_indx][0])))
                   if ((err = neighbor(elems,row,num_rows,
                                   &(elems[row][lcv]),
                                   &(elems[row][indx]),mask,bitmap,bitmap1,
                                   &is_neighbor)) == NO_ERROR)
                      if (is_neighbor == NEIGHBOR)
                         {
                          if (((err = enter_member(elems[row][0].member,
                                elems[row][lcv].center,
                                elems[row][indx].center)) != NO_ERROR) ||
                             ((err = enter_member(elems[row][lcv].member,
                                 elems[row][0].center,elems[row][indx].center))
                                 != NO_ERROR) ||
                             ((err = enter_member(elems[row][indx].member,
                                 elems[row][0].center,elems[row][lcv].center))
                                 != NO_ERROR))
                            return err;

                          elems[row][lcv].common->status |= NEIGHBOR;
                          elems[row][indx].common->status |= NEIGHBOR;
                          elems[elems[row][lcv].row_indx][indx].common->status
                              |= NEIGHBOR;

                          return NO_ERROR;}
              }
          elems[row][lcv].common->status |= NOT_NEIGHBOR;
        }
    }

 elems[row][0].common->status |= NO_MORE_NEIGHBORS;
 TRACE(printf("NO_MORE_NEIGHBORS...\n"))
 return NOT_FOUND;
}


/*-------------------------------------------------------------------------
 - ???

  return 1 if source overlaps with dest, else return 0.
-------------------------------------------------------------------------*/

int
is_overlap_but_not_joint(dest,source,source_dim)
 struct buf_2d_rec *dest,*source;
 struct int_bound_rec *source_dim;
{
 unsigned long row,col,col_bit,indx,start_bit,end_bit,cnt;
 static unsigned char _1_bit[] = {0x80,0x40,0x20,0x10,0x08,0x04,0x02,0x01};
 unsigned char source_byte,dest_byte;
 unsigned long start_byte,start_col_bit,stop_byte;
 unsigned long left,right,top,bottom;
 unsigned long loop,lcv,stop_row,stop_col,start_col,num_pts,overlap;
 struct int_bound_rec bound;

 setup_int_bound_rec(&bound,0,0,dest->dimx - 1,dest->dimy - 1);
 if (source_dim == NULL)
    valid_bound(&bound,&bound,&left,&right,&top,&bottom);
   else valid_bound(source_dim,&bound,&left,&right,&top,&bottom);

 start_byte = left / 8;
 start_col_bit = start_byte * 8;
 stop_byte = right / 8 + 1;
 for (row = top; row <= bottom; row++)
    {
     start_bit = left - start_col_bit;
     end_bit = 8;
     for (col_bit = start_col_bit,col = start_byte;
             col < stop_byte; col++,col_bit += 8,start_bit = 0)
        if (source_byte = source->buf[row][col])
           {dest_byte = dest->buf[row][col];
            if ((col_bit + 8) > right) end_bit = right - col_bit + 1;
            for (indx = col_bit + start_bit,cnt = start_bit;
                                cnt < end_bit; cnt++,indx++)
               if (source_byte & _1_bit[cnt])
                  {if (dest_byte & _1_bit[cnt])
                      {
                       if (row > 0) loop = row - 1;
                         else loop = 0;
                       if ((row + 1) < dest->dimy) stop_row = row + 1;
                         else stop_row = row;
                       if (indx > 0) start_col = indx - 1;
                         else start_col = indx;
                       if ((indx + 1) < dest->dimx) stop_col = indx + 1;
                          else stop_col = indx;
                       for (overlap = num_pts = 0; loop <= stop_row; loop++)
                          for (lcv = start_col; lcv <= stop_col; lcv++)
                             {num_pts++;
                              if (get_1d_buf_value(dest->buf[loop],lcv,
                                    dest->pixel_depth))
                                 overlap += 1;}
                       if (overlap == num_pts)
                          {
                           return 1;}
                      }
                  }
            }
    }
 return 0;
}


/*-------------------------------------------------------------------------
get_the_mid_pt_centers - ???
-------------------------------------------------------------------------*/

int
get_the_mid_pt_centers(member,center,x,y,center1,center2)
 struct neighbor_elem_rec *member;
 struct pixel_center_rec *center;
 unsigned long x,y;
 struct pixel_center_rec **center1,**center2;
{
 unsigned long cnt;
 long avex,avey,avex1,avey1,avex2,avey2,
              midx01,midx02,midy01,midy02;

 TRACE(printf("get_the_mid_pt_centers begin\n"))
 for (cnt = 0; cnt < member->num_neighbors; cnt++)
    {avex = center->avex;
     avey = center->avey;
     avex1 = member->neighbors[cnt].neighbor1->avex;
     avey1 = member->neighbors[cnt].neighbor1->avey;
     midx01 = (long) ULongRound((double) ((avex1 + avex) / (double) 2));
     midy01 = (long) ULongRound((double) ((avey1 + avey) / (double) 2));
     if ((midx01 == x) && (midy01 == y))
        {*center1 = center;
         *center2 = member->neighbors[cnt].neighbor1;
         return NO_ERROR;}

     avex2 = member->neighbors[cnt].neighbor2->avex;
     avey2 = member->neighbors[cnt].neighbor2->avey;
     midx02 = (long) ULongRound((double) ((avex + avex2) / (double) 2));
     midy02 = (long) ULongRound((double) ((avey + avey2) / (double) 2));
     if ((midx02 == x) && (midy02 == y))
        {*center1 = center;
         *center2 = member->neighbors[cnt].neighbor2;
         return NO_ERROR;}
    }
 TRACE(printf("get_the_mid_pt_centers end\n"))
 return CANCEL;

}


/*-------------------------------------------------------------------------
get_2_intersections - ???
-------------------------------------------------------------------------*/

int
get_2_intersections(x,y,angular_gradient,left,right,top,bottom,
                    x1,y1,x2,y2)
 int x,y,angular_gradient,left,right,top,bottom;
 int *x1,*y1,*x2,*y2;
{
 float gradient,c;
 int num;
 long pt[8],ok[4],lcv,cnt,found;

 PRT(printf("get_2_intersections x = %d y = %d angular_gradient = %d\n",x,y,angular_gradient))
 if ((angular_gradient % 180) == 0)
    {*x1 = left;
     *x2 = right;
     *y1 = *y2 = y;}
   else if ((angular_gradient % 90) == 0)
    {*x1 = *x2 = x;
     *y1 = top;
     *y2 = bottom;}
   else
    {gradient = (float) tan((double) radian((float) angular_gradient));
     c = y - gradient * x;

     pt[1] = top;
     pt[0]  = (long) rint((double) ((top - c) / gradient));
     pt[3] = bottom;
     pt[2] = (long) rint((double) ((bottom - c) / gradient));
     pt[4] = left;
     pt[5] = (long) rint((double) (gradient * left + c));
     pt[6] = right;
     pt[7] = (long) rint((double) (gradient * right + c));
     num = 8;
     for (lcv = 0; lcv < 4; lcv += 1)
        ok[lcv] = 1;
     for (lcv = 0; lcv <= 6; lcv += 2)
        for (cnt = lcv + 2; cnt <= 6; cnt += 2)
           if ((pt[lcv] == pt[cnt]) && (pt[lcv+1] == pt[cnt+1]))
              ok[cnt / 2] = 0;

     PRT(printf("gradient = %f   c = %f\n",gradient,c))
     PRT(
        for (lcv = 0; lcv < 8; lcv += 1)
           printf("%d) pt[] = %d ok = %d\n",lcv,pt[lcv],ok[lcv / 2]);
        )
     for (found = 0,lcv = 0; lcv <= 6; lcv += 2)
        if (ok[lcv / 2] &&
            (pt[lcv] >= left) && (pt[lcv] <= right) &&
            (pt[lcv + 1] >= top) && (pt[lcv + 1] <= bottom))
           {*x1 = pt[lcv];
            *y1 = pt[lcv + 1];
            PRT(printf("pick pt[%d] = %d  pt[%d] = %d\n",
                  lcv,pt[lcv],lcv+1,pt[lcv + 1]);)
            found = 1;
            break;}
     for (lcv += 2; lcv <= 6; lcv += 2)
        if (ok[lcv / 2] &&
            (pt[lcv] >= left) && (pt[lcv] <= right) &&
            (pt[lcv + 1] >= top) && (pt[lcv + 1] <= bottom))
           {*x2 = pt[lcv];
            *y2 = pt[lcv + 1];
            PRT(printf("pick == pt[%d] = %d  pt[%d] = %d\n",
                  lcv,pt[lcv],lcv+1,pt[lcv + 1]);)
            found += 1;
            break;}
     PRT(printf("x1 = %d y1 = %d    x2 = %d  y2 = %d\n",*x1,*y1,*x2,*y2))
     if (found == 2) return NO_ERROR;
     return CANCEL;
    }
 return NO_ERROR;
}


/*-------------------------------------------------------------------------
valid_intersection - ???
-------------------------------------------------------------------------*/

int
valid_intersection(elem,mask,bitmap,x,y,inter_x,inter_y)
 struct histo_table_elem_rec *elem;
 struct draw_mask_rec *mask;
 struct buf_2d_rec *bitmap;
 int x,y,*inter_x,*inter_y;
{
 int err;
 int pt[4],lcv,found;
 struct pixel_center_rec *center1,*center2;
 struct curve_rec curve;
 unsigned short status;
 struct int_bound_rec dimension;
 int angular_gradient;

 TRACE(printf("\nvalid_intersection BEGIN x = %d y = %d\n",x,y))
 if ((err = get_the_mid_pt_centers(elem->member,elem->center,
              x,y,&center1,&center2)) == NO_ERROR)
    {
     angular_gradient = (int) degree((float)
           ((atan2((double) center1->gradient,(double) 1.0) +
             atan2((double) center2->gradient,(double) 1.0)) / 2.0) );

     TRACE(printf("label center = %s center1 = %s center2 = %s\n",
             elem->center->roi_label,center1->roi_label,center2->roi_label))
     TRACE(printf("angular_gradient = %d...g1 = %f  g2 = %f\n",angular_gradient,
            center1->gradient,center2->gradient))
     get_2_intersections(x,y,angular_gradient,
          (unsigned long) 0,(unsigned long) (bitmap->dimx - 1),
          (unsigned long) 0,(unsigned long) (bitmap->dimy - 1),&(pt[0]),
          &(pt[1]),&(pt[2]),&(pt[3]));}
   else 
    {TRACE(printf("valid_intersection error....no mid pt\n"))
     return err;}
 TRACE(printf("valid_intersection ready pt[0] = %d pt1 = %d pt2 = %d p3 = %d\n",pt[0],pt[1],pt[2],pt[3]))

 init_curve_rec(&curve);
 for (found = 0,lcv = 0; (lcv <= 2) && !found && (err == NO_ERROR); lcv += 2)
    {
     if ((err = EnterCurvePts(&curve,&status,NULL,2,
             (double) x,(double) y,
             (double) pt[lcv],(double) pt[lcv+1])) == NO_ERROR)
        {clear_draw_masks(mask,1);
         if ((err = DrawCurveRec(mask,&curve,NULL,DRAW_LINE)) == 
                          NO_ERROR)
            {FindCurveLongMinMax(&curve,NULL,
                     &(dimension.left),&(dimension.right),
                     &(dimension.top),&(dimension.bottom),0,0,0,0);
             if (!is_overlap_but_not_joint(bitmap,&(mask->mask._2d),&dimension))
                {found = 1;
                 *inter_x = pt[lcv];
                 *inter_y = pt[lcv+1];}
            }
         free_curve_rec(&curve);}
    }

 free_curve_rec(&curve);
 TRACE(printf("valid_intersection end err = %d\n\n\n",err))
 if ((err == NO_ERROR) && !found) return CANCEL;
 return err;
}


/*-------------------------------------------------------------------------
get_corner_pts_for_left_side - ???
-------------------------------------------------------------------------*/

void
get_corner_pts_for_left_side(left,top,right,bottom,
                   x1,y1,x2,y2,ptsx,ptsy,num_pts)
 int left,top,right,bottom,x1,y1,x2,y2,ptsx[],ptsy[],*num_pts;
{
 if (x1 != left)
    {ptsx[*num_pts] = left;
     ptsy[(*num_pts)++] = top;}
 if (x2 != left)
    {ptsx[*num_pts] = left;
     ptsy[(*num_pts)++] = bottom;}

 if (x2 == right)
    {if (y2 != bottom)
        {ptsx[*num_pts] = right;
         ptsy[(*num_pts)++] = bottom;}
    }

 if (x1 == right)
    {if (y1 != top)
        {ptsx[*num_pts] = right;
         ptsy[(*num_pts)++] = top;}
    }

}


/*-------------------------------------------------------------------------
get_corner_pts_for_right_side - ???
-------------------------------------------------------------------------*/

void
get_corner_pts_for_right_side(left,top,right,bottom,
                   x1,y1,x2,y2,ptsx,ptsy,num_pts)
 int left,top,right,bottom,x1,y1,x2,y2,ptsx[],ptsy[],*num_pts;
{
 if (x1 != right)
    {ptsx[*num_pts] = right;
     ptsy[(*num_pts)++] = top;}
 if (x2 != right)
    {ptsx[*num_pts] = right;
     ptsy[(*num_pts)++] = bottom;}

 if (x2 == left)
    {if (y2 != bottom)
        {ptsx[*num_pts] = left;
         ptsy[(*num_pts)++] = bottom;}
    }

 if (x1 == left)
    {if (y1 != top)
        {ptsx[*num_pts] = left;
         ptsy[(*num_pts)++] = top;}
    }

}


/*-------------------------------------------------------------------------
get_corner_pts - ???
-------------------------------------------------------------------------*/

#define LEFT_SIDE 1
#define RIGHT_SIDE 2
#define TOP_SIDE 3
#define BOTTOM_SIDE 4

void
get_corner_pts(which_side,centerx,centery,left,top,right,bottom,
               ori_x1,ori_y1,ori_x2,ori_y2,ptsx,ptsy,num_pts)
 int which_side;
 int centerx,centery,left,top,right,bottom,ori_x1,ori_y1,ori_x2,ori_y2,
     ptsx[],ptsy[],*num_pts;
{
 int x1,y1,x2,y2;

 if (ori_y1 == ori_y2)
    {y1 = y2 = ori_y1;
     if (ori_x1 > ori_x2)
        {x1 = ori_x2;
         x2 = ori_x1;}
       else
        {x1 = ori_x1;
         x2 = ori_x2;}
    }
   else if (ori_y1 > ori_y2)
    {x1 = ori_x2;
     x2 = ori_x1;
     y1 = ori_y2;
     y2 = ori_y1;
    }
   else
    {x1 = ori_x1;
     x2 = ori_x2;
     y1 = ori_y1;
     y2 = ori_y2;
    }

 *num_pts = 0;
 if ((x1 == x2) && ((x1 == left) || (x1 == right))) return;
 if ((y1 == y2) && ((y1 == top) || (y1 == bottom))) return;
 if (which_side == TOP_SIDE)
    {ptsx[*num_pts] = left;
     ptsy[(*num_pts)++] = top;
     ptsx[*num_pts] = right;
     ptsy[(*num_pts)++] = top;}
   else if (which_side == BOTTOM_SIDE)
    {ptsx[*num_pts] = left;
     ptsy[(*num_pts)++] = bottom;
     ptsx[*num_pts] = right;
     ptsy[(*num_pts)++] = bottom;}
   else if (which_side == LEFT_SIDE)
     get_corner_pts_for_left_side(left,top,right,bottom,
                   x1,y1,x2,y2,ptsx,ptsy,num_pts);
   else if (which_side == RIGHT_SIDE)
        get_corner_pts_for_right_side(left,top,right,bottom,
                   x1,y1,x2,y2,ptsx,ptsy,num_pts);

}


/*-------------------------------------------------------------------------
get_center_side - ???
-------------------------------------------------------------------------*/

int
get_center_side(curve,mask,centerx,centery,left,top,right,bottom,which_size)
 struct curve_rec *curve;
 struct draw_mask_rec *mask;
 int centerx,centery;
 int left,top,right,bottom;
 int *which_size;
{
 unsigned long lcv;
 int value,err;

 clear_draw_masks(mask,1);
 if ((err = DrawCurveRec(mask,curve,NULL,DRAW_LINE)) == NO_ERROR)
    {
     for (lcv = left; lcv < centerx; lcv += 8)
        {
         value = get_1d_buf_value(mask->mask._2d.buf[centery],
                    lcv / 8,(unsigned long) 8);
         if ((lcv + 8) > centerx)
            value = value >> (lcv + 8 - centerx);

         if (value)
            {*which_size = RIGHT_SIDE;
             TRACE(printf("x = %d RIGHT_SIDE centery = %d value = %d\n",
                                  lcv,centery,value))
             TRACE(pr_curve_rec_pts(curve))
             return NO_ERROR;}
        }
     for (lcv = (centerx / 8) * 8;lcv < right; lcv += 8)
        {
         value = get_1d_buf_value(mask->mask._2d.buf[centery],
                    lcv / 8,(unsigned long) 8);
         if (lcv < centerx)
            {value = value << (centerx - lcv);
             value = value >> (centerx - lcv);}
         if ((lcv + 8) > right)
            value = value >> (lcv + 8 - right);

         if (value)
            {*which_size = LEFT_SIDE;
             TRACE(printf("x = %d LEFT_SIDE centery = %d value = %d\n",
                                  lcv,centery,value))
             TRACE(pr_curve_rec_pts(curve))
             return NO_ERROR;}
        }
     for (lcv = 0; lcv < centery; lcv += 8)
        {
         value = get_1d_buf_value(mask->mask._2d.buf[lcv],
                    centerx / 8,(unsigned long) 8);
         if ((lcv + 8) > centery)
            value = value >> (lcv + 8 - centery);

         if (value)
            {*which_size = BOTTOM_SIDE;
             TRACE(printf("y = %d BOTTOM_SIDE centerx = %d value = %d\n",
                                  lcv,centerx,value))
             TRACE(pr_curve_rec_pts(curve))
             return NO_ERROR;}
        }
     *which_size = TOP_SIDE;
     return NO_ERROR;
    }
 return err;

}

/********************
close_partition_1(curve,dimx,dimy,x1,y1,x2,y2,avex,avey)
 struct curve_rec *curve;
 int avex,avey,x1,y1,x2,y2;
 unsigned long dimx,dimy;
{
 int err;
 struct line_segment_rec segment;
 int sign;

 setup_line_segment_rec(&segment,x1,y1,x2,y2);
 sign = apply_line_equation(&(segment.eq),avex,avey);


}
*********************/


/*-------------------------------------------------------------------------
close_partition - ???
-------------------------------------------------------------------------*/

int
close_partition(curve,mask,avex,avey,dimx,dimy,x1,y1,x2,y2)
 struct curve_rec *curve;
 struct draw_mask_rec *mask;
 int avex,avey,x1,y1,x2,y2;
 unsigned long dimx,dimy;
{
 int err;
 struct curve_rec curve1;
 unsigned short status;
 int lcv,indx,temp_int,next,after_next;
 int ptsx[4],ptsy[4],num_pts;
 float temp,distance;
 int which_size;

 TRACE(printf("\nclose_partition...begin\n"))
 init_curve_rec(&curve1);

 TRACE(printf("init*****avex = %d avey = %d x1 = %d y1 = %d x2 = %d y2 = %d\n",avex,avey,x1,y1,x2,y2))
 TRACE(pr_curve_rec_pts(curve))
 TRACE(printf("*********************\n"))
 if ((err = EnterCurveXY(&curve1,
        (float) x1, (float) y1,&status,NULL,0,0,0)) == NO_ERROR)
    {
     for (lcv = 0; lcv < curve->num_xys; lcv++)
        if ((err = EnterCurveXY(&curve1,curve->xys[lcv].x,
            curve->xys[lcv].y,&status,NULL,0,0,0)) != NO_ERROR) break;
     if (err == NO_ERROR)
        err = EnterCurveXY(&curve1,(float) x2, (float) y2,&status,
                 NULL,0,0,0);
    }
 if ((err == NO_ERROR) &&
     ((err = get_center_side(&curve1,mask,avex,avey,
       0,0,dimx - 1,dimy - 1,&which_size)) == NO_ERROR))
    {
     get_corner_pts(which_size,avex,avey,
               0,0,dimx - 1,dimy - 1,
               x1,y1,x2,y2,ptsx,ptsy,&num_pts);
     if (num_pts > 0) 
        {distance = dimx * dimy;
         for (lcv = indx = 0; indx < num_pts; indx++)
            {TRACE(printf("ptsx[indx] = %d ptsy[indx] = %d\n",
                    ptsx[indx],ptsy[indx]))
             temp = (float) sqrt((double) (
                (ptsx[indx] - x2) * (ptsx[indx] - x2) + 
                (ptsy[indx] - y2) * (ptsy[indx] - y2)));
             if (temp < distance)
                {lcv = indx;
                 distance = temp;}
            }

/* to make sure any of the 4 corner points is always followed by its closest neighbor */
         if (num_pts == 3)
            {
             next = (lcv + 1) % num_pts;
             if ((ptsx[lcv] != ptsx[next]) && (ptsy[lcv] != ptsy[next]))
                {after_next = (next + 1) % num_pts;
                 temp_int = ptsx[next];
                 ptsx[next] = ptsx[after_next];
                 ptsx[after_next] = temp_int;
                 temp_int = ptsy[next];
                 ptsy[next] = ptsy[after_next];
                 ptsy[after_next] = temp_int;}
            }

         for (indx = 0; (indx < num_pts) && (err == NO_ERROR); 
              lcv = (lcv + 1) % num_pts,indx++)
            {if ((err = EnterCurvePts(&curve1,&status,NULL,1,
                 (double) ptsx[lcv], (double) ptsy[lcv])) != NO_ERROR)
                {free_curve_rec(&curve1);
                 return err;}
            }
        }
     if (err == NO_ERROR)
        {free_curve_rec(curve);
         for (lcv = 1; lcv < curve1.num_xys; lcv++)
            if ((err = EnterCurveXY(curve,curve1.xys[lcv].x,
               curve1.xys[lcv].y,&status,NULL,0,0,0)) != NO_ERROR) break;
         if (err == NO_ERROR)
            {err = EnterCurvePts(curve,&status,NULL,2,
               (double) x1,(double) y1,
               (double) curve->xys[0].x,(double) curve->xys[0].y);
             TRACE(printf("suceed*********************\n"))
             TRACE(pr_curve_rec_pts(curve))}
        }
    }
 free_curve_rec(&curve1);
 TRACE(printf("close_partition...end err = %d\n",err))
 return err;
}


/*-------------------------------------------------------------------------
closed_unbounded_centers - ???
-------------------------------------------------------------------------*/

int
closed_unbounded_centers(elem,curve,mask,bitmap)
 struct histo_table_elem_rec *elem;
 struct curve_rec *curve;
 struct draw_mask_rec *mask;
 struct buf_2d_rec *bitmap;
{
 int err;
 int x1,y1,x2,y2;

 TRACE(printf("closed_unbounded_centers BEGIN label = %s\n",elem->center->segmented_roi_label))
 if ((err = valid_intersection(elem,mask,bitmap,
       (int) rint((double) curve->xys[0].x),
       (int) rint((double) curve->xys[0].y),&x1,&y1)) != NO_ERROR)
    return err;
 if ((err = valid_intersection(elem,mask,bitmap,
       (int) rint((double) curve->xys[curve->num_xys - 1].x),
       (int) rint((double) curve->xys[curve->num_xys - 1].y),&x2,&y2))
     != NO_ERROR) return err;

 err = close_partition(curve,mask,
                       elem->center->avex,elem->center->avey,
                       bitmap->dimx,bitmap->dimy,x1,y1,x2,y2);
 TRACE(printf("closed_unbounded_centers end err = %d\n\n",err))
 return err;
}


/*-------------------------------------------------------------------------
get_pixel_type_regions - ???
-------------------------------------------------------------------------*/

int
get_pixel_type_regions(slice_roi,members,centers,num_centers)
 struct slice_roi_rec *slice_roi;
 struct neighbor_elem_rec *members;
 struct pixel_center_rec *centers;
{
 unsigned long lcv,cnt;
 long avex,avey,avex1,avey1,avex2,avey2,
              midx01,midx12,midx02,centerx,midy01,midy12,midy02,centery;
 int err;
 char *str,*str1,*str2;

 TRACE(printf("get_pixel_type_regions...begin\n"))
 err = NO_ERROR;
 str = str1 = str2 = NULL;
 for (lcv = 0; (lcv < num_centers) && (err == NO_ERROR); lcv++)
    {
     if ((err = concat_strs(&str,2,centers[lcv].roi_label,".TYPE"))
                 != NO_ERROR) break;
     avex = centers[lcv].avex;
     avey = centers[lcv].avey;
     for (cnt = 0; cnt < members[lcv].num_neighbors; cnt ++)
        {avex1 = members[lcv].neighbors[cnt].neighbor1->avex;
         avey1 = members[lcv].neighbors[cnt].neighbor1->avey;
         avex2 = members[lcv].neighbors[cnt].neighbor2->avex;
         avey2 = members[lcv].neighbors[cnt].neighbor2->avey;
         midx01 = (long) ULongRound((double) (avex1 + avex) / (double) 2);
         midx12 = (long) ULongRound((double) (avex1 + avex2) / (double) 2);
         midx02 = (long) ULongRound((double) (avex + avex2) / (double) 2);
         centerx = (long) ULongRound((double) (avex + avex1 + avex2) / (double) 3);

         midy01 = (long) ULongRound((double) (avey1 + avey) / (double) 2);
         midy12 = (long) ULongRound((double) (avey1 + avey2) / (double) 2);
         midy02 = (long) ULongRound((double) (avey + avey2) / (double) 2);
         centery = (long) ULongRound((double) (avey + avey1 + avey2) / (double) 3);

         if (is_colinear(avex1,avey1,avex2,avey2,avex,avey))
            {
             if ((err = concat_strs(&str1,2,
                members[lcv].neighbors[cnt].neighbor1->roi_label,".TYPE"))
                 != NO_ERROR) break;
             if ((err = concat_strs(&str2,2,
                members[lcv].neighbors[cnt].neighbor2->roi_label,".TYPE"))
                 != NO_ERROR) break;
             if ((err = enter_colinear_pts(slice_roi,avex,avey,str,
                   avex1,avey1,str1,avex2,avey2,str2,
                   midx01, midy01,midx02,midy02,midx12,midy12)) != NO_ERROR) 
                break;
             free_ptr(&str1);
             free_ptr(&str2);
            }
           else
            {if ((err = EnterPtsIntoSliceRoiRec(slice_roi,NULL,str,
                  (unsigned long) DRAW_LINE,0,3, (double) midx01, (double) midy01,
                  (double) centerx,(double) centery,
                  (double) midx02,(double) midy02)) != NO_ERROR) break;
            }
         next_curve_of_type(slice_roi,str,DRAW_LINE,1,0);
         free_ptr(&str);
        }
     free_ptr(&str);
    }
 if (str != NULL) free_ptr(&str);
 if (str1 != NULL) free_ptr(&str1);
 if (str2 != NULL) free_ptr(&str2);
 TRACE(printf("get_pixel_type_regions...end\n"))

 return err;
}


/*-------------------------------------------------------------------------
get_complete_pixel_type_regions - ???
-------------------------------------------------------------------------*/

int
get_complete_pixel_type_regions(slice_roi,elems,num_rows,
                                mask,bitmap)
 struct slice_roi_rec *slice_roi;
 struct histo_table_elem_rec **elems;
 struct draw_mask_rec *mask;
 struct buf_2d_rec *bitmap;
{
 unsigned long lcv,cnt;
 char *str;
 struct curve_2d_rec *pen;
 int err;
 struct roi_rec *roi;

 TRACE(printf("get_complete_pixel_type_regions...begin\n"))
 str = NULL;
 err = CANCEL;
        
 for (lcv = 0; lcv < num_rows; lcv++)
    {
     if ((err = concat_strs(&str,2,elems[lcv][0].center->roi_label,".TYPE"))
                 != NO_ERROR) break;
     roi = get_roi(slice_roi,str);
     err = joint_curves_of_type(slice_roi,str,DRAW_LINE,1);
     if ((err == NO_ERROR) && 
         ((roi = get_cur_roi(slice_roi)) != NULL) &&
         ((pen = get_cur_pen(slice_roi)) != NULL))
        {elems[lcv][0].center->segmented_roi_label = str;
         roi->status |= HISTO_SEGMENTED_ROI;
         for (cnt = 0; cnt < pen->num_curves; cnt++)
            if (!is_curve_closed(pen->curves[cnt]) &&
                 (pen->curves[cnt]->num_xys > 0))
               {TRACE(printf("\n\nbefore closed_unbounded_centers\n"))
                TRACE(pr_curve_rec_pts(pen->curves[cnt]))
                closed_unbounded_centers(&(elems[lcv][0]),
                          pen->curves[cnt],mask,bitmap);}
        }
       else 
        {c_free(str);
         break;}
    }
 if (str != NULL) c_free(str);
 TRACE(printf("get_complete_pixel_type_regions...end err = %d\n\n\n",err))
 return err;
}


/*-------------------------------------------------------------------------
PartitionPixelTypes - ???
-------------------------------------------------------------------------*/

int
PartitionPixelTypes(elems,num_rows,histo,members,slice_roi)
 struct histo_table_elem_rec **elems;
 unsigned long num_rows;
 struct histo_params_rec *histo;
 struct neighbor_elem_rec *members;
 struct slice_roi_rec *slice_roi;
{
 struct buf_2d_rec bitmap,bitmap1;
 struct draw_mask_rec mask;
 int first_time;
 unsigned long row,cnt;
 int err;

 TRACE(printf("PartitionPixelTypes...begin\n"))

 init_draw_mask_rec(&mask);
 init_buf_2d_rec(&bitmap);
 init_buf_2d_rec(&bitmap1);
 if (((err = SetupBuf2dRec(&bitmap,histo->_2d.dimx,histo->_2d.dimy,
                (unsigned long) 1,1)) != NO_ERROR) ||
     ((err = SetupBuf2dRec(&bitmap1,histo->_2d.dimx,histo->_2d.dimy,
                (unsigned long) 1,1)) != NO_ERROR) ||
     ((err = get_draw_mask_rec(&mask,
                 histo->_2d.dimx,histo->_2d.dimy)) != NO_ERROR))
    {free_buf_2d_rec(&bitmap);
     free_buf_2d_rec(&bitmap1);
     free_draw_mask_rec(&mask);
     return err;}

 first_time = 1;
 
 err = NO_ERROR;
 do
    {TRACE(printf("get_2_closest_neighbors...begin...\n"))
     for (cnt = 0,row = 0; (row < num_rows) && (cnt < num_rows); row++)
        if (get_2_closest_neighbors(elems,row,num_rows,
                          &mask,&bitmap,&bitmap1) != NO_ERROR)
            {if (elems[row][0].common->status & NO_MORE_NEIGHBORS) cnt++;
             continue;}
    }
 while ((cnt != num_rows) && (err == NO_ERROR));

 free_buf_2d_rec(&bitmap1);
 if (err == NO_ERROR)
    {
     if ((err = get_pixel_type_regions(slice_roi,
               members,histo->centers,histo->num_centers)) == NO_ERROR)
        get_complete_pixel_type_regions(slice_roi,
             elems,num_rows,&mask,&bitmap);
    }
 free_buf_2d_rec(&bitmap);
 free_draw_mask_rec(&mask);
 TRACE(printf("PartitionPixelTypes...err = %d end\n",err))
 return err;
}

