/****************************************************************************
label_roi.c - ???

Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.
****************************************************************************/

static const char rcsid[] = "$Id: label_roi.c,v 1.11 1999/07/07 17:41:17 hughett Exp $ Release $Name: nria_1_0_6 $";


#include <xview/xview.h>
#include <xview/cms.h>
#include <xview/canvas.h>
#include <xview/panel.h>
#include <xview/notice.h>
#include <xview/wmgr.h>

#include "mri_format.h"
#include "nria_procedures.h"

/*
#define PRINT(x) printf(x); fflush(stdout);
#define TRACE(x) x; fflush(stdout);
#define PRT(x) x; fflush(stdout);
*/
#define TRACE(x)
#define PRT(x)
#define PRINT(x)



#define CLEAN_UP_LABEL_ROI 0
#define SETUP_LABEL_ROI 1
#define GET_ALL_LABEL_ROI_PTR 2

/* for status */
#define SAVE_MAIN_LABELS 1
#define SAVE_SUB_LABELS 2
struct label_roi_rec
   {
    unsigned short status;
    struct image_display_window *win;
    struct display_win_rec *dwin;
    Frame owner_frame;
    int (*ready_proc)();
    void *ready_arg;
    int (*cancel_proc)();
    void *cancel_arg;
    Frame frame;
    Panel panel;
    Panel_item for_all_slices,main_roi,sub_roi,label,auto_add,auto_save,append;
    unsigned short which_roi_list;
   };

struct all_label_roi_recs
   {
    int num_labs;
    struct label_roi_rec **labs;
   };


/* Functions defined in this file */
void init_label_roi_rec(struct label_roi_rec *lab);
void free_label_roi_rec(struct label_roi_rec *lab);
void init_all_label_roi_recs(struct all_label_roi_recs *all_labs);
void lab_done_proc(Frame frame);
int save_proc(Panel_item item, Event *event);
int add_item_strs(Panel_item item, struct strs_rec *strs_rec_ptr, char *str,
    long *cur_str_indx, long set_cur);
int label_roi_action(struct image_display_window *win, char *text);
int label_roi_for_all_slices(struct image_display_window *win, char *text);
int name_roi_proc(Panel_item item, Event *event);
int label_cur_curve_for_all_slices(struct image_display_window *win,
    char *text);
int name_cur_curve_proc(Panel_item item, Event *event);
int add_to_proc(Panel_item item, Event *event);
void add_file_labels_proc(Panel_item item, Event *event);
void label_more_proc(Panel_item item, Event *event);
int read_main_sub_list_proc(struct label_roi_rec *lab, char *dir, char *fn);
void read_list_proc(Panel_item item, Event *event);
int save_main_sub_list_proc(struct label_roi_rec *lab, char *dir, char *fn);
void save_list_proc(Panel_item item, Event *event);
void free_label_roi_rec_it(struct label_roi_rec *lab, int which);
int ready_roi_proc(Panel_item item, Event *event);
void cancel_roi_proc(Panel_item item, Event *event);
void change_label_proc(Panel_item item, char *str, int row, Panel_list_op op,
    Event *event);
int copy_to_sub_proc(Panel_item item, Event *event);
void delete_main_label_proc(Panel_item item, Event *event);
void delete_sub_label_proc(Panel_item item, Event *event);
int win_show_sub_list_roi_labels(struct image_display_window *win,
    int is_rename_roi);
int setup_label_roi_rec(struct label_roi_rec *lab,
    struct image_display_window *win, Frame owner_frame, int (*ready_proc)(),
    void *ready_arg, int (*cancel_proc)(), void *cancel_arg, char *title,
    int which_roi_list);
struct label_roi_rec *find_label_roi_rec(struct all_label_roi_recs *all_labs,
    struct image_display_window *win, int owner_frame);
int setup_label_roi(struct image_display_window *win, Frame owner_frame,
    int (*ready_proc)(), void *ready_arg, int (*cancel_proc)(),
    void *cancel_arg, char *title, unsigned short which_roi_list);


/* (Known) Local functions */
static struct all_label_roi_recs *enter_label_roi(
    struct image_display_window *win, int status, Frame owner_frame,
    int (*ready_proc)(), void *ready_arg, int (*cancel_proc)(),
    void *cancel_arg, char *title, unsigned short which_roi_list);



/*-------------------------------------------------------------------------
init_label_roi_rec - ???
-------------------------------------------------------------------------*/

void
init_label_roi_rec(lab)
 struct label_roi_rec *lab;
{
 lab->status = 0;
 lab->frame = 0;
 lab->win = NULL;
 lab->dwin = NULL;
 lab->owner_frame = 0;
 lab->ready_proc = NULL;
 lab->ready_arg = NULL;
 lab->cancel_proc = NULL;
 lab->cancel_arg = NULL;
}


/*-------------------------------------------------------------------------
free_label_roi_rec - ???
-------------------------------------------------------------------------*/

void
free_label_roi_rec(lab)
 struct label_roi_rec *lab;
{
 struct all_label_roi_recs *all_labs;

 if (lab->frame != 0)
    {destroy_all_subframes(lab->frame);
     xv_destroy_safe(lab->frame);
    }
 all_labs = enter_label_roi(NULL, GET_ALL_LABEL_ROI_PTR, 
			    0, NULL, NULL, NULL, NULL, NULL, 0);
 c_free(lab);
 DropHandleAryPtr(&(all_labs->labs),&(all_labs->num_labs),lab);
}


/*-------------------------------------------------------------------------
init_all_label_roi_recs - ???
-------------------------------------------------------------------------*/

void
init_all_label_roi_recs(all_labs)
 struct all_label_roi_recs *all_labs;
{
 all_labs->num_labs = 0;
 all_labs->labs = NULL;
}


/*-------------------------------------------------------------------------
lab_done_proc - ???
-------------------------------------------------------------------------*/

void
lab_done_proc(frame)
 Frame frame;
{
 struct label_roi_rec *lab;

 lab = (struct label_roi_rec *) xv_get(frame,WIN_CLIENT_DATA);
 free_label_roi_rec(lab);
}


/*-------------------------------------------------------------------------
save_proc - ???
-------------------------------------------------------------------------*/

int
save_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct label_roi_rec *lab;
 struct strs_rec list;
 int err,err1,is_which_label;
 char *reason;

 lab = (struct label_roi_rec *) xv_get(item,PANEL_CLIENT_DATA);
 TRACE(printf("save_proc lab->status = %x\n",(unsigned long) lab->status))
 if (lab->status & SAVE_MAIN_LABELS)
    {TRACE(printf("SAVE_MAIN_LABELS...\n"))
     init_strs_rec(&list);
     if ((err = list_items(&list,lab->main_roi)) == NO_ERROR)
        {TRACE(pr_strs_rec(&list))
         if ((err = win_rec_save_labels(lab->win->owner,
                    lab->win->type2,&list,1,0)) == NO_ERROR)
            {lab->status &= ~SAVE_MAIN_LABELS;
             if (lab->win->type2 & HISTOGRAM_WINDOW) is_which_label = 3;
               else is_which_label = 1;
             if ((err1 = win_rec_label_file_access(lab->win->owner,
                      is_which_label,2,&reason)) != NO_ERROR)
                {will_not_save_err_msg(lab->frame,err1,
                       "The MAIN ROI LABEL LIST IS UPDATED !!!",reason);
                 c_free(reason);}
            }
        }
     if (err != NO_ERROR)
        {ShowMsgStrs(lab->frame,0,err,0,2,
               "The MAIN ROI LABEL LIST IS NOT UPDATED !!!",
               "AN ERROR OCCURRED DURING THE UPDATE",NULL,0);
         free_strs_rec(&list);}
    }
 if (lab->status & SAVE_SUB_LABELS)
    {TRACE(printf("SAVE_SUB_LABELS...\n"))
     init_strs_rec(&list);
     if ((err = list_items(&list,lab->sub_roi)) == NO_ERROR)
        {TRACE(pr_strs_rec(&list))
         if ((err = win_rec_save_labels(lab->win->owner,
                lab->win->type2,&list,1,1)) == NO_ERROR)
            {lab->status &= ~SAVE_SUB_LABELS;
             if (lab->win->type2 & HISTOGRAM_WINDOW) is_which_label = 4;
               else is_which_label = 2;
             if ((err1 = win_rec_label_file_access(lab->win->owner,
                      is_which_label,2,&reason)) != NO_ERROR)
                {will_not_save_err_msg(lab->frame,err1,
                       "The SUB ROI LABEL LIST IS UPDATED !!!",reason);
                 c_free(reason);}
            }
        }
     if (err != NO_ERROR)
        {ShowMsgStrs(lab->frame,0,err,0,2,
               "The SUB ROI LABEL LIST IS NOT UPDATED !!!",
               "AN ERROR OCCURRED DURING THE UPDATE",NULL,0);
         free_strs_rec(&list);}
    }
 return err;
}


/*-------------------------------------------------------------------------
add_item_strs - ???
-------------------------------------------------------------------------*/

int
add_item_strs(item,strs_rec_ptr,str,cur_str_indx,set_cur)
 Panel_item item;
 struct strs_rec *strs_rec_ptr;
 char *str;
 long *cur_str_indx,set_cur;
{
 struct strs_rec list,ori,temp;
 int err,is_update;
 long lcv;

 init_strs_rec(&list);
 list_item_ptrs(&list,item);
 err = NO_ERROR;
 if (strs_rec_ptr != NULL)
    {PRT(printf("add_item_strs...strs_rec_ptr != NULL\n"))
     if (strs_rec_ptr->num_strs == 0)
        {free_strs_rec_ptrs(&list);
         return NO_ERROR;}

     if (cur_str_indx != NULL)
        if ((*cur_str_indx >= 0) && (*cur_str_indx < list.num_strs))
           list.cur_str = *cur_str_indx;
     init_strs_rec(&ori);
     TRACE(printf("strs_rec_ptr...err = %d...",err))
     TRACE(pr_strs_rec(strs_rec_ptr))

     if ((err = cp_strs_rec_ptrs(&ori,&list)) == NO_ERROR) 
        err = add_str_ptr_ary_to_sorted_strs_rec(&list,
                 strs_rec_ptr->strs,strs_rec_ptr->num_strs,set_cur);
     
     TRACE(printf("\nlist...err = %d...",err))
     TRACE(pr_strs_rec(&list))
     if (err == NO_ERROR)
        {init_strs_rec(&temp);
         if ((err = diff_ptrs_in_sorted_strs_rec(&temp,&list,&ori))
                == NO_ERROR)
            {TRACE(printf("diff =..."))
             TRACE(pr_strs_rec(&temp))
             for (is_update = 0,lcv = 0; lcv < temp.num_strs; lcv++)
                if (temp.strs[lcv] != NULL)
                   {is_update = 1;
                    xv_set(item,PANEL_LIST_INSERT,lcv,
                          PANEL_LIST_STRING,lcv,temp.strs[lcv],0);}
                    
             if (is_update) update_list_client_data(item);
            }
         free_strs_rec_ptrs(&temp);
        }
     free_strs_rec_ptrs(&ori);
    }
   else if (str != NULL)
    {PRT(printf("add_item_strs...str != NULL\n"))
     if (FindStrInSortedStrsRec(&list,str,&(list.cur_str)) == NOT_FOUND)
        {xv_set(item,PANEL_LIST_INSERT,list.cur_str,
                          PANEL_LIST_STRING,list.cur_str,str,0);
         PRT(printf("INSERT str = %s   cur_str = %d\n",str,list.cur_str))
         update_list_client_data(item);
       
         if (cur_str_indx != NULL)
            if (set_cur)
               *cur_str_indx = list.cur_str;
              else if (list.cur_str <= *cur_str_indx)
               *cur_str_indx += 1;
        }
       else if (cur_str_indx != NULL)
        if (set_cur) *cur_str_indx = list.cur_str;
    }
 free_strs_rec_ptrs(&list);
 return err;
}


/*-------------------------------------------------------------------------
label_roi_action - ???
-------------------------------------------------------------------------*/

int
label_roi_action(win,text)
 struct image_display_window *win;
 char *text;
{
 char *str;

 if ((str = win_get_cur_roi_label(win)) == NULL) return CANCEL;
 if ((get_strlen(text) > 0) && (str_cmp(str,text) != 0))
    {
     return win_rename_cur_roi_label(win,text);
    }
 return CANCEL;
}


/*-------------------------------------------------------------------------
label_roi_for_all_slices - ???
-------------------------------------------------------------------------*/

int
label_roi_for_all_slices(win,text)
 struct image_display_window *win;
 char *text;
{
 int err;
 struct roi_rec *roi,*roi1;
 struct img_buf_rec *img;
 struct slice_roi_win_rec **sroi_wins;
 struct slice_roi_rec *slice_roi;
 long lcv,cnt;
 char *str;
 int is_clear;

 if ((img = win_rec_cur_img_buf(win->owner)) == NULL) return CANCEL;
 if ((roi = get_cur_roi(&(win->control.sroi.sroi_win->slice_roi))) == NULL)
    return CANCEL;
 if (str_cmp(roi->label,text) == 0) return NO_ERROR;
 if ((err = cp_str(&str,roi->label)) != NO_ERROR)
    return err;
 sroi_wins = img->file->slice_info.sroi_wins;
 for (lcv = 0; lcv < img->file->slice_info.num_sroi_wins; lcv++)
    {slice_roi = &(sroi_wins[lcv]->slice_roi);
     if ((roi = get_roi(slice_roi,str)) != NULL)
        {if ((roi1 = get_roi(slice_roi,text)) != NULL)
            err = JoinRoiRecs(roi1,roi,0,
                     &(slice_roi->status),1.0,1.0,NULL,NULL,NULL);
           else err = rename_roi_label(slice_roi,roi,text);
         if (err != NO_ERROR)
            {ShowMsgStrs(win->dwin->frame,0,err,0,1,
                     "Error labelling cur rois for all slices  !!!",NULL,0);
             return err;}
           else
            {for (cnt = 0; cnt < sroi_wins[lcv]->num_wins; cnt++)
                if (sroi_wins[lcv]->wins[cnt] != NULL)
                   {win_execute_proc_to_multiple_wins(
		       sroi_wins[lcv]->wins[cnt],win_redisplay_all_curves,
		       0,0,0,0,0);
                    is_clear = 1;
                    win_execute_proc_to_multiple_wins(
			sroi_wins[lcv]->wins[cnt],
			win_draw_enabled_label,&is_clear,
			0,0,0,0);
                   }
            }
        }
    }
 return NO_ERROR;
}


/*-------------------------------------------------------------------------
name_roi_proc - ???
-------------------------------------------------------------------------*/

int
name_roi_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct label_roi_rec *lab;
 int err,answer;
 char *str,*str1;
 struct roi_rec *roi;

 lab = (struct label_roi_rec *) xv_get(item,PANEL_CLIENT_DATA);
 if ((roi = get_cur_roi(&(lab->win->control.sroi.sroi_win->slice_roi)))
            == NULL)
    {notice_prompt(lab->win->dwin->frame,event,
            NOTICE_MESSAGE_STRINGS,"Sorry...",
                "No current roi has been selected yet !!!",
                0,
            NOTICE_BUTTON," OK !!",2,
            0);
      return NO_ERROR;}

 str = (char *) xv_get(lab->label,PANEL_VALUE);
 if ((str == NULL) || (get_strlen(str) == 0) ||
     (str_cmp(roi->label,str) == 0)) return CANCEL;

 if (xv_get(lab->for_all_slices,PANEL_VALUE))
    {
     if ((err = concat_strs(&str1,3,"to ",str," ?")) == NO_ERROR)
        {answer = (int) notice_prompt(lab->win->dwin->frame,event,
            NOTICE_MESSAGE_STRINGS,"Rename all rois of label...",
                roi->label,str1,0,
            NOTICE_BUTTON," CANCEL ?",1,
            NOTICE_BUTTON," YES !!",2,
            0);
         if (answer == 2)
            err = label_roi_for_all_slices(lab->win,str);
           else err = CANCEL;
         c_free(str1);}
    }
   else err = label_roi_action(lab->win,str);

 if (err == NO_ERROR)
    if (xv_get(lab->auto_add,PANEL_VALUE))
       add_to_proc(item,event);
 return err;
}


/*-------------------------------------------------------------------------
label_cur_curve_for_all_slices - ???
-------------------------------------------------------------------------*/

int
label_cur_curve_for_all_slices(win,text)
 struct image_display_window *win;
 char *text;
{
 int err,is_clear;
 struct roi_rec *roi,*roi1;
 struct img_buf_rec *img;
 struct slice_roi_win_rec **sroi_wins;
 struct curve_rec *curve;
 struct slice_roi_rec *slice_roi;
 struct curve_2d_rec *pen;
 long lcv,cnt;

 if ((img = win_rec_cur_img_buf(win->owner)) == NULL) return CANCEL;
 sroi_wins = img->file->slice_info.sroi_wins;
 for (lcv = 0; lcv < img->file->slice_info.num_sroi_wins; lcv++)
    {slice_roi = &(sroi_wins[lcv]->slice_roi);
     if ((roi = get_cur_roi(slice_roi)) != NULL)
        if ((str_cmp(roi->label,text) != 0) &&
            ((curve = get_cur_curve(slice_roi)) != NULL) &&
            ((pen = get_cur_pen(slice_roi)) != NULL) &&
            ((err = setup_more_rois(slice_roi,text,0)) == NO_ERROR))
           {if ((roi1 = get_roi(slice_roi,text)) != NULL)
               {err = JoinCurveToRoiRec(roi1,curve,pen->pen_type,1,
                     &(slice_roi->status),1.0,1.0,NULL,NULL,NULL);
                if (err != NO_ERROR)
                   {ShowMsgStrs(win->dwin->frame,0,err,0,1,
                     "Error labelling cur rois for all slices  !!!",NULL,0);
                    return err;}
                  else
                   {delete_cur_curve(pen,&(slice_roi->status),1);
                    for (cnt = 0; cnt < sroi_wins[lcv]->num_wins; cnt++)
                       if (sroi_wins[lcv]->wins[cnt] != NULL)
                          {win_execute_proc_to_multiple_wins(
                             sroi_wins[lcv]->wins[cnt],
                             win_redisplay_all_curves,
			     0,0,0,0,0);
                           is_clear = 1;
                           win_execute_proc_to_multiple_wins(
                             sroi_wins[lcv]->wins[cnt],
                             win_draw_enabled_label,&is_clear,
			     0,0,0,0);
                          }
                   }
               }
           }
    }
 return NO_ERROR;
}


/*-------------------------------------------------------------------------
name_cur_curve_proc - ???
-------------------------------------------------------------------------*/

int
name_cur_curve_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct label_roi_rec *lab;
 char *str;
 int err,answer;

 lab = (struct label_roi_rec *) xv_get(item,PANEL_CLIENT_DATA);
 str = (char *) xv_get(lab->label,PANEL_VALUE);
 if ((str == NULL) || (get_strlen(str) == 0) ) return CANCEL;
 if (xv_get(lab->for_all_slices,PANEL_VALUE))
    {answer = (int) notice_prompt(lab->win->dwin->frame,event,
       NOTICE_MESSAGE_STRINGS,"Label all current regions for all slices ?",0,
       NOTICE_BUTTON," CANCEL ?",1,
       NOTICE_BUTTON," YES !!",2,0);
     if (answer == 2)
        err = label_cur_curve_for_all_slices(lab->win,str);
       else err = CANCEL;
    }
   else err = win_join_curve_roi_rec(lab->win,str);
 if (err == NO_ERROR)
    if (xv_get(lab->auto_add,PANEL_VALUE))
       add_to_proc(item,event);
 return err;
}


/*-------------------------------------------------------------------------
add_to_proc - ???
-------------------------------------------------------------------------*/

int
add_to_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct label_roi_rec *lab;
 char *str;
 int err;

 lab = (struct label_roi_rec *) xv_get(item,PANEL_CLIENT_DATA);
 if ((str = (char *) xv_get(lab->label,PANEL_VALUE)) == NULL) return NO_ERROR;
 if (get_strlen(str) == 0) return NO_ERROR;

 PRT(printf("add_to_proc...main_roi\n"))
 err = add_item_strs(lab->main_roi,NULL,str,NULL,0);
 if (err == NO_ERROR)
    {lab->status |= SAVE_MAIN_LABELS;
     PRT(printf("add_to_proc sub_roi...\n"))
     err = add_item_strs(lab->sub_roi,NULL,str,NULL,0);
     if (err == NO_ERROR) lab->status |= SAVE_SUB_LABELS;
     if (xv_get(lab->auto_save,PANEL_VALUE))
        save_proc(item,event);
    }
 return err;
}


/*-------------------------------------------------------------------------
add_file_labels_proc - ???
-------------------------------------------------------------------------*/

void
add_file_labels_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct label_roi_rec *lab;
 struct strs_rec labels;
 int err;

 lab = (struct label_roi_rec *) xv_get(item,PANEL_CLIENT_DATA);
 init_strs_rec(&labels);
 WinGetAllRoiLabelPtrs(lab->win,&labels,0,1);
 if (labels.num_strs > 0)
    {sort_strs_rec_ascendingly(&labels);
     if ((err = add_item_strs(lab->main_roi,&labels,NULL,NULL,0)) == NO_ERROR)
        {lab->status |= SAVE_MAIN_LABELS;
         TRACE(printf("add_file_labels_proc to main_roi...\n"))
         if ((err = add_item_strs(lab->sub_roi,&labels,NULL,NULL,0)) == NO_ERROR)
            lab->status |= SAVE_SUB_LABELS;
        }
    }
 free_strs_rec_ptrs(&labels);
}


/*-------------------------------------------------------------------------
label_more_proc - ???
-------------------------------------------------------------------------*/

void
label_more_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct label_roi_rec *lab;

 lab = (struct label_roi_rec *) xv_get(item,PANEL_CLIENT_DATA);
 notice_prompt(lab->frame,(Event *) NULL,
            NOTICE_MESSAGE_STRINGS,
               "When the pen is enabled, pressing the middle or right...",
               "mouse button shows a list of up to 30 region names for",
               "selection.  That list is equal to the sub-list if it",
               "is present or equal to the main list otherwise...",
               "",
               "...middle-button = label the current region",
               "...right-button = label the current roi",
               "",
              0,
            NOTICE_BUTTON_YES," OK !",
            0);
}


/*-------------------------------------------------------------------------
read_main_sub_list_proc - ???
-------------------------------------------------------------------------*/

int
read_main_sub_list_proc(lab,dir,fn)
 struct label_roi_rec *lab;
 char *dir,*fn;
{
 int err,append;
 char *main,*sub,*str;
 struct strs_rec *roi_labels,labels,list;

 main = sub = str = NULL;
 if ((err = build_fn(&main,dir,fn,"")) != NO_ERROR) 
    {ShowMsgStrs(lab->frame,0,err,0,1,
             "Error setting up the input list fn  !!!",NULL,0);
     return err;}
   else if (((err = cp_str(&str,main)) == NO_ERROR) &&
            ((err = drop_suffix(str)) == NO_ERROR) &&
            ((err = concat_strs(&sub,2,str,".sub")) == NO_ERROR))
    {err = win_rec_rd_label_fns(lab->win->owner,lab->which_roi_list,main,sub);
     roi_labels = win_rec_get_roi_labels_PTR(lab->win->owner,lab->which_roi_list);
     append = (int) xv_get(lab->append,PANEL_VALUE);
     if (append) 
        {init_strs_rec(&list);
         list_items(&list,lab->main_roi);}
     delete_all_list_items(lab->main_roi);
     if (append)
        {init_strs_rec(&labels);
         cp_strs_rec_ptrs(&labels,&list);
         add_str_ptr_ary_to_sorted_strs_rec(&labels,roi_labels->strs,
                       roi_labels->num_strs,0);
         setup_list_items(&labels,lab->main_roi,0);
         free_strs_rec(&list);
         free_strs_rec_ptrs(&labels);
        }
       else setup_list_items(roi_labels,lab->main_roi,0);

     roi_labels = win_rec_get_sec_roi_labels_PTR(lab->win->owner,
                                 lab->which_roi_list);
     if (append) 
        {list_items(&list,lab->sub_roi);}
     delete_all_list_items(lab->sub_roi);
     if (append)
        {cp_strs_rec_ptrs(&labels,&list);
         add_str_ptr_ary_to_sorted_strs_rec(&labels,roi_labels->strs,
                       roi_labels->num_strs,0);
         setup_list_items(&labels,lab->sub_roi,0);
         free_strs_rec(&list);
         free_strs_rec_ptrs(&labels);
        }
       else setup_list_items(roi_labels,lab->sub_roi,0);
    }

 c_free(main);
 c_free(sub);
 c_free(str);
 return err;
}


/*-------------------------------------------------------------------------
read_list_proc - ???
-------------------------------------------------------------------------*/

void
read_list_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct label_roi_rec *lab;
 char *dir,*str,*main,*sub,*fn;

 lab = (struct label_roi_rec *) xv_get(item,PANEL_CLIENT_DATA);
 dir = get_prog_dir(lab->win->owner->root->prog);
 str = win_rec_win_id_title(lab->win->owner,"GET MAIN & SUB LABEL LIST");
 win_rec_get_label_fns(lab->win->owner,lab->which_roi_list,&main,&sub);
 cp_str(&fn,main);
 drop_dirfn(fn);
 ask_files(1,lab->frame,str,
       NULL,1,0,read_main_sub_list_proc,lab,
       NULL,NULL,0,"*.main",1,0,5,285,NULL,NULL,1,
       dir,NULL,NULL,
       "FN",fn,NULL,NULL
      );
 c_free(dir);
 c_free(fn);
 c_free(str);
}


/*-------------------------------------------------------------------------
save_main_sub_list_proc - ???
-------------------------------------------------------------------------*/

int
save_main_sub_list_proc(lab,dir,fn)
 struct label_roi_rec *lab;
 char *dir,*fn;
{
 int err;
 char *main,*sub,*str;

 main = sub = str = NULL;
 if ((err = build_fn(&main,dir,fn,"")) != NO_ERROR) 
    {ShowMsgStrs(lab->frame,0,err,0,1,
             "Error setting up the input list fn  !!!",NULL,0);
     return err;}
   else if (((err = cp_str(&str,main)) == NO_ERROR) &&
            ((err = drop_suffix(str)) == NO_ERROR) &&
            ((err = concat_strs(&sub,2,str,".sub")) == NO_ERROR))
    if ((err = win_rec_wr_label_fns(lab->win->owner,lab->which_roi_list,main,sub))
                   != NO_ERROR)
       {ShowMsgStrs(lab->frame,0,err,0,1,
             "Error when writing the main & sub lists !!!",NULL,0);
       }
 c_free(main);
 c_free(sub);
 c_free(str);
 return err;
}


/*-------------------------------------------------------------------------
save_list_proc - ???
-------------------------------------------------------------------------*/

void
save_list_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct label_roi_rec *lab;
 char *dir,*str,*main,*sub,*fn;

 lab = (struct label_roi_rec *) xv_get(item,PANEL_CLIENT_DATA);
 dir = get_prog_dir(lab->win->owner->root->prog);
 str = win_rec_win_id_title(lab->win->owner,"SAVE MAIN & SUB LABEL LIST");
 win_rec_get_label_fns(lab->win->owner,lab->which_roi_list,&main,&sub);
 cp_str(&fn,main);
 drop_dirfn(fn);
 ask_files(1,lab->frame,str,
       NULL,1,0,save_main_sub_list_proc,lab,
       NULL,NULL,0,"*.main",0,0,5,285,NULL,NULL,1,
       dir,NULL,NULL,
       "FN",fn,NULL,NULL
      );
 c_free(dir);
 c_free(fn);
 c_free(str);
}



/*-------------------------------------------------------------------------
free_label_roi_rec_it - ???
-------------------------------------------------------------------------*/

void
free_label_roi_rec_it(lab,which)
 struct label_roi_rec *lab;
 int which;
{
 (void) notify_set_itimer_func((Notify_client) lab,NOTIFY_FUNC_NULL,ITIMER_REAL,
             (struct itimerval *) 0,(struct itimerval *) 0);
 free_label_roi_rec(lab);
}


/*-------------------------------------------------------------------------
ready_roi_proc - ???
-------------------------------------------------------------------------*/

int
ready_roi_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct label_roi_rec *lab;
 char *str;
 int err,is_free;

 is_free = 1;
 err = NO_ERROR;
 lab = (struct label_roi_rec *) xv_get(item,PANEL_CLIENT_DATA);
 if ((lab->ready_proc != NULL) &&
     ((str = (char *) xv_get(lab->label,PANEL_VALUE)) != NULL) &&
     (get_strlen(str) > 0))
    err = lab->ready_proc(lab->ready_arg,str);
   else if (lab->ready_proc != NULL)
     {notice_prompt(lab->frame,(Event *) NULL,
            NOTICE_MESSAGE_STRINGS,
               "Please enter your roi label !!!!",
               "before clicking ready !!!",
               0,
            NOTICE_BUTTON_YES," OK !",
            0);
      is_free = 0;
     }
 if (is_free)
    (void) notify_set_itimer_func((Notify_client) lab,
             (Notify_func) free_label_roi_rec_it,ITIMER_REAL,
             &NOTIFY_POLLING_ITIMER,(struct itimerval *) 0);
 return err;
}


/*-------------------------------------------------------------------------
cancel_roi_proc - ???
-------------------------------------------------------------------------*/

void
cancel_roi_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct label_roi_rec *lab;

 lab = (struct label_roi_rec *) xv_get(item,PANEL_CLIENT_DATA);
 if (lab->cancel_proc != NULL)
    lab->cancel_proc(lab->cancel_arg);
 (void) notify_set_itimer_func((Notify_client) lab,
             (Notify_func) free_label_roi_rec_it,ITIMER_REAL,
             &NOTIFY_POLLING_ITIMER,(struct itimerval *) 0);

}


/*-------------------------------------------------------------------------
change_label_proc - ???
-------------------------------------------------------------------------*/

void
change_label_proc(item,str,row,op,event)
 Panel_item item;
 char *str;
 int row;
 Panel_list_op op;
 Event *event;
{
 struct label_roi_rec *lab;

 lab = (struct label_roi_rec *) xv_get(item,PANEL_CLIENT_DATA);
 TRACE(printf("op = %d row = %d  str = %s  \n",op,row,str))
 xv_set(lab->label,PANEL_VALUE,str,0);
}


/*-------------------------------------------------------------------------
copy_to_sub_proc - ???
-------------------------------------------------------------------------*/

int
copy_to_sub_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct label_roi_rec *lab;
 struct strs_rec list;
 int err;

 lab = (struct label_roi_rec *) xv_get(item,PANEL_CLIENT_DATA);
 init_strs_rec(&list);
 if ((err = selected_list_item_ptrs(&list,lab->main_roi)) == NO_ERROR)
    if (list.num_strs > 0)
       if ((err = add_item_strs(lab->sub_roi,&list,NULL,NULL,0)) == NO_ERROR)
          {lab->status |= SAVE_SUB_LABELS;
           if (xv_get(lab->auto_save,PANEL_VALUE))
              save_proc(item,event);}

 free_strs_rec_ptrs(&list);
 return err;
}

/********************
copy_to_sub_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct label_roi_rec *lab;
 struct strs_rec list,sub;
 int err;
 unsigned long num_strs;

 lab = (struct label_roi_rec *) xv_get(item,PANEL_CLIENT_DATA);
 init_strs_rec(&list);
 if ((err = selected_list_item_ptrs(&list,lab->main_roi)) == NO_ERROR)
    if (list.num_strs > 0)
       {init_strs_rec(&sub);
        if ((err = list_item_ptrs(&sub,lab->sub_roi)) == NO_ERROR)
           {num_strs = sub.num_strs;
            if ((err = add_str_ptr_ary_to_sorted_strs_rec(&sub,
                       list.strs,list.num_strs,0))  == NO_ERROR)
               if (num_strs != sub.num_strs)
                  {TRACE(printf("copy_to_sub_proc num_strs = %d sub.num_strs\n",num_strs,
                          sub.num_strs))
                   delete_all_list_items(lab->sub_roi);
                   format->cur_roi_indx = -1;
                   setup_list_items(&sub,lab->sub_roi,0);

                   if ((err = add_item_strs(lab->sub_roi,&list,NULL,NULL,0)) == NO_ERROR)
                      lab->status |= SAVE_SUB_LABELS;
       }
 free_strs_rec_ptrs(&list);
 return err;
}
********************/

/*******************
delete_label_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct label_roi_rec *lab;

 lab = (struct label_roi_rec *) xv_get(item,PANEL_CLIENT_DATA);
 delete_selected_list_items(lab->main_roi);
 delete_selected_list_items(lab->sub_roi);
 lab->status |= SAVE_MAIN_LABELS | SAVE_SUB_LABELS;
}
*******************/


/*-------------------------------------------------------------------------
delete_main_label_proc - ???
-------------------------------------------------------------------------*/

void
delete_main_label_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct label_roi_rec *lab;

 lab = (struct label_roi_rec *) xv_get(item,PANEL_CLIENT_DATA);
 delete_selected_list_items(lab->main_roi);
 lab->status |= SAVE_MAIN_LABELS;
 if (xv_get(lab->auto_save,PANEL_VALUE))
    save_proc(item,event);
}


/*-------------------------------------------------------------------------
delete_sub_label_proc - ???
-------------------------------------------------------------------------*/

void
delete_sub_label_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct label_roi_rec *lab;

 lab = (struct label_roi_rec *) xv_get(item,PANEL_CLIENT_DATA);
 delete_selected_list_items(lab->sub_roi);
 lab->status |= SAVE_SUB_LABELS;
 if (xv_get(lab->auto_save,PANEL_VALUE))
    save_proc(item,event);
}


/*-------------------------------------------------------------------------
win_show_sub_list_roi_labels - ???
-------------------------------------------------------------------------*/

int
win_show_sub_list_roi_labels(win,is_rename_roi)
 struct image_display_window *win;
 int is_rename_roi;
{
 struct strs_rec *roi_labels,*ptr;
 long total;
 int err;

 roi_labels = win_rec_get_sec_roi_labels_PTR(win->owner,win->type2);
 if (roi_labels->num_strs == 0)
    {roi_labels = win_rec_get_roi_labels_PTR(win->owner,win->type2);
     if (roi_labels->num_strs == 0)
        {ShowMsgStrs(win->dwin->frame,0,err,0,2,
                "No roi labels found...",
                "in both main & sub lists !!!",NULL,0);
         return NO_ERROR;}
    }

 if ((err = setup_strs_rec(&ptr,0,0)) == NO_ERROR)
    {total = roi_labels->num_strs;
     if (roi_labels->num_strs > 120) roi_labels->num_strs = 120;
     err = cp_strs_rec_ptrs(ptr,roi_labels);
     roi_labels->num_strs = total;
     if (err != NO_ERROR)
        {free_strs_rec_ptrs(ptr);
         c_free(ptr);
        }
    }
 if (err != NO_ERROR)
    {ShowMsgStrs(win->dwin->frame,0,err,0,1,
                "Insufficient Memory !!",NULL,0);
     return err;}
 if (is_rename_roi)
    return ShowMenuStrsProc(win->dwin->frame,NULL,"LABEL CUR ROI",ptr,
           USER_MENU_FREE_STR_PTRS_AND_PTR,label_roi_action,win,1,0,0);
 else
     return ShowMenuStrsProc(win->dwin->frame,NULL,"LABEL CUR REGION",ptr,
	   USER_MENU_FREE_STR_PTRS_AND_PTR,win_join_curve_roi_rec,win,1,0,0);
}


/*-------------------------------------------------------------------------
setup_label_roi_rec - ???
-------------------------------------------------------------------------*/

int
setup_label_roi_rec(lab,win,owner_frame,ready_proc,ready_arg,
                cancel_proc,cancel_arg,title,which_roi_list)
 struct label_roi_rec *lab;
 struct image_display_window *win;
 Frame owner_frame;
 int (*ready_proc)();
 void *ready_arg;
 int (*cancel_proc)();
 void *cancel_arg;
 char *title;
 unsigned short which_roi_list;
{
 Cms cms;
 Panel_item item,item1,item2,del_item;
 int row,offset,temp;
 char *str,*str1;
 Rect *rr,*rr1;
 struct strs_rec *roi_labels;

 cms = (Cms) xv_get(owner_frame,WIN_CMS);
 lab->frame = xv_create(owner_frame, FRAME,
			/*WIN_DYNAMIC_VISUAL,TRUE,*/
			FRAME_INHERIT_COLORS,TRUE,
			WIN_CLIENT_DATA,lab,
			0);
 if (lab->frame == 0)
    {free_label_roi_rec(lab);
     return ~NO_ERROR;}
 xv_set(lab->frame,		/* DRW 19970811 See display_init_free.c */
	WIN_CMS,cms,
	NULL);
 lab->win = win;
 lab->dwin = win->dwin;
 lab->owner_frame = owner_frame;
 lab->ready_proc = ready_proc;
 lab->ready_arg = ready_arg;
 lab->cancel_proc = cancel_proc;
 lab->cancel_arg = cancel_arg;

 if (title != NULL) str = win_rec_win_id_title(win->owner,title);
   else if (win->type2 & HISTOGRAM_WINDOW)
    {str1 = win_rec_win_id_title(win->owner,"HISTOGRAM");
     concat_strs(&str,2,str1,LABEL_ROI_AND_CURVE);
     c_free(str1);}
   else str = win_rec_win_id_title(win->owner,LABEL_ROI_AND_CURVE);
 xv_set(lab->frame,
              FRAME_SHOW_LABEL, TRUE,
              XV_LABEL,str,
              FRAME_NO_CONFIRM,TRUE,
              FRAME_DONE_PROC,lab_done_proc,
              0);
 c_free(str);
 xv_set(lab->frame,FRAME_CLOSED,FALSE,0);

 lab->panel = xv_create(lab->frame,PANEL,
               FRAME_INHERIT_COLORS,TRUE,
               0);
 cms = (Cms) xv_get(lab->panel,WIN_CMS);

 if (lab->panel == 0)
    {free_label_roi_rec(lab);
     return ~NO_ERROR;}
 row = 0;
 offset = xv_rows(lab->panel,1) - xv_rows(lab->panel,0) - 11;
 item1 = xv_create(lab->panel,PANEL_MESSAGE,PANEL_LABEL_STRING, "MAIN LIST",
                  XV_X,5,XV_Y,row+3,0);
 temp = row + offset;
 lab->main_roi = xv_create(lab->panel,PANEL_LIST,
                 PANEL_CHOOSE_ONE,FALSE,
                 PANEL_LIST_DISPLAY_ROWS,5,
                 PANEL_LIST_WIDTH,170,
                 PANEL_LAYOUT,PANEL_VERTICAL,
                  XV_X,5,XV_Y,temp,
                  PANEL_CLIENT_DATA,lab,
	          PANEL_NOTIFY_PROC,change_label_proc,
                 0);
 if (!which_roi_list) which_roi_list = win->type2;
 lab->which_roi_list = which_roi_list;
 roi_labels = win_rec_get_roi_labels_PTR(win->owner,which_roi_list);
 setup_list_items(roi_labels,lab->main_roi,0);

 rr = (Rect *) xv_get(lab->main_roi,XV_RECT);

 item = xv_create(lab->panel,PANEL_BUTTON,
	PANEL_LABEL_STRING,"<",
	PANEL_NOTIFY_PROC,delete_main_label_proc,
        XV_X,rr->r_left + rr->r_width + 2,XV_Y,temp,
        PANEL_CLIENT_DATA,lab,
	0);
 rr1 = (Rect *) xv_get(item,XV_RECT);
 item = xv_create(lab->panel,PANEL_MESSAGE,
	PANEL_LABEL_STRING,"DELETE",
        XV_X,rr1->r_left + rr1->r_width,XV_Y,temp+2,
	0);

 rr1 = (Rect *) xv_get(item,XV_RECT);
 del_item = xv_create(lab->panel,PANEL_BUTTON,
	PANEL_LABEL_STRING,">",
	PANEL_NOTIFY_PROC,delete_sub_label_proc,
        XV_X,rr1->r_left + rr1->r_width,XV_Y,temp,
        PANEL_CLIENT_DATA,lab,
	0);

 temp += offset;
 item = xv_create(lab->panel,PANEL_BUTTON,
	PANEL_LABEL_STRING,"COPY >>",
	PANEL_NOTIFY_PROC,copy_to_sub_proc,
        XV_X,rr->r_left + rr->r_width + 2,XV_Y,temp,
        PANEL_CLIENT_DATA,lab,
	0);

 temp += offset;
 xv_create(lab->panel,PANEL_BUTTON,
	PANEL_LABEL_STRING,"SAVE NOW",
	PANEL_NOTIFY_PROC,save_proc,
        XV_X,rr->r_left + rr->r_width + 2,XV_Y,temp,
        PANEL_CLIENT_DATA,lab,
	0);
 temp += offset;
 lab->auto_save = xv_create(lab->panel,PANEL_CHECK_BOX,
        PANEL_LABEL_STRING,"SAVE",
        XV_X,rr->r_left + rr->r_width + 2,XV_Y,temp,
        PANEL_VALUE,1,XV_Y,temp,
        PANEL_CLIENT_DATA,lab,
        0);
 rr = (Rect *) xv_get(del_item,XV_RECT);
 item2 = xv_create(lab->panel,PANEL_MESSAGE,PANEL_LABEL_STRING, "SUB LIST",
                  XV_X,rr->r_left + rr->r_width,XV_Y,row+3,0);
 temp = row + offset;
 lab->sub_roi = xv_create(lab->panel,PANEL_LIST,
                 PANEL_CHOOSE_ONE,FALSE,
                 PANEL_LIST_DISPLAY_ROWS,5,
                 PANEL_LIST_WIDTH,130,
                 PANEL_LAYOUT,PANEL_VERTICAL,
                 XV_X,rr->r_left + rr->r_width,XV_Y,temp,
                  PANEL_CLIENT_DATA,lab,
	          PANEL_NOTIFY_PROC,change_label_proc,
                 0);
 roi_labels = win_rec_get_sec_roi_labels_PTR(win->owner,which_roi_list);
 TRACE(printf("label roi 2...roi_labels = %u\n",roi_labels))
 setup_list_items(roi_labels,lab->sub_roi,0);
 TRACE(printf("label roi 3...roi_labels = %u\n",roi_labels))

 rr = (Rect *) xv_get(item1,XV_RECT);
 item = xv_create(lab->panel,PANEL_BUTTON,
	PANEL_LABEL_STRING,"ALL",
	PANEL_NOTIFY_PROC,all_list_proc,
        XV_X,rr->r_left + rr->r_width + 5,XV_Y,row,
        XV_KEY_DATA,1,&(lab->main_roi),
	0);

 rr = (Rect *) xv_get(item,XV_RECT);
 item = xv_create(lab->panel,PANEL_BUTTON,
	PANEL_LABEL_STRING,"NONE",
	PANEL_NOTIFY_PROC,none_list_proc,
        XV_X,rr->r_left + rr->r_width + 5,XV_Y,row,
        XV_KEY_DATA,1,&(lab->main_roi),
	0);
 rr = (Rect *) xv_get(item,XV_RECT);
 xv_create(lab->panel,PANEL_BUTTON,
	PANEL_LABEL_STRING,"MORE...",
	PANEL_NOTIFY_PROC,label_more_proc,
        XV_X,rr->r_left + rr->r_width + 5,XV_Y,row,
        PANEL_CLIENT_DATA,lab,
	0);

 rr = (Rect *) xv_get(item2,XV_RECT);
 item = xv_create(lab->panel,PANEL_BUTTON,
	PANEL_LABEL_STRING,"ALL",
	PANEL_NOTIFY_PROC,all_list_proc,
        XV_X,rr->r_left + rr->r_width + 5,XV_Y,row,
        XV_KEY_DATA,1,&(lab->sub_roi),
	0);

 rr = (Rect *) xv_get(item,XV_RECT);
 xv_create(lab->panel,PANEL_BUTTON,
	PANEL_LABEL_STRING,"NONE",
	PANEL_NOTIFY_PROC,none_list_proc,
        XV_X,rr->r_left + rr->r_width + 5,XV_Y,row,
        XV_KEY_DATA,1,&(lab->sub_roi),
	0);

 rr = (Rect *) xv_get(lab->main_roi,XV_RECT);
 row = rr->r_top + rr->r_height + 3;
 item = xv_create(lab->panel,PANEL_BUTTON,
	PANEL_LABEL_STRING,"ADD FILE LABELS",
	PANEL_NOTIFY_PROC,add_file_labels_proc,
        XV_X,5,XV_Y,row,
        PANEL_CLIENT_DATA,lab,
	0);
 rr = (Rect *) xv_get(item,XV_RECT);
 item = xv_create(lab->panel,PANEL_BUTTON,
	PANEL_LABEL_STRING,"SAVE LIST",
	PANEL_NOTIFY_PROC,save_list_proc,
        XV_X,rr->r_left + rr->r_width,XV_Y,row,
        PANEL_CLIENT_DATA,lab,
	0);
 rr = (Rect *) xv_get(item,XV_RECT);
 item = xv_create(lab->panel,PANEL_BUTTON,
	PANEL_LABEL_STRING,"READ LIST",        
        PANEL_NOTIFY_PROC,read_list_proc,
        XV_X,rr->r_left + rr->r_width,XV_Y,row,
        PANEL_CLIENT_DATA,lab,0);
 rr = (Rect *) xv_get(item,XV_RECT);
 lab->append = xv_create(lab->panel,PANEL_CHECK_BOX,
        PANEL_LABEL_STRING,"APPEND-READ",
        XV_X,rr->r_left + rr->r_width,XV_Y,row,
        PANEL_VALUE,0,
        0);

 row += offset;
 item1 = xv_create(lab->panel,PANEL_MESSAGE,
	PANEL_LABEL_STRING,"CLICK IN MAIN OR SUB TO SELECT A LABEL...",
        XV_X,5,XV_Y,row,
	0);
 row += offset - 5;
 lab->auto_add = xv_create(lab->panel,PANEL_CHECK_BOX,
        XV_X,5,XV_Y,row,
        PANEL_VALUE,1,XV_Y,row,
        PANEL_CLIENT_DATA,lab,
        0);
 rr = (Rect *) xv_get(lab->auto_add,XV_RECT);
 item = xv_create(lab->panel,PANEL_BUTTON,
	PANEL_LABEL_STRING,"ADD TO LIST",
	PANEL_NOTIFY_PROC,add_to_proc,
        XV_X,rr->r_left + rr->r_width - 5,XV_Y,row,
        PANEL_CLIENT_DATA,lab,
	0);
 rr = (Rect *) xv_get(item,XV_RECT);
 lab->label = xv_create(lab->panel,PANEL_TEXT,
        XV_X,rr->r_left + rr->r_width + 5,XV_Y,row + 2,
	PANEL_LABEL_STRING,"LABEL AS",
        PANEL_VALUE_DISPLAY_LENGTH,30,
        PANEL_VALUE_STORED_LENGTH,50,
        PANEL_CLIENT_DATA,lab,
	0);
 row += offset;

 if (lab->ready_proc == NULL)
    {
     item = xv_create(lab->panel,PANEL_BUTTON,
	PANEL_LABEL_STRING,"LABEL CUR ROI",
	PANEL_NOTIFY_PROC,name_roi_proc,
        XV_X,5,XV_Y,row,
        PANEL_CLIENT_DATA,lab,
	0);
     rr = (Rect *) xv_get(item,XV_RECT);
     item = xv_create(lab->panel,PANEL_BUTTON,
	PANEL_LABEL_STRING,"LABEL CUR REGION",
	PANEL_NOTIFY_PROC,name_cur_curve_proc,
        XV_X,rr->r_left + rr->r_width + 5,XV_Y,row,
        PANEL_CLIENT_DATA,lab,
	0);
     rr = (Rect *) xv_get(item,XV_RECT);
     lab->for_all_slices = item = xv_create(lab->panel,PANEL_CHECK_BOX,
	PANEL_CHOICE_STRINGS,"FOR ALL SLICES",NULL,
        XV_X,rr->r_left + rr->r_width + 5,XV_Y,row,
        PANEL_VALUE,0,
        PANEL_CLIENT_DATA,lab,
	0);
    }
   else
    {item = xv_create(lab->panel,PANEL_BUTTON,
	PANEL_LABEL_STRING,"READY",
	PANEL_NOTIFY_PROC,ready_roi_proc,
        XV_X,5,XV_Y,row,
        PANEL_CLIENT_DATA,lab,
	0);
     item = xv_create(lab->panel,PANEL_BUTTON,
	PANEL_LABEL_STRING,"CANCEL",
	PANEL_NOTIFY_PROC,cancel_roi_proc,
        XV_Y,row,
        PANEL_CLIENT_DATA,lab,
	0);
    }
 rr = (Rect *) xv_get(item,XV_RECT);
 window_fit(lab->panel);
 xv_set(lab->panel,XV_HEIGHT,rr->r_top + rr->r_height + 5,0);
 rr = (Rect *) xv_get(lab->panel,XV_RECT);
 xv_set(lab->panel,XV_WIDTH,rr->r_width - 10,0);

 window_fit(lab->frame);
 make_2_wins_adjacent(lab->frame,owner_frame,0);
 win_set_mouse_at_center(lab->frame);

 return NO_ERROR;
}


/*-------------------------------------------------------------------------
find_label_roi_rec - ???
-------------------------------------------------------------------------*/

struct label_roi_rec *
find_label_roi_rec(all_labs,win,owner_frame)
     struct all_label_roi_recs *all_labs;
     struct image_display_window *win;
{
 int lcv;

 for (lcv = 0; lcv < all_labs->num_labs; lcv++)
    if ((all_labs->labs[lcv]->win == win) &&
        (all_labs->labs[lcv]->owner_frame == owner_frame))
       return all_labs->labs[lcv];
 return NULL;
}


/*-------------------------------------------------------------------------
enter_label_roi - ???

FIXME:  This function can't decide whether it returns an int or a pointer.
Worse, some of its callers expect one and some expect the other.  In
addition, this seems to be a merger of two (or more) separate functions,
selected by the value of the status flag passed it.
-------------------------------------------------------------------------*/

static struct all_label_roi_recs all_labels;
static char all_labels_first_time = 1;

static struct all_label_roi_recs *
enter_label_roi(
    struct image_display_window *win,
    int status,
    Frame owner_frame,
    int (*ready_proc)(),
    void *ready_arg,
    int (*cancel_proc)(),
    void *cancel_arg,
    char *title,
    unsigned short which_roi_list)
{
 struct label_roi_rec *lab;

 if (status == GET_ALL_LABEL_ROI_PTR)
    {if (all_labels_first_time)
	return NULL;
    else
	return &all_labels;}
 else if (status == CLEAN_UP_LABEL_ROI)
   {if (!all_labels_first_time)
       if ((lab = find_label_roi_rec(&all_labels,win,owner_frame)) != NULL)
          free_label_roi_rec(lab);
    return NO_ERROR;}
 else {
     setup_label_roi(win,owner_frame,
                ready_proc,ready_arg,cancel_proc,cancel_arg,title,
                which_roi_list);
     /* FIXME:  Return value below not compatible with the declared
	type of this function.  -PWH 19990415 */
     return NO_ERROR; }
}


/*------------------------------------------------------------------------
setup_label_roi - ???
------------------------------------------------------------------------*/

int
setup_label_roi(
    struct image_display_window *win,
    Frame owner_frame,
    int (*ready_proc)(),
    void *ready_arg,
    int (*cancel_proc)(),
    void *cancel_arg,
    char *title,
    unsigned short which_roi_list)
{
 int err = NO_ERROR;
 struct label_roi_rec *lab;

 if (all_labels_first_time)
    {init_all_label_roi_recs(&all_labels);
     all_labels_first_time = 0;}
 else
    { 
     if ((lab = find_label_roi_rec(&all_labels,win,owner_frame)) != NULL)
        {
         xv_set(lab->frame,XV_SHOW,TRUE,FRAME_CLOSED,FALSE,WIN_ALARM,0);
         win_set_mouse_at_center(lab->frame);
         wmgr_top(lab->frame);
         return NO_ERROR;}
    }
 get_handle_memory(&(all_labels.labs),&(all_labels.num_labs),1,
     (size_t) sizeof(struct label_roi_rec),&err);
 if (err != NO_ERROR) return err;
 lab = all_labels.labs[all_labels.num_labs - 1];
 init_label_roi_rec(lab);
 if ((err = setup_label_roi_rec(lab,win,owner_frame,ready_proc,ready_arg,
         cancel_proc,cancel_arg,title,which_roi_list)) == NO_ERROR)
    xv_set(lab->frame,XV_SHOW,TRUE,FRAME_CLOSED,FALSE,0);

 return err;
}
