/****************************************************************************
slice_roi_rec.c - ???

Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.
****************************************************************************/

static const char rcsid[] = "$Id: slice_roi_rec.c,v 1.10 1999/07/07 17:41:19 hughett Exp $ Release $Name: nria_1_0_6 $";

#include <stdarg.h>
#include <stdio.h>
#include <string.h>

#include <sys/types.h>
#include <sys/timeb.h>

#include "error_constants.h"
#include "str_def.h"
#include "typedef.h"
#include "slice_roi_rec.h"
#include "geometry.h"
#include "ria_define_constants.h"

#include "nria_procedures.h"    


/*
#define PRT1(x) x; fflush(stdout);
#define PRINT(x) x; fflush(stdout);
#define TRACE(x) x; fflush(stdout);
#define PRT2(x) x; fflush(stdout);
#define PRT(x) x; fflush(stdout);
#define PRT3(x) x; fflush(stdout);
#define PRT4(x) x; fflush(stdout);
#define PRT5(x) x; fflush(stdout);
*/
#define PRT1(x)
#define PRINT(x)
#define TRACE(x)
#define PRT2(x)
#define PRT(x)
#define PRT3(x)
#define PRT4(x)
#define PRT5(x)



/* Functions defined in this file */
void free_curve_rec(struct curve_rec *curve);
void free_curve_recs(struct curve_rec **curves, unsigned long num_curves);
void free_curve_2d_rec(struct curve_2d_rec *region);
void free_curve_2d_recs(struct curve_2d_rec **pens, unsigned long num_pens);
void free_user_defined_rec(struct user_defined_rec *user);
void free_roi_rec(struct roi_rec *roi);
void free_slice_roi_rec(struct slice_roi_rec *slice_roi);
void init_curve_rec(struct curve_rec *curve);
void init_curve_2d_rec(struct curve_2d_rec *pen, int pen_type);
void init_user_defined_rec(struct user_defined_rec *user);
int init_roi_rec(struct roi_rec *roi, char *label);
void init_slice_roi_rec(struct slice_roi_rec *slice_roi);
void setup_slice_roi_rec_dim(struct slice_roi_rec *slice_roi,
    unsigned long dimx, unsigned long dimy);
int setup_curve_rec_ptr(struct curve_rec **curve, int pen_type);
int add_curve_rec_handle(struct curve_2d_rec *pen,
    unsigned long add_num_curves);
int setup_cur_curve_rec(struct curve_2d_rec *pen);
int setup_curve_2d_rec_ptr(struct curve_2d_rec **pen, int pen_type);
int add_curve_2d_rec_handle(struct roi_rec *roi, unsigned long add_num_pens);
int setup_cur_curve_2d_rec(struct roi_rec *roi, int pen_type, int set_cur_pen);
struct curve_2d_rec *get_pen(struct roi_rec *roi, int pen_type,
    int set_cur_pen, int *err);
int setup_roi_rec_ptr(struct roi_rec **roi, char *label);
int add_roi_rec_handle(struct slice_roi_rec *slice_roi,
    unsigned long add_num_rois);
int setup_cur_roi_rec(struct slice_roi_rec *slice_roi, char *label,
    int set_cur_roi);
int setup_more_rois(struct slice_roi_rec *slice_roi, char *label,
    int set_cur_roi);
int EnterCurveXY(struct curve_rec *curve, double x, double y,
    unsigned short *status, struct matrix_2d_rec *tolocal,
    int is_check_matrix, int use_xy_indx, unsigned long xy_indx);
int EnterCurvePts(struct curve_rec *curve, unsigned short *status,
    struct matrix_2d_rec *tolocal, unsigned long num_xys, ...);
int EnterRoiCurvePt(struct roi_rec *roi, int pen_type, int new_curve,
    double x, double y, unsigned short *status, struct matrix_2d_rec *tolocal);
int EnterPtsIntoRoiRec(struct roi_rec *roi, unsigned long pen_type,
    unsigned short *status, struct matrix_2d_rec *tolocal, int new_curve,
    unsigned long num_xys, ...);
int EnterPtAryIntoRoiRec(struct roi_rec *roi, unsigned long pen_type,
    unsigned short *status, struct matrix_2d_rec *tolocal, int new_curve,
    unsigned long num_xys, float *aryx, float *aryy);
int EnterSliceRoiRecPt(struct slice_roi_rec *slice_roi,
    struct matrix_2d_rec *tolocal, char *label, int pen_type, int new_curve,
    double x, double y);
int EnterPtsIntoSliceRoiRec(struct slice_roi_rec *slice_roi,
    struct matrix_2d_rec *tolocal, char *label, unsigned long pen_type,
    int new_curve, unsigned long num_xys, ...);
int EnterPtAryIntoSliceRoiRec(struct slice_roi_rec *slice_roi,
    struct matrix_2d_rec *tolocal, char *label, unsigned long pen_type,
    int new_curve, unsigned long num_xys, float *aryx, float *aryy);
int delete_pts(struct curve_rec *curve, unsigned long *pt_nums,
    unsigned long num_pts, unsigned short *status);
int DeleteCurveCurPts(struct curve_rec *curve, int use_start_pt,
    unsigned long start_pt, unsigned long num_pts, unsigned short *status);
int delete_curve_all_pts(struct curve_rec *curve, unsigned short *status);
int DeletePenCurPts(struct curve_2d_rec *pen, int use_start_pt, int start_pt,
    unsigned long num_pts, unsigned short *status);
int DeleteRoiCurPts(struct roi_rec *roi, int use_start_pt,
    unsigned long start_pt, unsigned long num_pts, unsigned short *status);
int DeleteSliceRoiCurPts(struct slice_roi_rec *slice_roi, int use_start_pt,
    unsigned long start_pt, unsigned long num_pts);
int delete_cur_curve(struct curve_2d_rec *pen, unsigned short *status,
    int is_free_curve);
int delete_cur_roi_curve(struct slice_roi_rec *slice_roi);
int delete_all_cur_pen_curves(struct roi_rec *roi, unsigned short *status);
int delete_all_roi_curves(struct roi_rec *roi, unsigned short *status);
int delete_cur_pen(struct roi_rec *roi, unsigned short *status);
int delete_cur_roi(struct slice_roi_rec *slice_roi);
int delete_roi(struct slice_roi_rec *slice_roi, struct roi_rec *roi);
int DeleteRoisOfType(struct slice_roi_rec *slice_roi, int roi_type,
    int select);
void delete_all_rois(struct slice_roi_rec *slice_roi);
int clean_curve_2d_rec(struct curve_2d_rec *pen);
int clean_roi_rec(struct roi_rec *roi);
int keep_only_non_empty_curves(struct slice_roi_rec *slice_roi);
int append_curves_to_curve_2d_rec(struct curve_2d_rec *pen,
    struct curve_rec **curves, unsigned long num_curves,
    unsigned short *status);
int append_curve_to_roi_rec(struct roi_rec *roi, struct curve_rec *curve,
    int pen_type, int set_cur, unsigned short *status);
int append_curves_to_roi_rec(struct roi_rec *roi, struct curve_rec **curves,
    unsigned long num_curves, unsigned short *status);
int JoinCurvesToCurve2dRec(struct curve_2d_rec *pen,
    struct curve_rec **curves, unsigned long num_curves,
    unsigned short *status, double sx, double sy,
    struct matrix_2d_rec *tolocal1, struct matrix_2d_rec *localto1,
    struct matrix_2d_rec *localto2);
int JoinCurveToRoiRec(struct roi_rec *roi, struct curve_rec *curve,
    int pen_type, int set_cur, unsigned short *status, double sx, double sy,
    struct matrix_2d_rec *tolocal1, struct matrix_2d_rec *localto1,
    struct matrix_2d_rec *localto2);
int JoinCurvesToRoiRec(struct roi_rec *roi, struct curve_rec **curves,
    unsigned long num_curves, unsigned short *status, double sx, double sy,
    struct matrix_2d_rec *tolocal1, struct matrix_2d_rec *localto1,
    struct matrix_2d_rec *localto2);
int JoinCurveRec(struct curve_rec *dest, struct curve_rec *src,
    int use_src_indx, unsigned long src_indx, unsigned short *status,
    double sx, double sy, struct matrix_2d_rec *tolocal1,
    struct matrix_2d_rec *localto1, struct matrix_2d_rec *localto2);
int JoinCurve2dRecs(struct curve_2d_rec *dest, struct curve_2d_rec *source,
    int set_cur, unsigned short *status, double sx, double sy,
    struct matrix_2d_rec *tolocal1, struct matrix_2d_rec *localto1,
    struct matrix_2d_rec *localto2);
int JoinRoiRecs(struct roi_rec *roi1, struct roi_rec *roi2, int set_cur,
    unsigned short *status, double sx, double sy,
    struct matrix_2d_rec *tolocal1, struct matrix_2d_rec *localto1,
    struct matrix_2d_rec *localto2);
int JoinRois(struct slice_roi_rec *slice_roi1,
    struct slice_roi_rec *slice_roi2, int set_cur, double sx, double sy,
    struct matrix_2d_rec *tolocal1, struct matrix_2d_rec *localto1,
    struct matrix_2d_rec *localto2);
int CpSliceRoiRec(struct slice_roi_rec *slice_roi1,
    struct slice_roi_rec *slice_roi2, struct matrix_2d_rec *tolocal1,
    struct matrix_2d_rec *localto1, struct matrix_2d_rec *localto2);
int JoinCurveToSliceRoiRec(struct slice_roi_rec *slice_roi, char *roi_label,
    int pen_type, struct curve_rec *curve, int set_cur_roi, int set_cur_pen,
    int set_cur_curve, double sx, double sy, struct matrix_2d_rec *tolocal1,
    struct matrix_2d_rec *localto1, struct matrix_2d_rec *localto2);
int JoinRoiToSliceRoiRec(struct slice_roi_rec *slice_roi, struct roi_rec *roi,
    int set_cur_roi, double sx, double sy, struct matrix_2d_rec *tolocal1,
    struct matrix_2d_rec *localto1, struct matrix_2d_rec *localto2);
int CanJointCurves(struct curve_rec *curve1, struct curve_rec *curve2,
    unsigned short *status, struct matrix_2d_rec *tolocal1,
    struct matrix_2d_rec *localto1, struct matrix_2d_rec *localto2);
int joint_curves(struct curve_2d_rec *pen, unsigned short *status);
int joint_curves_of_type(struct slice_roi_rec *slice_roi, char *roi_label,
    int pen_type, int set_cur);
int change_cur_curve_pen_type(struct slice_roi_rec *slice_roi, int pen_type);



/*--------------------------------------------------------------------------
free_curve_rec - ???
--------------------------------------------------------------------------*/

void
free_curve_rec(curve)
 struct curve_rec *curve;
{
 if (curve->total_xys > 0) c_free(curve->xys);
 init_curve_rec(curve);
}


/*--------------------------------------------------------------------------
free_curve_recs - ???
--------------------------------------------------------------------------*/

void
free_curve_recs(curves,num_curves)
 struct curve_rec **curves;
 unsigned long num_curves;
{
 unsigned long lcv;

 for (lcv = 0; lcv < num_curves; lcv++)
   free_curve_rec(curves[lcv]);
}


/*--------------------------------------------------------------------------
free_curve_2d_rec - ???
--------------------------------------------------------------------------*/

void
free_curve_2d_rec(region)
 struct curve_2d_rec *region;
{
 if (region->total_curves > 0)
    {free_curve_recs(region->curves,region->num_curves);
     free_handle_memory(region->curves,region->num_curves);
    }
 init_curve_2d_rec(region,0);
}


/*--------------------------------------------------------------------------
free_curve_2d_recs - ???
--------------------------------------------------------------------------*/

void
free_curve_2d_recs(pens,num_pens)
 struct curve_2d_rec **pens;
 unsigned long num_pens;
{
 unsigned long lcv;

 for (lcv = 0; lcv < num_pens; lcv++)
    free_curve_2d_rec(pens[lcv]);
}


/*--------------------------------------------------------------------------
free_user_defined_rec - ???
--------------------------------------------------------------------------*/

void
free_user_defined_rec(user)
 struct user_defined_rec *user;
{
 if (user->ptr != NULL)
    {if (user->exit != NULL)
        user->exit(&(user->ptr));
    }
 init_user_defined_rec(user);
}


/*--------------------------------------------------------------------------
free_roi_rec - ???
--------------------------------------------------------------------------*/

void
free_roi_rec(roi)
 struct roi_rec *roi;
{
 c_free(roi->label);
 if (roi->total_pens > 0)
    {free_curve_2d_recs(roi->pens,roi->num_pens);
     free_handle_memory(roi->pens,roi->num_pens);
     free_user_defined_rec(&(roi->user));
    }
 init_roi_rec(roi,NULL);
}


/*--------------------------------------------------------------------------
free_slice_roi_rec - ???
--------------------------------------------------------------------------*/

void
free_slice_roi_rec(slice_roi)
 struct slice_roi_rec *slice_roi;
{
 long lcv;

 if (slice_roi->total_rois > 0)
    {
     for (lcv = 0; lcv < slice_roi->num_rois; lcv++)
        {free_roi_rec(slice_roi->rois[lcv]);
         c_free(slice_roi->rois[lcv]);}
     c_free(slice_roi->rois);}
 init_slice_roi_rec(slice_roi);
}


/*--------------------------------------------------------------------------
init_curve_rec - ???
--------------------------------------------------------------------------*/

void
init_curve_rec(curve)
 struct curve_rec *curve;
{
 curve->status = 0;
 curve->more_status = CURVE_NEW;
 curve->total_xys = 0;
 curve->num_xys = 0;
 curve->xys = NULL;
 init_float_bound_rec(&(curve->fl_bound));
}


/*--------------------------------------------------------------------------
init_curve_2d_rec - ???
--------------------------------------------------------------------------*/

void
init_curve_2d_rec(pen,pen_type)
 struct curve_2d_rec *pen;
 unsigned short pen_type;
{
 pen->pen_type = pen_type;
 pen->total_curves = pen->cur_curve = pen->num_curves = 0;
 pen->curves = NULL;
}


/*--------------------------------------------------------------------------
init_user_defined_rec - ???
--------------------------------------------------------------------------*/

void
init_user_defined_rec(user)
 struct user_defined_rec *user;
{
 user->ptr = NULL;
 user->exit = NULL;
}


/*--------------------------------------------------------------------------
init_roi_rec - ???
--------------------------------------------------------------------------*/

int
init_roi_rec(roi,label)
 struct roi_rec *roi;
 char *label;
{
 int err;

 roi->label = NULL;
 if (label != NULL)
    {err = cp_str(&(roi->label),label);
    }
   else err = NO_ERROR;
 roi->status = ROI_VISIBLE | OWN_ROI;
 roi->prog_status = 0;
 roi->total_pens = roi->cur_pen = roi->num_pens = 0;
 roi->pens = NULL;
 init_user_defined_rec(&(roi->user));
 return err;
}


/*--------------------------------------------------------------------------
init_slice_roi_rec - ???
--------------------------------------------------------------------------*/

void
init_slice_roi_rec(slice_roi)
 struct slice_roi_rec *slice_roi;
{
 slice_roi->status = 0;
 slice_roi->prog_status = 0;
 slice_roi->xpixel_size = .0;
 slice_roi->ypixel_size = .0;
 slice_roi->dimx = 0;
 slice_roi->dimy = 0;
 slice_roi->total_rois = slice_roi->cur_roi = slice_roi->num_rois = 0;
 slice_roi->rois = NULL;
}


/*--------------------------------------------------------------------------
setup_slice_roi_rec_dim - ???
--------------------------------------------------------------------------*/

void
setup_slice_roi_rec_dim(slice_roi,dimx,dimy)
 struct slice_roi_rec *slice_roi;
 unsigned long dimx,dimy;
{
 slice_roi->dimx = dimx;
 slice_roi->dimy = dimy;
 slice_roi->status |= SLICE_ROI_DIM_VALID;
}


/*--------------------------------------------------------------------------
setup_curve_rec_ptr - ???
--------------------------------------------------------------------------*/

int
setup_curve_rec_ptr(curve,pen_type)
 struct curve_rec **curve;
 unsigned short pen_type;
{
 int err;

 if ((err = get_ptr(curve,NULL,(unsigned long) 1,(size_t) sizeof(**curve)))
        != NO_ERROR) 
    return err;
 init_curve_rec(*curve);
 (*curve)->status |= pen_type;
 return NO_ERROR;
}


/*--------------------------------------------------------------------------
add_curve_rec_handle - ???
--------------------------------------------------------------------------*/

int
add_curve_rec_handle(pen,add_num_curves)
 struct curve_2d_rec *pen;
 unsigned long add_num_curves;
{
 unsigned long cnt,num;
 int err;

 if (add_num_curves == 0) return NO_ERROR;
 if (pen->num_curves > pen->total_curves)
    pen->num_curves = pen->total_curves;
 if ((pen->num_curves + add_num_curves) > pen->total_curves)
    {num = pen->total_curves;
     if ((err = get_ptr(&(pen->curves),&(pen->total_curves),add_num_curves,
                    (size_t) sizeof(*(pen->curves)))) != NO_ERROR)
        return err;
     for (cnt = num; cnt < pen->total_curves; cnt++)
        pen->curves[cnt] = NULL;
    }
 return NO_ERROR;
}


/*--------------------------------------------------------------------------
setup_cur_curve_rec - ???
--------------------------------------------------------------------------*/

int
setup_cur_curve_rec(pen)
 struct curve_2d_rec *pen;
{
 struct curve_rec *curve;
 int err;

 if ((err = add_curve_rec_handle(pen,(unsigned long) 1)) != NO_ERROR) 
    return err;

 if ((err = setup_curve_rec_ptr(&curve,pen->pen_type)) != NO_ERROR)
    return err;

 pen->cur_curve = pen->num_curves;
 pen->curves[pen->num_curves] = curve;
 pen->num_curves += 1;
 return err;
}


/*--------------------------------------------------------------------------
setup_curve_2d_rec_ptr - ???
--------------------------------------------------------------------------*/

int
setup_curve_2d_rec_ptr(pen,pen_type)
 struct curve_2d_rec **pen;
 unsigned short pen_type;
{
 int err;

 if ((err = get_ptr(pen,NULL,(unsigned long) 1,(size_t) sizeof(**pen)))
       != NO_ERROR) 
    return err;
 init_curve_2d_rec(*pen,pen_type);
 return NO_ERROR;
}


/*--------------------------------------------------------------------------
add_curve_2d_rec_handle - ???
--------------------------------------------------------------------------*/

int
add_curve_2d_rec_handle(roi,add_num_pens)
 struct roi_rec *roi;
 unsigned long add_num_pens;
{
 unsigned long cnt,num;
 int err;

 if (add_num_pens == 0) return NO_ERROR;
 if (roi->num_pens > roi->total_pens)
    roi->num_pens = roi->total_pens;
 if ((roi->num_pens + add_num_pens) > roi->total_pens)
    {num = roi->total_pens;
     if ((err = get_ptr(&(roi->pens),&(roi->total_pens),add_num_pens,
                    (size_t) sizeof(*(roi->pens)))) != NO_ERROR)
        return err;
     for (cnt = num; cnt < roi->total_pens; cnt++)
        roi->pens[cnt] = NULL;
    }
 return NO_ERROR;
}


/*--------------------------------------------------------------------------
setup_cur_curve_2d_rec - ???
--------------------------------------------------------------------------*/

int
setup_cur_curve_2d_rec(roi,pen_type,set_cur_pen)
 struct roi_rec *roi;
 unsigned short pen_type;
 int set_cur_pen;
{
 int err;

 if ((err = add_curve_2d_rec_handle(roi,(unsigned long) 1)) != NO_ERROR)
    return err;
 if ((err = setup_curve_2d_rec_ptr(&(roi->pens[roi->num_pens]),
                pen_type)) != NO_ERROR)
     return err;
 if (set_cur_pen) roi->cur_pen = roi->num_pens;
 roi->num_pens += 1;
 return NO_ERROR;
}


/*--------------------------------------------------------------------------
get_pen - ???
--------------------------------------------------------------------------*/

struct curve_2d_rec *
get_pen(roi,pen_type,set_cur_pen,err)
 struct roi_rec *roi;
 unsigned short pen_type;
 int set_cur_pen;
 int *err;
{
 struct curve_2d_rec *pen;
 int cur_pen;

 *err = NO_ERROR;
 if ((pen = find_pen(roi,pen_type,set_cur_pen)) == NULL)
    {cur_pen = roi->cur_pen;
     if ((*err = setup_cur_curve_2d_rec(roi,pen_type,1)) != NO_ERROR)
        return NULL;
     pen = roi->pens[roi->cur_pen];
     if (!set_cur_pen) roi->cur_pen = cur_pen;
    }
 return pen;
}


/*--------------------------------------------------------------------------
setup_roi_rec_ptr - ???
--------------------------------------------------------------------------*/

int
setup_roi_rec_ptr(roi,label)
 struct roi_rec **roi;
 char *label;
{
 int err;

 if ((err = get_ptr(roi,NULL,(unsigned long) 1,(size_t) sizeof(**roi))) !=
        NO_ERROR)
    return err;
 init_roi_rec(*roi,label);
 return NO_ERROR;
}


/*--------------------------------------------------------------------------
add_roi_rec_handle - ???
--------------------------------------------------------------------------*/

int
add_roi_rec_handle(slice_roi,add_num_rois)
 struct slice_roi_rec *slice_roi;
 unsigned long add_num_rois;
{
 unsigned long cnt,num;
 int err;

 if (add_num_rois == 0) return NO_ERROR;
 if (slice_roi->num_rois > slice_roi->total_rois)
    slice_roi->num_rois = slice_roi->total_rois;
 if ((slice_roi->num_rois + add_num_rois) > slice_roi->total_rois)
    {num = slice_roi->total_rois;
     if ((err = get_ptr(&(slice_roi->rois),&(slice_roi->total_rois),add_num_rois,
                    (size_t) sizeof(*(slice_roi->rois)))) != NO_ERROR)
        return err;
     for (cnt = num; cnt < slice_roi->total_rois; cnt++)
        slice_roi->rois[cnt] = NULL;
    }
 return NO_ERROR;
}


/*--------------------------------------------------------------------------
setup_cur_roi_rec - ???
--------------------------------------------------------------------------*/

int
setup_cur_roi_rec(slice_roi,label,set_cur_roi)
 struct slice_roi_rec *slice_roi;
 char *label;
 int set_cur_roi;
{
 int err;

 if ((err = add_roi_rec_handle(slice_roi,1)) != NO_ERROR) return err;
 if ((err = setup_roi_rec_ptr(&(slice_roi->rois[slice_roi->num_rois]),
                label)) != NO_ERROR)
     return err;
 if (set_cur_roi) slice_roi->cur_roi = slice_roi->num_rois;
 slice_roi->num_rois += 1;
 return NO_ERROR;
}


/*--------------------------------------------------------------------------
setup_more_rois - ???
--------------------------------------------------------------------------*/

int
setup_more_rois(slice_roi,label,set_cur_roi)
 struct slice_roi_rec *slice_roi;
 char *label;
 int set_cur_roi;
{
 struct roi_rec *roi;

 roi = get_cur_roi(slice_roi);
 if (((roi == NULL) || (no_case_str_cmp(roi->label,label) != 0) ) &&
     (find_roi_rec(slice_roi,label,set_cur_roi) == NULL))
    {return setup_cur_roi_rec(slice_roi,label,set_cur_roi);
    }
 return NO_ERROR;
}


/*--------------------------------------------------------------------------
EnterCurveXY - ???
--------------------------------------------------------------------------*/

int
EnterCurveXY(curve,x,y,status,tolocal,is_check_matrix,use_xy_indx,xy_indx)
 struct curve_rec *curve;
 float x,y;
 unsigned short *status;
 struct matrix_2d_rec *tolocal;
 int is_check_matrix,use_xy_indx;
 unsigned long xy_indx;
{
 int err;
 float x1,y1,newx,newy;
 int yes;
 unsigned long indx;
 struct matrix_2d_rec mat,*pmat;

 PRT5(printf("EnterCurveXY( x = %f y = %f num_xys = %d use = %d indx = %d\n",
         x,y,curve->num_xys,use_xy_indx,xy_indx);)
 if ((is_check_matrix && (curve->status & CURVE_MATRIX_IN_USE)) ||
     (tolocal != NULL))
    {if (is_check_matrix && (curve->status & CURVE_MATRIX_IN_USE)) 
        curve_matrix(curve,tolocal,&mat,&pmat,0);
       else pmat = tolocal;
     PRT4(printf("pmat = %u\n",pmat))
     x1 = x;
     y1 = y;
     trans_2d_pt(pmat,x,y,&newx,&newy,0);
     x = newx;
     y = newy;
     PRT4(printf("EnterCurveXY transfrom x = %f y = %f\n",x,y);)
     yes = 1;
    }
   else yes = 0;
 if (use_xy_indx)
    {if (xy_indx > curve->num_xys) indx = curve->num_xys;
       else indx = xy_indx;}
   else indx = curve->num_xys;

 if ((curve->num_xys > 0) && (indx == curve->num_xys))
    { 
     if ((curve->xys[indx - 1].x == x) &&
         (curve->xys[indx - 1].y == y)) return NO_ERROR;
    }

 if (indx >= curve->total_xys)
    {if ((err = get_ptr(&(curve->xys),&(curve->total_xys),
         (unsigned long) ALLOCATE_MAX_XY_PTS,
         (size_t) sizeof(*(curve->xys)))) != NO_ERROR)
        return err;}

 curve->xys[indx].x = x;
 curve->xys[indx].y = y;
 PRT(printf("EnterCurveXY end indx = %d x = %d y = %d num_xys = %d\n",
        indx,(int) x,(int) y,curve->num_xys);)
 if (yes)
    {x = x1;
     y = y1;}
 if (curve->num_xys == 0)
    {curve->fl_bound.left = curve->fl_bound.right = x;
     curve->fl_bound.top = curve->fl_bound.bottom = y;
     curve->status |= COORDINATE_BOUND_COMPUTED;
    }
   else if (curve->status & COORDINATE_BOUND_COMPUTED)
    {if (curve->fl_bound.left > x) curve->fl_bound.left = x;
     if (curve->fl_bound.right < x) curve->fl_bound.right = x;
     if (curve->fl_bound.top > y) curve->fl_bound.top = y;
     if (curve->fl_bound.bottom < y) curve->fl_bound.bottom = y;}
 if (indx == curve->num_xys) curve->num_xys++;
 *status |= SLICE_ROI_NON_EMPTY | SLICE_ROI_TO_BE_SAVED;
 return NO_ERROR;
}



/*--------------------------------------------------------------------------
EnterCurvePts - ???

EnterCurvePts(curve,status,tolocal,num_xys,x1,y1,...,xn,yn)
--------------------------------------------------------------------------*/

int
EnterCurvePts(
    struct curve_rec *curve,
    unsigned short *status,
    struct matrix_2d_rec *tolocal,
    unsigned long num_xys,
    ...)
{
 va_list args;
 unsigned long lcv;
 float x,y;
 int err;

 va_start(args, num_xys);
 err = NO_ERROR;
 for (lcv = 0; lcv < num_xys; lcv++)
    {x = (float) va_arg(args,double);
     y = (float) va_arg(args,double);
     if ((err = EnterCurveXY(curve,x,y,status,tolocal,1,0,0)) != NO_ERROR)
        break;
    }
 va_end(args);

 return err;
}


/*--------------------------------------------------------------------------
EnterRoiCurvePt - ???
--------------------------------------------------------------------------*/

int
EnterRoiCurvePt(roi,pen_type,new_curve,x,y,status,tolocal)
 struct roi_rec *roi;
 unsigned short pen_type;
 int new_curve;
 float x,y;
 unsigned short *status;
 struct matrix_2d_rec *tolocal;
{
 struct curve_2d_rec *pen;
 int err;

 if ((pen = find_pen(roi,pen_type,1)) == NULL)
    {if ((err = setup_cur_curve_2d_rec(roi,pen_type,1)) != NO_ERROR)
        return err;
     pen = roi->pens[roi->cur_pen];
    }

 if (new_curve || (pen->cur_curve >= pen->num_curves))
    {err = setup_cur_curve_rec(pen);
    }
   else err = NO_ERROR;

 if (err == NO_ERROR)
    {
     err = EnterCurveXY(pen->curves[pen->cur_curve],
              x,y,status,tolocal,1,0,0);
     if (err == NO_ERROR) roi->prog_status |= ROI_MUST_UPDATE;
    }

 return err;
}


/*--------------------------------------------------------------------------
EnterPtsIntoRoiRec - ???

EnterPtsIntoRoiRec(roi,pen_type,status,tolocal,new_curve,num_xys,
    x1,y1,...,xn,yn)
--------------------------------------------------------------------------*/

int
EnterPtsIntoRoiRec(
    struct roi_rec *roi,
    unsigned long pen_type,
    unsigned short *status,
    struct matrix_2d_rec *tolocal,
    int new_curve,
    unsigned long num_xys,
    ...)
{
 va_list args;

 unsigned short pen_type1;
 float x,y;
 int err;
 unsigned long lcv;

 pen_type1 = pen_type;

 va_start(args, num_xys);
 err = CANCEL;
 for (lcv = 0; lcv < num_xys; lcv++,new_curve = 0)
    {x = (float) va_arg(args,double);
     y = (float) va_arg(args,double);
     if ((err = EnterRoiCurvePt(roi,
                     pen_type1,new_curve,x,y,status,tolocal)) != NO_ERROR)
        break;}
 va_end(args);
 return err;
}


/*--------------------------------------------------------------------------
EnterPtAryIntoRoiRec - ???
--------------------------------------------------------------------------*/

int
EnterPtAryIntoRoiRec(
    struct roi_rec *roi,
    unsigned long pen_type,
    unsigned short *status,
    struct matrix_2d_rec *tolocal,
    int new_curve,
    unsigned long num_xys,
    float *aryx,
    float *aryy)
{
 unsigned short pen_type1;
 float x,y;
 int err;
 unsigned long lcv;

 pen_type1 = pen_type;

 err = CANCEL;
 for (lcv = 0; lcv < num_xys; lcv++,new_curve = 0)
    {x = aryx[lcv];
     y = aryy[lcv];
     if ((err = EnterRoiCurvePt(roi,
                     pen_type1,new_curve,x,y,status,tolocal)) != NO_ERROR)
        break;}
 return err;
}


/*--------------------------------------------------------------------------
EnterSliceRoiRecPt - ???
--------------------------------------------------------------------------*/

int
EnterSliceRoiRecPt(slice_roi,tolocal,label,pen_type,new_curve,x,y)
 struct slice_roi_rec *slice_roi;
 struct matrix_2d_rec *tolocal;
 char *label;
 unsigned short pen_type;
 int new_curve;
 float x,y;
{
 int err;

 if ((err = setup_more_rois(slice_roi,label,1)) == NO_ERROR)
    {err = EnterRoiCurvePt(slice_roi->rois[slice_roi->cur_roi],
           pen_type,new_curve,x,y,&(slice_roi->status),tolocal);}
 return err;
}


/*--------------------------------------------------------------------------
EnterPtsIntoSliceRoiRec - ???

EnterPtsIntoSliceRoiRec(slice_roi,tolocal,label,pen_type,new_curve,
                             num_xys,x1,y1,...,xn,yn)
--------------------------------------------------------------------------*/

int
EnterPtsIntoSliceRoiRec(
    struct slice_roi_rec *slice_roi,
    struct matrix_2d_rec *tolocal,
    char *label,
    unsigned long pen_type,
    int new_curve,
    unsigned long num_xys,
    ...)
{
 va_list args;

 unsigned short pen_type1;
 float x,y;
 struct roi_rec *roi;
 int err;
 unsigned long lcv;

 pen_type1 = pen_type;

 va_start(args, num_xys);
 if ((err = setup_more_rois(slice_roi,label,1)) == NO_ERROR)
    {if ((roi = get_cur_roi(slice_roi)) != NULL)
       {
        for (lcv = 0; lcv < num_xys; lcv++,new_curve = 0)
           {x = (float) va_arg(args,double);
            y = (float) va_arg(args,double);
            if ((err = EnterRoiCurvePt(roi,
                pen_type1,new_curve,x,y,&(slice_roi->status),tolocal)) != NO_ERROR)
               break;
           }
       }
      else err = CANCEL;
     }
 va_end(args);
 return err;
}


/*--------------------------------------------------------------------------
EnterPtAryIntoSliceRoiRec - ???
--------------------------------------------------------------------------*/

int
EnterPtAryIntoSliceRoiRec(
    struct slice_roi_rec *slice_roi,  
    struct matrix_2d_rec *tolocal,
    char *label,
    unsigned long pen_type,
    int new_curve,
    unsigned long num_xys,
    float *aryx,
    float *aryy)
{
 unsigned short pen_type1;
 long new_curve1;
 float x,y;

 struct roi_rec *roi;
 int err;
 unsigned long lcv;

 pen_type1 = pen_type;
 new_curve1 = new_curve;

 if ((err = setup_more_rois(slice_roi,label,1)) == NO_ERROR)
     if ((roi = get_cur_roi(slice_roi)) != NULL) {
	 /* FIXME:  How is new_curve1 being used below?? -PWH 19990405 */
	 for (lcv = 0; lcv < num_xys; lcv++,new_curve1 = 0) {
	     x = aryx[lcv];
	     y = aryy[lcv];
	     if ((err = EnterRoiCurvePt(roi,pen_type1,new_curve1,x,y,
					&(slice_roi->status),tolocal))
		 != NO_ERROR)
		 break;}
     }
     else err = CANCEL;
 return err;
}


/*--------------------------------------------------------------------------
delete_pts - ???
--------------------------------------------------------------------------*/

int
delete_pts(curve,pt_nums,num_pts,status)
 struct curve_rec *curve;
 unsigned long *pt_nums;
 unsigned long num_pts;
 unsigned short *status;
{
 static unsigned char bits[] = {0x7f,0xbf,0xdf,0xef,0xf7,0xfb,0xfd,0xfe};
 static unsigned char _1_bit[] = {0x80,0x40,0x20,0x10,0x08,0x04,0x02,0x01};
 struct float_xy_rec *pts;
 long num_bytes,lcv,total,temp;
 unsigned char *ptr;
 int err;

 if ((curve->num_xys == 0) || (num_pts == 0)) return NO_ERROR;
 num_bytes = curve->num_xys / 8;
 if (curve->num_xys % 8) num_bytes += 1;
 if ((err = get_ptr(&ptr,NULL,num_bytes,(size_t) sizeof(*ptr))) != NO_ERROR)
    return err;
 memset(ptr,0xffffffff,(int) num_bytes);
 for (total = 0,lcv = 0; lcv < num_pts; lcv++)
    {if ((pt_nums[lcv] < curve->num_xys) &&
         (ptr[pt_nums[lcv] / 8] & _1_bit[pt_nums[lcv] % 8]))
        {ptr[pt_nums[lcv] / 8] &= bits[pt_nums[lcv] % 8];
         total += 1;}
    }
 if (total > 0)
    {total = curve->num_xys - total;
     if (total > 0)
        {temp = 0;
         if ((err = get_ptr(&pts,&temp,total, 
                    (size_t) sizeof(*pts))) == NO_ERROR)
            {
             for (total = 0,lcv = 0; lcv < curve->num_xys; lcv++)
                if (ptr[lcv / 8] & _1_bit[lcv % 8])
                   pts[total++] = curve->xys[lcv];
             c_free(curve->xys);
             curve->xys = pts;
             curve->total_xys = temp;
             curve->num_xys = total;
             curve->status &= ~COORDINATE_BOUND_COMPUTED;
            }
        }
       else free_curve_rec(curve);
     *status |= SLICE_ROI_TO_BE_SAVED;
     if (total > 0)
        *status |= SLICE_ROI_NON_EMPTY;
       else
        *status &= ~SLICE_ROI_NON_EMPTY;
    }
 c_free(ptr);
 return err;
}


/*--------------------------------------------------------------------------
DeleteCurveCurPts - ???
--------------------------------------------------------------------------*/

int
DeleteCurveCurPts(curve,use_start_pt,start_pt,num_pts,status)
 struct curve_rec *curve;
 int use_start_pt;
 unsigned long start_pt,num_pts;
 unsigned short *status;
{
 unsigned long last,lcv,cnt;

 PRT(printf("DeleteCurveCurPts...num_xys = %d use_start_pt = %d start_pt = %d num_pts = %d\n",
            curve->num_xys,use_start_pt,start_pt,num_pts))
 if (curve->num_xys > 0)
    {if (use_start_pt) last = start_pt + num_pts;
       else last = num_pts;
     if (last >= curve->num_xys)
        {if (use_start_pt) curve->num_xys = start_pt;
           else curve->num_xys = 0;}
       else if (use_start_pt)
        for (lcv = start_pt,cnt = last; cnt < curve->num_xys; lcv++,cnt++)
            curve->xys[lcv] = curve->xys[cnt];
       else curve->num_xys -= num_pts;

     if (curve->num_xys == 0)
        curve->status &= ~CURVE_MATRIX_IN_USE;
     curve->status &= ~COORDINATE_BOUND_COMPUTED;
     *status |= SLICE_ROI_NON_EMPTY | SLICE_ROI_TO_BE_SAVED;
    }
 return NO_ERROR;
}


/*--------------------------------------------------------------------------
delete_curve_all_pts - ???
--------------------------------------------------------------------------*/

int
delete_curve_all_pts(curve,status)
 struct curve_rec *curve;
 unsigned short *status;
{
 if ((curve != NULL) && (curve->num_xys > 0))
    {curve->num_xys = 0;
     curve->status &= ~CURVE_MATRIX_IN_USE;
     curve->status &= ~COORDINATE_BOUND_COMPUTED;
     *status |= SLICE_ROI_NON_EMPTY | SLICE_ROI_TO_BE_SAVED;
    }
 return NO_ERROR;
}


/*--------------------------------------------------------------------------
DeletePenCurPts - ???
--------------------------------------------------------------------------*/

int
DeletePenCurPts(pen,use_start_pt,start_pt,num_pts,status)
 struct curve_2d_rec *pen;
 int use_start_pt;
 unsigned long num_pts;
 unsigned short *status;
{
 if (pen->cur_curve >= pen->num_curves) return CANCEL;
 return DeleteCurveCurPts(pen->curves[pen->cur_curve],use_start_pt,
             start_pt,num_pts,status);
}


/*--------------------------------------------------------------------------
DeleteRoiCurPts - ???
--------------------------------------------------------------------------*/

int
DeleteRoiCurPts(roi,use_start_pt,start_pt,num_pts,status)
 struct roi_rec *roi;
 int use_start_pt;
 unsigned long start_pt,num_pts;
 unsigned short *status;
{
 int err;

 if (roi->cur_pen >= roi->num_pens) return CANCEL;
 if ((err = DeletePenCurPts(roi->pens[roi->cur_pen],
                use_start_pt,start_pt,num_pts,status)) == NO_ERROR)
    roi->prog_status |= ROI_MUST_UPDATE;
 return err;
}


/*--------------------------------------------------------------------------
DeleteSliceRoiCurPts - ???
--------------------------------------------------------------------------*/

int
DeleteSliceRoiCurPts(slice_roi,use_start_pt,start_pt,num_pts)
 struct slice_roi_rec *slice_roi;
 int use_start_pt;
 unsigned long start_pt,num_pts;
{
 if (slice_roi->cur_roi >= slice_roi->num_rois) return CANCEL;
 return DeleteRoiCurPts(slice_roi->rois[slice_roi->cur_roi],
         use_start_pt,start_pt,num_pts,&(slice_roi->status));
}


/*--------------------------------------------------------------------------
delete_cur_curve - ???
--------------------------------------------------------------------------*/

int
delete_cur_curve(pen,status,is_free_curve)
 struct curve_2d_rec *pen;
 unsigned short *status;
 int is_free_curve;
{
 unsigned long lcv;
 struct curve_rec *cur_curve;

 if (pen->num_curves == 0) return NO_ERROR;
 if (pen->cur_curve >= pen->num_curves)
    {pen->cur_curve = 0;
     return NO_ERROR;}
 
 if (is_free_curve)
    {cur_curve = pen->curves[pen->cur_curve];
     free_curve_rec(cur_curve);
     c_free(cur_curve);}

 pen->num_curves--;
 for (lcv = pen->cur_curve; lcv < pen->num_curves; lcv++)
    pen->curves[lcv] = pen->curves[lcv + 1];
 pen->curves[lcv] = NULL;

 if (pen->cur_curve >= pen->num_curves) pen->cur_curve = 0;

 *status |= SLICE_ROI_NON_EMPTY | SLICE_ROI_TO_BE_SAVED;

 return NO_ERROR;
}


/*--------------------------------------------------------------------------
delete_cur_roi_curve - ???
--------------------------------------------------------------------------*/

int
delete_cur_roi_curve(slice_roi)
 struct slice_roi_rec *slice_roi;
{
 int err;
 struct roi_rec *roi;
 struct curve_2d_rec *pen;

 if ((pen = get_cur_pen(slice_roi)) == NULL) return NO_ERROR;
 if ((err = delete_cur_curve(pen,&(slice_roi->status),1))
        == NO_ERROR)
    {if ((roi = get_cur_roi(slice_roi)) != NULL)
        roi->prog_status |= ROI_MUST_UPDATE;
    }
 return err;
}


/*--------------------------------------------------------------------------
delete_all_cur_pen_curves - ???
--------------------------------------------------------------------------*/

int
delete_all_cur_pen_curves(roi,status)
 struct roi_rec *roi;
 unsigned short *status;
{
 unsigned long lcv;
 struct curve_2d_rec *pen;

 if (roi->num_pens == 0) return NO_ERROR;
 if (roi->cur_pen >= roi->num_pens)
    {roi->cur_pen = 0;
     return NO_ERROR;}
 pen = roi->pens[roi->cur_pen];
 for (lcv = 0; lcv < pen->num_curves; lcv++)
    {free_curve_rec(pen->curves[lcv]);
     c_free(pen->curves[lcv]);
     pen->curves[lcv] = NULL;
    }
 pen->num_curves = pen->cur_curve = 0;
 if (status != NULL)
    *status |= SLICE_ROI_TO_BE_SAVED;

 return NO_ERROR;
}


/*--------------------------------------------------------------------------
delete_all_roi_curves - ???
--------------------------------------------------------------------------*/

int
delete_all_roi_curves(roi,status)
 struct roi_rec *roi;
 unsigned short *status;
{
 char *label;

 label = roi->label;
 roi->label = NULL;
 free_roi_rec(roi);
 roi->label = label;
 if (status != NULL)
    *status |= SLICE_ROI_TO_BE_SAVED;
 return NO_ERROR;
}


/*--------------------------------------------------------------------------
delete_cur_pen - ???
--------------------------------------------------------------------------*/

int
delete_cur_pen(roi,status)
 struct roi_rec *roi;
 unsigned short *status;
{
 unsigned long lcv;

/*
 TRACE(printf("delete_cur_pen...begin..."))
*/
 if ((roi == NULL) || (roi->num_pens == 0)) return NO_ERROR;
 if (roi->cur_pen >= roi->num_pens) 
    {roi->cur_pen = 0;
     return NO_ERROR;}
 free_curve_2d_rec(roi->pens[roi->cur_pen]);
 c_free(roi->pens[roi->cur_pen]);

 roi->num_pens--;
 for (lcv = roi->cur_pen; lcv < roi->num_pens; lcv++)
    roi->pens[lcv] = roi->pens[lcv + 1];
 roi->pens[lcv] = NULL;

 if (roi->cur_pen >= roi->num_pens) roi->cur_pen = 0;
/*
 TRACE(printf("delete_cur_pen...end...cur_pen = %d num_pens = %d\n",
        (int) roi->cur_pen,(int) roi->num_pens))
*/
 roi->prog_status |= ROI_MUST_UPDATE;
 *status |= SLICE_ROI_TO_BE_SAVED;
 return NO_ERROR;
}


/*--------------------------------------------------------------------------
delete_cur_roi - ???
--------------------------------------------------------------------------*/

int
delete_cur_roi(slice_roi)
 struct slice_roi_rec *slice_roi;
{
 unsigned long lcv;
 struct roi_rec *cur_roi;

 if (slice_roi->num_rois == 0) return NO_ERROR;
 if (slice_roi->cur_roi >= slice_roi->num_rois)
    {slice_roi->cur_roi = 0;
     return NO_ERROR;}
 cur_roi = slice_roi->rois[slice_roi->cur_roi];
 free_roi_rec(cur_roi);
 c_free(cur_roi);

 slice_roi->num_rois--;
 for (lcv = slice_roi->cur_roi; lcv < slice_roi->num_rois; lcv++)
    slice_roi->rois[lcv] = slice_roi->rois[lcv + 1];
 slice_roi->rois[lcv] = NULL;

 if (slice_roi->cur_roi >= slice_roi->num_rois) slice_roi->cur_roi = 0;

 if (is_slice_roi_non_empty(slice_roi))
    slice_roi->status |= SLICE_ROI_NON_EMPTY;
   else slice_roi->status &= ~SLICE_ROI_NON_EMPTY;
 slice_roi->status |= SLICE_ROI_TO_BE_SAVED;

 return NO_ERROR;
}


/*--------------------------------------------------------------------------
delete_roi - ???
--------------------------------------------------------------------------*/

int
delete_roi(slice_roi,roi)
 struct slice_roi_rec *slice_roi;
 struct roi_rec *roi;
{
 struct roi_rec *cur_roi;
 unsigned long indx;
 int err;

 indx = find_handle_element(slice_roi->rois,slice_roi->num_rois,roi,&err);
 if (err == NO_ERROR)
    {cur_roi = get_cur_roi(slice_roi);
     slice_roi->cur_roi = indx;
     if ((err = delete_cur_roi(slice_roi)) == NO_ERROR)
        {if ((cur_roi != NULL) && (cur_roi != roi))
            slice_roi->cur_roi = find_handle_element(slice_roi->rois,
                                   slice_roi->num_rois,cur_roi,&err);
        }
       else return err;
    }
 return NO_ERROR;
}


/*--------------------------------------------------------------------------
DeleteRoisOfType - ???

if select & 1 then use the roi->status
if select & 2 then use the roi->prog_status
--------------------------------------------------------------------------*/

int
DeleteRoisOfType(slice_roi,roi_type,select)
 struct slice_roi_rec *slice_roi;
 unsigned short roi_type;
{
 unsigned long lcv,temp;
 int again;
 int err;

 do
    {
     for (again = 0,lcv = 0; lcv < slice_roi->num_rois; lcv++)
        {if ((select & 1) && (slice_roi->rois[lcv]->status & roi_type))
            again = 1;
           else if ((select & 2) && (slice_roi->rois[lcv]->prog_status & roi_type))
            again = 1;
         if (again)
            {temp = slice_roi->num_rois;
             if ((err = delete_roi(slice_roi,slice_roi->rois[lcv])) != NO_ERROR)
                return err;
             if (temp == slice_roi->num_rois) again = 0;
             break;}
        }
    }
 while (again);
 return NO_ERROR;
}


/*--------------------------------------------------------------------------
delete_all_rois - ???
--------------------------------------------------------------------------*/

void
delete_all_rois(slice_roi)
 struct slice_roi_rec *slice_roi;
{
 free_slice_roi_rec(slice_roi);
 slice_roi->status |= SLICE_ROI_TO_BE_SAVED;
}


/*--------------------------------------------------------------------------
clean_curve_2d_rec - ???
--------------------------------------------------------------------------*/

int
clean_curve_2d_rec(pen)
 struct curve_2d_rec *pen;
{
 unsigned long lcv,cnt;
 struct curve_rec *curve;

 for (lcv = 0; lcv < pen->num_curves; )
    if (pen->curves[lcv]->num_xys > 0) lcv++;
      else
       {pen->num_curves--;
        curve = pen->curves[lcv];
        free_curve_rec(curve);
        c_free(curve);
        for (cnt = lcv; cnt < pen->num_curves; cnt++)
           pen->curves[cnt] = pen->curves[cnt + 1];
        pen->curves[cnt] = NULL;
       }
 return pen->num_curves;
}


/*--------------------------------------------------------------------------
clean_roi_rec - ???
--------------------------------------------------------------------------*/

int
clean_roi_rec(roi)
 struct roi_rec *roi;
{
 unsigned long lcv,cnt;

 for (lcv = 0; lcv < roi->num_pens; )
    if (clean_curve_2d_rec(roi->pens[lcv])) lcv++;
      else
       {roi->num_pens--;
        free_curve_2d_rec(roi->pens[lcv]);
        c_free(roi->pens[lcv]);
        for (cnt = lcv; cnt < roi->num_pens; cnt++)
           roi->pens[cnt] = roi->pens[cnt + 1];
        roi->pens[cnt] = NULL;
       }
 return roi->num_pens;
}


/*--------------------------------------------------------------------------
keep_only_non_empty_curves - ???
--------------------------------------------------------------------------*/

int
keep_only_non_empty_curves(slice_roi)
 struct slice_roi_rec *slice_roi;
{
 unsigned long lcv;
 int err;

 for (lcv = 0; lcv < slice_roi->num_rois; )
    if (clean_roi_rec(slice_roi->rois[lcv])) lcv++;
      else
       if ((err = delete_roi(slice_roi,slice_roi->rois[lcv])) != NO_ERROR)
          break;        
 return slice_roi->num_rois;
}



/*--------------------------------------------------------------------------
append_curves_to_curve_2d_rec - ???
--------------------------------------------------------------------------*/

int
append_curves_to_curve_2d_rec(pen,curves,num_curves,status)
 struct curve_2d_rec *pen;
 struct curve_rec **curves;
 unsigned long num_curves;
 unsigned short *status;
{
 unsigned long lcv;
 int err;

 PRT1(printf("append_curves_to_curve_2d_rec...num_curves = %d\n",num_curves);)
 if ((err = add_curve_rec_handle(pen,num_curves)) != NO_ERROR) return err;

 for (lcv = 0; lcv < num_curves; lcv++)
    {curves[lcv]->status &= ~DRAW_MODE;
     curves[lcv]->status |= pen->pen_type;
     pen->curves[(pen->num_curves)++] = curves[lcv];
    }
 *status |= SLICE_ROI_NON_EMPTY | SLICE_ROI_TO_BE_SAVED;
 PRT1(printf("append_curves_to_curve_2d_rec...end\n");)
 return NO_ERROR;
}


/*--------------------------------------------------------------------------
append_curve_to_roi_rec - ???
--------------------------------------------------------------------------*/

int
append_curve_to_roi_rec(roi,curve,pen_type,set_cur,status)
 struct roi_rec *roi;
 struct curve_rec *curve;
 unsigned short pen_type;
 int set_cur;
 unsigned short *status;
{
 struct curve_2d_rec *pen;
 int err;

 pen = get_pen(roi,pen_type,0,&err);
 if (pen == NULL) return err;

 if ((err = append_curves_to_curve_2d_rec(pen,&curve,(unsigned long) 1,status))
           != NO_ERROR) return err;
 if (set_cur) pen->cur_curve = pen->num_curves - 1;
 roi->prog_status |= ROI_MUST_UPDATE;
 return NO_ERROR;
}


/*--------------------------------------------------------------------------
append_curves_to_roi_rec - ???
--------------------------------------------------------------------------*/

int
append_curves_to_roi_rec(roi,curves,num_curves,status)
 struct roi_rec *roi;
 struct curve_rec **curves;
 unsigned long num_curves;
 unsigned short *status;
{
 struct curve_2d_rec *pen;
 int err;
 struct curve_rec **temp_curves;
 long indx,cnt,lcv,num;

 if (num_curves == 0) return NO_ERROR;
 if ((err = get_ptr(&temp_curves,NULL,num_curves,
      (size_t) sizeof(struct curve_rec *))) != NO_ERROR) 
    return err;
 for (cnt = num_curves - 1,num = indx = lcv = 0; lcv < num_curves; lcv++)
    {
     if (curves[lcv] != NULL)
        {if (is_curve_closed(curves[lcv]))
            {temp_curves[indx++] = curves[lcv];
            }
           else
            {num += 1;
             temp_curves[cnt--] = curves[lcv];
            }
        }
    }

 if (indx > 0)
    {if ((pen = get_pen(roi,DRAW_FREE_CLOSED,0,&err)) != NULL)
        err = append_curves_to_curve_2d_rec(pen,temp_curves,
             (unsigned long) indx,status);
    }
 if ((num > 0) && (err == NO_ERROR))
    {
     if ((pen = get_pen(roi,DRAW_FREE,0,&err)) != NULL)
        err = append_curves_to_curve_2d_rec(pen,
             &(temp_curves[num_curves - num]),
             (unsigned long) num,status);
    }
 c_free(temp_curves);
 if (err == NO_ERROR) roi->prog_status |= ROI_MUST_UPDATE;
 return err;
}


/*--------------------------------------------------------------------------
JoinCurvesToCurve2dRec - ???
--------------------------------------------------------------------------*/

int
JoinCurvesToCurve2dRec(pen,curves,num_curves,status,sx,sy,tolocal1,localto1,localto2)
 struct curve_2d_rec *pen;
 struct curve_rec **curves;
 unsigned long num_curves;
 unsigned short *status;
 float sx,sy;
 struct matrix_2d_rec *tolocal1,*localto1,*localto2;
{
 unsigned long lcv,cnt;
 int err;

 for (cnt = 0,lcv = 0; lcv < num_curves; lcv++)
    if (curves[lcv]->num_xys > 0) cnt++;
 if (cnt == 0) return CANCEL;
 PRT2(printf("JoinCurvesToCurve2dRec...num_curves = %d cnt = %d\n",num_curves,cnt))
 if ((err = add_curve_rec_handle(pen,cnt)) != NO_ERROR) return err;

 for (lcv = 0; lcv < num_curves; lcv++)
    if (curves[lcv]->num_xys > 0)
       {if ((err = setup_curve_rec_ptr(&(pen->curves[pen->num_curves]),
                     pen->pen_type)) != NO_ERROR)
           return err;
        err = JoinCurveRec(pen->curves[pen->num_curves],
                        curves[lcv],0,0,status,sx,sy,tolocal1,localto1,localto2);
        pen->num_curves += 1;
        if (err != NO_ERROR) return err;
       }
 return NO_ERROR;
}


/*--------------------------------------------------------------------------
JoinCurveToRoiRec - ???
--------------------------------------------------------------------------*/

int
JoinCurveToRoiRec(roi,curve,pen_type,set_cur,status,sx,sy,tolocal1,localto1,localto2)
 struct roi_rec *roi;
 struct curve_rec *curve;
 unsigned short pen_type;
 int set_cur;
 unsigned short *status;
 float sx,sy;
 struct matrix_2d_rec *tolocal1,*localto1,*localto2;
{
 struct curve_2d_rec *pen;
 int err;

 pen = get_pen(roi,pen_type,0,&err);
 if (pen == NULL) return err;
 if ((err = JoinCurvesToCurve2dRec(pen,&curve,(unsigned long) 1,
                status,sx,sy,tolocal1,localto1,localto2)) ==
     NO_ERROR)
    {roi->prog_status |= ROI_MUST_UPDATE;
     if (set_cur) pen->cur_curve = pen->num_curves - 1;}

 return err;
}


/*--------------------------------------------------------------------------
JoinCurvesToRoiRec - ???
--------------------------------------------------------------------------*/

int
JoinCurvesToRoiRec(roi,curves,num_curves,status,sx,sy,tolocal1,localto1,localto2)
 struct roi_rec *roi;
 struct curve_rec **curves;
 unsigned long num_curves;
 unsigned short *status;
 float sx,sy;
 struct matrix_2d_rec *tolocal1,*localto1,*localto2;
{
 struct curve_2d_rec *pen;
 struct curve_rec **temp_curves;
 long indx,cnt,lcv,num;
 int err;

 if (num_curves == 0) return NO_ERROR;
 if ((err = get_ptr(&temp_curves,NULL,num_curves,
      (size_t) sizeof(struct curve_rec *))) != NO_ERROR)
    return err;
 for (cnt = num_curves - 1,num = indx = lcv = 0; lcv < num_curves; lcv++)
    {
     if (curves[lcv] != NULL)
        {
         if (curves[lcv]->num_xys > 0)
            {if (is_curve_closed(curves[lcv]))
                {temp_curves[indx++] = curves[lcv];
                }
               else
                {num += 1;
                 temp_curves[cnt--] = curves[lcv];}
            }
        }
    }

 if (indx > 0)
    {if ((pen = get_pen(roi,DRAW_FREE_CLOSED,0,&err)) != NULL)
        err = JoinCurvesToCurve2dRec(pen,temp_curves,
             (unsigned long) indx,status,sx,sy,tolocal1,localto1,localto2);
    }
 if ((num > 0) && (err == NO_ERROR))
    {
     if ((pen = get_pen(roi,DRAW_FREE,0,&err)) != NULL)
        err = JoinCurvesToCurve2dRec(pen,
             &(temp_curves[num_curves - num]),
             (unsigned long) num,status,sx,sy,tolocal1,localto1,localto2);
    }
 c_free(temp_curves);
 if (err == NO_ERROR) roi->prog_status |= ROI_MUST_UPDATE;
 return err;
}


/*--------------------------------------------------------------------------
JoinCurveRec - ???
--------------------------------------------------------------------------*/

int
JoinCurveRec(dest,src,use_src_indx,src_indx,status,sx,sy,tolocal1,localto1,localto2)
 struct curve_rec *dest,*src;
 int use_src_indx;
 unsigned long src_indx;
 unsigned short *status;
 float sx,sy;
 struct matrix_2d_rec *tolocal1,*localto1,*localto2;
{
 unsigned long temp,lcv,amt,start;
 struct matrix_2d_rec matrix;
 int err;
 float x,y,x1,y1;
 struct matrix_2d_rec mat,*pmat;

 if (use_src_indx) start = src_indx;
   else start = 0;
 if (start >= src->num_xys) return NO_ERROR;
 lcv = start;
 curve_matrix(src,localto2,&mat,&pmat,1);
 if (!(src->status & DRAW_CIRCLE_OR_ELLIPSE_OR_RECT) &&
     (dest->num_xys > 0))
    {
     temp = dest->num_xys - 1;
     CurvePtByIndx(dest,temp,&x,&y,localto1);
     for (; lcv < src->num_xys; )
        {if (pmat != NULL)
            trans_2d_pt(pmat,src->xys[lcv].x,src->xys[lcv].y,&x1,&y1,0);
           else
            {x1 = src->xys[lcv].x;
             y1 = src->xys[lcv].y;}
         if ((x == x1) && (y == y1)) lcv++;
           else break;}
     if (lcv == src->num_xys) return NO_ERROR;
    }
 temp = src->num_xys - lcv;
 if ((temp + dest->num_xys) > dest->total_xys)
    {amt = temp + dest->num_xys - dest->total_xys;
     PRT2(printf("JoinCurveRec...amt = %d\n",amt))
     if ((err = get_ptr(&(dest->xys),&(dest->total_xys),amt,
             (size_t) sizeof(*(dest->xys)))) != NO_ERROR)
           return err;
    }
 PRT3(printf("JoinCurveRec...lcv = %d src->num_xys = %d\n",lcv,src->num_xys))
 PRT3(printf("src->status = %x dest->status = %x\n",
        (unsigned long) src->status,(unsigned long) dest->status);)
 if (src->status & DRAW_CIRCLE_OR_ELLIPSE_OR_RECT)
    {dest->status &= ~CURVE_MATRIX_IN_USE;
     for (; lcv < src->num_xys; lcv++)
        if ((err = EnterCurveXY(dest,src->xys[lcv].x,
             src->xys[lcv].y,status,tolocal1,0,0,0)) != NO_ERROR)
           return err;
     if (src->status & CURVE_MATRIX_IN_USE)
        {dest->status |= CURVE_MATRIX_IN_USE;
         dest->matrix = src->matrix;
         dest->inv = src->inv;}
    }
   else
    for (; lcv < src->num_xys; lcv++)
       {if (pmat != NULL)
           trans_2d_pt(pmat,src->xys[lcv].x,src->xys[lcv].y,&x1,&y1,0);
          else
           {x1 = src->xys[lcv].x;
            y1 = src->xys[lcv].y;}
        if ((err = EnterCurveXY(dest,x1,
             y1,status,tolocal1,1,0,0)) != NO_ERROR)
           return err;
       }
 if ((sx != 1.0) || (sy != 1.0))
    {setup_scale_2d_matrix_from_xy(&matrix,sx,sy,.0,.0);
     set_curve_matrix(dest,&matrix,1,status);
    }
 return NO_ERROR;

}


/*--------------------------------------------------------------------------
JoinCurve2dRecs - ???
--------------------------------------------------------------------------*/

int
JoinCurve2dRecs(dest,source,set_cur,status,sx,sy,tolocal1,localto1,localto2)
 struct curve_2d_rec *dest,*source;
 int set_cur;
 unsigned short *status;
 float sx,sy;
 struct matrix_2d_rec *tolocal1,*localto1,*localto2;
{
 int err;

 PRT2(printf("JoinCurve2dRecs...source->num_curves = %d\n",source->num_curves))
 if ((err = JoinCurvesToCurve2dRec(dest,source->curves,
       source->num_curves,status,sx,sy,tolocal1,localto1,localto2)) != NO_ERROR)
    return err;
 PRT2(printf("JoinCurve2dRecs...\n"))

 dest->pen_type = source->pen_type;
 if (set_cur)
    {dest->cur_curve = source->cur_curve;
     if (dest->cur_curve > dest->num_curves)
        dest->cur_curve = 0;
    }
 return NO_ERROR;
}


/*--------------------------------------------------------------------------
JoinRoiRecs - ???
--------------------------------------------------------------------------*/

int
JoinRoiRecs(roi1,roi2,set_cur,status,sx,sy,tolocal1,localto1,localto2)
 struct roi_rec *roi1,*roi2;
 int set_cur;
 unsigned short *status;
 float sx,sy;
 struct matrix_2d_rec *tolocal1,*localto1,*localto2;
{
 unsigned long lcv;
 int err;
 struct curve_2d_rec *pen;

 /* Changed NO_ERROR to return NO_ERROR below.  -PWH 19990405 */
 if (roi2->num_pens == 0) return NO_ERROR;
 PRT2(printf("JoinRoiRecs...roi2->num_pens = %d\n",roi2->num_pens))
 for (lcv = 0; lcv < roi2->num_pens; lcv++)
    {if ((pen = get_pen(roi1,roi2->pens[lcv]->pen_type,0,&err)) == NULL)
        return err;
     if ((err = JoinCurve2dRecs(pen,roi2->pens[lcv],set_cur,
                  status,sx,sy,tolocal1,localto1,localto2)) != NO_ERROR)
        return err;
    }
 PRT2(printf("JoinRoiRecs...end\n"))

 if (set_cur)
    {roi1->cur_pen = roi2->cur_pen;
     if (roi1->cur_pen >= roi1->num_pens)
        roi1->cur_pen = 0;
    }
 roi1->status = OWN_ROI | ROI_VISIBLE;
 roi1->prog_status |= ROI_MUST_UPDATE;
 return NO_ERROR;
}


/*--------------------------------------------------------------------------
JoinRois - ???
--------------------------------------------------------------------------*/

int
JoinRois(slice_roi1,slice_roi2,set_cur,sx,sy,tolocal1,localto1,localto2)
 struct slice_roi_rec *slice_roi1,*slice_roi2;
 int set_cur;
 float sx,sy;
 struct matrix_2d_rec *tolocal1,*localto1,*localto2;
{
 unsigned long lcv,cnt;
 struct roi_rec *roi;
 int err;

 if (slice_roi2->num_rois == 0) return NO_ERROR;
 if (slice_roi2->dimx > slice_roi1->dimx) slice_roi1->dimx = slice_roi2->dimx;
 if (slice_roi2->dimy > slice_roi1->dimy) slice_roi1->dimy = slice_roi2->dimy;
 PRT2(printf("JoinRois...= %d\n",slice_roi2->num_rois))

 for (cnt = 0,lcv = 0; lcv < slice_roi2->num_rois; lcv++)
    if (find_roi_rec(slice_roi1,slice_roi2->rois[lcv]->label,0) == NULL)
       cnt++;
 PRT2(printf("JoinRois...cnt = %d\n",cnt))
 if (cnt > 0)
    {if ((err = add_roi_rec_handle(slice_roi1,cnt)) != NO_ERROR) return err;
    }
 for (lcv = 0; lcv < slice_roi2->num_rois; lcv++)
    {if ((roi = find_roi_rec(slice_roi1,slice_roi2->rois[lcv]->label,0)) 
         == NULL)
        { 
         PRT2(printf("setup_roi_rec_ptr...\n"))
         if ((err = setup_roi_rec_ptr(&roi,slice_roi2->rois[lcv]->label))
                 != NO_ERROR)
            return err;
         slice_roi1->rois[slice_roi1->num_rois] = roi;
         slice_roi1->num_rois += 1;
        }
     if ((err = JoinRoiRecs(roi,slice_roi2->rois[lcv],set_cur,
             &(slice_roi1->status),sx,sy,tolocal1,localto1,localto2)) != NO_ERROR)
        return err;
    }
 PRT2(printf("JoinRois...end\n"))
 if (set_cur)
    {find_roi_rec(slice_roi1,
             slice_roi2->rois[slice_roi2->cur_roi]->label,1);
/*
     slice_roi1->cur_roi = slice_roi2->cur_roi;
     if (slice_roi1->cur_roi > slice_roi1->num_rois)
        slice_roi1->cur_roi = 0;
*/
    }

 return NO_ERROR;
}


/*--------------------------------------------------------------------------
CpSliceRoiRec - ???
--------------------------------------------------------------------------*/

int
CpSliceRoiRec(slice_roi1,slice_roi2,tolocal1,localto1,localto2)
 struct slice_roi_rec *slice_roi1,*slice_roi2;
 struct matrix_2d_rec *tolocal1,*localto1,*localto2;
{
 int err;

 free_slice_roi_rec(slice_roi1);
 slice_roi1->dimx = slice_roi2->dimx;
 slice_roi1->dimy = slice_roi2->dimy;
 if ((err = JoinRois(slice_roi1,slice_roi2,1,1.0,1.0,
              tolocal1,localto1,localto2)) != NO_ERROR)
    free_slice_roi_rec(slice_roi1);
 return err;
}


/*--------------------------------------------------------------------------
JoinCurveToSliceRoiRec - ???
--------------------------------------------------------------------------*/

int
JoinCurveToSliceRoiRec(slice_roi,roi_label,pen_type,curve,set_cur_roi,set_cur_pen,
                       set_cur_curve,sx,sy,tolocal1,localto1,localto2)
 struct slice_roi_rec *slice_roi;
 char *roi_label;
 unsigned short pen_type;
 struct curve_rec *curve;
 int set_cur_roi,set_cur_pen,set_cur_curve;
 float sx,sy;
 struct matrix_2d_rec *tolocal1,*localto1,*localto2;
{
 struct roi_rec *roi;
 int err;

 PRT1(printf("JoinCurveToSliceRoiRec...slice_roi = %u roi_label = %s pen_type = %x\n",
       slice_roi,roi_label,(unsigned long) pen_type);)
 PRT1(printf("JoinCurveToSliceRoiRec...num_rois = %d\n",slice_roi->num_rois);)
 if ((err = setup_more_rois(slice_roi,roi_label,set_cur_roi)) != NO_ERROR)
    return err;
 PRT1(printf("JoinCurveToSliceRoiRec...0\n");)

 if (((roi = get_roi(slice_roi,roi_label)) == NULL) ||
     (no_case_str_cmp(roi->label,roi_label) != 0))
    return CANCEL;
 PRT3(printf("JoinCurveToSliceRoiRec...1\n");)
 if ((err = JoinCurveToRoiRec(roi,curve,pen_type,set_cur_curve,
                &(slice_roi->status),sx,sy,tolocal1,localto1,localto2)) ==
           NO_ERROR)
    {
     set_roi_saved(slice_roi);
     if (set_cur_pen) find_pen(roi,pen_type,set_cur_pen);}
 PRT1(printf("JoinCurveToSliceRoiRec...err = %d\n",err);)
 return err;
}


/*--------------------------------------------------------------------------
JoinRoiToSliceRoiRec - ???
--------------------------------------------------------------------------*/

int
JoinRoiToSliceRoiRec(slice_roi,roi,set_cur_roi,sx,sy,tolocal1,localto1,localto2)
 struct slice_roi_rec *slice_roi;
 struct roi_rec *roi;
 int set_cur_roi;
 float sx,sy;
 struct matrix_2d_rec *tolocal1,*localto1,*localto2;
{
 struct roi_rec *roi1;
 int err;

 if ((err = setup_more_rois(slice_roi,roi->label,set_cur_roi)) != NO_ERROR)
    return err;
 if ((roi1 = get_roi(slice_roi,roi->label)) == NULL)
    return CANCEL;
 if ((err = JoinRoiRecs(roi1,roi,1,&(slice_roi->status),
       sx,sy,tolocal1,localto1,localto2)) == NO_ERROR)
    if (set_cur_roi) find_roi_rec(slice_roi,roi1->label,set_cur_roi);
 return err;
}


/*--------------------------------------------------------------------------
CanJointCurves - ???
--------------------------------------------------------------------------*/

int
CanJointCurves(curve1,curve2,status,tolocal1,localto1,localto2)
 struct curve_rec *curve1,*curve2;
 unsigned short *status;
 struct matrix_2d_rec *tolocal1,*localto1,*localto2;
{
 long num_pts,lcv,cnt,total,last;
 struct curve_rec curv;
 struct float_xy_rec *temp_pts;
 int err,joint;
 float x10,y10,x11,y11,x20,y20,x21,y21,x1,y1;

 PRT1(printf("CanJointCurves...curve1->num_xys = %d curve2->num_xys = %d\n",
          curve1->num_xys,curve2->num_xys))
 if (curve2->num_xys == 0) return NO_ERROR;
 joint = 0;
 err = NO_ERROR;
 FindTheCurveMinMax(curve1,localto1,curve1->status,0,0,0);
 if (curve1->num_xys == 0) joint = 5;
   else
    {CurvePtByIndx(curve1,0,&x10,&y10,localto1);
     last = curve1->num_xys - 1;
     CurvePtByIndx(curve1,last,&x11,&y11,localto1);

     CurvePtByIndx(curve2,0,&x20,&y20,localto2);
     last = curve2->num_xys - 1;
     CurvePtByIndx(curve2,last,&x21,&y21,localto2);
     if ((x10 == x21) && (y10 == y21))
        joint = 2;
       else if ((x11 == x20) && (y11 == y20))
        joint = 1;
       else if ((x20 == x10) && (y20 == y10))
        joint = 3;
       else if ((x21 == x11) && (y21 == y11))
        joint = 4;
     }
 TRACE(printf("***joint = %d curve1->num_xys = %d curve2->num_xys = %d\n",
             joint,curve1->num_xys,curve2->num_xys))
 if (joint)
    {if (joint == 5) num_pts = curve2->num_xys;
       else num_pts = curve1->num_xys + curve2->num_xys - 1;
     if ((joint == 2) || (joint == 3))
        {
         init_curve_rec(&curv);
         if ((err = get_ptr(&(curv.xys),&(curv.total_xys),(unsigned long) 
                num_pts, (size_t) sizeof(*(curv.xys)))) != NO_ERROR)
            return err;
        }
       else if (num_pts > curve1->total_xys)
        {num_pts = num_pts - curve1->total_xys;
         if ((err = get_ptr(&(curve1->xys),&(curve1->total_xys),(unsigned long) 
                 num_pts, (size_t) sizeof(*(curve1->xys)))) != NO_ERROR)
            return err;}

     if (joint == 2)
        {
         memcpy(curv.xys,curve2->xys,(size_t)
           (curve2->num_xys * (size_t) sizeof(*(curv.xys))));
         temp_pts = curv.xys;
         total = curv.total_xys;
         curv = *curve2;
         curv.xys = temp_pts;
         curv.total_xys = total;
         for (lcv = 1; lcv < curve1->num_xys; lcv++)
            {CurvePtByIndx(curve1,lcv,&x1,&y1,localto1);
             if ((err = EnterCurveXY(&curv,x1,
                            y1,status,tolocal1,1,0,0)) != NO_ERROR)
                {free_curve_rec(&curv);
                 return err;}
            }
         free_curve_rec(curve1);
         *curve1 = curv;
        }
       else if (joint == 5)
        {temp_pts = curve1->xys;
         total = curve1->total_xys;
         *curve1 = *curve2;
         curve1->xys = temp_pts;
         curve1->total_xys = total;
         memcpy(curve1->xys,curve2->xys,(size_t)
           (curve2->num_xys * (size_t) sizeof(*(curve1->xys))));
        }
       else if (joint == 1)
        {for (lcv = 1; lcv < curve2->num_xys; lcv++)
            {CurvePtByIndx(curve2,lcv,&x1,&y1,localto2);
             if ((err = EnterCurveXY(curve1,x1,
                   y1,status,tolocal1,1,0,0)) != NO_ERROR)
                return err;
            }
        }
       else if (joint == 3)
        {
         for (cnt = 0,lcv = curve1->num_xys - 1; lcv >= 0; lcv--)
            curv.xys[cnt++] = curve1->xys[lcv];
         temp_pts = curv.xys;
         total = curv.total_xys;
         curv = *curve1;
         curv.xys = temp_pts;
         curv.total_xys = total;
         for (lcv = 1; lcv < curve2->num_xys; lcv++)
            {CurvePtByIndx(curve2,lcv,&x1,&y1,localto2);
             if ((err = EnterCurveXY(&curv,x1,
                   y1,status,tolocal1,1,0,0)) != NO_ERROR)
                {free_curve_rec(&curv);
                 return err;}
            }
         free_curve_rec(curve1);
         *curve1 = curv;
        }
       else if (joint == 4)
        {
         for (lcv = curve2->num_xys - 1; lcv >= 0; lcv--)
            {CurvePtByIndx(curve2,lcv,&x1,&y1,localto2);
             if ((err = EnterCurveXY(curve1,x1,
                   y1,status,tolocal1,1,0,0)) != NO_ERROR)
                return err;
            }
        }
     TRACE(printf("curve1->num_xys = %d joint = %d\n",curve1->num_xys,joint))

     *status |= SLICE_ROI_NON_EMPTY | SLICE_ROI_TO_BE_SAVED;
     
     return NO_ERROR;}

 return CANCEL;
}


/*--------------------------------------------------------------------------
joint_curves - ???
--------------------------------------------------------------------------*/

int
joint_curves(pen,status)
 struct curve_2d_rec *pen;
 unsigned short *status;
{
 unsigned long lcv,cnt,cur_curve;
 int again,err;

 cur_curve = pen->cur_curve;
 err = NO_ERROR;
 do
    {TRACE(printf("joint_curves pen->num_curves = %d\n",pen->num_curves))

     for (again = lcv = 0; (lcv < pen->num_curves) && !again;lcv++)
         for (cnt = lcv + 1; cnt < pen->num_curves; cnt++)
            if (pen->curves[lcv] != pen->curves[cnt])
              if ((err = CanJointCurves(pen->curves[lcv],pen->curves[cnt],
                     status,NULL,NULL,NULL)) == NO_ERROR)
                 {
                  pen->cur_curve = cnt;
                  delete_cur_curve(pen,status,1);
                  again = 1;
                  break;}
    }
 while (again);
 if (cur_curve > pen->num_curves) pen->cur_curve = 0;
   else pen->cur_curve = cur_curve;
 TRACE(printf("end joint_curves pen->num_curves = %d\n",pen->num_curves))
 return err;
}


/*--------------------------------------------------------------------------
joint_curves_of_type - ???
--------------------------------------------------------------------------*/

int
joint_curves_of_type(slice_roi,roi_label,pen_type,set_cur)
 struct slice_roi_rec *slice_roi;
 char *roi_label;
 unsigned short pen_type;
 int set_cur;
{
 struct roi_rec *cur_roi,*roi;
 int err;
 struct curve_2d_rec *pen;

 err = NO_ERROR;
 TRACE(printf("joint_curves_of_type roi_label = %s\n",roi_label))
 if (!set_cur) cur_roi = get_cur_roi(slice_roi);
 if ((roi = next_cur_roi(slice_roi,roi_label)) != NULL)
    {if ((err = set_cur_pen(slice_roi,pen_type,0)) == NO_ERROR)
        if ((pen = get_cur_pen(slice_roi)) != NULL)
           {if ((err = joint_curves(pen,&(slice_roi->status))) == NO_ERROR)
               roi->status |= HISTO_EDITED_ROI;}
          else err = CANCEL;
     if ((!set_cur) && (cur_roi != NULL) && (roi != cur_roi))
        next_cur_roi(slice_roi,cur_roi->label);
    }
 TRACE(printf("joint_curves_of_type end"))
 return err;
}


/*--------------------------------------------------------------------------
change_cur_curve_pen_type - ???
--------------------------------------------------------------------------*/

int
change_cur_curve_pen_type(slice_roi,pen_type)
 struct slice_roi_rec *slice_roi;
 unsigned short pen_type;
{
 struct curve_rec *curve;
 struct curve_2d_rec *pen,*pen1;
 struct roi_rec *roi;
 int delete_ok;
 int err;

 if ((roi = get_cur_roi(slice_roi)) == NULL)
    {if (((err = setup_more_rois(slice_roi,SCRATCH_STR,1)) != NO_ERROR) ||
         ((roi = get_cur_roi(slice_roi)) == NULL))
        {
         return err;}
    }
 if ((pen = get_cur_pen(slice_roi)) == NULL) 
    {pen = get_pen(roi,pen_type,1,&err);
     return err;}
 if (pen->pen_type == pen_type) return NO_ERROR;
 curve = get_cur_curve(slice_roi);
 if ((pen1 = find_pen(roi,pen_type,1)) == NULL)
    {if ((pen1 = get_pen(roi,pen_type,1,&err)) == NULL)
        return err;
    }

 delete_ok = 0;
 if (curve != NULL)
    {if ((err = delete_cur_curve(pen,&(slice_roi->status),0)) == NO_ERROR)
        delete_ok = 1;
    }
 if (delete_ok) 
    {err = append_curves_to_curve_2d_rec(pen1,&curve,1,
            &(slice_roi->status));
     
     if (err != NO_ERROR)
        {c_free(curve);
         return err;}
     pen1->cur_curve = pen1->num_curves - 1;
    }
 return NO_ERROR;
}

