/****************************************************************************
text_output.c - ???

Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.
****************************************************************************/

static const char rcsid[] = "$Id: text_output_rec.c,v 1.7 1999/07/07 17:41:20 hughett Exp $ Release $Name: nria_1_0_6 $";

#include "mri_format.h"
#include "text_format.h"
#include "debug.h"
#include "nria_procedures.h"


#define TEXT_FORMAT_COPY 1
#define TEXT_FORMAT_APPEND 2
#define TEXT_FORMAT_COPY_AND_CP_INDX 5
#define TEXT_FORMAT_APPEND_AND_CP_INDX 6



/* Functions defined in this file */
void init_collection_rec(struct collection_rec *collection);
void init_page_rec(struct page_rec *page);
void init_text_output_analysis_group_rec(
   struct text_output_analysis_group_rec *group);
void init_all_collection_rec(struct all_collection_rec *analysis);
void init_output_field_rec(struct output_field_rec *group);
void init_output_count_field_value_rec(
   struct output_count_field_value_rec *field);
void init_group_count_field_value_rec(struct output_count_field_rec *groups,
    unsigned int num_groups);
void init_output_count_field_rec(struct output_count_field_rec *group);
void init_output_count_group_rec(struct output_count_group_rec *count);
void init_output_text_format_rec(struct output_text_format_rec *format);
void init_text_output_rec(struct text_output_rec *text,
    struct input_file_rec *file);
void free_collection_rec(struct collection_rec *collection);
void free_text_output_analysis_group_rec(
   struct text_output_analysis_group_rec *group);
void free_analysis_group_recs(struct text_output_analysis_group_rec *groups,
    unsigned int num_groups);
void free_page_rec(struct page_rec *page);
void free_all_collection_rec(struct all_collection_rec *analysis);
void free_output_field_rec(struct output_field_rec *group);
void free_output_count_field_rec(struct output_count_field_rec *group);
void free_output_count_group_rec(struct output_count_group_rec *count);
void flush_counts(struct text_output_rec *text);
void free_output_text_format_rec(struct output_text_format_rec *format);
void free_text_output_rec(struct text_output_rec *text);
void delete_the_text_output(struct text_output_rec *text);
int copy_text_output_analysis_group_rec(
   struct text_output_analysis_group_rec *dest,
    struct text_output_analysis_group_rec *source, int is_cp_indx);
int cp_text_output_analysis_group_recs(
   struct text_output_analysis_group_rec **dest, 
   unsigned int *dest_num_groups,
    struct text_output_analysis_group_rec *source,
    unsigned int source_num_groups, int option);
int copy_text_format_only(struct text_output_rec *dest_text,
    struct text_output_rec *source_text, int option);
int RdTextFormat(char *fn, struct text_output_analysis_group_rec **groups,
    unsigned int *num_groups, struct output_text_format_rec *format,
    int is_count_format);
int WrTextFormat(char *fn, struct output_text_format_rec *format,
    struct text_output_analysis_group_rec *groups, unsigned long num_groups);
int get_out_format_labels(struct strs_rec *labels,
    struct text_output_analysis_group_rec *groups, unsigned int num_groups,
    int option);
int get_out_format_roi_nums(struct output_text_format_rec *format);
int sort_type_index_rec_order(struct pixel_type_order_rec *type1,
    struct pixel_type_order_rec *type2);
int get_out_format_pixel_type_nums(struct output_text_format_rec *format);
int get_memory_for_indx_nums(unsigned int **indx_nums, unsigned int *total,
    unsigned int num);
int assign_text_group_num(struct strs_rec *labels, unsigned int *nums,
    struct text_output_analysis_group_rec *groups, unsigned int num_groups,
    int option);
void get_output_format_default_params(struct output_text_format_rec *format);
int setup_output_count_group_rec(struct output_text_format_rec *format,
    unsigned int total_slices);
int setup_output_field_rec(struct output_text_format_rec *format,
    struct text_output_analysis_group_rec *groups, unsigned int num_groups);
int setup_output_text_format_rec(struct output_text_format_rec *format,
    struct hdr_rec *hdr, struct text_output_analysis_group_rec *groups,
    unsigned int num_groups);
int same_output_text_format_rec(struct output_text_format_rec *new,
    struct output_text_format_rec *old);
void pr_output_text_format_rec(struct output_text_format_rec *format);
void pr_text_output_analysis_group_recs(
   struct text_output_analysis_group_rec *groups, unsigned int num_groups);
int setup_output_format(struct text_output_rec *text);
int setup_text_output_params(struct text_output_rec *text);



/*--------------------------------------------------------------------------
init_collection_rec - ???
--------------------------------------------------------------------------*/

void
init_collection_rec(collection)
 struct collection_rec *collection;
{
 collection->total_buf_size = collection->buf_size = 0;
 collection->buf = NULL;
}


/*--------------------------------------------------------------------------
init_page_rec - ???
--------------------------------------------------------------------------*/

void
init_page_rec(page)
 struct page_rec *page;
{
 page->num_collects = 0;
 page->collects = NULL;
}


/*--------------------------------------------------------------------------
init_text_output_analysis_group_rec - ???
--------------------------------------------------------------------------*/

void
init_text_output_analysis_group_rec(group)
 struct text_output_analysis_group_rec  *group;
{
 group->num_rois = 0;
 group->roi_labels = NULL;
 group->total_rois = 0;
 group->roi_nums = NULL;
 group->num_pixel_types = 0;
 group->pixel_type_labels = NULL;
 group->total_pixel_types = 0;
 group->pixel_type_nums = NULL;
}


/*--------------------------------------------------------------------------
init_all_collection_rec - ???
--------------------------------------------------------------------------*/

void
init_all_collection_rec(analysis)
 struct all_collection_rec *analysis;
{
 analysis->cur_collect = analysis->num_collects = 0;
 analysis->collects = NULL;
 analysis->text_temp_fn = NULL;
 analysis->num_pages = 0;
 analysis->pages = NULL;
}


/*--------------------------------------------------------------------------
init_output_field_rec - ???
--------------------------------------------------------------------------*/

void
init_output_field_rec(group)
 struct output_field_rec *group;
{
 group->pixel_type_indx = 0;
 group->num_roi_indxs = 0;
}


/*--------------------------------------------------------------------------
init_output_count_field_value_rec - ???
--------------------------------------------------------------------------*/

void
init_output_count_field_value_rec(field)
 struct output_count_field_value_rec *field;
{
 field->valid = COUNT_ROI_NOT_COMPUTED;
}


/*--------------------------------------------------------------------------
init_group_count_field_value_rec - ???
--------------------------------------------------------------------------*/

void
init_group_count_field_value_rec(groups,num_groups)
 struct output_count_field_rec *groups;
 unsigned int num_groups;
{
 unsigned int cnt,indx;

 for (cnt = 0; cnt < num_groups; cnt++)
    {
     for (indx = 0; indx < groups[cnt].num_fields; indx++)
        init_output_count_field_value_rec(&(groups[cnt].fields[indx]));}
}


/*--------------------------------------------------------------------------
init_output_count_field_rec - ???
--------------------------------------------------------------------------*/

void
init_output_count_field_rec(group)
 struct output_count_field_rec *group;
{
 group->num_fields = 0;
}


/*--------------------------------------------------------------------------
init_output_count_group_rec - ???
--------------------------------------------------------------------------*/

void
init_output_count_group_rec(count)
 struct output_count_group_rec *count;
{
 count->num_groups = 0;
}


/*--------------------------------------------------------------------------
init_output_text_format_rec - ???
--------------------------------------------------------------------------*/

void
init_output_text_format_rec(format)
 struct output_text_format_rec *format;
{
 format->status = OUT_FORMAT_COUNT_NEG;
 init_strs_rec(&(format->roi_labels));
 format->roi_nums = NULL;
 initial_unit_2d_rec(&(format->img_unit));
 init_strs_rec(&(format->pixel_type_labels));
 format->pixel_type_nums = NULL;
 format->num_groups = 0;
 format->groups = NULL;
 format->num_slices = 0;
 format->counts = NULL;
 format->lt_percent = format->gt_percent = .0;
 format->roi_lt_percent = format->roi_gt_percent = .0;
 get_output_format_default_params(format);
}


/*--------------------------------------------------------------------------
init_text_output_rec - ???
--------------------------------------------------------------------------*/

void
init_text_output_rec(text,file)
 struct text_output_rec *text;
 struct input_file_rec *file;
{
 text->status = 0;
 text->file = file;
 text->text_printing_time = NULL;
 text->num_groups = 0;
 text->groups = NULL;
 init_output_text_format_rec(&(text->format));

 init_all_collection_rec(&(text->analysis));
}


/*--------------------------------------------------------------------------
free_collection_rec - ???
--------------------------------------------------------------------------*/

void
free_collection_rec(collection)
 struct collection_rec *collection;
{
 if (collection->total_buf_size > 0)
    c_free(collection->buf);
 init_collection_rec(collection);
}



/*--------------------------------------------------------------------------
free_text_output_analysis_group_rec - ???
--------------------------------------------------------------------------*/

void
free_text_output_analysis_group_rec(group)
     struct text_output_analysis_group_rec *group;
{
  TRACE(printf("free_text_output_analysis_group_rec()\n");
	printf("	group->num_rois = %d\n", group->num_rois););
  if (group->num_rois > 0)
  {
    free_handle_memory(group->roi_labels,group->num_rois);
    c_free(group->roi_nums);
  }

  TRACE(printf("	group->num_pixel_types = %d\n",
	       group->num_pixel_types););
  if (group->num_pixel_types)
  {
    free_handle_memory(group->pixel_type_labels,group->num_pixel_types);
    c_free(group->pixel_type_nums);
  }
  init_text_output_analysis_group_rec(group);
}



/*--------------------------------------------------------------------------
free_analysis_group_recs - ???
--------------------------------------------------------------------------*/

void
free_analysis_group_recs(groups,num_groups)
 struct text_output_analysis_group_rec *groups;
 unsigned int num_groups;
{
 unsigned int lcv;

 if (num_groups > 0)
    {for (lcv = 0; lcv < num_groups; lcv++)
        free_text_output_analysis_group_rec(&(groups[lcv]));
    }
}


/*--------------------------------------------------------------------------
free_page_rec - ???
--------------------------------------------------------------------------*/

void
free_page_rec(page)
 struct page_rec *page;
{
 if (page->num_collects > 0)
    c_free(page->collects);
 init_page_rec(page);
}


/*--------------------------------------------------------------------------
free_all_collection_rec - ???
--------------------------------------------------------------------------*/

void
free_all_collection_rec(analysis)
 struct all_collection_rec *analysis;
{
 unsigned int lcv;

 if (analysis->num_collects > 0)
    {for (lcv = 0; lcv < analysis->num_collects; lcv++)
        free_collection_rec(&(analysis->collects[lcv]));
     c_free(analysis->collects);
    }
 if (analysis->num_pages > 0)
    {for (lcv = 0; lcv < analysis->num_pages; lcv++)
        free_page_rec(&(analysis->pages[lcv]));
     c_free(analysis->pages);
    }
 c_free(analysis->text_temp_fn);
 init_all_collection_rec(analysis);
}


/*--------------------------------------------------------------------------
free_output_field_rec - ???
--------------------------------------------------------------------------*/

void
free_output_field_rec(group)
 struct output_field_rec *group;
{
 if (group->num_roi_indxs > 0)
    c_free(group->roi_indxs);
}


/*--------------------------------------------------------------------------
free_output_count_field_rec - ???
--------------------------------------------------------------------------*/

void
free_output_count_field_rec(group)
 struct output_count_field_rec *group;
{
 if (group->num_fields > 0)
    c_free(group->fields);
}


/*--------------------------------------------------------------------------
free_output_count_group_rec - ???
--------------------------------------------------------------------------*/

void
free_output_count_group_rec(count)
 struct output_count_group_rec *count;
{
 unsigned int lcv;

 for (lcv = 0; lcv < count->num_groups; lcv++)
    free_output_count_field_rec(&(count->groups[lcv]));
}


/*--------------------------------------------------------------------------
flush_counts - ???
--------------------------------------------------------------------------*/

void
flush_counts(text)
 struct text_output_rec *text;
{
 text->format.status &= ~OUT_FORMAT_COUNT_DEFINED;
}


/*--------------------------------------------------------------------------
free_output_text_format_rec - ???
--------------------------------------------------------------------------*/

void
free_output_text_format_rec(format)
 struct output_text_format_rec *format;
{
 unsigned int lcv;

 free_strs_rec(&(format->roi_labels));
 c_free(format->roi_nums);

 free_strs_rec(&(format->pixel_type_labels));
 c_free(format->pixel_type_nums);

 for (lcv = 0; lcv < format->num_groups; lcv++)
    free_output_field_rec(&(format->groups[lcv]));
 c_free(format->groups);

 for (lcv = 0; lcv < format->num_slices; lcv++)
    free_output_count_group_rec(&(format->counts[lcv]));
 c_free(format->counts);
 init_output_text_format_rec(format);
}


/*--------------------------------------------------------------------------
free_text_output_rec - ???
--------------------------------------------------------------------------*/

void
free_text_output_rec(text)
 struct text_output_rec *text;
{
 if (text->num_groups > 0)
    {free_analysis_group_recs(text->groups,text->num_groups);
     c_free(text->groups);}
 free_output_text_format_rec(&(text->format));
 free_all_collection_rec(&(text->analysis));
 init_text_output_rec(text,text->file);
}


/*--------------------------------------------------------------------------
delete_the_text_output - ???
--------------------------------------------------------------------------*/

void
delete_the_text_output(text)
 struct text_output_rec *text;
{
 free_all_collection_rec(&(text->analysis));
 text->status &= TEXT_OUTPUT_DELETED;
}


/*--------------------------------------------------------------------------
copy_text_output_analysis_group_rec - ???
--------------------------------------------------------------------------*/

int
copy_text_output_analysis_group_rec(dest,source,is_cp_indx)
 struct text_output_analysis_group_rec *dest,*source;
 int is_cp_indx;
{
 unsigned int lcv;
 int err;

 free_text_output_analysis_group_rec(dest);
 if (source->num_rois > 0)
    {if ((err = copy_strs(&(dest->roi_labels),&(dest->num_rois),
                          source->roi_labels,source->num_rois)) != NO_ERROR)
        return err;
     if (is_cp_indx)
        if (source->roi_nums != NULL)
           {if ((err = get_memory_for_indx_nums(&(dest->roi_nums),
                 &(dest->total_rois),source->num_rois)) != NO_ERROR)
               return err;
            for (lcv = 0; lcv < source->num_rois; lcv++)
               dest->roi_nums[lcv] = source->roi_nums[lcv];
           }
    }
 if (source->num_pixel_types > 0)
    {if ((err = copy_strs(&(dest->pixel_type_labels),&(dest->num_pixel_types),
               source->pixel_type_labels,source->num_pixel_types)) != NO_ERROR)
        return err;
     if (is_cp_indx)
        if (source->pixel_type_nums != NULL)
           {if ((err = get_memory_for_indx_nums(&(dest->pixel_type_nums),
                  &(dest->total_pixel_types),
                  source->num_pixel_types)) != NO_ERROR)
               return err;
            for (lcv = 0; lcv < source->num_pixel_types; lcv++)
               dest->pixel_type_nums[lcv] = source->pixel_type_nums[lcv];
           }
    }
 return NO_ERROR;
}




/*--------------------------------------------------------------------------
cp_text_output_analysis_group_recs - ???

 before calling this routine,dest_num_groups has to be initialized to some 
     value
--------------------------------------------------------------------------*/

int
cp_text_output_analysis_group_recs(dest,dest_num_groups,
                                   source,source_num_groups,option)
     struct text_output_analysis_group_rec **dest,*source;
     unsigned int *dest_num_groups,source_num_groups;
     int option;
{
  unsigned int lcv,num_groups,num;
  struct text_output_analysis_group_rec *groups;
  unsigned long total;
  int err,is_cp_indx;

  TRACE(printf("cp_text_output_analysis_group_recs...1\n"));
  groups = NULL;
  if (option & TEXT_FORMAT_COPY)
  {
    num_groups = 0;
    num = source_num_groups;
  }
  else
  {
    num_groups = *dest_num_groups;
    num = source_num_groups + *dest_num_groups;
  }
  if (option & 4)
    is_cp_indx = 1;
  else
    is_cp_indx = 0;

  TRACE(printf("cp_text_output_analysis_group_recs...2\n"));
  total = 0;
  if ((err = get_ptr(&groups,&total,num,(size_t) sizeof(*groups))) != NO_ERROR)
    return err;
  for (lcv = 0; lcv < total; lcv++)
    init_text_output_analysis_group_rec(&(groups[lcv]));
  if (!(option & TEXT_FORMAT_COPY))
    for (lcv = 0; lcv < num_groups; lcv++)
      if ((err = copy_text_output_analysis_group_rec(
						     &(groups[lcv]),
						     &((*dest)[lcv]),
						     is_cp_indx))
	  != NO_ERROR)
      {
	free_analysis_group_recs(groups,lcv);
	c_free(groups);
	return err;
      }

  TRACE(printf("cp_text_output_analysis_group_recs...3\n"));
  for (lcv = 0; lcv < source_num_groups; lcv++,num_groups++)
  {
    if ((err = copy_text_output_analysis_group_rec(
						   &(groups[num_groups]),
						   &(source[lcv]),is_cp_indx))
	!= NO_ERROR)
    {
      free_analysis_group_recs(groups,num_groups);
      c_free(groups);
      return err;
    }
  }
  TRACE(printf("cp_text_output_analysis_group_recs...4\n"));
  if (*dest_num_groups > 0)
  {
    free_analysis_group_recs(*dest,*dest_num_groups);
    c_free(*dest);
  }
  *dest = groups;
  *dest_num_groups = total;
  TRACE(printf("cp_text_output_analysis_group_recs...5\n"));
  return NO_ERROR;
}



/*--------------------------------------------------------------------------
copy_text_format_only - ???
--------------------------------------------------------------------------*/

int
copy_text_format_only(dest_text,source_text,option)
 struct text_output_rec *dest_text,*source_text;
 int option;
{
 int err;

 if ((err = cp_text_output_analysis_group_recs(
            &(dest_text->groups),&(dest_text->num_groups),
            source_text->groups,source_text->num_groups,option)) != NO_ERROR)
    return err;

 dest_text->format.page_num_rows = source_text->format.page_num_rows;
 dest_text->format.page_row_size = source_text->format.page_row_size;
 dest_text->format.num_field_spaces = source_text->format.num_field_spaces;
 dest_text->format.num_collect_spaces = source_text->format.num_collect_spaces;

 return NO_ERROR;
}


/*--------------------------------------------------------------------------
 RdTextFormat- ???
--------------------------------------------------------------------------*/

int
RdTextFormat(fn,groups,num_groups,format,is_count_format)
 char *fn;
 struct text_output_analysis_group_rec **groups;
 unsigned int *num_groups;
 struct output_text_format_rec *format;
 int is_count_format;
{
 FILE *fid;
 int num,ary[5];
 unsigned int total,lcv;
 int err;

 PRT1(printf("RdTextFormat..*num_groups = %d  *groups = %u fn = %s\n",
         *num_groups,*groups,fn))
 if (fn == NULL) return FILE_NAME_ERROR;
 if ((fid = fopen(fn,"r")) == NULL)
    {TRACE(printf("\nCould not open the text format file : %s\n",fn))
     return FILE_ACCESS_ERROR;}
 free_analysis_group_recs(*groups,*num_groups);
 c_free(*groups);
 *num_groups = 0;
 *groups = NULL;
 total = 5;
 err = RdText(fid,ary,&total,RD_INT,0);
 PRT1(printf("RdTextFormat..1.err = %d\n",err))
 if (total != 5) err = CANCEL;
 if (err != NO_ERROR)
    {fclose(fid);
     return err;}
 num = ary[4];
 if (num == 0) 
    {fclose(fid);
     return NO_ERROR;}
 if (is_count_format) num = 1;
 if ((err = get_ptr(groups,num_groups,(unsigned int) num,
        (size_t) sizeof(**groups))) != NO_ERROR)
    {fclose(fid);
     return err;}
 for (lcv = 0; lcv < num; lcv++)
    init_text_output_analysis_group_rec(&((*groups)[lcv]));
 PRT1(printf("RdTextFormat..2.err = %d\n",err))

 for (lcv = 0; lcv < num; lcv++)
    {
     if ((err = rd_text_fields(fid,&((*groups)[lcv].roi_labels),
               &((*groups)[lcv].num_rois))) != NO_ERROR)
        {fclose(fid);
         return err;}
     PRT1(printf("RdTextFormat.lcv = %d\n",lcv))
     if (!is_count_format)
        if ((err = rd_text_fields(fid,&((*groups)[lcv].pixel_type_labels),
                         &((*groups)[lcv].num_pixel_types))) != NO_ERROR)
           {fclose(fid);
            return err;}
    }
 PRT1(printf("RdTextFormat..3.err = %d\n",err))
 if ((err == NO_ERROR) && (ferror(fid) == 0))
    {format->page_num_rows = ary[0];
     format->page_row_size = ary[1];
     format->num_field_spaces = ary[2];
     format->num_collect_spaces = ary[3];
     err = NO_ERROR;}
   else err = CANCEL;
 fclose(fid);
 PRT1(pr_text_output_analysis_group_recs(*groups,*num_groups))
 PRT1(printf("RdTextFormat...err = %d\n",err))

 return err;
}


/*--------------------------------------------------------------------------
WrTextFormat - ???
--------------------------------------------------------------------------*/

int
WrTextFormat(fn,format,groups,num_groups)
 char *fn;
 struct output_text_format_rec *format;
 unsigned long num_groups;
 struct text_output_analysis_group_rec *groups;
{
 FILE *fid;
 unsigned int lcv,cnt,blank;
 char str[200];
 int err;

 if (fn == NULL) return FILE_NAME_ERROR;
 if ((fid = fopen(fn,"w")) == NULL)
    {TRACE(printf("\nCould not open the text format file : %s\n",fn))
     return FILE_ACCESS_ERROR;}

 sprintf(str,"%ld # # of page rows",format->page_num_rows);
 fprintf(fid,"%s\n",str);
 sprintf(str,"%ld # # of columns in a row",format->page_row_size);
 fprintf(fid,"%s\n",str);
 sprintf(str,"%ld # # of spaces in between fields in a collection",
              format->num_field_spaces);
 fprintf(fid,"%s\n",str);
 sprintf(str,"%ld # # of spaces in between collections\n",
              format->num_collect_spaces);
 fprintf(fid,"%s\n",str);

 sprintf(str,"%ld # # of groups of collections\n",num_groups);
 fprintf(fid,"%s\n",str);

 for (lcv = 0; lcv < num_groups; lcv++)
    {
     sprintf(str,"%ld # # of rois",groups[lcv].num_rois);
     fprintf(fid,"%s\n",str);
     for (cnt = 0; cnt < groups[lcv].num_rois; cnt++)
        {
         if (blank_in_str(groups[lcv].roi_labels[cnt],&blank) == NO_ERROR)
            sprintf(str,"\"%s\" # roi label #%d",
                   groups[lcv].roi_labels[cnt],cnt);
           else
            sprintf(str,"%s # roi label #%d",
                   groups[lcv].roi_labels[cnt],cnt);
         fprintf(fid,"%s\n",str);
        }

     if (groups[lcv].num_pixel_types > 0)
        {sprintf(str,"\n%ld # # of pixel types",groups[lcv].num_pixel_types);
         fprintf(fid,"%s\n",str);
         for (cnt = 0; cnt < groups[lcv].num_pixel_types; cnt++)
            {
             if (blank_in_str(groups[lcv].pixel_type_labels[cnt],&blank)
                   == NO_ERROR) 
                sprintf(str,"\"%s\" # pixel_type label #%d",
                   groups[lcv].pixel_type_labels[cnt],cnt);
               else
                sprintf(str,"%s # pixel_type label #%d",
                   groups[lcv].pixel_type_labels[cnt],cnt);
             fprintf(fid,"%s\n",str);
            }
         }
     fprintf(fid,"\n");

    }
 if (ferror(fid)) err = WRITE_ERROR;
   else err = NO_ERROR;
 fclose(fid);

 return err;
}


/*--------------------------------------------------------------------------
get_out_format_labels - ???
--------------------------------------------------------------------------*/

#define GET_TEXT_ROI 1
#define GET_TEXT_PIXEL_TYPE 2

int
get_out_format_labels(labels,groups,num_groups,option)
 struct strs_rec *labels;
 struct text_output_analysis_group_rec *groups;
 unsigned int num_groups;
 int option;
{
 unsigned int lcv,num,cnt;
 int err;
 char *str;

 TRACE(printf("get_out_format_labels...num_groups = %d option = %d\n",num_groups,option))
 if (num_groups == 0) return CANCEL;
 free_strs_rec(labels);
 for (lcv = 0; lcv < num_groups; lcv++)
    {if (option == GET_TEXT_ROI) num = groups[lcv].num_rois;
       else num = groups[lcv].num_pixel_types;
     for (cnt = 0; cnt < num; cnt++)
       {
        if (option == GET_TEXT_ROI) str = groups[lcv].roi_labels[cnt];
          else str = groups[lcv].pixel_type_labels[cnt];
        FindStrInHandleAryByContent(labels->strs,
                              labels->num_strs,str,&err);
        if (err == NOT_FOUND)
           {if ((err = add_str_to_strs_rec(labels,str)) != NO_ERROR)
               return err;
           }
       }
    }
 TRACE(pr_strs_rec(labels))
 TRACE(printf("get_out_format_labels...END\n\n"))
 return NO_ERROR;
}


/*--------------------------------------------------------------------------
get_out_format_roi_nums - ???
--------------------------------------------------------------------------*/

int
get_out_format_roi_nums(format)
 struct output_text_format_rec *format;
{
 unsigned int lcv,total,exist;
 int err;

 TRACE(printf("get_out_format_roi_nums...begin num_strs = %d\n",
              format->roi_labels.num_strs))
 if (format->roi_labels.num_strs == 0) return CANCEL;

 if (format->roi_nums == NULL)
    {TRACE(printf("init roi_nums...\n"))
     total = 0;
     get_ptr_memory(&(format->roi_nums),&total,format->roi_labels.num_strs,
        (size_t) sizeof(*(format->roi_nums)),&err);
     if (err) return err;
     TRACE(printf("get_out_format_roi_nums...readytotal = %d\n",total))
    }
 for (exist = 1,lcv = 0; lcv < format->roi_labels.num_strs; lcv++)
    {TRACE(printf("lcvc = %d\n",lcv))
     format->roi_nums[lcv] = lcv + 1;
    }
 TRACE(printf("get_out_format_roi_nums...end\n"))
 return NO_ERROR;
}


/*--------------------------------------------------------------------------
sort_type_index_rec_order - ???
--------------------------------------------------------------------------*/

int
sort_type_index_rec_order(type1,type2)
 struct pixel_type_order_rec *type1,*type2;
{
 if (type1->order > type2->order) return 1;
 if (type1->order < type2->order) return -1;
 return 0;
}


/*--------------------------------------------------------------------------
get_out_format_pixel_type_nums - ???
--------------------------------------------------------------------------*/

int
get_out_format_pixel_type_nums(format)
 struct output_text_format_rec *format;
{
 unsigned int lcv,total;
 int err;

 TRACE(printf("get_out_format_pixel_type_nums...begin num_strs = %d\n",
           format->pixel_type_labels.num_strs))
 if (format->pixel_type_labels.num_strs == 0) return CANCEL;
/*
 total = 0;
 get_ptr_memory(&types,&total,
         format->pixel_type_labels.num_strs,
        (size_t) sizeof(*types),&err);
 if (err) return err;
 num_types = 0;
*/
 if (format->pixel_type_nums == NULL)
    {TRACE(printf("init pixel_type_nums...format->pixel_type_labels.num_strs = %d\n",format->pixel_type_labels.num_strs))

     total = 0;
     get_ptr_memory(&(format->pixel_type_nums),&total,
         format->pixel_type_labels.num_strs,
        (size_t) sizeof(*(format->pixel_type_nums)),&err);
     if (err) return err;
    }

 for (lcv = 0; lcv < format->pixel_type_labels.num_strs; lcv++)
    {
     format->pixel_type_nums[lcv] = lcv + 1;
    }
 TRACE(printf("get_out_format_pixel_type_nums...end\n"))
 return err;
}


/*--------------------------------------------------------------------------
get_memory_for_indx_nums - ???
--------------------------------------------------------------------------*/

int
get_memory_for_indx_nums(indx_nums,total,num)
 unsigned int **indx_nums;
 unsigned int *total,num;
{
 if ((*indx_nums == NULL) || (*total < num))
    {
     c_free(*indx_nums);
     *indx_nums = NULL;
     *total = 0;
     return get_ptr(indx_nums,total,num,(size_t) sizeof(unsigned int));
    }
 return NO_ERROR;
}


/*--------------------------------------------------------------------------
assign_text_group_num - ???
--------------------------------------------------------------------------*/

#define GET_TEXT_ROI 1
#define GET_TEXT_PIXEL_TYPE 2

int
assign_text_group_num(labels,nums,groups,num_groups,option)
 struct strs_rec *labels;
 unsigned int *nums;
 struct text_output_analysis_group_rec *groups;
 unsigned int num_groups;
 int option;
{
 unsigned int lcv,indx,num,cnt;
 int err,found;
 char *str;

 TRACE(printf("assign_text_group_num...BEGIN num_groups = %d option = %d\n",
        num_groups,option))
 TRACE(pr_strs_rec(labels))

 found = 0;
 for (lcv = 0; lcv < num_groups; lcv++)
    {if (option == GET_TEXT_ROI)
        {
         num = groups[lcv].num_rois;
         if ((err = get_memory_for_indx_nums(&(groups[lcv].roi_nums),
            &(groups[lcv].total_rois),groups[lcv].num_rois)) != NO_ERROR)
            return err;
        }
       else
        {
         num = groups[lcv].num_pixel_types;
         if ((err = get_memory_for_indx_nums(&(groups[lcv].pixel_type_nums),
               &(groups[lcv].total_pixel_types),
               groups[lcv].num_pixel_types)) != NO_ERROR)
            return err;
        }
     for (cnt = 0; cnt < num; cnt++)
       {
        if (option == GET_TEXT_ROI) str = groups[lcv].roi_labels[cnt];
          else str = groups[lcv].pixel_type_labels[cnt];
        indx = FindStrInHandleAryByContent(labels->strs,labels->num_strs,
                      str,&err);
        TRACE(printf("cnt = %d %s indx = %d err = %d\n",cnt,str,indx,err))
        if (err == NO_ERROR)
           {if (option == GET_TEXT_ROI)
               groups[lcv].roi_nums[cnt] = nums[indx];
             else groups[lcv].pixel_type_nums[cnt] = nums[indx];

            TRACE(printf("group# = %d type# = %d label = %s indx = %d num = %d\n",
              lcv,cnt,str,indx,nums[indx]))
            found = 1;
           }
       }
    }
 if (!found) return CANCEL;
 TRACE(printf("assign_text_group_num...END\n\n"))
 return NO_ERROR;
}
  

  
/*
 after the output format "struct text_output_analysis_group_rec *groups" is
     updated, we need to update the orders of the roi_nums & pixel_type_nums
*/

/********************
update_text_output_params(text)
 struct text_output_rec *text;
{
 int err;

 TRACE(printf("update_text_output_params...BEGIN\n"))
 if (((err = get_out_format_roi_nums(text)) == NO_ERROR) &&
     ((err = get_out_format_pixel_type_nums(&(text->format))) == NO_ERROR) &&
     ((err = assign_text_group_num(&(text->roi_labels),text->roi_nums,
                     text->groups,text->num_groups,GET_TEXT_ROI)) == NO_ERROR))
    err = assign_text_group_num(&(text->pixel_type_labels),
                     text->pixel_type_nums,
                     text->groups,text->num_groups,GET_TEXT_PIXEL_TYPE);
 TRACE(printf("update_text_output_params...end err = %d\n",err))
 return err;
}
***********************/


/*--------------------------------------------------------------------------
get_output_format_default_params - ???
--------------------------------------------------------------------------*/

void
get_output_format_default_params(format)
 struct output_text_format_rec *format;
{
 format->num_field_spaces = NUM_FILED_SPACES;
 format->num_collect_spaces = NUM_COLLECT_SPACES;
 format->page_row_size = PAGE_ROW_SIZE;
 format->page_num_rows = PAGE_NUM_ROWS;
}


/*--------------------------------------------------------------------------
setup_output_count_group_rec - ???
--------------------------------------------------------------------------*/

int
setup_output_count_group_rec(format,total_slices)
 struct output_text_format_rec *format;
 unsigned int total_slices;
{
 int err;
 unsigned int lcv,cnt;

 TRACE(printf("setup_output_count_group_rec...total_slices = %d\n",total_slices))

 format->status &= ~OUT_FORMAT_COUNT_DEFINED;
 get_ptr_memory(&(format->counts),&(format->num_slices),total_slices,
         (size_t) sizeof(struct output_count_group_rec),&err);
 if (err != NO_ERROR) return err;
 for (lcv = 0; lcv < format->num_slices; lcv++)
    init_output_count_group_rec(&(format->counts[lcv]));

 for (lcv = 0; lcv < format->num_slices; lcv++)
    {get_ptr_memory(&(format->counts[lcv].groups),
             &(format->counts[lcv].num_groups),format->num_groups,
            (size_t) sizeof(struct output_count_field_rec),&err);
     if (err != NO_ERROR) return err;
     for (cnt = 0; cnt < format->counts[lcv].num_groups; cnt++)
        init_output_count_field_rec(&(format->counts[lcv].groups[cnt]));
     for (cnt = 0; cnt < format->counts[lcv].num_groups; cnt++)
        {get_ptr_memory(&(format->counts[lcv].groups[cnt].fields),
             &(format->counts[lcv].groups[cnt].num_fields),
             format->groups[cnt].num_roi_indxs,
            (size_t) sizeof(*(format->counts[lcv].groups[cnt].fields)),&err);
         if (err != NO_ERROR) return err;
        }
     init_group_count_field_value_rec(format->counts[lcv].groups,
                  format->counts[lcv].num_groups);
    }

 return NO_ERROR;
}


/*--------------------------------------------------------------------------
setup_output_field_rec - ???
--------------------------------------------------------------------------*/

int
setup_output_field_rec(format,groups,num_groups)
 struct output_text_format_rec *format;
 struct text_output_analysis_group_rec *groups;
 unsigned int num_groups;
{
 unsigned int lcv,cnt,num,num_fields,indx;
 int err;

 TRACE(printf("setup_output_field_rec num_groups = %d\n",num_groups))

 for (num = lcv = 0; lcv < num_groups; lcv++)
    {
     TRACE(printf("setup_output_field_rec total_rois = %d groups[lcv].total_pixel_types  = %d\n",groups[lcv].total_rois,groups[lcv].total_pixel_types))
     for (cnt = 0; cnt < groups[lcv].total_rois; cnt++)
       {TRACE(printf("cnt = %d groups[lcv].roi_nums[cnt] = %d\n",cnt,
            groups[lcv].roi_nums[cnt]))

         if (groups[lcv].roi_nums[cnt])
           {for (cnt = 0; cnt < groups[lcv].total_pixel_types; cnt++)
               if (groups[lcv].pixel_type_nums[cnt]) num += 1;
            break;}
        }
    }
 TRACE(printf("setup_output_field_rec...num = %d format->num_groups = %d\n",num,
       format->num_groups))
 format->num_groups = 0;
 if (num == 0) return CANCEL;

 get_ptr_memory(&(format->groups),&(format->num_groups),num,
       (size_t) sizeof(struct output_field_rec),&err);
 if (err != NO_ERROR) return err;
 for (lcv = 0; lcv < format->num_groups; lcv++)
    init_output_field_rec(&(format->groups[lcv]));

 TRACE(printf("setup_output_field_rec...format->num_groups = %d\n",format->num_groups))
 for (num = lcv = 0; lcv < num_groups; lcv++)
    {for (num_fields = cnt = 0; cnt < groups[lcv].total_rois; cnt++)
        if (groups[lcv].roi_nums[cnt]) num_fields += 1;
     if (num_fields > 0)
        {for (indx = 0; indx < groups[lcv].total_pixel_types; indx++)
            if (groups[lcv].pixel_type_nums[indx])
               {get_ptr_memory(&(format->groups[num].roi_indxs),
                     &(format->groups[num].num_roi_indxs),num_fields,
                    (size_t) sizeof(*(format->groups[num].roi_indxs)),&err);
                if (err != NO_ERROR) return err;
                format->groups[num].pixel_type_indx =
                       groups[lcv].pixel_type_nums[indx];

                for (num_fields = cnt = 0; cnt < groups[lcv].total_rois; cnt++)
                   if (groups[lcv].roi_nums[cnt])
                      format->groups[num].roi_indxs[num_fields++] = 
                          groups[lcv].roi_nums[cnt];

                num += 1;
               }
        }
    }
 return NO_ERROR;
}


/*--------------------------------------------------------------------------
setup_output_text_format_rec - ???
--------------------------------------------------------------------------*/

int
setup_output_text_format_rec(format,hdr,groups,num_groups)
 struct output_text_format_rec *format;
 struct hdr_rec *hdr;
 struct text_output_analysis_group_rec *groups;
 unsigned int num_groups;
{
 int err;
 long num_slices;

 format->status = 0;
 if ((err = get_out_format_labels(&(format->roi_labels),groups,
                    num_groups,GET_TEXT_ROI)) != NO_ERROR)
    return err;
 num_slices = NumberOfSliceCuts(&(hdr->data_info));
 if (((err = get_out_format_labels(&(format->pixel_type_labels),
        groups,num_groups,GET_TEXT_PIXEL_TYPE)) != NO_ERROR) ||
     ((err = get_out_format_roi_nums(format)) != NO_ERROR) ||
     ((err = get_out_format_pixel_type_nums(format)) != NO_ERROR) ||
     ((err = assign_text_group_num(&(format->roi_labels),
           format->roi_nums,
           groups,num_groups,GET_TEXT_ROI)) != NO_ERROR) ||
     ((err = assign_text_group_num(&(format->pixel_type_labels),
           format->pixel_type_nums,
           groups,num_groups,GET_TEXT_PIXEL_TYPE)) != NO_ERROR) ||
     ((err = setup_output_field_rec(format,groups,
                    num_groups)) != NO_ERROR) ||
     ((err = setup_output_count_group_rec(format,num_slices)) != NO_ERROR))
    {if (err == CANCEL)
        format->status |= OUT_FORMAT_CHECK_FORMAT;
     return err;}
 format->xpixel_size = hdr->data_info.xpixel_size;
 format->ypixel_size = hdr->data_info.ypixel_size;
 format->pixel_thickness = hdr->data_info.pixel_thickness;
 format->voxel_size = hdr->data_info.xpixel_size *
       hdr->data_info.ypixel_size * 
         format->pixel_thickness;
 if (!(format->status & OUT_FORMAT_LT_PERCENT_MEAN)) format->lt_percent = .0;
 if (!(format->status & OUT_FORMAT_GT_PERCENT_MEAN)) format->gt_percent = .0;
 if (!(format->status & OUT_FORMAT_LT_PERCENT_ROI_MEAN)) 
    format->roi_lt_percent = .0;
 if (!(format->status & OUT_FORMAT_GT_PERCENT_ROI_MEAN)) 
    format->roi_gt_percent = .0;
 format->status |= OUT_FORMAT_DEFINED;
 TRACE(printf("setup_output_text_format_rec...end\n"))
 return NO_ERROR;
}


/*--------------------------------------------------------------------------
same_output_text_format_rec - ???
--------------------------------------------------------------------------*/

int
same_output_text_format_rec(new,old)
 struct output_text_format_rec *new,*old;
{
 int result,err;
 long lcv,cnt;

 if (((err = cmp_str_arys(new->roi_labels.strs,new->roi_labels.num_strs,
     old->roi_labels.strs,old->roi_labels.num_strs,&result
            )) != NO_ERROR) ||
     (result != 0)) return 0;

 if (((err = cmp_str_arys(new->pixel_type_labels.strs,
       new->pixel_type_labels.num_strs,
       old->pixel_type_labels.strs,
       old->pixel_type_labels.num_strs,&result)) != NO_ERROR) ||
     (result != 0)) return 0;

 for (lcv = 0; lcv < new->roi_labels.num_strs; lcv++)
    {
     if (new->roi_nums[lcv] != old->roi_nums[lcv]) return 0;}

 for (lcv = 0; lcv < new->pixel_type_labels.num_strs; lcv++)
    {
     if (new->pixel_type_nums[lcv] != old->pixel_type_nums[lcv])
        return 0;
    }
 if ((new->xpixel_size != old->xpixel_size) ||
     (new->ypixel_size != old->ypixel_size) ||
     (new->pixel_thickness != old->pixel_thickness))
    return 0;
 if (new->num_groups != old->num_groups) return 0;
 for (lcv = 0; lcv < new->num_groups; lcv++)
    {if (old->groups[lcv].pixel_type_indx != new->groups[lcv].pixel_type_indx)
        {/*printf("old pixel_type_indx= %d %d\n",
             old->groups[lcv].pixel_type_indx,new->groups[lcv].pixel_type_indx);*/
         return 0;}
     if (old->groups[lcv].num_roi_indxs != new->groups[lcv].num_roi_indxs)
        {/*printf("old num_roi_indxs= %d %d\n",
             old->groups[lcv].num_roi_indxs,new->groups[lcv].num_roi_indxs);*/
         return 0;}
     for (cnt = 0; cnt < old->groups[lcv].num_roi_indxs; cnt++)
        if (old->groups[lcv].roi_indxs[cnt] != new->groups[lcv].roi_indxs[cnt])
           {/*printf("old roi_indxs= %d %d\n",
             old->groups[lcv].roi_indxs[cnt],new->groups[lcv].roi_indxs[cnt]);*/
            return 0;}
    }
 return 1;
}


/*--------------------------------------------------------------------------
pr_output_text_format_rec - ???
--------------------------------------------------------------------------*/

void
pr_output_text_format_rec(format)
 struct output_text_format_rec *format;
{
 long lcv,indx;

 printf("pr_output_text_format_rec... status = %lx\n",
	(unsigned long)format->status);
 printf("format->roi_labels...\n");
 pr_strs_rec(&(format->roi_labels));

 printf("format->pixel_type_labels...\n");
 pr_strs_rec(&(format->pixel_type_labels));

 for (lcv = 0; lcv < format->roi_labels.num_strs; lcv++)
    {printf("roi_nums%ld) %ld ...\n",lcv,format->roi_nums[lcv]);
    }
 for (lcv = 0; lcv < format->pixel_type_labels.num_strs; lcv++)
    {printf("pixel_type_nums%ld) %ld ...\n",lcv,format->pixel_type_nums[lcv]);
    }

 printf("output_field_rec...format->num_groups = %ld\n",format->num_groups);
 for (lcv = 0; lcv < format->num_groups; lcv++)
    for (indx = 0; indx < format->groups[lcv].num_roi_indxs; indx++)
       {printf("%ld) type = %ld ...roi = %ld\n",
                lcv,format->groups[lcv].pixel_type_indx,
                format->groups[lcv].roi_indxs[indx]);
       }

 printf("output_count_group_rec...format->num_slices = %ld\n",
	format->num_slices);
/*
 for (lcv = 0; lcv < format->num_slices; lcv++)
    for (indx = 0; indx < format->counts[lcv].num_fields; indx++)
       {printf("%d) type = %d ...roi = %d\n",
*/
 printf("pr_output_text_format_rec... end\n\n");

}


/*--------------------------------------------------------------------------
pr_text_output_analysis_group_recs - ???
--------------------------------------------------------------------------*/

void
pr_text_output_analysis_group_recs(groups,num_groups)
 struct text_output_analysis_group_rec *groups;
 unsigned int num_groups;
{
 long lcv,indx;

 printf("pr_text_output_analysis_group_recs...num_groups = %d\n",num_groups);
 for (lcv = 0; lcv < num_groups; lcv++)
    {printf("   num_rois = %ld total = %ld roi_nums = %p\n",
	    groups[lcv].num_rois, groups[lcv].total_rois,groups[lcv].roi_nums);

     for (indx = 0; indx < groups[lcv].num_rois; indx++)
        {printf("g#%ld - f#%ld) roi-label = %s  ",lcv,indx,
                groups[lcv].roi_labels[indx]);
         if (groups[lcv].roi_nums != NULL)
            printf("num = %ld\n",groups[lcv].roi_nums[indx]);
           else printf("\n");
        }

     printf("   num_pixel_types = %ld total = %ld pixel_type_nums = %p\n",
	    groups[lcv].num_pixel_types, groups[lcv].total_pixel_types,
	    groups[lcv].pixel_type_nums);
     for (indx = 0; indx < groups[lcv].num_pixel_types; indx++)
        {printf("g#%ld - f#%ld) type-label = %s \n",lcv,indx,
                groups[lcv].pixel_type_labels[indx]);
         if (groups[lcv].pixel_type_nums != NULL)
            printf("num = %ld\n", groups[lcv].pixel_type_nums[indx]);
           else printf("\n");
        }
    }
 printf("pr_text_output_analysis_group_recs...end\n\n");
}



/*--------------------------------------------------------------------------
setup_output_format - ???
--------------------------------------------------------------------------*/

int
setup_output_format(text)
     struct text_output_rec *text;
{
  struct output_text_format_rec format;
  struct text_output_rec text1;
  int err;

  TRACE(printf("setup_output_format...begin status = %x\n",
	       (int) text->format.status));

  init_output_text_format_rec(&format);
  if (text->num_groups == 0)
  {
    TRACE(printf("setup_output_format text->num_groups = %d....%s\n",
		 text->num_groups,
		 text->file->txt_format_fn););
    init_text_output_rec(&text1,text->file);
    if ((err = RdTextFormat(text->file->txt_format_fn,
			    &(text1.groups),&(text1.num_groups),
			    &(text1.format),0)) != NO_ERROR)
    {
      TRACE(printf("setup_output_format err = %d\n",err));
      free_text_output_rec(&text1);
      TRACE(printf("setup_output_format 1 err = %d\n",err));
      return err;
    }
    free_text_output_rec(text);
    *text = text1;
    TRACE(printf("text->num_groups = %d....\n",text->num_groups));
    TRACE(pr_output_text_format_rec(&(text->format)););

    return NO_ERROR;
  }

  if ((err = setup_output_text_format_rec(&format,&(text->file->hdr),
					  text->groups,
					  text->num_groups)) != NO_ERROR)
    return err;

  TRACE(printf("setup_output_format...begin 1\n"));

 delete_the_text_output(text);
 if (text->format.status & OUT_FORMAT_DEFINED)
    {TRACE(pr_output_text_format_rec(&format);
           pr_output_text_format_rec(&(text->format));)
     TRACE(printf("same_output_text_format_rec..check..\n");)

     if (same_output_text_format_rec(&format,&(text->format)))
       {flush_counts(text);
        free_output_text_format_rec(&format);
        TRACE(printf("same_output_text_format_rec....\n");)
        return NO_ERROR;}
    }
 TRACE(printf("setup_output_format...new\n");)

 free_output_text_format_rec(&(text->format));
 TRACE(printf("setup_output_format...begin 3\n"))
 text->format = format;
 return NO_ERROR;
}


/*--------------------------------------------------------------------------
setup_text_output_params - ???
--------------------------------------------------------------------------*/

int
setup_text_output_params(text)
 struct text_output_rec *text;
{
 int err;

 TRACE(printf("setup_text_output_params..txt_format_fn = %s\n",
            text->file->txt_format_fn))
 if (!(text->status & TEXT_OUTPUT_FORMAT_SETUP) &&
     (text->file->txt_format_fn != NULL))
    {TRACE(printf("RdTextFormat..\n"))
     if ((err = RdTextFormat(text->file->txt_format_fn,
          &(text->groups),&(text->num_groups),&(text->format),0)) != NO_ERROR)
        {free_text_output_rec(text);
         TRACE(printf("setup_text_output_params..err = %d\n",err))
         return err;}

     text->status |= TEXT_OUTPUT_FORMAT_SETUP;
     text->status |= TEXT_OUTPUT_PARAMS_SETUP;
     text->status |= TEXT_OUTPUT_PIXEL_TYPE_ANALYSIS;
    }

 TRACE(printf("setup_text_output_params..end\n"))
 return NO_ERROR;
}



/***************
setup_pixel_type_output(format,file)
 struct output_text_format_rec *format;
 struct input_file_rec *file;
{
 char *str,*str1;
 long slnum,gnum,fnum,lcv,cnt;
 struct slice_roi_win_rec *slice_roi_wins;
 struct slice_roi_rec *slice_roi;
 struct roi_rec *roi;
 char *label;
 struct output_count_field_rec *group;
 struct pixel_type_cnt_rec *ptr;
 unsigned long roi_indx,pixel_type_indx,type_indx;
 int err,found;

 TRACE(printf("setup_pixel_type_output...num_slices = %d counts = %u\n",
             format->num_slices,format->counts))
 TRACE(pr_strs_rec(&(format->roi_labels)))
 TRACE(pr_strs_rec(&(format->pixel_type_labels)))
 TRACE(pr_strs_rec(&(file->pixel_type.order)))
 slice_roi_wins = file->slice_info.slice_roi_wins;
 for (slnum = 0; slnum < format->num_slices; slnum++)
    {slice_roi = &(slice_roi_wins[slnum].slice_roi);
     TRACE(printf("...slnum = %d\n",slnum))

     for (gnum = 0; gnum < format->counts[slnum].num_groups; gnum++)
        {group = &(format->counts[slnum].groups[gnum]);
         for (fnum = 0; fnum < group->num_fields; fnum++)
            group->fields[fnum].valid = 0;
        }
     for (lcv = 0; lcv < format->roi_labels.num_strs; lcv++)
        {
         TRACE(printf(" label = %s ",format->roi_labels.strs[lcv]))
         if ((roi = get_roi(slice_roi,format->roi_labels.strs[lcv])) != NULL)
            {
             ptr = (struct pixel_type_cnt_rec *) roi->user.ptr;
            }
           else ptr = NULL;
         roi_indx = format->roi_nums[lcv];
         TRACE(printf("  ptr = %u  roi = %u roi_indx = %d\n",ptr,roi,roi_indx))
         for (gnum = 0; gnum < format->counts[slnum].num_groups; gnum++)
            {
             group = &(format->counts[slnum].groups[gnum]);
             if (pixel_type_indx = format->groups[gnum].pixel_type_indx)
                {TRACE(printf("   pixel_type_indx = %d type = %s ",pixel_type_indx,
                         format->pixel_type_labels.strs[pixel_type_indx-1]))
                 str1 = format->pixel_type_labels.strs[pixel_type_indx-1];
                 if ((found = find_str_in_strs_rec(&(file->pixel_type.order),
                        str1,&type_indx)) != NO_ERROR)
                    {if ((err = concat_strs(&str,2,str1,".TYPE")) != NO_ERROR)
                        {TRACE(printf("\n"))
                         return err;}
                 
                     found = find_str_in_strs_rec(&(file->pixel_type.order),
                           str,&type_indx);
                     c_free(str);
                    }
                 if (found != NO_ERROR)
                    {
                     TRACE(printf("\n"))
                     for (fnum = 0; fnum < group->num_fields; fnum++)
                            group->fields[fnum].valid |= COUNT_PIXEL_TYPE_NOT_DEFINED; 
                     continue;}
                }
               else
                {for (fnum = 0; fnum < group->num_fields; fnum++)
                    group->fields[fnum].valid |= COUNT_PIXEL_TYPE_NOT_DEFINED;
                 continue;}

             for (fnum = 0; fnum < group->num_fields; fnum++)
                {
                 if (roi_indx != format->groups[gnum].roi_indxs[fnum]) continue;
                 if (roi == NULL) continue;

                 group->fields[fnum].valid = COUNT_ROI_DEFINED;

                 printf("  gnum = %d fnum = %d roinum = %d va = %d ",gnum,fnum,
                      format->groups[gnum].roi_indxs[fnum],(int)group->fields[fnum].valid);

                 if ((ptr == NULL) ||
                     !(ptr->status & PIXEL_TYPE_CNTS_COMPUTED))
                    group->fields[fnum].valid |= COUNT_ROI_NOT_COMPUTED;
                   else
                    {
                     group->fields[fnum].value = 
                                    ptr->pixel_type_cnts[type_indx+1];

                     TRACE(printf("gnum = %d fnum = %d type = %d type_indx = %d val = %d valid = %d",
                               gnum,fnum,pixel_type_indx,type_indx+1,
                               group->fields[fnum].value,
                               (unsigned long) group->fields[fnum].valid))
                 
                     }
                  TRACE(printf("\n"))

                  printf("  valid = %d  value = %d\n",
                       (int)group->fields[fnum].valid,group->fields[fnum].value);

                 }
            }
         }
     }
 format->status |= OUT_FORMAT_COUNT_DEFINED;
 TRACE(printf("setup_pixel_type_output...end\n"))
 return NO_ERROR;
}
******************/
