/****************************************************************************
win_text.c - ???

Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.
****************************************************************************/

static const char rcsid[] = "$Id: win_text.c,v 1.10 1999/07/07 17:41:20 hughett Exp $ Release $Name: nria_1_0_6 $";


/* FIXME:  Not needed?
#include <time.h>
#include <sys/timeb.h>
*/

#include <X11/X.h>
#include <X11/Xlib.h>
#include <xview/xv_xrect.h>
#include <xview/cms.h>
#include <xview/font.h>

#include "mri_format.h"
#include "error_constants.h"
#include "rect.h"
#include "typedef.h"
#include "nria_procedures.h"

/*
#define TRACE(x) x; fflush(stdout);
*/
#define TRACE(x)


/* Functions defined in this file */
int get_strs_dim(Xv_Window win, char *strs[], int num_strs, int *width,
    int *height, int *line_height);
int largest_ch_dim(Xv_Window win, int *width, int *height);
int writeTextToXY(Xv_window win, Pixmap pm, int is_wr_to_win, int x, int y,
    char *text);
int center_text_in_rect(Xv_window win, Pixmap pm, int is_wr_to_win, Rect *rr,
    char *text);
int center_text_in_xdim(Xv_window win, Pixmap pm, int is_wr_to_win, int left,
    int rightx, int y, char *text);
int center_text_in_ydim(Xv_window win, Pixmap pm, int is_wr_to_win, int x,
    int top, int bottomy, char *text);
int user_text(Xv_Window win, Pixmap pm, int is_wr_to_win, int x, int y,
    int op, char *font_name, char *text);
int draw_int_rect(Xv_Window win, Pixmap pm, int is_wr_to_win,
    struct int_rect_rec *rr, int color, int is_polygon);
int win_center_int_rect_text(Xv_Window win, Pixmap pm, int is_wr_to_win,
    struct int_rect_rec *rr, int text_op, char *font_name, char *text,
    int text_color);
void win_center_text_in_int_rect(Xv_Window win, Pixmap pm, int is_wr_to_win,
    struct int_rect_rec *rr, int rr_frame_color, int rr_bkg_color,
    char *font_name, char *text, int text_color, int text_op, int erase,
    int trans_x, int trans_y);



/*------------------------------------------------------------------------
get_strs_dim - ???

Get maximum width, total height, and individual line height of
a certain text, given a certain font
------------------------------------------------------------------------*/

int
get_strs_dim( win, strs, num_strs, width, height, line_height )
     Xv_Window win;
     char *strs[];
     int num_strs;
     int *width, *height, *line_height;	/* output arguments */
{
  int lcv;
  Xv_Font xvfont;
  Font_string_dims dims;


  *width = *height = 0;
  if ((xvfont = (Xv_Font) xv_get(win,XV_FONT)) == 0)
    return ~NO_ERROR;
  for (lcv = 0; lcv < num_strs; lcv++)
  {
    if (strs[lcv] != NULL)
    {
      xv_get(xvfont,FONT_STRING_DIMS,strs[lcv],&dims);
      if (*width < dims.width)
	*width = dims.width;
      *height += dims.height;
    }
  }

  /*  This shows an obvious bug in callers' algorithms.  */
  if (*height == 0)
    *line_height = 0;
  else
    *line_height = dims.height;

  return NO_ERROR;
}



/*------------------------------------------------------------------------
largest_ch_dim - ???
------------------------------------------------------------------------*/

int
largest_ch_dim(win,width,height)
 Xv_Window win;
 int *width,*height;
{
 char *str;
 int line_height;

 str = "m";
 return get_strs_dim(win,&str,1,width,height,&line_height);
}


/*------------------------------------------------------------------------
writeTextToXY - ???

RW 961111: Write text left-aligned at x,y.

FIXME:  Convert to "ToRect", and add arg: Alignment, with possible values
    1 thru 9, which correspond respectively with: NW, N, NE, W, CENTER, E,
    SW, S, SE.
------------------------------------------------------------------------*/

int
writeTextToXY( win, pm, is_wr_to_win, x, y, text )
     Xv_window win;
     Pixmap pm;
     int is_wr_to_win;
     int x, y;
     char *text;
{
  Xv_Font xvfont;
  XFontStruct *font;
  /* Font_string_dims dims;   FIXME:  For future use  */
  Display *dp;
  XID xid;
  XGCValues gc_val;
  GC gc;
  Cms cms;
  unsigned long fg;


  if (text == NULL)
    return ~NO_ERROR;
  if ((xvfont = (Xv_Font) xv_get(win,XV_FONT)) == 0)
    return ~NO_ERROR;
  if ((dp = (Display *) xv_get(win,XV_DISPLAY)) == NULL)
    return ~NO_ERROR;

  /*
   *  Get XID of X window to which we will draw the text.
   *    Note that a canvas is not directly drawn on; instead,
   *    we need to get the PAINT window of the canvas.
   */
  if (is_wr_to_win == 0)
    xid = (XID) pm;
  else if ((Xv_pkg *) xv_get(win,XV_TYPE) == CANVAS) {
    if ((xid = (XID) xv_get(canvas_paint_window(win),XV_XID)) == 0)
      return ~NO_ERROR;
  }
  else {
    if (( xid = (XID) xv_get(win,XV_XID)) == 0)
      return ~NO_ERROR;
  }

  font = (XFontStruct *) xv_get(xvfont,FONT_INFO);
  gc_val.font = font->fid;
  gc = XCreateGC(dp,xid,GCFont,&gc_val);
  if ((cms = (Cms) xv_get(win,WIN_CMS)) != 0) {
    fg = (unsigned long) xv_get(win,WIN_FOREGROUND_COLOR);
    fg = (unsigned long) xv_get(cms,CMS_PIXEL,fg);
    XSetForeground(dp,gc,fg);
  }
  XDrawString(dp,xid,gc,x,y,text,strlen(text));
  XFreeGC(dp,gc);
  return NO_ERROR;
}


/*------------------------------------------------------------------------
center_text_in_rect - ???
------------------------------------------------------------------------*/

int
center_text_in_rect(win,pm,is_wr_to_win,rr,text)
 Xv_window win;
 Pixmap pm;
 int is_wr_to_win;
 Rect *rr;
 char *text;
{
#if 1
 Xv_Font xvfont;
 int x, y;
 Font_string_dims dims;


 if ((xvfont = (Xv_Font) xv_get(win,XV_FONT)) == 0)
   return ~NO_ERROR;
 xv_get(xvfont,FONT_STRING_DIMS,text,&dims);
 x = rr->r_left + rr->r_width / 2 - dims.width / 2;
 y = rr->r_top + rr->r_height / 2 - dims.height / 2;
 return writeTextToXY( win, pm, is_wr_to_win, x, y, text );
#else
 /*  DRW 961111: Obsolete version of center_text_in_rect()  */
 Xv_Font xvfont;
 XFontStruct *font;
 int x,y;
 Font_string_dims dims;
 Display *dp;
 XID xid;
 XGCValues gc_val;
 GC gc;
 Cms cms;
 unsigned long fg;

 if (text == NULL) return ~NO_ERROR;
 if ((xvfont = (Xv_Font) xv_get(win,XV_FONT)) == NULL) return ~NO_ERROR;
 xv_get(xvfont,FONT_STRING_DIMS,text,&dims);
 x = rr->r_left + rr->r_width / 2 - dims.width / 2;
 y = rr->r_top + rr->r_height / 2 - dims.height / 2;
 if ((dp = (Display *) xv_get(win,XV_DISPLAY)) == NULL) return ~NO_ERROR;

 if (is_wr_to_win == 0)
    xid = (XID) pm;
   else if ((Xv_pkg *) xv_get(win,XV_TYPE) == CANVAS)
    {
     if ((xid = (XID) xv_get(canvas_paint_window(win),XV_XID)) == NULL)
        return ~NO_ERROR;
    }
   else
    {
    if (( xid = (XID) xv_get(win,XV_XID)) == NULL) return ~NO_ERROR;
    }
 font = (XFontStruct *) xv_get(xvfont,FONT_INFO);
 gc_val.font = font->fid;
 gc = XCreateGC(dp,xid,GCFont,&gc_val);
 if ((cms = (Cms) xv_get(win,WIN_CMS)) != NULL)
    {fg = (unsigned long) xv_get(win,WIN_FOREGROUND_COLOR);
     fg = (unsigned long) xv_get(cms,CMS_PIXEL,fg);
     XSetForeground(dp,gc,fg);}

 XDrawString(dp,xid,gc,x,y,text,strlen(text));
 XFreeGC(dp,gc);
 return NO_ERROR;
#endif
}


/*------------------------------------------------------------------------
center_text_in_xdim - ???
------------------------------------------------------------------------*/

int
center_text_in_xdim(win,pm,is_wr_to_win,left,rightx,y,text)
 Xv_window win;
 Pixmap pm;
 int is_wr_to_win;
 int left,rightx,y;
 char *text;
{
 Rect rr;

 rr.r_left = left;
 rr.r_width = rightx - left;
 rr.r_top = 0;
 rr.r_height = 2 * y;
 return center_text_in_rect(win,pm,is_wr_to_win,&rr,text);
}


/*------------------------------------------------------------------------
center_text_in_ydim - ???
------------------------------------------------------------------------*/

int
center_text_in_ydim(win,pm,is_wr_to_win,x,top,bottomy,text)
 Xv_window win;
 Pixmap pm;
 int is_wr_to_win;
 int x,top,bottomy;
 char *text;
{
 Rect rr;

 rr.r_left = 0;
 rr.r_width = 2 * x;
 rr.r_top = top;
 rr.r_height = bottomy - top;
 return center_text_in_rect(win,pm,is_wr_to_win,&rr,text);
}


/*------------------------------------------------------------------------
user_text - ???
------------------------------------------------------------------------*/

int
user_text(win,pm,is_wr_to_win,x,y,op,font_name,text)
 Xv_Window win;
 Pixmap pm;
 int is_wr_to_win;
 int x,y,op;
 char *font_name;
 char *text;
{
 Display *dp;
 Drawable xid;
 GC gc;
 XGCValues gc_val;
 XFontStruct *font;
 Xv_Font the_font;
 int screen_num;
 int func;
 Cms cms;
 unsigned long fg;

 dp = (Display *) xv_get(win,XV_DISPLAY);
 if (is_wr_to_win == 0)
    xid = (XID) pm;
   else if ((Xv_pkg *) xv_get(win,XV_TYPE) == CANVAS)
    {
     if ((xid = (Drawable) xv_get(canvas_paint_window(win),XV_XID)) == 0)
        return ~NO_ERROR;
    }
   else if ((xid = (Drawable) xv_get(win,XV_XID)) == 0)
        return ~NO_ERROR;
 screen_num = XDefaultScreen(dp);

 gc_val.function = op;
 func = GCFunction;
 if (font_name != NULL)
    {if ((font = XLoadQueryFont(dp,font_name)) != NULL)
        {
         gc_val.font = font->fid;
         func |= GCFont;}
    }
   else if ((the_font = (Xv_Font) xv_get(win,XV_FONT)) != 0)
        {
         gc_val.font = (Font) xv_get(the_font,XV_XID);
         func |= GCFont;}

 gc = XCreateGC(dp,xid,func,&gc_val);
 if ((cms = (Cms) xv_get(win,WIN_CMS)) != 0)
    {fg = (unsigned long) xv_get(win,WIN_FOREGROUND_COLOR);
     fg = (unsigned long) xv_get(cms,CMS_PIXEL,fg);
     XSetForeground(dp,gc,fg);}

 XDrawString(dp,xid,gc,x,y,text,get_strlen(text));
 XFreeGC(dp,gc);
 return NO_ERROR;
}


/*------------------------------------------------------------------------
draw_int_rect - ???
------------------------------------------------------------------------*/

int
draw_int_rect(win,pm,is_wr_to_win,rr,color,is_polygon)
 Xv_Window win;
 Pixmap pm;
 int is_wr_to_win;
 struct int_rect_rec *rr;
 int color;
 int is_polygon;
{
 Display *dp;
 Drawable xid;
 GC gc;
 XGCValues gc_val;
 Cms cms;
 unsigned int num_real_color_indxs;
 unsigned long *real_color_indxs;

 dp = (Display *) xv_get(win,XV_DISPLAY);
 if (is_wr_to_win == 0)
    xid = (XID) pm;
   else if ((Xv_pkg *) xv_get(win,XV_TYPE) == CANVAS)
    {
     if ((xid = (Drawable) xv_get(canvas_paint_window(win),XV_XID)) == 0)
        return ~NO_ERROR;
    }
   else if ((xid = (Drawable) xv_get(win,XV_XID)) == 0)
        return ~NO_ERROR;

 if ((cms = (Cms) xv_get(win,WIN_CMS)) == 0)
    return ~NO_ERROR;
 real_color_indxs = (unsigned long *) xv_get(cms,CMS_INDEX_TABLE);
 num_real_color_indxs = (unsigned int) xv_get(cms,CMS_SIZE);
 if (num_real_color_indxs == 0) return ~NO_ERROR;
 if (color >= num_real_color_indxs)
    color = num_real_color_indxs - 1;

 gc_val.foreground = real_color_indxs[color];
 gc_val.line_style = LineSolid;
 gc = XCreateGC(dp,xid,GCForeground | GCLineStyle,&gc_val);
 if (is_polygon)
     XFillRectangle(dp,xid,gc,rr->left,rr->top,rr->width,rr->height);
   else
     XDrawRectangle(dp,xid,gc,rr->left,rr->top,rr->width,rr->height);
 XFreeGC(dp,gc);
 return NO_ERROR;
}


/*------------------------------------------------------------------------
win_center_int_rect_text - ???
------------------------------------------------------------------------*/

int
win_center_int_rect_text(win,pm,is_wr_to_win,
                         rr,text_op,font_name,text,text_color)
 Xv_Window win;
 Pixmap pm;
 int is_wr_to_win;
 struct int_rect_rec *rr;
 int text_op;
 char *font_name;
 char *text;
 int text_color;
{
 int row_height,col_width,x,y,len;
 Display *dp;
 Drawable xid;
 GC gc;
 XGCValues gc_val;
 XFontStruct *font;
 Xv_Font the_font;
 int func;
 Cms cms;
 unsigned int num_real_color_indxs;
 unsigned long *real_color_indxs;

 if (text == NULL) return NO_ERROR;
 dp = (Display *) xv_get(win,XV_DISPLAY);
 if (is_wr_to_win == 0)
    xid = (XID) pm;
   else if ((Xv_pkg *) xv_get(win,XV_TYPE) == CANVAS)
    {
     if ((xid = (Drawable) xv_get(canvas_paint_window(win),XV_XID)) == 0)
        return ~NO_ERROR;
    }
   else if ((xid = (Drawable) xv_get(win,XV_XID)) == 0)
        return ~NO_ERROR;

 if ((cms = (Cms) xv_get(win,WIN_CMS)) == 0) return ~NO_ERROR;
 real_color_indxs = (unsigned long *) xv_get(cms,CMS_INDEX_TABLE);
 num_real_color_indxs = xv_get(cms,CMS_SIZE);
 if (num_real_color_indxs == 0) return ~NO_ERROR;
 if (text_color >= num_real_color_indxs) text_color = num_real_color_indxs - 1;

 gc_val.function = text_op;
 gc_val.foreground = real_color_indxs[text_color];
 func = GCFunction | GCForeground;
 if (font_name != NULL)
    {if ((font = XLoadQueryFont(dp,font_name)) != NULL)
        {
         gc_val.font = font->fid;
         func |= GCFont;}
    }
   else if ((the_font = (Xv_Font) xv_get(win,XV_FONT)) != 0)
        {
         gc_val.font = (Font) xv_get(the_font,XV_XID);
         func |= GCFont;}
 func |= GCForeground;
 gc = XCreateGC(dp,xid,func,&gc_val);

 col_width = (int) xv_get(win,WIN_COLUMN_WIDTH)
           + (int) xv_get(win,WIN_COLUMN_GAP);
 row_height = (int) xv_get(win,WIN_ROW_HEIGHT);

 len = strlen(text);
 x = (rr->left + rr->width / 2) - (col_width * len) / 2;
 y = (rr->top + rr->height / 2) + row_height / 2;
 XDrawString(dp,xid,gc,x,y,text,len);
 XFreeGC(dp,gc);
 return NO_ERROR;

}


/*------------------------------------------------------------------------
win_center_text_in_int_rect - ???
------------------------------------------------------------------------*/

void
win_center_text_in_int_rect(win,pm,is_wr_to_win,rr,rr_frame_color,rr_bkg_color,
                                  font_name,text,text_color,text_op,
                                  erase,trans_x,trans_y)
 Xv_Window win;
 Pixmap pm;
 int is_wr_to_win;
 struct int_rect_rec *rr;
 char *font_name;
 char *text;
 int rr_frame_color,rr_bkg_color,text_color;
 int text_op,erase;
 int trans_x,trans_y;
{

 rr->top += trans_y;
 rr->left += trans_x;
 if (erase) draw_int_rect(win,pm,is_wr_to_win,rr,rr_bkg_color,1);
 draw_int_rect(win,pm,is_wr_to_win,rr,rr_frame_color,0);
 win_center_int_rect_text(win,pm,is_wr_to_win,
                          rr,text_op,font_name,text,text_color);
 rr->left -= trans_x;
 rr->top -= trans_y;

}

