/****************************************************************************
dates.c - ???

Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.
****************************************************************************/

static const char rcsid[] = "$Id: dates.c,v 1.7 1999/07/07 18:07:14 hughett Exp $ Release $Name: nria_1_0_6 $";

#include <stdio.h>
#include <string.h>


/* dates.c */
char *canonizeDate(const char *old, char *new);
int translateDate(const char *datestring, unsigned int *year,
    unsigned int *month, unsigned int *day);


/*------------------------------------------------------------------------
canonizeDate - convert a date string to canonical form

Make all date formats fit canonical Year-2000 compatible format YYYYMMDD

 Possible styles:
   xx-xxx-xx (unknown)
   YYYY.MM.DD
   MM/DD/YY
   DD-Mon-YY
   YYYYMMDD
------------------------------------------------------------------------*/

char *
canonizeDate( 
    const char * old,
    char * new )
{
  unsigned int year, month, day;
  static char canonDate[10];

  if (!new)
    new = canonDate;

  if (translateDate( old, &year, &month, &day ) < 0)
    sprintf( new, "00000000" );
  else
    sprintf( new, "%0.4d%0.2d%0.2d", year, month, day );

  return new;
}



/*------------------------------------------------------------------------
translateDate - ???
------------------------------------------------------------------------*/

int
translateDate( const char * datestring,
	       unsigned int * year,
	       unsigned int * month,
	       unsigned int * day )
{
  int date;

  /*  Problematic  */
  if (!datestring || *datestring == '\0')
    return -1;

  /*  "xx-xxx-xx"  */
  if (strcmp( datestring, "xx-xxx-xx" ) == 0) {
    return -1;
  }

  /*  YYYY.MM.DD  */
  if (strchr( datestring, '.' )) {
    if (sscanf( strrchr( datestring, '.' ) + 1, "%d", day ) != 1)
      *day = 0;
    if (sscanf( strchr( datestring, '.' ) + 1, "%d", month ) != 1)
      *month = 0;
    if (sscanf( datestring, "%d", year ) != 1)
      *year = 0;
    return 0;
  }

  /*  MM/DD/YY  */
  if (strchr( datestring, '/' )) {
    if (sscanf( strrchr( datestring, '/' ) + 1, "%d", year ) != 1)
      *year = 0;
    else
      *year += 1900;
    if (sscanf( strchr( datestring, '/' ) + 1, "%d", day ) != 1)
      *day = 0;
    if (sscanf( datestring, "%d", month ) != 1)
      *month = 0;
    return 0;
  }

  /*  DD-Mon-YY  */
  if (strchr( datestring, '-' )) {
    char monthstr[4];

    if (sscanf( strrchr( datestring, '-' ) + 1, "%d", year ) != 1)
      *year = 0;
    else
      *year += 1900;
    if (sscanf( datestring, "%d", day ) != 1)
      *day = 0;
    strncpy( monthstr, strchr( datestring, '-' ) + 1, 3 );
    switch( monthstr[0] ) {
    case 'J': case 'j':
      if (monthstr[1] == 'a')
	*month = 1;
      else if (monthstr[2] == 'n')
	*month = 6;
      else
	*month = 7;
      break;
    case 'F': case 'f':
      *month = 2;
      break;
    case 'M': case 'm':
      if (monthstr[2] == 'r')
	*month = 3;
      else
	*month = 5;
      break;
    case 'A': case 'a':
      if (monthstr[1] == 'p')
	*month = 4;
      else
	*month = 8;
      break;
    case 'S': case 's':
      *month = 9;
      break;
    case 'O': case 'o':
      *month = 10;
      break;
    case 'N': case 'n':
      *month = 11;
      break;
    case 'D': case 'd':
      *month = 12;
      break;
    default:
      *month = 0;
      break;
    }
    return 0;
  }

  /*  YYYYMMDD  */
  if (sscanf( datestring, "%d", &date ) != 1)
    return -1;

  *year = date / 10000;
  date -= (*year * 10000);
  *month = date / 100;
  *day = date - (*month * 100);
  return 0;
}



/*------------------------------------------------------------------------
 - ???
------------------------------------------------------------------------*/

#ifdef TESTING
int
main( int argc, char *argv[] )
{
  char datestring[10];
  int args = 1;

  while (--argc) {
    canonizeDate( argv[args], datestring );
    printf( "%s   ->   %s\n", argv[args], datestring );
    args++;
  }
}
#endif
