/****************************************************************************
examine_hdrs.c - Print out NRIA header summary(ies)

Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.
****************************************************************************/

static const char rcsid[] = "$Id: examine_hdrs.c,v 1.12 1999/07/07 18:36:58 hughett Exp $ Release $Name: nria_1_0_6 $";

#include <stdio.h>
#include <stdlib.h>

#include <sys/types.h>
#include <unistd.h>

#include <grp.h>     /* FIXME:  What are these?  -PWH 19990430 */
#include <pwd.h>

#include "mri_format.h"
#include "debug.h"
#include "limits.h"
#include "string.h"
#include "dates.h"
#include "nria_lib.h"



/* examine_hdrs.c */
void rotate(char *ch);
void compute_spaces(struct strs_rec *ptr, long *spaces, long total_fields,
    long num_lines);
int wr_line_token(FILE *fid, char *str, unsigned long str_len,
    unsigned long *cnt_spaces, unsigned long *num_lines, int max_line_chs);
int wr_lineup_lines(FILE *fid, struct strs_rec *ptr, long total_fields,
    long num_lines, int max_line_chs, int is_title);
int wr_hdr_info(struct strs_rec *out, char *hdrfn, int hidden);
int sysadminuser(void);
void showusage(char *argv0);
int main(int argc, char *argv[]);


/* Global variables */
int debugFlag = 0;

static int restricted = 1;


/*--------------------------------------------------------------------------
rotate - ???

FIXME: The rotate function appears in both pr_ia_hdr.c and 
examine_hdrs.c; it really ought to be a common library routine. 
--------------------------------------------------------------------------*/

void
rotate( char *ch )
{
  char nums[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789nopqrstuvwxyzabcdefghijklmNOPQRSTUVWXYZABCDEFGHIJKLM!@#{}&()[]";
  char *index;
  int numchars = sizeof(nums) - 1;
  int rotateby = numchars / 2;


  while (*ch) {
    if (index = strchr( nums, *ch ))
      *ch = nums[((index - nums) + rotateby) % numchars];
    ch++;
  }
}



/*--------------------------------------------------------------------------
compute_spaces - ???
--------------------------------------------------------------------------*/

void
compute_spaces( struct strs_rec * ptr,
		long * spaces,
		long total_fields,
		long num_lines )
{
  long lcv,cnt,indx,len;

  for (lcv = 0; lcv < total_fields; lcv++,indx++) spaces[lcv] = 0;

  for (indx = cnt = 0; cnt < num_lines; cnt++)
  {
    for (lcv = 0; lcv < total_fields; lcv++,indx++)
    {
      len = strlen(ptr->strs[indx]);
      if (len > spaces[lcv])
	spaces[lcv] = len;
    }
  }
}



/*--------------------------------------------------------------------------
wr_line_token - ???
--------------------------------------------------------------------------*/

int
wr_line_token( FILE * fid,
	       char * str,
	       unsigned long str_len,
	       unsigned long * cnt_spaces,
	       unsigned long * num_lines,
	       int max_line_chs )
{
  long len;
  char ch;

  *cnt_spaces += str_len;
  if (*cnt_spaces >= max_line_chs)
  {
    len = str_len + max_line_chs - *cnt_spaces;
    ch = str[len];
    str[len] = '\0';
    fprintf(fid,"%s\n",str);
    *cnt_spaces -= max_line_chs;
    str[len] = ch;
    fprintf(fid,"%s ",&(str[len]));
    *num_lines += 1;
  }
  else
  {
    fprintf(fid,"%s ",str);
    *cnt_spaces += 1;
  }
  return ferror(fid); 
}



/*--------------------------------------------------------------------------
wr_lineup_lines - ???
--------------------------------------------------------------------------*/

int
wr_lineup_lines( FILE * fid,
		 struct strs_rec * ptr,
		 long total_fields,
		 long num_lines,
		 int max_line_chs,
		 int is_title )
{
  int err;
  char *str;
  int lcv,cnt,indx,max;
  long total,*spaces,loop;
  unsigned long cnt_spaces, num;

  if (ptr->num_strs == 0)
  {
    return NO_ERROR;
  }
  total = 0;
  get_ptr_memory(&spaces,&total,total_fields,(size_t) sizeof(*spaces),&err);
  if (err != NO_ERROR)
    return err;
  compute_spaces(ptr,spaces,total_fields,num_lines);

  clearerr(fid);
  err = NO_ERROR;
  for (max = 0,lcv = 0; lcv < total_fields; lcv++)
    if (max < spaces[lcv]) max = spaces[lcv];
  total = 0;
  get_ptr_memory(&str,&total,max + 1,1,&err);
  if (err != NO_ERROR)
    return err;
  for (num = 0,indx = cnt = 0; cnt < num_lines; cnt++)
  {
    for (cnt_spaces = lcv = 0; lcv < total_fields; lcv++,indx++)
    {
      sprintf(str,"%s",ptr->strs[indx]);
      set_str_len(str,spaces[lcv]);
      if ((err = wr_line_token( fid, str, spaces[lcv],
				&cnt_spaces, &num,
				max_line_chs)) != NO_ERROR)
      {
	c_free(str);
	return err;
      }
    }
    fprintf(fid,"\n");
    if ((cnt == 0) && is_title)
    {
      for (cnt_spaces = lcv = 0; lcv < total_fields; lcv++)
      {
	for (loop = 0; loop < spaces[lcv]; loop++)     
	  str[loop] = '-';
	str[loop] = '\0';
	if ((err = wr_line_token(fid,str,spaces[lcv],&cnt_spaces,&num,
				 max_line_chs)) != NO_ERROR)
	{
	  c_free(str);
	  return err;
	}
      }
      fprintf(fid,"\n");
    }
    num += 1;
  }
  c_free(str);
  return err;
}



/*--------------------------------------------------------------------------
wr_hdr_info - ???
--------------------------------------------------------------------------*/

int
wr_hdr_info( struct strs_rec * out,
	     char * hdrfn,
	     int hidden )
{
  int err;
  struct hdr_rec hdr;
  struct data_info_rec *info;
  char mhcrc[6],study_num[9],te[13],date[13];
  char *fn1;

  if ((err = rd_the_hdr(hdrfn,&hdr)) != NO_ERROR)
  {
    printf("Error reading the %s file...err = %d\n",hdrfn,err);
    return err;
  }

  info = &(hdr.data_info);
  if (fn1 = strrchr( hdrfn, '/' ))
    fn1 += 1;
  else
    fn1 = hdrfn;

  if (hidden) {

    if (hdr.comment_size == 24 && hdr.comment &&
	*(hdr.comment) == '$' && *(hdr.comment + 1) == '$') {
      rotate( hdr.comment );

      strncpy( mhcrc, hdr.comment + 2, 5 );
      mhcrc[5] = '\0';

      strncpy( study_num, hdr.comment + 7, 8 );
      study_num[8] = '\0';

      strncpy( date, hdr.comment + 15, 8 );
      date[8] = '\0';
    }
    else {
      strcpy( mhcrc, "*****" );
      strcpy( study_num, "********" );
      strcpy( date, "********" );
    }
  }
  else {
    trim_str_blanks(info->patient.patient_num);
    if (get_strlen(info->patient.patient_num) == 0)
      sprintf(mhcrc,"00000");
    else {
      int pnum;

      if (sscanf( info->patient.patient_num, "%d", &pnum ) != 1)
	sprintf( mhcrc, "00000" );
      else
	sprintf( mhcrc, "%0.5d", pnum );
    }

    trim_str_blanks(info->date_of_study);
    if (get_strlen(info->date_of_study) == 0)
      sprintf(date,"00000000");
    else {
      canonizeDate( info->date_of_study, date );
    }

    trim_str_blanks(info->patient.study_num);
    if (get_strlen(info->patient.study_num) == 0)
      sprintf(study_num,"00000000");
    else {
      int snum;

      /* If not a number (should be), then simply copy verbatim */
      if (sscanf( info->patient.study_num, "%d", &snum ) != 1)
	sprintf( study_num, "00000000" );
      else
	sprintf( study_num, "%0.8d", snum );
    }
  }

  trim_str_blanks(info->te);
  if (get_strlen(info->te) == 0)
    sprintf(te,".");
  else
    str_ncpy(te,sizeof(te),info->te);

  if (((err = add_str_to_strs_rec(out,fn1)) == NO_ERROR) &&
      ((err = add_str_to_strs_rec(out,mhcrc)) == NO_ERROR) &&
      ((err = add_str_to_strs_rec(out,study_num)) == NO_ERROR) &&
      ((err = add_str_to_strs_rec(out,date)) == NO_ERROR) &&
      ((err = add_double_to_strs_rec(out,info->min_pixel,2)) == NO_ERROR) &&
      ((err = add_double_to_strs_rec(out,info->max_pixel,2)) == NO_ERROR) &&
      ((err = add_double_to_strs_rec(out, (double) info->pixel_thickness,1))
       == NO_ERROR) &&
      ((err = add_double_to_strs_rec(out, (double) info->xpixel_size,4))
       == NO_ERROR) &&
      ((err = add_double_to_strs_rec(out, (double) info->ypixel_size,4))
       == NO_ERROR) &&
      ((err = add_long_to_strs_rec(out,(long) info->pixel_depth_bits))
       == NO_ERROR) &&
      ((err = add_long_to_strs_rec(out,(long) info->max_scans))
       == NO_ERROR))
    err = add_long_to_strs_rec(out,(long) info->num_echoes);

  return err;
}



/*--------------------------------------------------------------------------
sysadminuser - ???

If user is in sysadmin group, return 1, otherwise return 0.
--------------------------------------------------------------------------*/

int
sysadminuser( void )
{
  struct group *groupptr;
  struct passwd *passwdptr;
  char **groupmem;
  char *username;
  uid_t myuid;


  myuid = getuid();

  setpwent();
  passwdptr = getpwuid( myuid );
  username = passwdptr->pw_name;
  endpwent();

  setgrent();
  groupptr = getgrnam( "sysadmin" );
  groupmem = groupptr->gr_mem;
  endgrent();

  while (*groupmem && (strcmp(*groupmem,username) != 0)) {
    groupmem++;
  }

  if (!(*groupmem)) {
    return 0;
  }

  return 1;
}



/*--------------------------------------------------------------------------
showusage - ???
--------------------------------------------------------------------------*/

void
showusage( char *argv0 )
{
  if (restricted) {
    printf( "Usage: %s [ -d ] nria_header [...]\n", argv0 );
    printf( "\n-d = Show descriptive header line\n" );
  }
  else {
    printf( "Usage: %s [ -d ] [ -h ] nria_header [...]\n", argv0 );
    printf( "\n-d = Show descriptive header line\n" );
    printf( "\n-h = Show hidden, instead of public, information\n" );
  }

  printf("\nExamples :\n");
  printf("\nSummarize all mri headers in 1998 subdirectory:\n");
  printf("	%s 1998/mri*.nhdr\n", argv0);
  printf("\nSummarize all headers in current directory:\n");
  printf("	%s *.nhdr\n", argv0);
  printf("\nSummarize 10880's header:\n");
  printf("	%s mri10880am0.nhdr\n", argv0);
}



/*--------------------------------------------------------------------------
main - ???
--------------------------------------------------------------------------*/

int
main( int argc,
      char * argv[] )
{
  struct strs_rec out;
  int c;
  int header = 0;
  int hidden = 0;
  const char resoptions[] = "d";
  const char alloptions[] = "dh";
  const char *options = resoptions;

  if (sysadminuser())
    restricted = 0;

  if (!restricted) {
    options = alloptions;
  }

  while ((c = getopt( argc, argv, options )) != EOF) {
    switch (c) {

    case 'd':
      header = 1;
      break;

    case 'h':
      hidden = 1;
      break;

    case '?':
      showusage( argv[0] );
      exit( 1 );
    }
  }


  if (optind == argc) {
    showusage( argv[0] );
    exit( 0 );
  }    

  init_strs_rec( &out );
  if (header) {
    char *str_ary[12] = {
      "FNAME", "PID", "SID", "DOMRI", "MIN", "MAX",
      "Z", "X", "Y", "B", "SL", "E"
    };
    add_str_ary_to_strs_rec(&out,str_ary,12);
  }

  while (argc - optind) {
    if (wr_hdr_info( &out, argv[optind], hidden ) != NO_ERROR) {
      printf( "Error processing %s\n", argv[optind] );
    }
    optind++;
  }

  if (out.num_strs > (header ? 12 : 0)) {
    wr_lineup_lines( stdout, &out, 12, out.num_strs / 12, 100, 0 );
  }

  free_strs_rec(&out);

  return 0;
}
