/****************************************************************************
modify_nhdr.c - ???

Author:  Dan Widyono(?) 19980325

Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.

Modify nhdr:

  -a: anonymize:

  remove name, ssid, patient id, sex, age, height, weight
  for neatness, remove time of study, default data filename
  if desired by user, also remove patient mhcrc id, study id, date of study
  if reversible is desired, simply rotate (see rotate() below)
  study id, mhcrc id, and date of study


  saves old nhdr in <name>.nhdr_old in case of crash, etc.
****************************************************************************/

#ifndef lint
static const char rcsid[] = "$Id: modify_nhdr.c,v 1.6 1999/04/30 21:29:04 hughett Exp $ Release $Name: nria_1_0_6 $";
#endif

#include <stdio.h>
#include <stdlib.h>
#include <grp.h>
#include <sys/types.h>
#include <unistd.h>
#include <pwd.h>
#include <ctype.h>
#include <limits.h>

#include "typedef.h"
#include "mri_format.h"
#include "ia_hdr.h"
#include "dates.h"
#include "nria_lib.h"


/* modify_nhdr.c */
void rotate(char *ch);
int anonymize(struct hdr_rec *hdr, int anonlevel);
int sysadminuser(void);
void showusage(char *argv0);
int main(int argc, char *argv[]);



static int restricted = 1;


/*---------------------------------------------------------------------------
rotate - ???

FIXME:  I think that this function is duplicated elsewhere; it ought to be
a shared library function.  -PWH 19990430
---------------------------------------------------------------------------*/

void
rotate( 
    char *ch )
{
  char nums[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789nopqrstuvwxyzabcdefghijklmNOPQRSTUVWXYZABCDEFGHIJKLM!@#{}&()[]";
  char *index;
  int numchars = sizeof(nums) - 1;
  int rotateby = numchars / 2;


  while (*ch) {
    if (index = strchr( nums, *ch ))
      *ch = nums[((index - nums) + rotateby) % numchars];
    ch++;
  }
}



/*---------------------------------------------------------------------------
anonymize - ???
---------------------------------------------------------------------------*/

int
anonymize( 
    struct hdr_rec * hdr,
    int anonlevel )
{
  struct data_info_rec * info = &(hdr->data_info);


  /*  Always!  */
  *(info->patient.name) = '\0';
  *(info->patient.patient_id) = '\0';
  *(info->patient.ssnum) = '\0';
  info->patient.sex = 'X';
  info->patient.age = 0;
  info->patient.height = 0;
  info->patient.weight = 0;
  *(info->time_of_study) = '\0';
  *(info->default_datafn) = '\0';

  if (anonlevel > 1) {
    char tempstring[11];
    unsigned int tempint;


    if (*(info->patient.patient_num) == '\0') {
      free( info->patient.patient_num );
      info->patient.patient_num = strdup( "00000" );
    }
    rotate( info->patient.patient_num );

    /*  Make sure it's zero-filled and eight digits long.  */
    if (*(info->patient.study_num) == '\0') {
      free( info->patient.study_num );
      info->patient.study_num = strdup( "00000000" );
    }
    else if (sscanf( info->patient.study_num, "%d", &tempint ) == 1) {
      sprintf( tempstring, "%0.8d", tempint );
      free( info->patient.study_num );
      info->patient.study_num = strdup( tempstring );
    }
    rotate( info->patient.study_num );

    /*  Make canonical; also, size of string may increase.  */
    if (*(info->date_of_study) == '\0') {
      free( info->date_of_study );
      info->date_of_study = strdup( "00000000" );
    }
    else if (translateDate( info->date_of_study, &tempint,
		       &tempint, &tempint ) == 0) {
      canonizeDate( info->date_of_study, tempstring );
      free( info->date_of_study );
      info->date_of_study = strdup( tempstring );
    }
    rotate( info->date_of_study );

    /*
     * If we already placed coded information in comment field,
     *  then decode it and swap places with actual fields.
     */
    if (hdr->comment_size == 24 && hdr->comment &&
	*(hdr->comment) == '$' && *(hdr->comment + 1) == '$') {
      rotate( hdr->comment );

      strncpy( tempstring, info->patient.patient_num, 5 );
      strncpy( info->patient.patient_num, (hdr->comment + 2), 5 );
      strncpy( (hdr->comment + 2), tempstring, 5 );

      strncpy( tempstring, info->patient.study_num, 8 );
      strncpy( info->patient.study_num, (hdr->comment + 7), 8 );
      strncpy( (hdr->comment + 7), tempstring, 8 );

      strncpy( tempstring, info->date_of_study, 8 );
      strncpy( info->date_of_study, (hdr->comment + 15), 8 );
      strncpy( (hdr->comment + 15), tempstring, 8 );
    }
    else {
      if (hdr->comment_size && hdr->comment) {
	free( hdr->comment );
      }

      hdr->comment = (char *) malloc( 24 );
      strcpy( hdr->comment, "$$" );
      strncpy( hdr->comment + 2, info->patient.patient_num, 5 );
      strncpy( hdr->comment + 7, info->patient.study_num, 8 );
      strncpy( hdr->comment + 15, info->date_of_study, 8 );
      *(hdr->comment + 23) = '\0';
      hdr->comment_size = 24;

      strcpy( info->patient.patient_num, "00000" );
      strcpy( info->patient.study_num, "00000000" );
      strcpy( info->date_of_study, "00000000" );
    }

  }

  if (anonlevel > 2) {
    *(info->patient.patient_num) = '\0';
    *(info->patient.study_num) = '\0';
    *(info->date_of_study) = '\0';
  }

  return NO_ERROR;         /* Always succeeds */
}



/*---------------------------------------------------------------------------
sysadminuser - ???

If user is in sysadmin group, return 1, otherwise return 0.

FIXME:  I think this routine is duplicated elsewhere.  It probably
ought to be a shared library routine.  -PWH 19990430
---------------------------------------------------------------------------*/

int
sysadminuser( void )
{
  struct group *groupptr;
  struct passwd *passwdptr;
  char **groupmem;
  char *username;
  uid_t myuid;


  myuid = getuid();

  setpwent();
  passwdptr = getpwuid( myuid );
  username = passwdptr->pw_name;
  endpwent();

  setgrent();
  groupptr = getgrnam( "sysadmin" );
  groupmem = groupptr->gr_mem;
  endgrent();

  while (*groupmem && (strcmp(*groupmem,username) != 0)) {
    groupmem++;
  }

  if (!(*groupmem)) {
    return 0;
  }

  return 1;
}




/*---------------------------------------------------------------------------
showusage - ???
---------------------------------------------------------------------------*/

void
showusage(
    char *argv0 )
{
  printf("%s version $Name: nria_1_0_6 $\n", argv0);

  if (restricted)
    printf("Usage: %s [ -a ] [ -i mhcrc id ] nria_header [...]\n",
	   argv0);
  else {
    printf("Usage: %s [ -a | -aa ] [ -i mhcrc id ] [ -s study id ]\n",
	   argv0);
    printf("	[ -d date of mri ] nria_header [...]\n");
  }

  printf("\nExample : %s mri10010a.nhdr\n", argv0);
  printf("-a = Anonymize reversibly\n");
  if (!restricted) {
    printf("-aa = Anonymize completely and irreversibly all id info.\n");
  }

  printf("mhcrc id: 0-99999\n");
  if (!restricted) {
    printf("study id: 0-99999999\n");
    printf("date of mri: YYYYMMDD (e.g. 19980315)\n");
  }
}



/*---------------------------------------------------------------------------
main - ???
---------------------------------------------------------------------------*/

int
main( int argc, char *argv[] )
{
  int err;
  int retval = 0;
  int c;
  int anonlevel = 1;
  int anonhigh = 2;
  int overwrite = 0;
  struct hdr_rec hdr;
  char *id = NULL;
  int studyid = -1;
  int domri = -1;
  char tempname[PATH_MAX];
  char tempstring[20];
  const char alloptions[] = "aoi:s:d:";
  const char resoptions[] = "ai:";
  const char *options = resoptions;


  if (sysadminuser())
    restricted = 0;

  if (!restricted) {
    options = alloptions;
    anonhigh = 3;
  }

  while ((c = getopt( argc, argv, options )) != EOF) {
    switch (c) {

    case 'a':
      anonlevel++;
      if (anonlevel > anonhigh) {
	fprintf( stderr, "Largest supported anonlevel is %d.  Using %d.\n",
		 anonhigh, anonhigh );
	anonlevel = anonhigh;
      }
      break;

    case 'i': {
      char *ch = optarg;
      char *ch2;

      ch2 = id = strdup( optarg );
      while (*ch) {
	if (isalnum(*ch))
	  *ch2++ = *ch;
	ch++;
      }
      *ch2 = '\0';
      break;
    }

    case 's':
      if (sscanf( optarg, "%d", &studyid ) != 1) {
	fprintf( stderr, "Bad STUDYID option; ignoring.\n" );
	studyid = -1;
      }
      if (studyid < 0 || studyid > 99999999) {
	fprintf( stderr,
		 "STUDYID must be between 0 and 99999999 inclusive.\n" );
	studyid = -1;
      }
      break;

    case 'd':
      if (sscanf( optarg, "%d", &domri ) != 1) {
	fprintf( stderr, "Bad DOMRI option; ignoring.\n" );
	domri = -1;
      }
      if (domri < 1000000 && domri > 830101) {
	fprintf( stderr, "DOMRI anachronism; use YYYYMMDD, not YYMMDD.\n" );
	domri += 19000000;
	fprintf( stderr, "  Using %0.8d instead.\n", domri );
      }
      break;

    case 'o':
      overwrite = 1;
      break;

    case '?':
      showusage( argv[0] );
      exit( 1 );
    }
  }

  if (optind == argc) {
    showusage( argv[0] );
    exit( 0 );
  }

  while (optind < argc) {

    if ((err = rd_the_hdr( argv[optind], &hdr )) != NO_ERROR) {
      fprintf( stderr, "Error reading \"%s\".  Skipping.\n",  argv[optind] );
      optind++;
      retval = 1;
      continue;
    }

    /*  -a: anonymize  */
    if (anonymize( &hdr, anonlevel ))
      retval = 1;

    /*  -i: change mhcrc id  */
    if (id) {
      if (hdr.data_info.patient.patient_num)
	free( hdr.data_info.patient.patient_num );
      hdr.data_info.patient.patient_num = id;
    }

    /*  RESTRICTED: -s: change study id  */
    if (studyid != -1) {
      sprintf( tempstring, "%0.8d", studyid );
      if (hdr.data_info.patient.study_num)
	free( hdr.data_info.patient.study_num );
      hdr.data_info.patient.study_num = strdup( tempstring );
    }

    /*  RESTRICTED: -d: change date of mri field  */
    if (domri != -1) {
      if (domri < 19800101) {
	fprintf( stderr, "Illegal date.  Ignoring.\n" );
      }
      else {
	sprintf( tempstring, "%0.8d", domri );
	if (hdr.data_info.date_of_study)
	  free( hdr.data_info.date_of_study );
	hdr.data_info.date_of_study = strdup( tempstring );
      }
    }

    if (!overwrite) {
      strcpy( tempname, argv[optind] );
      strcat( tempname, "_old" );

      if (rename( argv[optind], tempname ) != 0) {
	fprintf( stderr, "Problem with temporary renaming of %s\n",
		 argv[optind] );
	fprintf( stderr, "  to %s.  Skipping.\n", tempname );
	optind++;
	retval = 1;
	continue;
      }
    }
      
    if ((err = wr_the_hdr( argv[optind], &hdr )) != NO_ERROR) {
      fprintf( stderr, "Error writing to \"%s\".  Skipping.\n",
	       argv[optind] );
      retval = 1;
    }

    optind++;
  }

  return retval;
}
