/****************************************************************************
rev_images.c - ???

Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.
****************************************************************************/

static const char rcsid[] = "$Id: rev_images.c,v 1.6 1999/07/07 18:07:14 hughett Exp $ Release $Name: nria_1_0_6 $";

#include <stdio.h>
#include <unistd.h>

#include <sys/types.h>

#include "mri_format.h"
#include "mark_hdr.h"
#include "ia_hdr.h"
#include "error_constants.h"
#include "nria_lib.h"


int debugFlag = 0;           /* Controls debugging output from library */


extern char *locate_suffix();

/* rev_images.c */
unsigned long get_row_bytes(unsigned long dimx, unsigned long pixel_depth);
unsigned long get_image_size(unsigned long dimx, unsigned long dimy,
    unsigned long pixel_depth);
int reverse_img(FILE *fout, FILE *fid, struct hdr_rec *hdr);
int reverse_images(char *fn);
void prog_usage(int argc, char *argv[]);
void main(int argc, char *argv[]);


/*------------------------------------------------------------------------
get_row_bytes - ???
------------------------------------------------------------------------*/

unsigned long 
get_row_bytes(dimx,pixel_depth)
 unsigned long dimx,pixel_depth;
{
 unsigned long row_bytes;

 row_bytes = dimx * pixel_depth;
 if (row_bytes % 8) row_bytes = row_bytes / 8 + 1;
   else row_bytes /= 8;
 return row_bytes;
}


/*------------------------------------------------------------------------
get_image_size - ???
------------------------------------------------------------------------*/

unsigned long 
get_image_size(dimx,dimy,pixel_depth)
 unsigned long dimx,dimy,pixel_depth;
{
 unsigned long size;

 size = dimy * get_row_bytes(dimx,pixel_depth);
 return size;
}


/*------------------------------------------------------------------------
reverse_img - ???
------------------------------------------------------------------------*/

int
reverse_img(fout,fid,hdr)
 FILE *fout,*fid;
 struct hdr_rec *hdr;
{
 long cnt,lcv,num_slice;
 unsigned char *buf;
 unsigned long size,slice_size;
 struct hdr_scan_entry_rec *cur;
 int err;

 slice_size = get_image_size(hdr->data_info.dimx,hdr->data_info.dimy,
              hdr->data_info.pixel_depth_bits);
 size = 0;
 if ((err = get_ptr(&buf,&size,slice_size,(size_t) sizeof(char))) != NO_ERROR) 
    {printf("error allocating the buffer memory...\n");
     return err;}
 num_slice = hdr->data_info.max_scans / hdr->data_info.num_echoes;
 for (lcv = num_slice - 1; lcv >= 0; lcv--)
    {
     for (cnt = 0; cnt < hdr->data_info.num_echoes; cnt++)
        {
         cur = &(hdr->scans[lcv * hdr->data_info.num_echoes + cnt]);
         if (fseek(fid,(size_t) cur->img_addr,0) != 0)
            {err = ILLEGAL_FILE_ADDRESS;
             break;}
         if (fread(buf,slice_size,(size_t) 1,fid) != 1)
            {err = READ_ERROR;
             printf("error reading the input images..."
		    "slice# = %ld echo# = %ld\n",
		    lcv + 1,cnt + 1);
             break;}
         if (fwrite(buf,slice_size,(size_t) 1,fout) != 1)
            {err = WRITE_ERROR;
             printf("error writing the output images..."
		    "slice# = %ld echo# = %ld\n",
		    lcv + 1,cnt + 1);
             break;}
        }
     if (err != NO_ERROR) break;
    }
 c_free(buf);
 return err;
}


/*------------------------------------------------------------------------
reverse_images - ???
------------------------------------------------------------------------*/

int
reverse_images(fn)
 char *fn;
{
 int err;
 struct hdr_rec hdr;
 char *suffix,*str,*str1,*str2;
 FILE *fout,*fid;

 init_hdr_rec(&hdr);
 str2 = str1 = str = NULL;
 if (((suffix = locate_suffix(fn)) == NULL) || 
     (str_cmp(suffix,".nhdr") != 0))
    {printf("The fn = %s must be of type *.nhdr\n",fn);
     return CANCEL;}
   else if (((err = rd_the_hdr(fn,&hdr)) == NO_ERROR) &&
            ((err = cp_str(&str,fn)) == NO_ERROR))
    {drop_suffix(str);
     if (((err = concat_strs(&str1,2,str,".img")) == NO_ERROR) &&
         ((err = concat_strs(&str2,2,str1,".tmp")) == NO_ERROR))
        {if ((fid = fopen(str1,"r")) == NULL)
            {printf("error opening the image file = %s\n",str1);
             err = FILE_ACCESS_ERROR;
            }
           else
            {
             if ((fout = fopen(str2,"w")) == NULL)
                {printf("error opening the temp image file = %s\n",str2);
                 err = FILE_ACCESS_ERROR;
                }
               else
                {if ((err = reverse_img(fout,fid,&hdr)) != NO_ERROR)
                    {printf("error when reversing the images...\n");
                     printf("image fn = %s  temp = %s err = %d\n",
                                 str1,str2,err);}
                 fclose(fout);
                 if (err != NO_ERROR) unlink(str2);
                }
             fclose(fid);
             if (err == NO_ERROR)
                {unlink(str1);
                 if (rename(str2,str1) != 0)
                    {printf("Failed to rename temp = %s to image fn = %s\n",
                                 str1,str2);
                     printf("Please rename it yourself again !!!\n");
                    }
                }
            }
        }
       else printf("error setting up the filenames...err = %d\n",err);
    }
 free_hdr_rec(&hdr);
 c_free(str);
 c_free(str1);
 c_free(str2);
 return err;
}


/*------------------------------------------------------------------------
prog_usage - ???
------------------------------------------------------------------------*/

void
prog_usage(argc,argv)
 int argc;
 char *argv[];
{
 if (argc != 2)
    {printf("Usage : %s \"*.nhdr\"\n",argv[0]);
     printf("1) INPUT : \"*.nhdr\"\n");
     printf("2) INPUT : \"*.img\"\n");
     printf("3) OUTPUT : \"*.img\"\n");
     exit(1);}
}


/*------------------------------------------------------------------------
main - ???
------------------------------------------------------------------------*/

void
main(argc,argv)
 int argc;
 char *argv[];
{
 int err;
 char **fns;
 unsigned int num_fns;
 unsigned long lcv;
 char *dirfn,fn[200];

 prog_usage(argc,argv);
 dirfn = (char *) locate_dirfn(argv[1]);
 drop_dirfn(argv[1]);
 if (dirfn == NULL) dirfn = ".";
 if ((err = get_fns(dirfn,NULL,0,0,NULL,&fns,&num_fns,1,argv[1],
         NULL,0,0,NULL)) != NO_ERROR) exit(err);
 if (num_fns == 0)
    {printf("Sorry...No file names...\n");
     sprintf(fn,"%s/%s",dirfn,argv[1]);
     printf("...Fn specification processed = %s\n",fn);
     exit(CANCEL);
    }
   else
    {printf("\n");
     for (lcv = 0; lcv < num_fns; lcv++)
        {sprintf(fn,"%s/%s",dirfn,fns[lcv]);
         printf("%ld) processing %s\n",lcv,fn);
         if ((err = reverse_images(fn)) != NO_ERROR)
            printf("...error processing %s\n",fn);
           else printf("...%s reversed successfully !!!\n",fn);
        }
    }
 exit(0);
}
