/****************************************************************************
scale2mrvbr.c - ???

Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.
****************************************************************************/

static const char rcsid[] = "$Id: scale2mrvbr.c,v 1.6 1999/07/07 18:07:14 hughett Exp $ Release $Name: nria_1_0_6 $";

#include <math.h>
#include <stdio.h>

#include <sys/types.h>

#include "ia_hdr.h"
#include "common.h"
#include "error_constants.h"
#include "mark_hdr.h"
#include "nria_lib.h"

#ifdef DEBUG
#  define PRT(x) x; fflush(stdout)
#else
#  define PRT(x) 
#endif

/* The library debugging code checks the following variable */
int debugFlag;


/* scale2mrvbr.c */
char *which_data_type(unsigned long type);
int scale_img_pixels(FILE *fid, FILE *outfid, char *imgfn, char *outimgfn,
    struct hdr_rec *hdr, unsigned long data_type, unsigned long num_bits);
int output_hdr(char *fn, struct hdr_rec *hdr, int is_replace);
int scan_file(char *fn, char *prefix, unsigned long data_type,
    unsigned long num_bits);
int prog_usage(int argc, char *argv[], unsigned long *data_type,
    unsigned long *num_bits);
void main(int argc, char *argv[]);



/*--------------------------------------------------------------------------
which_data_type - ???
--------------------------------------------------------------------------*/

char *
which_data_type(type)
     unsigned long type;
{
 if (type & INPUT_DATA_FLOAT)
   return "FLOAT";

 else if (type & INPUT_DATA_DOUBLE)
   return "DOUBLE";

 else if (type & INPUT_DATA_SIGNED)
   return "SIGNED INT";

 return "UNSIGNED INT";
}




/*--------------------------------------------------------------------------
scale_img_pixels - ???
--------------------------------------------------------------------------*/

int
scale_img_pixels(fid,outfid,imgfn,outimgfn,hdr,data_type,num_bits)
     FILE *fid,*outfid;
     char *imgfn,*outimgfn;
     struct hdr_rec *hdr;
     unsigned long data_type,num_bits;
{
  struct buf_2d_rec _2d,_2d1;
  struct data_info_rec *info;
  unsigned long lcv;
  double min,max,newmin,newmax,diff,diff1;
  int err;

  init_buf_2d_rec(&_2d);
  init_buf_2d_rec(&_2d1);
  info = &(hdr->data_info);

  if (((err = SetupBuf2dRec(&_2d,info->dimx,info->dimy,
			    info->pixel_depth_bits,
			    InputDataTypeToBufType(info))) !=NO_ERROR) ||
      ((err = SetupBuf2dRec(&_2d1,info->dimx,info->dimy,
			    num_bits,
			    VarInputDataTypeToBufType(data_type))) !=NO_ERROR))
    {printf("Memory error...\n");
     free_buf_2d_rec(&_2d);
     free_buf_2d_rec(&_2d1);
     return err;}
  GetMinMaxValues(&_2d1,&min,&max);

  /*  DRW 960401  Change for MRVBR images, must be 0-250, not 0-255 */
  /*  Program was named scale2mrvbr; doesn't collide with "scale_img_pixel" */
  max = 250;

  PRT(printf("scale_img_pixels min = %f max = %f\n",min,max));
  PRT(pr_buf_2d_rec(&_2d));
  PRT(pr_buf_2d_rec(&_2d1));

  if ((min <= hdr->data_info.min_pixel) &&
      (max >= hdr->data_info.max_pixel))
    {
      PRT(printf("...1\n"));
      newmin = hdr->data_info.min_pixel;
      newmax = hdr->data_info.max_pixel;
    }
  else
    {diff = hdr->data_info.max_pixel - hdr->data_info.min_pixel;
     diff1 = max - min;
     PRT(printf("...diff = %f  diff1 = %f\n",diff,diff1));
     if (min > hdr->data_info.min_pixel)
        {newmin = min;
         PRT(printf("...2\n"));
         if (diff <= diff1) newmax = newmin + diff; 
           else newmax = max;
        }
       else if (max < hdr->data_info.max_pixel)
        {
         PRT(printf("...3\n"));
         newmax = max;
         if (diff <= diff1) newmin = newmax - diff; 
           else newmin = min;
        }
       else
        {newmin = min;
         newmax = max;
        }
     PRT(printf("...newmin = %f  newmax = %f\n",newmin,newmax));
    }

  if ((hdr->data_info.min_pixel != newmin) ||
      (hdr->data_info.max_pixel != newmax)) {
    printf("	%ld slices, %ld-bit data (scaling to %ld-bit data)\n",
	   info->max_scans,
	   info->pixel_depth_bits,
	   num_bits);
    printf("	Pixel range: [%.0f - %.0f] (scaling to [%.0f - %.0f])\n",
	   hdr->data_info.min_pixel,
	   hdr->data_info.max_pixel,
	   newmin,
	   newmax);
  } else {
    printf("	%ld slices, %ld-bit data\n",
	   info->max_scans,
	   info->pixel_depth_bits);
    printf("	Pixel range: [%f - %f]\n",
	   hdr->data_info.min_pixel,
	   hdr->data_info.max_pixel);
  }
  printf("	Output filename: \"%s\"\n", outimgfn);


 for (lcv = 0; lcv < info->max_scans; lcv++)
    {if (fread(_2d.buf[0],_2d.total_bytes,1,fid) != 1)
        {printf("\nRead error occurred when reading \"%s\"...\n",imgfn);
         err = READ_ERROR;
         break;
        }
     if ((hdr->data_info.min_pixel != newmin) || 
         (hdr->data_info.max_pixel != newmax))
       {
	 PRT(printf("reading, scaling and writing slice# %d...\n",lcv+1););
	 printf("	Scaling slice #%ld\r", lcv + 1);
	 fflush(stdout);
	 Scale2dValues( &_2d1, &_2d,
		       hdr->data_info.min_pixel,
		       hdr->data_info.max_pixel,
		       0, (double) 0.0, (double) 0.0,
		       newmin, newmax );
       }
     else 
       {
	 PRT(printf("reading, assigning and writing slice# %d...\n",lcv+1););
	 printf("	Copying slice #%ld\r", lcv + 1);
	 fflush(stdout);
         Assign2d(&_2d1,&_2d);
       }
     if (fwrite(_2d1.buf[0],_2d1.total_bytes,1,outfid) != 1)
        {printf("Write error occurred when writing to %s...\n",outimgfn);
         err = WRITE_ERROR;
         break;
        }
    }
 if (err == NO_ERROR) {

#if 0
   printf("   Input data...min = %f  max = %f num_pixel_bits = %d data_type = %s\n",
	  hdr->data_info.min_pixel,hdr->data_info.max_pixel,
	  info->pixel_depth_bits,which_data_type(info->var_use_status));
   printf("   Output data...min = %f  max = %f num_pixel_bits = %d data_type = %s\n",
	  newmin,newmax,num_bits,which_data_type(data_type));
#endif

     info->pixel_depth_bits = num_bits;
     info->var_use_status &= ~INPUT_DATA_TYPE;
     info->var_use_status |= data_type;
     info->min_pixel = newmin;
     info->max_pixel = newmax;}

 free_buf_2d_rec(&_2d);
 free_buf_2d_rec(&_2d1);
 return err;
}




/*--------------------------------------------------------------------------
output_hdr - ???

FIXME:  This function is called elsewhere in this file by other functions
which appear to expect that is_replace actually means something.
    -PWH 19990430
--------------------------------------------------------------------------*/

int
output_hdr(
 char *fn,
 struct hdr_rec *hdr,
 int is_replace)
{
 struct mark_hdr mark;
 char *nhdrfn,*hdrfn,str[250];
 int err;

 strcpy(str,fn);
 drop_suffix(str);
 nhdrfn = hdrfn = NULL;
 if (((err = concat_strs(&nhdrfn,2,str,".nhdr")) != NO_ERROR) ||
     ((err = concat_strs(&hdrfn,2,str,".hdr")) != NO_ERROR))
    {c_free(nhdrfn);
     printf("Memory ERROR ...\n");
     c_free(hdrfn);
     return err;}
 if (((err = build_extra_file_info_rec(&(hdr->extra_info)))
              != NO_ERROR) ||
     ((err = BuildRiaHdr(hdr,0)) != NO_ERROR))
    {printf("ERROR generating the RIA header file = %s ... err = %d\n",
                 nhdrfn,err);
     c_free(nhdrfn);
     c_free(hdrfn);
     return err;}

 if ((err = wr_the_hdr(nhdrfn,hdr)) != NO_ERROR)
    {printf("ERROR writing the RIA header file = %s ... err = %d\n",nhdrfn,err);
     c_free(nhdrfn);
     c_free(hdrfn);
     return err;}

#if 0
 printf("...RIA header file %s was generated...\n",nhdrfn);
#endif

 init_mark_hdr(&mark);
 if (assign_nhdr_to_hdr(&mark,hdr) != NO_ERROR)
    printf("Warning : ERROR generating the Analyse header file = %s ... \n",
              hdrfn);
   else if (wr_hdr_file(hdrfn,&mark) != NO_ERROR) 
     printf("Warning : error writing the Analyse header file = %s ...\n",
              hdrfn);

#if 0
 else
   printf("...Analyse header file %s was generated...\n",hdrfn);
#endif

 c_free(nhdrfn);
 c_free(hdrfn);
 return err;
}


/*--------------------------------------------------------------------------
scan_file - ???
--------------------------------------------------------------------------*/

int
scan_file(fn,prefix,data_type,num_bits)
 char *fn,*prefix;
 unsigned long data_type,num_bits;
{
  struct hdr_rec hdr;
  FILE *fid,*outfid;
  char *imgfn,*outimgfn,*outhdrfn,*str,*pref;
  int err,is_replace;

  init_hdr_rec(&hdr);
  if ((err = rd_the_hdr(fn,&hdr)) != NO_ERROR)
    {printf("Error reading the input nhdr file %s\n",fn);
     return err;}

  if (get_strlen(prefix) == 0)
    {pref = "zzz-new-&&";
     is_replace = 1;}
  else
    {pref = prefix;
     is_replace = 0;}
  imgfn = outimgfn = outhdrfn = str = NULL;
  if (((err = cp_str(&str,fn)) == NO_ERROR) &&
      ((err = drop_suffix(str)) ==  NO_ERROR) &&
      ((err = drop_dirfn(str)) ==  NO_ERROR) &&
      ((err = concat_strs(&imgfn,2,str,".img")) ==  NO_ERROR) &&
      ((err = concat_strs(&outhdrfn,3,pref,str,".nhdr")) ==  NO_ERROR) &&
      ((err = concat_strs(&outimgfn,3,pref,str,".img")) ==  NO_ERROR))
    {fid = outfid = NULL;
     if ((fid = fopen(imgfn,"r")) == NULL)
       {printf("Error !!! : Unable to open up the input img file %s\n",imgfn);
	err = FILE_ACCESS_ERROR;}
     else if ((outfid = fopen(outimgfn,"w")) == NULL)
       {printf("Error !!! : Unable to open up the output img file %s\n",outimgfn);
	fclose(fid);
	err = FILE_ACCESS_ERROR;}
     else
       {
         if (((err = scale_img_pixels(fid,outfid,imgfn,outimgfn,
				      &hdr,data_type,
				      num_bits)) == NO_ERROR) &&
             ((err = output_hdr(outhdrfn,&hdr,is_replace)) == NO_ERROR))
	   {
	   }
         fclose(fid);
         fclose(outfid);
         if (err != NO_ERROR)
	   delete_file(outimgfn);
	 else if (is_replace) {
	   if (rename(imgfn,outimgfn) != 0) {
	     printf("Sorry..failed to rename %s to %s\n",outimgfn,imgfn);
#if 0
	     printf("output_img imgfn = %s\n",outimgfn);
#endif
	   }
#if 0
	 else
	   printf("output_img imgfn = %s\n",imgfn);
#endif
	 }
#if 0
	 else
	   printf("output_img imgfn = %s\n",outimgfn);
#endif
       }
    }
 c_free(str);
 c_free(imgfn);
 c_free(outhdrfn);
 c_free(outimgfn);
 free_hdr_rec(&hdr);

 return err;
}


/*--------------------------------------------------------------------------
prog_usage - ???
--------------------------------------------------------------------------*/

int
prog_usage(argc,argv,data_type,num_bits)
 int argc;
 char *argv[];
 unsigned long *data_type,*num_bits;
{
 if (argc == 5)
    {sscanf(argv[3],"%ld",data_type);
     sscanf(argv[4],"%ld",num_bits);
     if (*data_type == 1)
        {if (*num_bits > 32) *num_bits = 32;
         *data_type = INPUT_DATA_SIGNED;
        }
       else if (*data_type == 2)
        {*num_bits = 32;
         *data_type = INPUT_DATA_FLOAT;
        }
       else if (*data_type == 3)
        {*num_bits = 64;
         *data_type = INPUT_DATA_DOUBLE;
        }
       else
        {if (*num_bits > 32) *num_bits = 32;
         *data_type = 0;
        }
     return NO_ERROR;}

 printf("\nUsage : %s fn.nhdr prefix data_type num_output_pixel_bits\n",argv[0]);
 printf("\n");
 printf("if data_type == 1 then pixel is signed, use num_output_pixel_bits <= 32\n");
 printf("if data_type == 2 then pixel is float, use default num_output_pixel_bits = 32\n");
 printf("if data_type == 3 then pixel is double, use default num_output_pixel_bits = 64\n");
 printf("otherwise data_type = 0 and pixel is unsigned, use num_output_pixel_bits <= 32\n");
 printf("\n");
 printf("Example 1 : %s fn.nhdr new 0 16\n",argv[0]);
 printf("   Input ==> 1) fn.nhdr\n");
 printf("             2) fn.img\n");
 printf("   Output ==> 1) newfn.hdr\n");
 printf("              2) newfn.nhdr\n");
 printf("              3) newfn.img\n\n");
 printf("\n");
 printf("Note : if you use '*' in the file specification, please\n");
 printf("       enclose that file specification by double quotes.\n");
 exit(1);
}


/*--------------------------------------------------------------------------
main - ???
--------------------------------------------------------------------------*/

void
main(argc,argv)
     int argc;
     char *argv[];
{
  int err;
  char **fns;
  unsigned int num_fns;
  unsigned long lcv;
  char *dirfn, fn[200];
  unsigned long data_type,num_bits;
  char *prefix;



  prog_usage(argc,argv,&data_type,&num_bits);
  prefix = argv[2];

  /*  DRW 960401  scale2mrvbr changes: default values */
  data_type = 0;
  num_bits = 8;
  prefix = "8";

  dirfn = (char *) locate_dirfn(argv[1]);
  drop_dirfn(argv[1]);
  if (dirfn == NULL) dirfn = ".";
  if ((err = get_fns(dirfn,NULL,0,0,NULL,&fns,&num_fns,1,argv[1],
		     NULL,0,0,NULL)) != NO_ERROR) exit(err);
  if (num_fns == 0)
    {printf("Sorry...No file names...\n");
     sprintf(fn,"%s/%s",dirfn,argv[1]);
     printf("...Fn specification processed = %s\n",fn);
   }
  else
    {printf("\n");
     for (lcv = 0; lcv < num_fns; lcv++)
       {sprintf(fn,"%s/%s",dirfn,fns[lcv]);
	printf("Scaling \"%s\" ...\n",fn);
	if ((err = scan_file(fn,prefix,data_type,num_bits)) != NO_ERROR)
	  {printf("	ERROR in processing %s\n",fn);}
      }
   }
  exit(1);
}
